Package: equatiomatic
Title: Transform Models into 'LaTeX' Equations
Version: 0.4.8
Authors@R: 
    c(person(given = "Daniel",
             family = "Anderson",
             role = "aut",
             email = "daniela@uoregon.edu",
             comment = c(ORCID = "0000-0003-4699-4680")),
      person(given = "Andrew",
             family = "Heiss",
             role = "aut",
             email = "andrew@andrewheiss.com",
             comment = c(ORCID = "0000-0002-3948-3914")),
      person(given = "Jay",
             family = "Sumners",
             role = "aut",
             email = "jay.sumners@gmail.com"),
      person(given = "Joshua",
             family = "Rosenberg",
             role = "ctb",
             email = "jmrosenberg@utk.edu",
             comment = c(ORCID = "0000-0003-2170-0447")),
      person(given = "Jonathan",
             family = "Sidi",
             role = "ctb",
             email = "yonis@metrumrg.com",
             comment = c(ORCID = "0000-0002-4222-1819")),
      person(given = "Ellis",
             family = "Hughes",
             role = "ctb",
             email = "ellishughes@live.com",
             comment = c(ORCID = "0000-0003-0637-4436")),
      person(given = "Thomas",
             family = "Fung",
             role = "ctb",
             email = "thomas.fung.dr@gmail.com",
             comment = c(ORCID = "0000-0003-2601-0728")),
      person(given = "Reza",
             family = "Norouzian",
             role = "ctb",
             email = "rnorouzian@gmail.com",
             comment = c(ORCID = "0000-0002-0531-6886")),
      person(given = "Indrajeet",
             family = "Patil",
             role = "ctb",
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531")),
      person(given = "Quinn",
             family = "White",
             role = "ctb",
             email = "quinnarlise@gmail.com",
             comment = c(ORCID = "0000-0001-5399-0237")),
      person(given = "David",
             family = "Kane",
             role = "ctb",
             email = "dave.kane@gmail.com"),
      person(given = "Philippe",
             family = "Grosjean",
             role = "cre",
             email = "phgrosjean@sciviews.org",
             comment = c(ORCID = "0000-0002-2694-9471")),
      person(given = "Teun",
             family = "van den Brand",
             role = "ctb",
             email = "t.vandenbrand@nki.nl",
             comment = c(ORCID = "0000-0002-9335-7468")),
      person(given = "Guyliann",
             family = "Engels",
             role = "ctb",
             email = "guyliann.engels@umons.ac.be",
             comment = c(ORCID = "0000-0001-9514-1014")))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Description: The goal of 'equatiomatic' is to reduce the pain
    associated with writing 'LaTeX' formulas from fitted models. The
    primary function of the package, extract_eq(), takes a fitted model
    object as its input and returns the corresponding 'LaTeX' code for the
    model.
License: CC BY 4.0
Depends: R (>= 3.3.0)
URL: https://github.com/datalorax/equatiomatic,
        https://datalorax.github.io/equatiomatic/
BugReports: https://github.com/datalorax/equatiomatic/issues
Imports: broom (>= 0.7.0), broom.mixed, shiny, knitr, rmarkdown, stats,
        utils
Suggests: covr, shinyWidgets, forecast (>= 8.13), ggplot2 (>= 3.3.3),
        latex2exp (>= 0.4.0), lme4, MASS, ordinal, parsnip, recipes,
        workflows, testthat (>= 3.0.0), gtsummary, spelling
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Config/testthat/edition: 3
Config/testthat/parallel: true
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-10 16:47:37 UTC; phgrosjean
Author: Daniel Anderson [aut] (ORCID: <https://orcid.org/0000-0003-4699-4680>),
  Andrew Heiss [aut] (ORCID: <https://orcid.org/0000-0002-3948-3914>),
  Jay Sumners [aut],
  Joshua Rosenberg [ctb] (ORCID: <https://orcid.org/0000-0003-2170-0447>),
  Jonathan Sidi [ctb] (ORCID: <https://orcid.org/0000-0002-4222-1819>),
  Ellis Hughes [ctb] (ORCID: <https://orcid.org/0000-0003-0637-4436>),
  Thomas Fung [ctb] (ORCID: <https://orcid.org/0000-0003-2601-0728>),
  Reza Norouzian [ctb] (ORCID: <https://orcid.org/0000-0002-0531-6886>),
  Indrajeet Patil [ctb] (ORCID: <https://orcid.org/0000-0003-1995-6531>),
  Quinn White [ctb] (ORCID: <https://orcid.org/0000-0001-5399-0237>),
  David Kane [ctb],
  Philippe Grosjean [cre] (ORCID:
    <https://orcid.org/0000-0002-2694-9471>),
  Teun van den Brand [ctb] (ORCID:
    <https://orcid.org/0000-0002-9335-7468>),
  Guyliann Engels [ctb] (ORCID: <https://orcid.org/0000-0001-9514-1014>)
Repository: CRAN
Date/Publication: 2026-04-10 17:20:02 UTC
Built: R 4.5.2; ; 2026-04-10 18:15:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(create_l1,glmerMod)
S3method(equation,character)
S3method(equation,default)
S3method(extract_eq,default)
S3method(extract_eq,forecast_ARIMA)
S3method(extract_eq,glmerMod)
S3method(extract_eq,list)
S3method(extract_eq,lmerMod)
S3method(extract_eq,model_fit)
S3method(extract_eq,workflow)
S3method(format,equation)
S3method(knit_print,equation)
S3method(print,equation)
S3method(print,inline_equation)
export(eqOutput)
export(eq_)
export(eq__)
export(equation)
export(extract_eq)
export(preview_eq)
export(renderEq)
importFrom(knitr,asis_output)
importFrom(knitr,is_html_output)
importFrom(knitr,is_latex_output)
importFrom(knitr,knit_print)
importFrom(shiny,createRenderFunction)
importFrom(shiny,helpText)
importFrom(shiny,htmlOutput)
importFrom(shiny,installExprFunction)
importFrom(shiny,withMathJax)
importFrom(stats,coef)
importFrom(stats,formula)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(utils,tail)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# equatiomatic <img src="man/figures/logo.png" align="right"/>

<!-- badges: start -->

[![lifecycle](https://lifecycle.r-lib.org/articles/figures/lifecycle-stable.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R build
status](https://github.com/datalorax/equatiomatic/workflows/R-CMD-check/badge.svg)](https://github.com/datalorax/equatiomatic/actions)
[![Codecov test
coverage](https://codecov.io/gh/datalorax/equatiomatic/branch/master/graph/badge.svg)](https://app.codecov.io/gh/datalorax/equatiomatic?branch=master)
[![Covrpage
Summary](https://img.shields.io/badge/covrpage-Last_Build_2020_08_24-brightgreen.svg)](https://github.com/datalorax/equatiomatic/tree/master/tests)
[![](https://cranlogs.r-pkg.org/badges/grand-total/equatiomatic)](https://cranlogs.r-pkg.org/badges/grand-total/equatiomatic)
<!-- badges: end -->

The goal of {equatiomatic} is to reduce the pain associated with writing
LaTeX code from a fitted model. The package aims to support any model
supported by [{broom}](https://cran.r-project.org/package=broom). See
the [introduction to
equatiomatic](https://datalorax.github.io/equatiomatic/articles/equatiomatic.html#other-models)
for currently supported models.

## Installation

Install from CRAN with:

``` r
install.packages("equatiomatic")
```

Or get the development version from GitHub with:

``` r
remotes::install_github("datalorax/equatiomatic")
```

## Basic usage

![](https://github.com/datalorax/equatiomatic/raw/master/img/equatiomatic.gif)

The gif above shows the basic functionality.

To convert a model to LaTeX, feed a model object to `extract_eq()`:

``` r
library(equatiomatic)

# Fit a simple model
mod1 <- lm(mpg ~ cyl + disp, data = mtcars)

# Give the results to extract_eq
extract_eq(mod1)
```

    #> $$
    #> \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    #> $$

$$\operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon$$

The model can be built in any standard way. It can handle shortcut
syntax:

``` r
mod2 <- lm(mpg ~ ., data = mtcars)
extract_eq(mod2)
```

    #> $$
    #> \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp}) + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs}) + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
    #> $$

$$\operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp}) + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs}) + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon$$

When using categorical variables, it will include the levels of the
variables as subscripts.

``` r
data("penguins", package = "equatiomatic")
mod3 <- lm(body_mass_g ~ bill_length_mm + species, data = penguins)
extract_eq(mod3)
```

    #> $$
    #> \operatorname{body\_mass\_g} = \alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}}) + \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}}) + \epsilon
    #> $$

$$\operatorname{body\_mass\_g} = \alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}}) + \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}}) + \epsilon$$

It helpfully preserves the order the variables are supplied in the
formula:

``` r
set.seed(8675309)
d <- data.frame(cat1  = rep(letters[1:3], 100),
                cat2  = rep(LETTERS[1:3], each = 100),
                cont1 = rnorm(300, 100, 1),
                cont2 = rnorm(300, 50, 5),
                out   = rnorm(300, 10, 0.5))
mod4 <- lm(out ~ cont1 + cat2 + cont2 + cat1, data = d)
extract_eq(mod4)
```

    #> $$
    #> \operatorname{out} = \alpha + \beta_{1}(\operatorname{cont1}) + \beta_{2}(\operatorname{cat2}_{\operatorname{B}}) + \beta_{3}(\operatorname{cat2}_{\operatorname{C}}) + \beta_{4}(\operatorname{cont2}) + \beta_{5}(\operatorname{cat1}_{\operatorname{b}}) + \beta_{6}(\operatorname{cat1}_{\operatorname{c}}) + \epsilon
    #> $$

$$\operatorname{out} = \alpha + \beta_{1}(\operatorname{cont1}) + \beta_{2}(\operatorname{cat2}_{\operatorname{B}}) + \beta_{3}(\operatorname{cat2}_{\operatorname{C}}) + \beta_{4}(\operatorname{cont2}) + \beta_{5}(\operatorname{cat1}_{\operatorname{b}}) + \beta_{6}(\operatorname{cat1}_{\operatorname{c}}) + \epsilon$$

## Appearance

You can wrap the equations so that a specified number of terms appear on
the right-hand side of the equation using `terms_per_line` (defaults to
4):

``` r
extract_eq(mod2, wrap = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\ + \\
    #> &\quad \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\ + \\
    #> &\quad \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\ + \\
&\quad \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\ + \\
&\quad \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
\end{aligned}$$

``` r
extract_eq(mod2, wrap = TRUE, terms_per_line = 6)
```

    #> $$
    #> \begin{aligned}
    #> \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp}) + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt})\ + \\
    #> &\quad \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs}) + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp}) + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt})\ + \\
&\quad \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs}) + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
\end{aligned}$$

When wrapping, you can change whether the lines end with trailing math
operators like `+` (the default), or if they should begin with them
using `operator_location = "end"` or `operator_location = "start"`:

``` r
extract_eq(mod2, wrap = TRUE, terms_per_line = 4, operator_location = "start")
```

    #> $$
    #> \begin{aligned}
    #> \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\\
    #> &\quad + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\\
    #> &\quad + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\\
&\quad + \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\\
&\quad + \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
\end{aligned}$$

By default, all text in the equation is wrapped in `\operatorname{}`.
You can optionally have the variables themselves be italicized (i.e. not
be wrapped in `\operatorname{}`) with `ital_vars = TRUE`:

``` r
extract_eq(mod2, wrap = TRUE, ital_vars = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> mpg &= \alpha + \beta_{1}(cyl) + \beta_{2}(disp) + \beta_{3}(hp)\ + \\
    #> &\quad \beta_{4}(drat) + \beta_{5}(wt) + \beta_{6}(qsec) + \beta_{7}(vs)\ + \\
    #> &\quad \beta_{8}(am) + \beta_{9}(gear) + \beta_{10}(carb) + \epsilon
    #> \end{aligned}
    #> $$

$$\begin{aligned}
mpg &= \alpha + \beta_{1}(cyl) + \beta_{2}(disp) + \beta_{3}(hp)\ + \\
&\quad \beta_{4}(drat) + \beta_{5}(wt) + \beta_{6}(qsec) + \beta_{7}(vs)\ + \\
&\quad \beta_{8}(am) + \beta_{9}(gear) + \beta_{10}(carb) + \epsilon
\end{aligned}$$

## R Markdown and previewing

If you include `extract_eq()` in an R Markdown chunk, {knitr} will
render the equation. If you’d like to see the LaTeX code wrap the call
in `print()`.

You can also use the `preview_eq()` function to preview the equation in
RStudio:

``` r
preview_eq(mod1)
```

![](man/figures/preview1.png)

Both `extract_eq()` and `preview_eq()` work with base R or {magrittr}
pipes, so you can do something like this:

``` r
#library(magrittr)  # if you want to use %>% instead of |>

extract_eq(mod1) |>
  preview_eq()
# Or simply: preview_eq(mod1)
```

## Extra options

There are several extra options you can enable with additional arguments
to `extract_eq()`.

### Actual coefficients

You can return actual numeric coefficients instead of Greek letters with
`use_coefs = TRUE`:

``` r
extract_eq(mod1, use_coefs = TRUE)
```

    #> $$
    #> \operatorname{\widehat{mpg}} = 34.66 - 1.59(\operatorname{cyl}) - 0.02(\operatorname{disp})
    #> $$

$$\operatorname{\widehat{mpg}} = 34.66 - 1.59(\operatorname{cyl}) - 0.02(\operatorname{disp})$$

By default, it will remove doubled operators like “+ -”, but you can
keep those in (which is often useful for teaching) with
`fix_signs = FALSE`:

``` r
extract_eq(mod1, use_coefs = TRUE, fix_signs = FALSE)
```

    #> $$
    #> \operatorname{\widehat{mpg}} = 34.66 + -1.59(\operatorname{cyl}) + -0.02(\operatorname{disp})
    #> $$

$$\operatorname{\widehat{mpg}} = 34.66 + -1.59(\operatorname{cyl}) + -0.02(\operatorname{disp})$$

This works in longer wrapped equations:

``` r
extract_eq(mod2, wrap = TRUE, terms_per_line = 3, use_coefs = TRUE,
  fix_signs = FALSE)
```

    #> $$
    #> \begin{aligned}
    #> \operatorname{\widehat{mpg}} &= 12.3 + -0.11(\operatorname{cyl}) + 0.01(\operatorname{disp})\ + \\
    #> &\quad -0.02(\operatorname{hp}) + 0.79(\operatorname{drat}) + -3.72(\operatorname{wt})\ + \\
    #> &\quad 0.82(\operatorname{qsec}) + 0.32(\operatorname{vs}) + 2.52(\operatorname{am})\ + \\
    #> &\quad 0.66(\operatorname{gear}) + -0.2(\operatorname{carb})
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\operatorname{\widehat{mpg}} &= 12.3 + -0.11(\operatorname{cyl}) + 0.01(\operatorname{disp})\ + \\
&\quad -0.02(\operatorname{hp}) + 0.79(\operatorname{drat}) + -3.72(\operatorname{wt})\ + \\
&\quad 0.82(\operatorname{qsec}) + 0.32(\operatorname{vs}) + 2.52(\operatorname{am})\ + \\
&\quad 0.66(\operatorname{gear}) + -0.2(\operatorname{carb})
\end{aligned}$$

## Beyond `lm()`

You’re not limited to just `lm` models! {equatiomatic} supports many
other models, including logistic regression, probit regression, and
ordered logistic regression (with `MASS::polr()`).

### Logistic regression with `glm()`

``` r
model_logit <- glm(sex ~ bill_length_mm + species, data = penguins,
  family = binomial(link = "logit"))
extract_eq(model_logit, wrap = TRUE, terms_per_line = 3)
```

    #> $$
    #> \begin{aligned}
    #> \log\left[ \frac { P( \operatorname{sex} = \operatorname{male} ) }{ 1 - P( \operatorname{sex} = \operatorname{male} ) } \right] &= \alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}})\ + \\
    #> &\quad \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}})
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\log\left[ \frac { P( \operatorname{sex} = \operatorname{male} ) }{ 1 - P( \operatorname{sex} = \operatorname{male} ) } \right] &= \alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}})\ + \\
&\quad \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}})
\end{aligned}$$

### Probit regression with `glm()`

``` r
model_probit <- glm(sex ~ bill_length_mm + species, data = penguins,
  family = binomial(link = "probit"))
extract_eq(model_probit, wrap = TRUE, terms_per_line = 3)
```

    #> $$
    #> \begin{aligned}
    #> P( \operatorname{sex} = \operatorname{male} ) &= \Phi[\alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}})\ + \\
    #> &\qquad\ \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}})]
    #> \end{aligned}
    #> $$

$$\begin{aligned}
P( \operatorname{sex} = \operatorname{male} ) &= \Phi[\alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}})\ + \\
&\qquad\ \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}})]
\end{aligned}$$

### Ordered logistic regression with `MASS::polr()`

``` r
set.seed(1234)
df <- data.frame(
  outcome      = ordered(rep(LETTERS[1:3], 100), levels = LETTERS[1:3]),
  continuous_1 = rnorm(300, 100, 1),
  continuous_2 = rnorm(300, 50, 5))

model_ologit <- MASS::polr(outcome ~ continuous_1 + continuous_2,
  data = df, Hess = TRUE, method = "logistic")
model_oprobit <- MASS::polr(outcome ~ continuous_1 + continuous_2,
  data = df, Hess = TRUE, method = "probit")

extract_eq(model_ologit, wrap = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2}) \\
    #> \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2}) \\
\log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})
\end{aligned}$$

``` r
extract_eq(model_oprobit, wrap = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})] \\
    #> P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})]
    #> \end{aligned}
    #> $$

$$\begin{aligned}
P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})] \\
P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})]
\end{aligned}$$

### Ordered regression (logit and probit) with `ordinal::clm()`

``` r
set.seed(1234)
df <- data.frame(
  outcome      = ordered(rep(LETTERS[1:3], 100), levels = LETTERS[1:3]),
  continuous_1 = rnorm(300, 1, 1),
  continuous_2 = rnorm(300, 5, 5))

model_ologit <- ordinal::clm(outcome ~ continuous_1 + continuous_2,
  data = df, link = "logit")
model_oprobit <- ordinal::clm(outcome ~ continuous_1 + continuous_2,
  data = df, link = "probit")

extract_eq(model_ologit, wrap = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2}) \\
    #> \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})
    #> \end{aligned}
    #> $$

$$\begin{aligned}
\log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2}) \\
\log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})
\end{aligned}$$

``` r
extract_eq(model_oprobit, wrap = TRUE)
```

    #> $$
    #> \begin{aligned}
    #> P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})] \\
    #> P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})]
    #> \end{aligned}
    #> $$

$$\begin{aligned}
P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})] \\
P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})]
\end{aligned}$$

## Extension

If you would like to contribute to this package, we’d love your help! We
are particularly interested in extending to more models. We hope to
support any model supported by
[{broom}](https://cran.r-project.org/package=broom) in the future.

## Code of Conduct

Please note that the ‘equatiomatic’ project is released with a
[Contributor Code of
Conduct](https://github.com/datalorax/equatiomatic/blob/master/CODE_OF_CONDUCT.md).
By contributing to this project, you agree to abide by its terms.

## A note of appreciation

We’d like to thank the authors of the
[{palmerpenguins}](https://allisonhorst.github.io/palmerpenguins/index.html)
dataset for generously allowing us to incorporate the `penguins` dataset
in our package for example usage.

Horst AM, Hill AP, Gorman KB (2020). *palmerpenguins: Palmer Archipelago
(Antarctica) penguin data*. R package version 0.1.0.
<https://allisonhorst.github.io/palmerpenguins/>

Tests and Coverage
================
14 March, 2022 16:40:53

  - [Coverage](#coverage)
  - [Unit Tests](#unit-tests)

This output is created by
[covrpage](https://github.com/yonicd/covrpage).

## Coverage

Coverage summary is created using the
[covr](https://github.com/r-lib/covr) package.

| Object                                                    | Coverage (%) |
| :-------------------------------------------------------- | :----------: |
| equatiomatic                                              |    95.80     |
| [R/shiny.R](../R/shiny.R)                                 |     0.00     |
| [R/print.R](../R/print.R)                                 |    24.00     |
| [R/distribution-wrapping.R](../R/distribution-wrapping.R) |    90.91     |
| [R/extract\_lhs.R](../R/extract_lhs.R)                    |    94.74     |
| [R/merMod.R](../R/merMod.R)                               |    95.25     |
| [R/extract\_eq.R](../R/extract_eq.R)                      |    96.41     |
| [R/utils.R](../R/utils.R)                                 |    97.87     |
| [R/extract\_rhs.R](../R/extract_rhs.R)                    |    99.31     |
| [R/create\_eq.R](../R/create_eq.R)                        |    99.51     |
| [R/helpers\_forecast.R](../R/helpers_forecast.R)          |    100.00    |

<br>

## Unit Tests

Unit Test summary is created using the
[testthat](https://github.com/r-lib/testthat) package.

| file                                                              |  n |  time | error | failed | skipped | warning | icon |
| :---------------------------------------------------------------- | -: | ----: | ----: | -----: | ------: | ------: | :--- |
| [test-clm.R](testthat/test-clm.R)                                 | 19 | 0.057 |     0 |      0 |       0 |       0 |      |
| [test-fontsize.R](testthat/test-fontsize.R)                       |  9 | 0.004 |     0 |      0 |       0 |       0 |      |
| [test-forecast-arima.R](testthat/test-forecast-arima.R)           |  4 | 0.004 |     0 |      0 |       0 |       0 |      |
| [test-glm.R](testthat/test-glm.R)                                 | 23 | 0.009 |     0 |      0 |       0 |       0 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R)                       |  9 | 0.003 |     0 |      0 |       0 |       0 |      |
| [test-lm.R](testthat/test-lm.R)                                   | 18 | 0.010 |     0 |      0 |       0 |       0 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R)                         | 41 | 0.012 |     0 |      0 |       0 |       0 |      |
| [test-polr.R](testthat/test-polr.R)                               | 13 | 0.004 |     0 |      0 |       0 |       0 |      |
| [test-print.R](testthat/test-print.R)                             |  4 | 0.002 |     0 |      0 |       0 |       0 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R)             | 19 | 0.007 |     0 |      0 |       0 |       2 | ⚠️   |
| [test-utils.R](testthat/test-utils.R)                             |  8 | 0.003 |     0 |      0 |       0 |       0 |      |
| [test-wrapping-formatting.R](testthat/test-wrapping-formatting.R) | 10 | 0.004 |     0 |      0 |       0 |       0 |      |

<details open>

<summary> Show Detailed Test Results </summary>

| file                                                               | context | test                                                        | status  | n |  time | icon |
| :----------------------------------------------------------------- | :------ | :---------------------------------------------------------- | :------ | -: | ----: | :--- |
| [test-clm.R](testthat/test-clm.R#)                                 |         | colorizing works                                            | PASS    | 1 | 0.038 |      |
| [test-clm.R](testthat/test-clm.R#)                                 |         | Renaming Variables works                                    | PASS    | 2 | 0.012 |      |
| [test-clm.R](testthat/test-clm.R#)                                 |         | Math extraction works                                       | PASS    | 4 | 0.002 |      |
| [test-clm.R](testthat/test-clm.R#)                                 |         | Collapsing clm factors works                                | PASS    | 6 | 0.002 |      |
| [test-clm.R](testthat/test-clm.R#)                                 |         | Ordered models with clm work                                | PASS    | 5 | 0.002 |      |
| [test-clm.R](testthat/test-clm.R#)                                 |         | Unsupported CLMs create a message                           | PASS    | 1 | 0.001 |      |
| [test-fontsize.R](testthat/test-fontsize.R#)                       |         | font-size changes, lm                                       | PASS    | 3 | 0.001 |      |
| [test-fontsize.R](testthat/test-fontsize.R#)                       |         | font-size changes, lmer                                     | PASS    | 3 | 0.001 |      |
| [test-fontsize.R](testthat/test-fontsize.R#)                       |         | font-size changes, arima                                    | PASS    | 3 | 0.002 |      |
| [test-forecast-arima.R](testthat/test-forecast-arima.R#)           |         | Basic ARIMA model functions                                 | PASS    | 2 | 0.002 |      |
| [test-forecast-arima.R](testthat/test-forecast-arima.R#)           |         | Regression w/ ARIMA Errors functions                        | PASS    | 2 | 0.002 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | colorizing works                                            | PASS    | 4 | 0.002 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Renaming Variables works                                    | PASS    | 1 | 0.000 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Math extraction works                                       | PASS    | 2 | 0.001 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Collapsing glm factors works                                | PASS    | 4 | 0.001 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Logistic regression works                                   | PASS    | 1 | 0.000 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Probit regression works                                     | PASS    | 2 | 0.001 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Unsupported GLMs create a message                           | PASS    | 1 | 0.000 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Distribution-based equations work                           | PASS    | 3 | 0.002 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | Weights work                                                | PASS    | 1 | 0.001 |      |
| [test-glm.R](testthat/test-glm.R#)                                 |         | non-binomial regression works                               | PASS    | 4 | 0.001 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | Checking for random/fixed effects works                     | PASS    | 1 | 0.000 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | colorizing works                                            | PASS    | 2 | 0.000 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | Renaming Variables works                                    | PASS    | 1 | 0.000 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | Standard Poisson regression models work                     | PASS    | 2 | 0.001 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | Poisson regression models with an offset work               | PASS    | 2 | 0.001 |      |
| [test-glmerMod.R](testthat/test-glmerMod.R#)                       |         | Binomial Logistic Regression models work                    | PASS    | 1 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Overlapping names don’t result in an error                  | PASS    | 1 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Dropping intercept notation works                           | PASS    | 1 | 0.000 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | colorizing works                                            | PASS    | 1 | 0.000 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Renaming Variables works                                    | PASS    | 1 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Math extraction works                                       | PASS    | 3 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Collapsing lm factors works                                 | PASS    | 2 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Labeling works                                              | PASS    | 1 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Simple lm models work                                       | PASS    | 3 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Interactions work                                           | PASS    | 2 | 0.002 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Custom Greek works                                          | PASS    | 2 | 0.001 |      |
| [test-lm.R](testthat/test-lm.R#)                                   |         | Hat is escaped correctly                                    | PASS    | 1 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Checking for random/fixed effects works                     | PASS    | 1 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | colorizing works                                            | PASS    | 3 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Math extraction works                                       | PASS    | 4 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Implicit ID variables are handled                           | PASS    | 1 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Renaming Variables works                                    | PASS    | 1 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Really big models work                                      | PASS    | 1 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Categorical variable level parsing works (from issue \#140) | PASS    | 1 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Unconditional lmer models work                              | PASS    | 3 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Level 1 predictors work                                     | PASS    | 2 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Mean separate works as expected                             | PASS    | 2 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Wrapping works as expected                                  | PASS    | 1 | 0.000 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Unstructured variance-covariances work as expected          | PASS    | 5 | 0.002 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Group-level predictors work as expected                     | PASS    | 3 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Interactions work as expected                               | PASS    | 5 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Alternate random effect VCV structures work                 | PASS    | 3 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | Nested model syntax works                                   | PASS    | 3 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | use\_coef works                                             | PASS    | 1 | 0.001 |      |
| [test-lmerMod.R](testthat/test-lmerMod.R#)                         |         | return variances works                                      | PASS    | 1 | 0.001 |      |
| [test-polr.R](testthat/test-polr.R#)                               |         | colorizing works                                            | PASS    | 1 | 0.001 |      |
| [test-polr.R](testthat/test-polr.R#)                               |         | Renaming Variables works                                    | PASS    | 1 | 0.000 |      |
| [test-polr.R](testthat/test-polr.R#)                               |         | Math extraction works                                       | PASS    | 2 | 0.000 |      |
| [test-polr.R](testthat/test-polr.R#)                               |         | Collapsing polr factors works                               | PASS    | 4 | 0.001 |      |
| [test-polr.R](testthat/test-polr.R#)                               |         | Ordered logistic regression works                           | PASS    | 5 | 0.002 |      |
| [test-print.R](testthat/test-print.R#)                             |         | Equation is printed correctly                               | PASS    | 2 | 0.001 |      |
| [test-print.R](testthat/test-print.R#)                             |         | Equation is knit\_print-ed correctly                        | PASS    | 2 | 0.001 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R#)             |         | se\_subscripts argument works with numeric coefficients     | PASS    | 3 | 0.001 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R#)             |         | se\_subscripts argument works with wrapping                 | PASS    | 4 | 0.001 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R#)             |         | se\_subscripts argument works with Greek letter terms       | PASS    | 4 | 0.002 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R#)             |         | se\_subscripts argument works with transformed variables    | PASS    | 4 | 0.002 |      |
| [test-se-subscripts.R](testthat/test-se-subscripts.R#L178_L182)    |         | se\_subscripts works with factor variable                   | WARNING | 4 | 0.001 | ⚠️   |
| [test-utils.R](testthat/test-utils.R#)                             |         | Strict mapply\_\* functions work                            | PASS    | 8 | 0.003 |      |
| [test-wrapping-formatting.R](testthat/test-wrapping-formatting.R#) |         | Coefficient digits work correctly                           | PASS    | 2 | 0.001 |      |
| [test-wrapping-formatting.R](testthat/test-wrapping-formatting.R#) |         | Wrapping works correctly                                    | PASS    | 8 | 0.003 |      |

| Failed | Warning | Skipped |
| :----- | :------ | :------ |
| 🛑      | ⚠️      | 🔶       |

</details>

<details>

<summary> Session Info </summary>

| Field    | Value                             |                                                                                                                                                                                                                                                                         |
| :------- | :-------------------------------- | :---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| Version  | R version 4.1.3 (2022-03-10)      |                                                                                                                                                                                                                                                                         |
| Platform | x86\_64-apple-darwin17.0 (64-bit) | <a href="https://github.com/datalorax/equatiomatic/commit/6e71b4707529a2e38d226f254a5022e976ed1094/checks" target="_blank"><span title="Built on Github Actions">![](https://github.com/metrumresearchgroup/covrpage/blob/actions/inst/logo/gh.png?raw=true)</span></a> |
| Running  | macOS Big Sur/Monterey 10.16      |                                                                                                                                                                                                                                                                         |
| Language | en\_US                            |                                                                                                                                                                                                                                                                         |
| Timezone | UTC                               |                                                                                                                                                                                                                                                                         |

| Package  | Version |
| :------- | :------ |
| testthat | 3.1.2   |
| covr     | 3.5.1   |
| covrpage | 0.1     |

</details>

<!--- Final Status : skipped/warning --->

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
#  library(equatiomatic)
#  library(shiny)
#  library(shinyWidgets)
#  library(gtsummary)
#  library(gt)
#  
#  ui <- fluidPage(
#    titlePanel("equatiomatic w/Shiny"),
#    sidebarLayout(
#      sidebarPanel(
#        multiInput(
#          inputId = "xvars", label = "Select predictor variables :",
#          choices = names(penguins)[-3],
#          selected = "island"
#        )
#      ),
#      mainPanel(
#        gt_output("tbl")
#      )
#    )
#  )
#  
#  server <- function(input, output) {
#    model <- reactive({
#      form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#      lm(as.formula(form), penguins)
#    })
#  
#    output$tbl <- render_gt({
#      as_gt(tbl_regression(model()))
#    })
#  
#  }
#  
#  shinyApp(ui = ui, server = server)

## -----------------------------------------------------------------------------
#  library(equatiomatic)
#  library(shiny)
#  library(shinyWidgets)
#  library(gtsummary)
#  library(gt)
#  
#  ui <- fluidPage(
#    titlePanel("equatiomatic w/Shiny"),
#    withMathJax(), # Initialize mathJax so the equation renders properly
#    sidebarLayout(
#      sidebarPanel(
#        multiInput(
#          inputId = "xvars", label = "Select predictor variables :",
#          choices = names(penguins)[-3],
#          selected = "island"
#        )
#      ),
#      mainPanel(
#        eqOutput("equation"),
#        gt_output("tbl")
#      )
#    )
#  )
#  
#  server <- function(input, output) {
#    model <- reactive({
#      form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#      lm(as.formula(form), penguins)
#    })
#  
#    output$equation <- renderEq(
#      extract_eq(model())
#    )
#  
#    output$tbl <- render_gt({
#      as_gt(tbl_regression(model()))
#    })
#  
#  }
#  
#  shinyApp(ui = ui, server = server)

## -----------------------------------------------------------------------------
#  library(equatiomatic)
#  library(shiny)
#  library(shinyWidgets)
#  library(gtsummary)
#  library(gt)
#  
#  ui <- fluidPage(
#    titlePanel("equatiomatic w/Shiny"),
#    withMathJax(),
#    sidebarLayout(
#      sidebarPanel(
#        multiInput(
#          inputId = "xvars",
#          label = "Select predictor variables :",
#          choices = names(penguins)[-3],
#          selected = "island"
#        )
#      ),
#      mainPanel(
#        eqOutput("eq2"),
#        eqOutput("equation"),
#        gt_output("tbl")
#      )
#    )
#  )
#  
#  server <- function(input, output) {
#    model <- reactive({
#      form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#      lm(as.formula(form), penguins)
#    })
#  
#    output$equation <- renderEq(
#        extract_eq(
#          model(),
#          wrap = TRUE,
#          terms_per_line = 2,
#          use_coefs = TRUE,
#          font_size = "Huge"
#      )
#    )
#  
#    output$tbl <- render_gt({
#      as_gt(tbl_regression(model()))
#    })
#  
#  }
#  
#  shinyApp(ui = ui, server = server)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

options(rmarkdown.html_vignette.check_title = FALSE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)

## -----------------------------------------------------------------------------
# library(equatiomatic)
# library(shiny)
# library(shinyWidgets)
# library(gtsummary)
# library(gt)
# 
# data("penguins", package = "equatiomatic")
# 
# ui <- fluidPage(
#   titlePanel("equatiomatic w/Shiny"),
#   sidebarLayout(
#     sidebarPanel(
#       multiInput(
#         inputId = "xvars", label = "Select predictor variables :",
#         choices = names(penguins)[-3],
#         selected = "island"
#       )
#     ),
#     mainPanel(
#       gt_output("tbl")
#     )
#   )
# )
# 
# server <- function(input, output) {
#   model <- reactive({
#     form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#     lm(as.formula(form), penguins)
#   })
# 
#   output$tbl <- render_gt({
#     as_gt(tbl_regression(model()))
#   })
# 
# }
# 
# shinyApp(ui = ui, server = server)

## -----------------------------------------------------------------------------
# library(equatiomatic)
# library(shiny)
# library(shinyWidgets)
# library(gtsummary)
# library(gt)
# 
# ui <- fluidPage(
#   titlePanel("equatiomatic w/Shiny"),
#   withMathJax(), # Initialize mathJax so the equation renders properly
#   sidebarLayout(
#     sidebarPanel(
#       multiInput(
#         inputId = "xvars", label = "Select predictor variables :",
#         choices = names(penguins)[-3],
#         selected = "island"
#       )
#     ),
#     mainPanel(
#       eqOutput("equation"),
#       gt_output("tbl")
#     )
#   )
# )
# 
# server <- function(input, output) {
#   model <- reactive({
#     form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#     lm(as.formula(form), penguins)
#   })
# 
#   output$equation <- renderEq(
#     extract_eq(model())
#   )
# 
#   output$tbl <- render_gt({
#     as_gt(tbl_regression(model()))
#   })
# 
# }
# 
# shinyApp(ui = ui, server = server)

## -----------------------------------------------------------------------------
# library(equatiomatic)
# library(shiny)
# library(shinyWidgets)
# library(gtsummary)
# library(gt)
# 
# ui <- fluidPage(
#   titlePanel("equatiomatic w/Shiny"),
#   withMathJax(),
#   sidebarLayout(
#     sidebarPanel(
#       multiInput(
#         inputId = "xvars",
#         label = "Select predictor variables :",
#         choices = names(penguins)[-3],
#         selected = "island"
#       )
#     ),
#     mainPanel(
#       eqOutput("eq2"),
#       eqOutput("equation"),
#       gt_output("tbl")
#     )
#   )
# )
# 
# server <- function(input, output) {
#   model <- reactive({
#     form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
#     lm(as.formula(form), penguins)
#   })
# 
#   output$equation <- renderEq(
#       extract_eq(
#         model(),
#         wrap = TRUE,
#         terms_per_line = 2,
#         use_coefs = TRUE,
#         font_size = "Huge"
#     )
#   )
# 
#   output$tbl <- render_gt({
#     as_gt(tbl_regression(model()))
#   })
# 
# }
# 
# shinyApp(ui = ui, server = server)

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

colorize_text <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color,
      x)
  } else x
}

## -----------------------------------------------------------------------------
library(equatiomatic)
slr <- lm(bill_length_mm ~ body_mass_g, data = penguins)

## -----------------------------------------------------------------------------
extract_eq(
  model = slr, 
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g = "firebrick"
  )
)

## -----------------------------------------------------------------------------
extract_eq(
  model = slr, 
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g = "firebrick"
  ),
  greek_colors = c(
    "#3bd100", rep("black", 2)
  )
)

## -----------------------------------------------------------------------------
greek_col <- c("#1b9e77", "#d95f02", "#7570b3")

extract_eq(
  model = slr, 
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g = "firebrick"
  ),
  greek_colors = greek_col
)

## -----------------------------------------------------------------------------
m_interaction <- lm(bill_length_mm ~ body_mass_g * flipper_length_mm, 
                    data = penguins)

extract_eq(
  m_interaction,
  var_colors = c(
    body_mass_g = "#ffa91f",
    flipper_length_mm = "#00d1ab"
  ),
  greek_colors = c(
    "black", "#3A21B3", "#58A1D9", "#FF7582", "black"
  ),
  wrap = TRUE,
  terms_per_line = 3
)

## -----------------------------------------------------------------------------
extract_eq(
  m_interaction,
  var_colors = c(
    body_mass_g = "#ffa91f",
    flipper_length_mm = "#00d1ab"
  ),
  greek_colors = c(
    "black", "#3A21B3", "#58A1D9", "#FF7582", "black"
  ),
  subscript_colors = c(
     NA_character_, "#3A21B3", "#58A1D9", "#FF7582", NA_character_
  ),
  wrap = TRUE,
  terms_per_line = 3
)

## -----------------------------------------------------------------------------
m_categorical <- lm(bill_length_mm ~ species + island, data = penguins)

extract_eq(
  model = m_categorical,
  var_colors = c(
    species = "#FB2C4B",
    island = "#643B77"
  ),
  var_subscript_colors = c(
    species = "#0274B2",
    island = "#FBA640"
  )
)

## -----------------------------------------------------------------------------
extract_eq(
  model = m_interaction,
  swap_var_names = c(
    "bill_length_mm" = "Bill Length (MM)",
    "body_mass_g" = "Body Mass (grams)",
    "flipper_length_mm" = "Flipper Length (MM)"
  ),
  var_colors = c(
    flipper_length_mm = "firebrick",
    body_mass_g = "cornflowerblue"
  ),
  wrap = TRUE,
  terms_per_line = 3
)

## -----------------------------------------------------------------------------
extract_eq(
  model = m_categorical,
  swap_var_names = c(
    "bill_length_mm" = "Bill Length (MM)",
    "species" = "Species",
    "island" = "Island"
  ),
  swap_subscript_names = c(
    Chinstrap = "little buddy",
    Gentoo = "happy feet"
  ),
  var_colors = c(
    species = "#FB2C4B",
    island = "#643B77"
  ),
  var_subscript_colors = c(
    species = "#0274B2",
    island = "#FBA640"
  ),
  wrap = TRUE,
  terms_per_line = 3
)

## ----eval = FALSE-------------------------------------------------------------
#  create_eq(m_categorical) %>%
#    swap_var_names(
#      "bill_length_mm" = "Bill Length (MM)",
#      "species" = "Species",
#      "island" = "Island"
#    ) %>%
#    swap_subscript_names(
#      Chinstrap = "little buddy",
#      Gentoo = "happy feet"
#    ) %>%
#    colorize_variables(
#      species = "#FB2C4B",
#      island = "#643B77"
#    ) %>%
#    colorize_variable_subscripts(
#      species = "#0274B2",
#      island = "#FBA640"
#    ) %>%
#    wrap(terms_per_line = 3)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
options(rmarkdown.html_vignette.check_title = FALSE)

## ----include=FALSE------------------------------------------------------------
library(equatiomatic)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----warning=FALSE, results='markup'------------------------------------------
data("penguins", package = "equatiomatic")
penguins_lm <- lm(bill_length_mm ~ bill_depth_mm, penguins)
penguins_lm

## ----echo=FALSE---------------------------------------------------------------
library(equatiomatic)
extract_eq(penguins_lm)

## ----fit-m1-------------------------------------------------------------------
library(equatiomatic)

# fit a basic multiple linear regression model
m <- lm(bill_length_mm ~ bill_depth_mm + flipper_length_mm, data = penguins)

## ----extract_eq1--------------------------------------------------------------
extract_eq(m)

## ----extract_eq1-no-echo, echo=FALSE------------------------------------------
extract_eq(m)

## ----eq2----------------------------------------------------------------------
m2 <- lm(bill_length_mm ~ bill_depth_mm * island, data = penguins)
extract_eq(m2)

## ----eq2-wrap-----------------------------------------------------------------
extract_eq(m2, wrap = TRUE) # default terms_per_line = 4
extract_eq(m2, wrap = TRUE, terms_per_line = 2)

## ----eq2-intercept-beta-------------------------------------------------------
extract_eq(m2, wrap = TRUE, intercept = "beta")

## ----eq2-ital-vars------------------------------------------------------------
extract_eq(m2, wrap = TRUE, ital_vars = TRUE)

## ----raw-tex------------------------------------------------------------------
extract_eq(m2, wrap = TRUE, intercept = "\\hat{\\phi}", greek = "\\hat{\\gamma}",
  raw_tex = TRUE)

## ----use_coefs----------------------------------------------------------------
extract_eq(m2, wrap = TRUE, use_coefs = TRUE)

## ----echo=FALSE---------------------------------------------------------------
supported <- data.frame(
  model = c(
    "linear regression",
    "logistic regression",
    "probit regression",
    "ordinal logistic regression",
    "ordinal probit regression",
    "auto-regressive integrated moving average",
    "regression with auto-regressive integrated moving average errors"
  ),
  packages = c(
    "`stats::lm`",
    "`stats::glm(family = binomial(link = 'logit'))`",
    "`stats::glm(family = binomial(link = 'probit'))`",
    "`MASS::polr(method = 'logistic')`; `ordinal::clm(link = 'logit')`",
    "`MASS::polr(method = 'probit')`; `ordinal::clm(link = 'probit')`",
    "`forecast::Arima`",
    "`forecast::Arima`"
  )
)

knitr::kable(supported, col.names = c("Model", "Packages/Functions"))

## ----log-reg1-----------------------------------------------------------------
lr <- glm(sex ~ species * bill_length_mm, data = penguins,
  family = binomial(link = "logit"))

extract_eq(lr, wrap = TRUE)

## ----log-reg2-----------------------------------------------------------------
extract_eq(lr, wrap = TRUE, show_distribution = TRUE)

## ----prob-reg1----------------------------------------------------------------
pr <- glm(sex ~ species * bill_length_mm, data = penguins,
  family = binomial(link = "probit"))

extract_eq(pr, wrap = TRUE)

## ----prob-reg2----------------------------------------------------------------
extract_eq(pr, wrap = TRUE, show_distribution = TRUE)

## ----install-ordinal, eval=FALSE----------------------------------------------
# install.packages("ordinal")

## ----wine-data, echo=FALSE----------------------------------------------------
knitr::kable(head(ordinal::wine), align = "l")

## ----mass-ologit--------------------------------------------------------------
mass_ologit <- MASS::polr(rating ~ temp * contact, data = ordinal::wine)
extract_eq(mass_ologit, wrap = TRUE, terms_per_line = 2)

## ----mass-oprobit-------------------------------------------------------------
mass_oprobit <- MASS::polr(rating ~ temp * contact, data = ordinal::wine,
  method = "probit")
extract_eq(mass_oprobit, wrap = TRUE, terms_per_line = 2)

## ----ordinal-ologit-----------------------------------------------------------
ordinal_ologit <- ordinal::clm(rating ~ temp * contact, data = ordinal::wine,
  link = "logit")
extract_eq(ordinal_ologit, wrap = TRUE, terms_per_line = 2)

## ----ordinal-oprobit----------------------------------------------------------
ordinal_probit <- ordinal::clm(rating ~ temp * contact, data = ordinal::wine,
  link = "probit")
extract_eq(ordinal_probit, wrap = TRUE, terms_per_line = 2)

## ----list---------------------------------------------------------------------
# Create a series of regressions and place results in a data.frame
# (note: this is much better handled with a tibble, but we avoid
# extra dependencies here)
data(iris)
res <- data.frame(name = c("lm1", "lm2", "lm3"))
models <- list(
  lm1 = lm(Sepal.Length ~ Sepal.Width, data = iris),
  lm2 = lm(Sepal.Length ~ Sepal.Width + Petal.Length, data = iris),
  lm3 = lm(Sepal.Length ~ Sepal.Width * Petal.Length, data = iris)
)
# Add a column in the data.frame with the three corresponding equations
res$equation <- extract_eq(models)
knitr::kable(res)

## ----include=FALSE------------------------------------------------------------
library(equatiomatic)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----iris---------------------------------------------------------------------
data(iris)
# Attach `label` and `units` attributes to the variables
# Note, this is done manually, but there are functions in various packages to
# ease the process (see packages {Hmisc}, {labelled}, {LabelR}, {labelVector})
attr(iris$Sepal.Length, "label") <- "Sepal length"
attr(iris$Sepal.Length, "units") <- "cm"
attr(iris$Sepal.Width, "label") <- "Sepal width"
attr(iris$Sepal.Width, "units") <- "cm"
attr(iris$Petal.Length, "label") <- "Petal length"
attr(iris$Petal.Length, "units") <- "cm"
attr(iris$Petal.Width, "label") <- "Petal width"
attr(iris$Petal.Width, "units") <- "cm"

str(iris)

## -----------------------------------------------------------------------------
lm1 <- lm(Sepal.Length ~ Sepal.Width, data = iris)
lm1

## ----extracteq----------------------------------------------------------------
extract_eq(lm1)

## ----equation-iris------------------------------------------------------------
equation(lm1)

## -----------------------------------------------------------------------------
equation(lm1, var_colors = c(Sepal.Width  = "red"),
  ital_vars = TRUE, use_coefs = TRUE, coef_digits = 3)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", message = FALSE,
  warning = FALSE, error = FALSE)

## ----setup_arima--------------------------------------------------------------
library(equatiomatic)
library(forecast)

# Build Arima (no regression)
simple_ts_mod <- Arima(simple_ts, order = c(1, 1, 1), seasonal = c(1, 0, 1),
  include.constant = TRUE)

## ----extract_arima------------------------------------------------------------
extract_eq(simple_ts_mod)

## ----setup_arima_errors-------------------------------------------------------
# Build Exogenous Regressors
xregs <- as.matrix(data.frame(
  x1 = ts_reg_list$x1 + 5,
  x2 = ts_reg_list$x2 * 5
))

# Build Regression Model
ts_reg_mod <- Arima(ts(ts_reg_list$ts_rnorm, freq = 4), order = c(1, 1, 1),
  seasonal = c(1, 0, 1), xreg = xregs, include.constant = TRUE)

## ----extract_arima_errors-----------------------------------------------------
extract_eq(ts_reg_mod)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", message = FALSE,
  warning = FALSE, error = FALSE)

## ----packages-----------------------------------------------------------------
library(ggplot2)
library(latex2exp)
library(equatiomatic)

## ----model_setup--------------------------------------------------------------
data("penguins", package = "equatiomatic")
# Fit an lm model
m <- lm(body_mass_g ~ bill_length_mm, penguins)

## ----equation_from_equatiomatic-----------------------------------------------
extract_eq(m, use_coefs = TRUE)

## -----------------------------------------------------------------------------
ggplot(penguins, aes(x = bill_length_mm, y = body_mass_g)) +
  geom_point() +
  geom_smooth(method = "lm")

## -----------------------------------------------------------------------------
# extract equation with `ital_vars = TRUE` to avoid the use of `\operatorname{}`
m_eq <- extract_eq(m, use_coef = TRUE, ital_vars = TRUE)

# swap escaped underscores for dashes
prep_eq <- gsub("\\\\_", "-", m_eq)

# swap display-style $$ with inline-style $
prep_eq <- paste("$", as.character(prep_eq), "$", sep = "")

## -----------------------------------------------------------------------------
ggplot(penguins, aes(x = bill_length_mm, y = body_mass_g)) +
  geom_point() +
  geom_smooth(method = "lm") +
  labs(
    title    = "Relation between bill length and body mass",
    subtitle = TeX(prep_eq))

## -----------------------------------------------------------------------------
ggplot(penguins, aes(x = bill_length_mm, y = body_mass_g)) +
  geom_point() +
  geom_smooth(method = "lm", color = "#ab00fa") +
  annotate("text",
    label = as.character(TeX(prep_eq)), x = 35, y = 2500, hjust = 0,
    color = "#ab00fa", parse = TRUE) +
  labs(title = "Relation between bill length and body mass") +
  ylim(2000, 6500)

## ----base_graphics------------------------------------------------------------
plot(penguins$bill_length_mm, penguins$body_mass_g,
  pch = 20, col = "gray80", xlab = "Bill length", ylab = "Body Mass",
  main = "Relation between bill length and body mass",
  sub = TeX(prep_eq))
lines(na.omit(penguins$bill_length_mm), predict(m), col = "#ab00fa")

## -----------------------------------------------------------------------------
plot(penguins$bill_length_mm, penguins$body_mass_g,
  pch = 20, col = "gray80", xlab = "Bill length", ylab = "Body Mass",
  main = "Relation between bill length and body mass")
lines(na.omit(penguins$bill_length_mm), predict(m), col = "#ab00fa")
text(x = 41, y = 6000, TeX(prep_eq))

## ----include=FALSE------------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

colorize_text <- function(x, color) {
  if (knitr::is_latex_output()) {
    sprintf("\\textcolor{%s}{%s}", color, x)
  } else if (knitr::is_html_output()) {
    sprintf("<span style='color: %s;'>%s</span>", color, x)
  } else {
    x
  }
}

## -----------------------------------------------------------------------------
library(equatiomatic)
data("penguins", package = "equatiomatic")
slr <- lm(bill_length_mm ~ body_mass_g, data = penguins)

## -----------------------------------------------------------------------------
extract_eq(slr,
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g    = "firebrick"
  )
)

## -----------------------------------------------------------------------------
extract_eq(slr,
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g    = "firebrick"),
  greek_colors = c(
    "#3bd100", rep("black", 2)
  )
)

## -----------------------------------------------------------------------------
greek_col <- c("#1b9e77", "#d95f02", "#7570b3")

extract_eq(slr, 
  var_colors = c(
    bill_length_mm = "cornflowerblue",
    body_mass_g    = "firebrick"),
  greek_colors = greek_col
)

## -----------------------------------------------------------------------------
m_interaction <- lm(bill_length_mm ~ body_mass_g * flipper_length_mm,
  data = penguins)

extract_eq(m_interaction,
  var_colors = c(
    body_mass_g       = "#ffa91f",
    flipper_length_mm = "#00d1ab"),
  greek_colors = c(
    "black", "#3A21B3", "#58A1D9", "#FF7582", "black"),
  wrap = TRUE, terms_per_line = 3
)

## -----------------------------------------------------------------------------
extract_eq(m_interaction,
  var_colors = c(
    body_mass_g       = "#ffa91f",
    flipper_length_mm = "#00d1ab"),
  greek_colors = c(
    "black", "#3A21B3", "#58A1D9", "#FF7582", "black"),
  subscript_colors = c(
     NA_character_, "#3A21B3", "#58A1D9", "#FF7582", NA_character_),
  wrap = TRUE, terms_per_line = 3
)

## -----------------------------------------------------------------------------
m_categorical <- lm(bill_length_mm ~ species + island, data = penguins)

extract_eq(m_categorical,
  var_colors = c(
    species = "#FB2C4B",
    island  = "#643B77"),
  var_subscript_colors = c(
    species = "#0274B2",
    island  = "#FBA640")
)

## -----------------------------------------------------------------------------
extract_eq(m_interaction,
  swap_var_names = c(
    "bill_length_mm"    = "Bill Length [mm]",
    "body_mass_g"       = "Body Mass [g])",
    "flipper_length_mm" = "Flipper Length [mm]"),
  var_colors = c(
    flipper_length_mm   = "firebrick",
    body_mass_g         = "cornflowerblue"),
  wrap = TRUE, terms_per_line = 3
)

## -----------------------------------------------------------------------------
extract_eq(m_categorical,
  swap_var_names = c(
    "bill_length_mm" = "Bill Length [mm]",
    "species"        = "Species",
    "island"         = "Island"),
  swap_subscript_names = c(
    Chinstrap        = "little buddy",
    Gentoo           = "happy feet"),
  var_colors = c(
    species          = "#FB2C4B",
    island           = "#643B77"),
  var_subscript_colors = c(
    species          = "#0274B2",
    island           = "#FBA640"),
  wrap = TRUE, terms_per_line = 3
)

## ----eval=FALSE---------------------------------------------------------------
# create_eq(m_categorical) |>
#   swap_var_names(
#     "bill_length_mm" = "Bill Length [mm]",
#     "species"        = "Species",
#     "island"         = "Island"
#   ) |>
#   swap_subscript_names(
#     Chinstrap        = "little buddy",
#     Gentoo           = "happy feet"
#   ) |>
#   colorize_variables(
#     species          = "#FB2C4B",
#     island           = "#643B77"
#   ) |>
#   colorize_variable_subscripts(
#     species          = "#0274B2",
#     island           = "#FBA640"
#   ) |>
#   wrap(terms_per_line = 3)

#' Generic function for extracting the right-hand side from a model
#'
#' @keywords internal
#'
#' @param model A fitted model
#' @param \dots additional arguments passed to the specific extractor
#' @noRd

extract_rhs <- function(model, ...) {
  UseMethod("extract_rhs", model)
}

#' Extract right-hand side
#'
#' Extract a data frame with list columns for the primary terms and subscripts
#' from all terms in the model
#'
#' @keywords internal
#'
#' @param model A fitted model
#'
#' @return A list with one element per future equation term. Term components
#'   like subscripts are nested inside each list element. List elements with two
#'   or more terms are interactions.
#' @noRd
#' @examples
#' \dontrun{
#' data("penguins", package = "equatiomatic")
#' mod1 <- lm(body_mass_g ~ bill_length_mm + species * flipper_length_mm, penguins)
#'
#' extract_rhs(mod1)
#' # > # A tibble: 7 x 8
#' # >                                 term     estimate ...           primary  subscripts
#' # > 1                        (Intercept) -3341.615846 ...
#' # > 2                     bill_length_mm    59.304539 ...    bill_length_mm
#' # > 3                   speciesChinstrap   -27.292519 ...           species   Chinstrap
#' # > 4                      speciesGentoo -2215.913323 ...           species      Gentoo
#' # > 5                  flipper_length_mm    24.962788 ... flipper_length_mm
#' # > 6 speciesChinstrap:flipper_length_mm    -3.484628 ... flipper_length_mm Chinstrap,
#' # > 7    speciesGentoo:flipper_length_mm    11.025972 ... flipper_length_mm    Gentoo,
#'
#' str(extract_rhs(mod1))
#' # > Classes ‘lm’ and 'data.frame':	7 obs. of  8 variables:
#' # >  $ term      : chr  "(Intercept)" "bill_length_mm" "speciesChinstrap" "speciesGentoo" ...
#' # >  $ estimate  : num  -3341.6 59.3 -27.3 -2215.9 25 ...
#' # >  $ std.error : num  810.14 7.25 1394.17 1328.58 4.34 ...
#' # >  $ statistic : num  -4.1247 8.1795 -0.0196 -1.6679 5.7534 ...
#' # >  $ p.value   : num  4.69e-05 5.98e-15 9.84e-01 9.63e-02 1.97e-08 ...
#' # >  $ split     :List of 7
#' # >   ..$ : chr "(Intercept)"
#' # >   ..$ : chr "bill_length_mm"
#' # >   ..$ : chr "speciesChinstrap"
#' # >   ..$ : chr "speciesGentoo"
#' # >   ..$ : chr "flipper_length_mm"
#' # >   ..$ : chr  "speciesChinstrap" "flipper_length_mm"
#' # >   ..$ : chr  "speciesGentoo" "flipper_length_mm"
#' # >  $ primary   :List of 7
#' # >   ..$ : chr
#' # >   ..$ : chr "bill_length_mm"
#' # >   ..$ : chr "species"
#' # >   ..$ : chr "species"
#' # >   ..$ : chr "flipper_length_mm"
#' # >   ..$ : chr  "species" "flipper_length_mm"
#' # >   ..$ : chr  "species" "flipper_length_mm"
#' # >  $ subscripts:List of 7
#' # >   ..$ : chr ""
#' # >   ..$ : chr ""
#' # >   ..$ : chr "Chinstrap"
#' # >   ..$ : chr "Gentoo"
#' # >   ..$ : chr ""
#' # >   ..$ : Named chr  "Chinstrap" ""
#' # >   .. ..- attr(*, "names")= chr [1:2] "species" "flipper_length_mm"
#' # >   ..$ : Named chr  "Gentoo" ""
#' # >   .. ..- attr(*, "names")= chr [1:2] "species" "flipper_length_mm"
#' }
#'
extract_rhs.default <- function(model, index_factors, ...) {
  check_dots(...)
  # Extract RHS from formula
  formula_rhs <- labels(terms(formula(model)))

  # Extract unique (primary) terms from formula (no interactions)
  formula_rhs_terms <- formula_rhs[!grepl(":", formula_rhs)]

  # Extract coefficient names and values from model
  full_rhs <- broom::tidy(model)

  # Split interactions split into character vectors
  full_rhs$split <- strsplit(full_rhs$term, ":")

  full_rhs$primary <- extract_primary_term(
    formula_rhs_terms,
    full_rhs$term
  )

  full_rhs$subscripts <- extract_all_subscripts(
    full_rhs$primary,
    full_rhs$split
  )
  if (index_factors) {
    full_rhs <- distinct(full_rhs, "primary")
    unique_ss <- unique(unlist(full_rhs$subscripts))
    unique_ss <- unique_ss[vapply(unique_ss, nchar, FUN.VALUE = integer(1)) > 0]
    replacement_ss <- letters[seq(9, (length(unique_ss) + 8))]
    full_rhs$subscripts <- lapply(full_rhs$subscripts, function(x) {
      out <- replacement_ss[match(x, unique_ss)]
      ifelse(is.na(out), "", out)
    })
  }
  class(full_rhs) <- c("data.frame", class(model))
  full_rhs
}

#' @noRd
extract_rhs.lmerMod <- function(model, return_variances, ...) {

  check_dots(...)
    
  # Extract RHS from formula
  formula_rhs <- labels(terms(formula(model)))

  # Extract unique (primary) terms from formula (no interactions)
  formula_rhs_terms <- formula_rhs[!grepl(":", formula_rhs)]
  formula_rhs_terms <- gsub("^`?(.+)`$?", "\\1", formula_rhs_terms)

  # Extract coefficient names and values from model
  if(return_variances) {
    full_rhs <- broom.mixed::tidy(model, scales = c("vcov", NA))
    
    # Make the names like they are sdcor, so it doesn't break other code
    full_rhs$term <- gsub("var__", "sd__", full_rhs$term)
    full_rhs$term <- gsub("cov__", "cor__", full_rhs$term)
  } else {
    full_rhs <- broom.mixed::tidy(model)  
  }
  
  full_rhs$term <- vapply(full_rhs$term, order_interaction,
    FUN.VALUE = character(1)
  )
  full_rhs$group <- recode_groups(full_rhs)

  full_rhs$original_order <- seq_len(nrow(full_rhs))
  full_rhs$term <- gsub("^`?(.+)`$?", "\\1", full_rhs$term)

  # Split interactions split into character vectors
  full_rhs$split <- strsplit(full_rhs$term, ":")
  
  full_rhs$primary <- lapply(full_rhs$term, function(x) "")
  full_rhs$primary[full_rhs$effect == "fixed"] <- extract_primary_term(
    formula_rhs_terms,
    full_rhs$term[full_rhs$effect == "fixed"]
  )

  # make sure split and primary are in the same order
  full_rhs$primary[full_rhs$effect == "fixed"] <- Map(
    function(prim, splt) {
      ord <- vapply(prim, function(x) grep(x, splt, fixed = TRUE), FUN.VALUE = integer(1))
      names(sort(ord))
    },
    full_rhs$primary[full_rhs$effect == "fixed"],
    full_rhs$split[full_rhs$effect == "fixed"]
  )
  
  full_rhs$subscripts <- lapply(full_rhs$term, function(x) "")
  full_rhs$subscripts[full_rhs$effect == "fixed"] <- extract_all_subscripts(
    full_rhs$primary[full_rhs$effect == "fixed"],
    full_rhs$split[full_rhs$effect == "fixed"]
  )
  
  randvars <- pull_randvar_names(full_rhs)
  fixedvars <- unique(full_rhs$term[full_rhs$effect == "fixed"])
  
  if(any(!randvars %in% fixedvars)) {
    stop(
      paste(
        paste0(
          "{equatiomatic} only supports models where each random effect ",
          "has a corresponding fixed effect. You specified the following ",
          "variables as randomly varying without including the ",
          "corresponding fixed effect:"), 
        paste0(setdiff(randvars, fixedvars), collapse = ", ")
      ),
      call. = FALSE
    )
  }
  
  group_coefs <- detect_group_coef(model, full_rhs)
  all_terms <- unique(unlist(full_rhs$primary[full_rhs$effect == "fixed"]))
  l1_terms <- setdiff(all_terms, names(group_coefs))
  l1_terms <- setNames(rep("l1", length(l1_terms)), l1_terms)
  
  var_levs <- c(l1_terms, group_coefs)
  
  full_rhs$pred_level <- lapply(full_rhs$primary, function(x) {
    var_levs[names(var_levs) %in% x]
  })

  full_rhs$pred_level[full_rhs$effect == "fixed"] <- Map(
    function(predlev, splt) {
      ord <- vapply(names(predlev), 
                    function(x) grep(x, splt, fixed = TRUE), 
                    FUN.VALUE = integer(1))
      ord <- names(sort(ord))
      predlev[ord]
    },
    full_rhs$pred_level[full_rhs$effect == "fixed"],
    full_rhs$split[full_rhs$effect == "fixed"]
  )

  full_rhs$l1 <- vapply(full_rhs$pred_level, function(x) {
    length(x) > 0 & all(x == "l1")
  }, FUN.VALUE = logical(1))
  full_rhs$l1 <- ifelse(full_rhs$term == "(Intercept)",
    TRUE,
    full_rhs$l1
  )

  full_rhs$crosslevel <- detect_crosslevel(
    full_rhs$primary,
    full_rhs$pred_level
  )

  class(full_rhs) <- c("data.frame", class(model))
  full_rhs
}

#' @noRd
extract_rhs.glmerMod <- function(model, ...) {
  extract_rhs.lmerMod(model, ...)
}

pull_randvar_names <- function(rhs) {
  rows_selected <- (rhs$effect == "ran_pars") & (rhs$group != "Residual")
  random <- rhs[rows_selected, "term", drop = TRUE]
  random <- random[!grepl("^cor", random)]
  unique(gsub("sd__", "", random))
}

#' Extract right-hand side of an forecast::Arima object
#'
#' Extract a dataframe of S/MA components
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A dataframe
#' @noRd
extract_rhs.forecast_ARIMA <- function(model, ...) {
  # RHS of ARIMA is the Moving Average side
  # Consists of a Non-Seasonal MA (p), Seasonal MA (P), Seasonal Differencing.

  # This is more than needed, but we"re being explicit for readability.
  # Orders structure in Arima model: c(p, q, P, Q, m, d, D)
  ords <- model$arma
  names(ords) <- c("p", "q", "P", "Q", "m", "d", "D")

  # Following the rest of the package.
  # Pull the full model with broom::tidy
  full_mdl <- broom::tidy(model)

  # Filter down to only the MA terms and seasonal drift
  full_rhs <- full_mdl[grepl("^s?ma", full_mdl$term), ]

  # Add a Primary column and set it to the backshift operator.
  full_rhs$primary <- "B"

  # Get the superscript for the backshift operator.
  ## This is equal to the number on the term for MA
  ## and the number on the term * the seasonal frequency for SMA.
  ## Powers of 1 are replaced with an empty string.
  rhs_super <- as.numeric(gsub("^s?ma", "", full_rhs$term))
  rhs_super[grepl("^sma", full_rhs$term)] <- rhs_super[grepl("^sma", full_rhs$term)] * ords["m"]

  rhs_super <- as.character(rhs_super)

  full_rhs$superscript <- rhs_super

  # The RHS (MA side) has no differencing.
  # Previous versions of this function were erroneous
  # in that it included a seasonal difference on this side.

  # Reduce any "1" superscripts to not show the superscript
  full_rhs[full_rhs$superscript == "1", "superscript"] <- ""

  # Set subscripts so that create_term works later
  full_rhs$subscripts <- ""

  # Set the class
  class(full_rhs) <- c(class(model), "data.frame")

  # Explicit return
  return(full_rhs)
}


order_interaction <- function(interaction_term) {
  if (grepl("^cor__", interaction_term)) {
    ran_part <- gsub("(.+\\.).+", "\\1", interaction_term)
    interaction_term <- gsub(ran_part, "", interaction_term, fixed = TRUE)
  } else if (grepl("^sd__", interaction_term)) {
    ran_part <- "sd__"
    interaction_term <- gsub(paste0("^", ran_part), "", interaction_term)
  }
  terms <- strsplit(interaction_term, ":")[[1]]
  terms_ordered <- sort(terms)
  out <- paste0(terms_ordered, collapse = ":")

  if (exists("ran_part")) {
    # check/handle if there's an interaction in the random part
    # sd or cor
    type <- gsub("(^.+__).+", "\\1", ran_part)

    # remove type and period at end
    ran <- gsub(type, "", ran_part)
    ran <- gsub("\\.$", "", ran)

    # handle interaction (if present)
    ran <- strsplit(ran, ":")[[1]]
    ran <- paste0(sort(ran), collapse = ":")

    # paste it all back together
    if (grepl("^cor", ran_part)) {
      out <- paste0(type, ran, ".", out)
    } else {
      out <- paste0(type, ran, out)
    }
  }
  out
}

recode_groups <- function(rhs) {
  rhs_splt <- split(rhs, rhs$group)
  rhs_splt <- rhs_splt[!grepl("Residual", names(rhs_splt))]

  names_collapsed <- collapse_groups(names(rhs_splt))

  intercept_vary <- vapply(rhs_splt, function(x) {
    any(grepl("sd__(Intercept)", x$term, fixed = TRUE))
  }, FUN.VALUE = logical(1))

  check <- split(intercept_vary, names_collapsed)

  # collapse these groups
  collapse <- vapply(check, all, FUN.VALUE = logical(1))

  collapse_term <- function(term, v) {
    ifelse(grepl(term, v), collapse_groups(v), v)
  }

  out <- rhs$group
  for (i in seq_along(collapse[!collapse])) {
    out <- collapse_term(names(collapse[!collapse])[i], out)
  }
  out
}

collapse_groups <- function(group) {
  gsub("(.+)\\.\\d\\d?$", "\\1", group)
}

order_split <- function(split, pred_level) {
  if (length(pred_level) == 0) {
    return(pred_level)
  }
  var_order <- vapply(names(pred_level), function(x) {
    exact <- split %in% x
    detect <- grepl(x, split)

    # take exact if it's there, if not take detect
    if (any(exact)) {
      out <- exact
    } else {
      out <- detect
    }

    seq_along(out)[out]
  }, FUN.VALUE = integer(1))

  split[var_order]
}

#' Pull just the random variables
#' @param rhs output from \code{extract_rhs}
#' @keywords internal
#' @noRd
extract_random_vars <- function(rhs) {
  order <- rhs[rhs$group != "Residual", ]
  order <- sort(tapply(order$original_order, order$group, min))

  vc <- rhs[rhs$group != "Residual" & rhs$effect == "ran_pars", ]
  splt <- split(vc, vc$group)[names(order)]

  lapply(splt, function(x) {
    vars <- x[!grepl("cor__", x$term), ]
    gsub("sd__(.+)", "\\1", vars$term)
  })
}


detect_crosslevel <- function(primary, pred_level) {
  mapply_lgl(function(prim, predlev) {
    if (length(prim) > 1) {
      if (length(prim) != length(predlev)) {
        TRUE
      } else if (length(unique(predlev)) != 1) {
        TRUE
      } else {
        FALSE
      }
    } else {
      FALSE
    }
  },
  prim = primary,
  predlev = pred_level
  )
}

#### Consider refactoring the below too
detect_covar_level <- function(predictor, group) {
  nm <- names(group)
  if (is.numeric(predictor)) {
    if (is.matrix(predictor)) {
      predictor <- predictor[ ,1]
    }
    predictor <- round(predictor, 5)
  }
  v <- paste(predictor, group[, 1], sep = " _|_ ")
  unique_v <- unique(v)
  test <- gsub(".+\\s\\_\\|\\_\\s(.+)", "\\1", unique_v)

  if (all(!duplicated(test))) {
    return(nm)
  }
}

detect_X_level <- function(X, group) {
  lapply(X, detect_covar_level, group)
}

collapse_list <- function(x, y) {
  null_x <- vapply(x, function(x) {
    if (any(is.null(x))) {
      return(is.null(x))
    } else {
      return(is.na(x))
    }
  }, FUN.VALUE = logical(1))

  null_y <- vapply(y, function(x) {
    if (any(is.null(x))) {
      return(is.null(x))
    } else {
      return(is.na(x))
    }
  }, FUN.VALUE = logical(1))

  y[null_x & !null_y] <- y[null_x & !null_y]
  y[!null_x & null_y] <- x[!null_x & null_y]
  y[!null_x & !null_y] <- x[!null_x & !null_y]

  unlist(lapply(y, function(x) ifelse(is.null(x), NA_character_, x)))
}

detect_group_coef <- function(model, rhs) {
  outcome <- all.vars(formula(model))[1]
  d <- model@frame
  
  random_lev_names <- names(extract_random_vars(rhs))
  random_levs <- unlist(strsplit(random_lev_names, ":"))
  random_levs <- gsub("^\\(|\\)$", "", random_levs)
  random_levs <- unique(collapse_groups(random_levs))

  random_lev_ids <- d[random_levs]
  ranef_order <- vapply(random_lev_ids, function(x) {
    length(unique(x))
  }, FUN.VALUE = numeric(1))
  ranef_order <- rev(sort(ranef_order))
  random_lev_ids <- random_lev_ids[, names(ranef_order), drop = FALSE]

  # Make sure there are explicit ids
  random_lev_ids <- make_explicit_id(random_lev_ids)
  
  X <- d[!(names(d) %in% c(random_levs, outcome))]

  lev_assign <- vector("list", length(random_levs))
  for (i in seq_along(random_lev_ids)) {
    lev_assign[[i]] <- detect_X_level(X, random_lev_ids[, i, drop = FALSE])
  }

  levs <- Reduce(collapse_list, rev(lev_assign))

  # reassign acutal names (in cases where ranef contains ":")
  out <- random_lev_names[match(levs, random_levs)]
  names(out) <- names(levs)

  unlist(out[!is.na(out)])
}

row_paste <- function(d) {
  apply(d, 1, paste, collapse = "-")
}

#' Makes the grouping variables explicit, which is neccessary for
#' detecting group-level predictors
#' @param ranef_df A data frame that includes only the random 
#'   effect ID variables (i.e., random_lev_ids)
#' @noRd

make_explicit_id <- function(ranef_df) {
  for(i in seq_along(ranef_df)[-length(ranef_df)]) {
    ranef_df[[i]] <- row_paste(ranef_df[ ,i:length(ranef_df)])
  }
  ranef_df
}


#' Extract the primary terms from all terms
#'
#' @inheritParams detect_primary
#'
#' @keywords internal
#'
#' @param all_terms A list of all the equation terms on the right hand side,
#'   usually the result of \code{broom::tidy(model, quick = TRUE)$term}.
#' @examples
#' \dontrun{
#' primaries <- c("partyid", "age", "race")
#'
#' full_terms <- c(
#'   "partyidDon't know", "partyidOther party", "age",
#'   "partyidNot str democrat", "age", "raceBlack", "age", "raceBlack"
#' )
#'
#' extract_primary_term(primaries, full_terms)
#' }
#' @noRd

extract_primary_term <- function(primary_term_v, all_terms) {
  detected <- lapply(all_terms, detect_primary, primary_term_v)
  lapply(detected, function(pull) primary_term_v[pull])
}

#' Detect if a given term is part of a vector of full terms
#'
#' @keywords internal
#'
#' @param full_term The full name of a single term, e.g.,
#'   \code{"partyidOther party"}
#' @param primary_term_v A vector of primary terms, e.g., \code{"partyid"}.
#'   Usually the result of \code{formula_rhs[!grepl(":", formula_rhs)]}
#'
#' @return A logical vector the same length of \code{primary_term_v} indicating
#'   whether the \code{full_term} is part of the given \code{primary_term_v}
#'   element
#'
#' @examples
#' \dontrun{
#' detect_primary("partyidStrong republican", c("partyid", "age", "race"))
#' detect_primary("age", c("partyid", "age", "race"))
#' detect_primary("raceBlack", c("partyid", "age", "race"))
#' }
#' @noRd

detect_primary <- function(full_term, primary_term_v) {
  if (full_term %in% primary_term_v) {
    primary_term_v %in% full_term
  } else {
    # escape parens
    primary_term_v <- gsub("(", "\\(", primary_term_v, fixed = TRUE)
    primary_term_v <- gsub(")", "\\)", primary_term_v, fixed = TRUE)
    
    splt <- strsplit(full_term, ":")[[1]]
      
    m_logical <- vapply(primary_term_v, function(indiv_term) {
      vapply(splt, function(x) grepl(paste0("^", indiv_term), x),
             FUN.VALUE = logical(1))
    },
    logical(length(splt))
    )
    if (is.null(dim(m_logical))) {
      return(m_logical)
    }
    apply(m_logical, 2, any)
  }
}


#' Extract all subscripts
#'
#' @keywords internal
#'
#' @param primary_list A list of primary terms
#' @param full_term_list A list of full terms
#'
#' @return A list with the subscripts. If full term has no subscript,
#' returns \code{""}.
#'
#' @examples
#' \dontrun{
#' p_list <- list(
#'   "partyid",
#'   c("partyid", "age"),
#'   c("age", "race"),
#'   c("partyid", "age", "race")
#' )
#'
#' ft_list <- list(
#'   "partyidNot str republican",
#'   c("partyidInd,near dem", "age"),
#'   c("age", "raceBlack"),
#'   c("partyidInd,near dem", "age", "raceBlack")
#' )
#'
#' extract_all_subscripts(p_list, ft_list)
#' }
#' @noRd

extract_all_subscripts <- function(primary_list, full_term_list) {
  Map(extract_subscripts, primary_list, full_term_list)
}


#' Extract the subscripts from a given term
#'
#' @keywords internal
#'
#' @param primary A single primary term, e.g., \code{"partyid"}
#' @param full_term_v A vector of full terms, e.g.,
#'   \code{c("partyidDon't know", "partyidOther party"}. Can be of length 1.
#' @examples
#' \dontrun{
#' extract_subscripts("partyid", "partyidDon't know")
#' extract_subscripts(
#'   "partyid",
#'   c(
#'     "partyidDon't know", "partyidOther party",
#'     "partyidNot str democrat"
#'   )
#' )
#' }
#' @noRd

extract_subscripts <- function(primary, full_term_v) {
  out <- switch(as.character(length(primary)),
    "0" = "",
    "1" = gsub(primary, "", full_term_v, fixed = TRUE),
    mapply_chr(function(x, y) gsub(x, "", y, fixed = TRUE),
      x = primary,
      y = full_term_v
    )
  )
  out
}

#' Generic function for wrapping the RHS of a model equation in something, like
#' how the RHS of probit is wrapped in φ()
#'
#' @keywords internal
#'
#' @param model A fitted model
#' @param tex The TeX version of the RHS of the model (as character), built as
#'   \code{rhs_combined} or \code{eq_raw$rhs} in \code{extract_eq()}
#' @param \dots additional arguments passed to the specific extractor
#' @noRd

wrap_rhs <- function(model, tex, ...) {
  UseMethod("wrap_rhs", model)
}

#' @keywords internal
#' @noRd
wrap_rhs.default <- function(model, tex, ...) {
  return(tex)
}

#' @keywords internal
#' @noRd
wrap_rhs.glm <- function(model, tex, ...) {
  if (model$family$link == "probit") {
    rhs <- probitify(tex)
  } else {
    rhs <- tex
  }

  return(rhs)
}

#' @keywords internal
#' @noRd
wrap_rhs.polr <- function(model, tex, ...) {
  if (model$method == "probit") {
    rhs <- probitify(tex)
  } else {
    rhs <- tex
  }

  return(rhs)
}

#' @keywords internal
#' @noRd
wrap_rhs.clm <- function(model, tex, ...) {
  if (model$info$link == "probit") {
    rhs <- probitify(tex)
  } else {
    rhs <- tex
  }

  return(rhs)
}

#' @keywords internal
#' @noRd
probitify <- function(tex) {
  # Replace existing beginning-of-line \quad space with `\\qquad\` to account for \Phi
  tex <- gsub("&\\\\quad", "&\\\\qquad\\\\", tex)

  # It would be cool to use \left[ and \right] someday, but they don't work when
  # the equation is split across multiple lines (see
  # https://tex.stackexchange.com/q/21290/11851)
  paste0("\\Phi[", tex, "]")
}

#' Simulated data for time-series regression
#'
#' Output from \code{set.seed(42); ts_reg_list <- list(x1 = rnorm(1000), x2 = rnorm(1000), ts_rnorm = rnorm(1000))}.
#'
#' @format A tibble with 1000 rows and 8 variables:
#' \describe{
#'   \item{x1}{Random normal simulated data.}
#'   \item{x2}{Random normal simulated data.}
#'   \item{ts_rnorm}{Random normal simulated data.}
#' }
"ts_reg_list"

#' Utility function to wrap things as normally distributed
#' @keywords internal
#' @param mean The LaTeX code that should go into the mean part
#' @param sigma The LaTeX code that should go into the variance part.
#'   Defaults to sigma squared
#' @noRd
wrap_normal_dist <- function(mean, sigma = "\\sigma^2") {
  paste0("N \\left(", mean, ", ", sigma, " \\right)")
}

#' @noRd
wrap_binomial_dist <- function(p, n = 1) {
  paste0("\\operatorname{Binomial}(n = ", n, ", ", p, " = \\widehat{P})")
}

#' @noRd
create_logit <- function() {
  "\\log\\left[\\frac{\\hat{P}}{1 - \\hat{P}} \\right]"
}

#' @noRd
create_poisson_dist <- function() {
  paste0("\\operatorname{Poisson}(\\lambda_i)")
}



#### Helpers for distributions

#' For glmer models, check which family was used
#' @noRd
which_family <- function(model) {
  model@resp$family$family
}

#' @noRd
which_link <- function(model) {
  model@resp$family$link
}

#' Check if a glmer model uses an offset
#' @noRd
is_exposure_modeled <- function(model) {
  all(model@resp$offset != 0)
}

get_offset <- function(model, ital_vars) {
  offset_term <- as.character(as.list(model@call)$offset)
  
  if(length(offset_term) == 2) {
    offset_term[1] <- ifelse(offset_term[1] == "log", "\\log", offset_term[1])
    offset_term[2] <- add_tex_ital(escape_tex(offset_term[2]), ital_vars)
    offset_term <- paste0(offset_term[1], "(", offset_term[2], ")")
  } else if (length(offset_term) == 1) {
    offset_term <- add_tex_ital(escape_tex(offset_term), ital_vars)
  }
  
  offset_term
}



### Full distribution wrapping
binomial_logit_l1 <- function(model, lhs, l1, ital_vars) {
  outcome <- escape_tex(all.vars(formula(model))[1])
  out_v <- model@frame[[outcome]]
  if (is.factor(out_v)) {
    ss <- escape_tex(levels(out_v)[2])
  } else {
    ss <- 1
  }
  
  p <- paste0(
    "\\operatorname{prob}",
    add_tex_subscripts(
      paste0(
        add_tex_ital_v(outcome, ital_vars), " = ",
        ifelse(grepl("\\d", ss), ss, add_tex_ital_v(ss, ital_vars))
      )
    )
  )
  
  out <- paste0(lhs, " \\sim ", wrap_binomial_dist(p),
                " \\\\\n    ", create_logit(), " &=", l1)
}

poisson_log_l1 <- function(lhs, l1) {
  paste0(lhs, " \\sim ", create_poisson_dist(),
         " \\\\\n    ", "\\log(\\lambda_i)", " &=", l1)
}

#' Get a 'LaTeX' equation from a model or from 'LaTeX' code in a R Markdown or Quarto document
#'
#' @description
#' Extract or create a 'LaTeX' equation to describe a model, or directly from
#' 'LaTeX' code. All objects supported by [extract_eq()] are also supported by
#' the default method description. If labels (and units) are defined as
#' attributes to the data, they are automatically used in the equation in place
#' of the variable names.
#' @param object An object with a model whose the equation is constructed. If a
#'   **character** object is provided, it is concatenated into a single
#'   character string and the **equation** class, otherwise it is not
#'   transformed (it is supposed to be a valid 'LaTeX' equation). Remember that
#'   backslashes must be doubled in regular R strings, or use the `r"(...)"`
#'   notation, which may be more comfortable here, see examples.
#' @param ... Further parameters passed to [extract_eq()].
#'
#' @return An object of class `c("equation", "character")`.
#' @export
#'
#' @details
#' There are slight differences between `equation()`, `eq_()` and `eq__()`:
#' * `equation()` returns a string with 'LaTeX' code and prints 'LaTeX' code at
#'    the R console.
#' * `eq_()` returns the 'LaTeX' code surrounded by a dollar sign `$...$` and is
#'    suitable to build inline equations in R Markdown/Quarto documents by using
#'    inline R code. It prints the rendered inline equation in the RStudio
#'    Viewer or in the browser. So it is advised to rapidly preview the
#'    resulting equation.
#' * `eq__()` returns the 'LaTeX' code _not_ surrounded by dollar signs in a
#'    single character string. It also prints the rendered inline equation in
#'    the RStudio Viewer or in the browser.
#'    It should be used in an R inline chunk inside a `$$...$$`
#'    construct in a Markdown text. The result is a display equation that can
#'    also be cross referenced in Quarto in the usual way if you label it, e.g.,
#'    you use `$$...$$ {#eq-label}`.
#'
#' @examples
#' iris_lm <- lm(data = iris, Petal.Length ~ Sepal.Length + Species)
#' summary(iris_lm)
#' equation(iris_lm)
#' # Providing directly the LaTeX code of the equation (variance of a sample)
#' equation("S^2_y = \\sum_{i=1}^{n-1} \\frac{(y_i - \\bar{Y})^2}{n}")
#' # Easier to write like this (avoiding the double backslashes):
#' eq1 <- equation(r"(S^2_y = \sum_{i=1}^{n-1} \frac{(y_i - \bar{Y})^2}{n})")
#' # Print raw text:
#' eq1
#' # Get a preview of the equation
#' if (interactive())
#'   eq__(eq1)
#' # The same function can be used inside a `$$...$$ {#eq-label}` construct in
#' # R Markdown or Quarto to calcule a display equation that is also recognized
#' # by the cross referencing system of Quarto.
#' # Get a string suitable for inclusion inline in R Markdown with `r eq__(eq1)`
#' # (inside Markdown text and without the dollar signs around it)
#' # For inline equations in a Markdown text, you are better to use `r eq_(eq1)`
#' if (interactive())
#'   eq_(eq1)
equation <- function(object, ...) {
  UseMethod("equation")
}

#' @rdname equation
#' @export
#' @method equation default
#' @param auto.labs If `TRUE` (by default), use labels (and units) automatically
#'   from data or `origdata=`.
#' @param origdata The original data set this model was fitted to. By default it
#'   is `NULL` and no label is used.
#' @param labs Labels to change the names of elements in the `term` column of
#'   the table. By default it is `NULL` and no term is changed.
#' @param swap_var_names Change the variable names for these values, regardless
#'   of the values in `auto.labs=` or `labs=` that are ignored if this argument
#'   is used. Provide a named character string with name being the variables and
#'   strings the new names. You can use `^` or `_` to indicate next character,
#'   or next integer should be super- or subscript in the equation.
equation.default <- function(object, auto.labs = TRUE, origdata = NULL,
    labs = NULL, swap_var_names = NULL, ...) {
  # Extract labels off data or origdata
  if (missing(swap_var_names)) {
    if (isTRUE(auto.labs)) {
      labs <- try(.labels2(x = object, origdata = origdata, labs = labs),
        silent = TRUE)
      if (inherits(labs, "try-error")) {
        #warning("Cannot extract labels from the object")
        labs <- NULL
      }
    } else {
      labs <- .labels2(x = NULL, labs = labs)
    }
    if (!is.null(labs)) {
      res <- extract_eq(model = object, swap_var_names =  labs, ...)
    } else {
      res <- extract_eq(model = object, ...)
    }
  } else {# swap_var_names provided
    res <- extract_eq(model = object, swap_var_names =  swap_var_names, ...)
  }
  
  # In case we use ^ or _, we mean "put next character in super or subscript"
  # but equatiomatic tries hard to preserve the character in LaTeX
  # -> make the correction now
  # Restore ^ for superscript
  res <- gsub("\\texttt{\\^{}}", "^", res, fixed = TRUE)
  res <- gsub("\\texttt{^}", "^", res, fixed = TRUE)
  # Transform the result from ~ into _ (LaTeX subscript)
  res <- gsub("\\_", "_", res, fixed = TRUE)
  # Make sure all digits (and leading minus sign are super- or subscript)
  res <- gsub("([\\^_])(-?[0-9]+)", "\\1{\\2}", res)
  
  # If it appears there is a beta_1 term, but no beta_0 or beta_2 terms, just-
  # rename it beta
  if (grepl("\\beta_{1}", res, fixed = TRUE) &&
      !grepl("\\beta_{0}", res, fixed = TRUE) &&
      !grepl("\\beta_{2}", res, fixed = TRUE)) {
    res <- gsub("\\beta_{1}", "\\beta_{}", res, fixed = TRUE)
  }
  
  res
}

#' @rdname equation
#' @export
#' @method equation character
equation.character <- function(object, ...) {
  eq <- paste(object, collapse = "\n")
  class(eq) <- c("equation", "character")
  eq
}

#' @rdname equation
#' @export
eq__ <- function(object, ...) {
  eq <-  equation(object, ...)
  class(eq) <- c("inline_equation", "character")
  eq
}

#' @rdname equation
#' @export
eq_ <- function(object, ...) {
  eq <-  equation(object, ...)
  eq <- paste0("$", paste(eq, collapse = "\n"), "$")
  class(eq) <- c("inline_equation", "character")
  eq
}

#' @rdname equation
#' @export
#' @method print inline_equation
#' @param x An **inline_equation** object generated by `eq_()` or `eq__()`.
print.inline_equation <- function(x, ...) {
  if (grepl("^\\$.+\\$$", x)) {
    eq <- substring(x, 2L, nchar(x) - 1L)
  } else {
    eq <- x
  }
  eq <- structure(gsub("\n", " ", eq), class = c("equation", "character"))
  preview_eq(eq)
  invisible(x)
}

.labels <- function(x, units = TRUE, ...) {
  
  .get_label_with_units <- function(x, units = TRUE) {
    lab <- attr(x, "label")
    if (is.null(lab))
      return("")

    if (isTRUE(units)) {
      un <- attr(x, "units")
      if (!is.null(un))
        lab <- paste0(lab, " [", un, "]")
    }
    lab
  }
  
  labels <- sapply(x, .get_label_with_units, units = units)
  
  if (any(labels != "")) {
    # Use a \n before labels and the units
    if (isTRUE(units))
      labels <- sub(" +\\[([^]]+)\\]$", "\n [\\1]", labels)
    # set names if empty
    labels[labels == ""] <- names(x)[labels == ""]
    # Specific case for I() using in a formula
    for (i in seq_along(labels)) {
      # check if I() includes an exponent.
      if (grepl("^I\\(.*\\^.*\\)$", names(labels)[i])) {
        # extract the exponent of I()
        vec <- sub("^I\\(.*(\\^.*)\\).*", "\\1", names(labels)[i])
        
        if (isTRUE(units) && grepl("\n", labels[i])) {
          # vec[i] <- sub(" \\n", paste0(sous_chaine, "\\\n"), vec[i])
          labels[i] <- gsub("(.*)(\\n \\[)(.*)(\\])",
            paste0("\\1", vec, "\\2", "\\3", vec,"\\4"), labels[i])
        } else {
          labels[i] <- paste0(labels[i], vec)
        }
      }
    }
    labels[grepl("^I\\(([^\\^]*)\\)$", names(labels))] <- names(labels)[
      grepl("^I\\(([^\\^]*)\\)$", names(labels))]
  }
  
  if (all(labels == ""))
    labels <- NULL
  
  labels
}

.labels2 <- function(x, origdata = NULL, labs = NULL) {
  
  if (is.null(origdata)) {
    labs_auto <- .labels(x$model)
  } else {
    labs_auto <- .labels(origdata)
  }
  
  if (!is.null(labs)) {
    if (!is.character(labs))
      stop("labs is not character vector")
    if (is.null(names(labs)))
      stop("labs must be named character vector")
    if (any(names(labs) %in% ""))
      stop("all element must be named")
    labs_res <- c(labs, labs_auto[!names(labs_auto) %in% names(labs)])
  } else {
    labs_res <- labs_auto
  }
  
  labs_res
}

#' Print 'LaTeX' equations
#' 
#' Print 'LaTeX' equations built with \code{\link{extract_eq}}.
#'
#' @method print equation
#' @export
#'
#' @param x 'LaTeX' equation built with \code{\link{extract_eq}}
#' @param ... not used
#' @return The unmodified object 'x' is returned invisibly. The function is
#'   used for its side effect of printing the equation.

print.equation <- function(x, ...) {
  cat(format(x), sep = "")
  invisible(x)
}

#' Print 'LaTeX' equations in R Markdown environments
#'
#' Print 'LaTeX' equations built with \code{\link{extract_eq}} nicely in R Markdown environments.
#'
#' @method knit_print equation
#' @export
#'
#' @param x 'LaTeX' equation built with \code{\link{extract_eq}}
#' @param ... not used
#' @param tex_packages A string with LaTeX code to include in the header,
#'   usually to include LaTeX packages in the output.
#' 
#' @return A string with the equation formatted according to R Markdown's output
#'   format (different output for HTML, PDF, docx, gfm, markdown_strict). The
#'   format is detected automatically, so, you do not have to worry about it.
#'
#' @importFrom knitr knit_print asis_output
#'
knit_print.equation <- function(x, ..., tex_packages = "\\renewcommand*\\familydefault{\\rmdefault}") {
  eq <- format(x)
  if (isTRUE(knitr::opts_knit$get("rmarkdown.pandoc.to") %in% "markdown_strict")) {
    # Just print raw LaTeX
    print(eq)
  } else {
    return(asis_output(eq))
  }
}

#' Format 'LaTeX' equations
#'
#' Format 'LaTeX' equations built with \code{\link{extract_eq}}.
#'
#' @export
#' @method format equation
#'
#' @param x 'LaTeX' equation built with \code{\link{extract_eq}}
#' @param ... not used
#' @param latex Logical, whether the output is LaTeX or not. The default
#'   value uses [knitr::is_latex_output()] to determine the current output format.
#' @return A character string with the equation formatted either as proper
#'   LaTeX code, or as a display equation tag (surrounded by `$$...$$`) for R
#'   Markdown or Quarto documents.
#'
format.equation <- function(x, ..., latex = knitr::is_latex_output()) {
  if (isTRUE(latex)) {
    header <- paste0(attr(x, "latex_define_colors"), collapse = "\n")
    eq <- paste0("\n\n\\begin{equation}\n", x, "\n\\end{equation}")
    paste0(header, eq)
  } else {
    paste0("$$\n", x, "\n$$\n")
  }
}

#' Preview an equation in a web browser
#'
#' Preview 'LaTeX' equations built with \code{\link{extract_eq}}.
#'
#' @param x 'LaTeX' equation built with \code{\link{extract_eq}}
#' @param ... not used
#'
#' @returns The path to the temporary html file that was created to preview the
#'   equation is returned invisibly.
#' @export
#'
#' @examples
#' if (interactive()) {
#'   mod1 <- lm(mpg ~ cyl + disp, mtcars)
#'   eq1 <- extract_eq(mod1)
#'   eq1 # Not that nice
#'   preview_eq(eq1)
#'   # or easier...
#'   preview_eq(mod1)
#' }
preview_eq <- function(x, ...) {
  if (!rmarkdown::pandoc_available())
    stop("Pandoc is not available. Please install it to use this function.",
      call. = FALSE)
  
  if (!inherits(x, c("equation", "list")))
    x <- extract_eq(x)
  
  rmd <- tempfile(fileext = ".Rmd")
  cat(sprintf("---\ntitle: \"&nbsp;\"\n---\n$$\n%s\n$$\n", x), file = rmd)
  rmarkdown::render(rmd, output_format = rmarkdown::html_document(...),
    quiet = TRUE)
  
  file_out <- sub("\\.Rmd$", ".html", rmd)
  viewer <- getOption("viewer", default = getOption("browser", NULL))
  if (!is.function(viewer))
    viewer <- utils::browseURL
  viewer(file_out)
  invisible(file_out)
}

# PhG: I am not a fan of stating all the possible arguments in the generic
# function like it is currently the case for extract_eq(). May be keep only the
# common and most used ones and allow for varying arguments with ... However,
# changing this could possibly break backward compatibility. So, I leave it as
# it is for now... but I had to homogenize for all methods, otherwise CRAN would
# reject the package!

#' 'LaTeX' equation for R models
#' 
#' Extract the variable names from a model to produce a 'LaTeX' equation.
#' Supports any model where there is a [broom::tidy()] method. This is a generic
#' function with methods for lmerMod objects obtained with [lme4::lmer()],
#' glmerMod objects with [lme4::glmer()], forecast_ARIMA with
#' [forecast::Arima()] and default, with the later further covering most "base"
#' R models implemented in [broom::tidy()] like lm objects with [stats::lm()],
#' glm objects with [stats::glm()] or polr objects with [MASS::polr()]. The
#' default method also supports clm objects obtained with [ordinal::clm()].
#'
#' @param model A fitted model
#' @param intercept How should the intercept be displayed? Default is \code{"alpha"},
#'   but can also accept \code{"beta"}, in which case the it will be displayed
#'   as beta zero.
#' @param greek What notation should be used for
#'   coefficients? Currently only accepts \code{"beta"} (with plans for future
#'   development). Can be used in combination with \code{raw_tex} to use any
#'   notation, e.g., \code{"\\hat{\\beta}"}.
#' @param greek_colors The colors of the greek notation in the equation. Must
#'   be a single color (named or HTML hex code) or a vector of colors (which
#'   will be recycled if smaller than the number of terms in the model). When
#'   rendering to PDF, I suggest using HTML hex codes, as not all named colors
#'   are recognized by LaTeX, but equatiomatic will internally create the
#'   color definitions for you if HTML codes are supplied. Note that this is 
#'   not yet implemented for mixed effects models (lme4). 
#' @param subscript_colors The colors of the subscripts for the greek notation.
#'   The argument structure is equivalent to \code{greek_colors} (i.e., see
#'   above for more detail).
#' @param var_colors The color of the variable names. This takes a named vector
#'   of the form \code{c("variable" = "color")}. For example 
#'   \code{c("bill_length_mm" = "#00d4fa", "island" = "#00fa85")}. Colors can
#'   be names (e.g., \code{"red"}) or HTML hex codes, as shown in the example.
#' @param var_subscript_colors The colors of the factor subscripts for 
#'   categorical variables. The interface for this is equivalent to 
#'   \code{var_colors}, and all subscripts for a given variable will be 
#'   displayed in the provided color. For example, the code 
#'   \code{c("island" = "green")} would result in the subscripts for "Dream"
#'   and "Torgersen" being green (assuming "Biscoe" was the reference group).
#' @param raw_tex Logical. Is the greek code being passed to denote coefficients
#' raw tex code?
#' @param swap_var_names A vector of the form c("old_var_name" = "new name"). 
#'   For example: c("bill_length_mm" = "Bill Length (MM)").
#' @param swap_subscript_names A vector of the form 
#'   c("old_subscript_name" = "new name"). For example: 
#'   c("f" = "Female").
#' @param ital_vars Logical, defaults to \code{FALSE}. Should the variable names
#'   not be wrapped in the \code{\\operatorname{}} command?
#' @param label A label for the equation, which can then be used for in-text 
#'   references. See example [here](https://www.overleaf.com/learn/latex/Cross_referencing_sections,_equations_and_floats#Referencing_equations.2C_figures_and_tables).
#'   Note that this **only works for PDF output**. The in-text references also 
#'   must match the label exactly, and must be formatted as 
#'   \code{\\ref{eq: label}}, where \code{label} is a place holder for the
#'   specific label. Notice the space after the colon before the label. This
#'   also must be there, or the cross-reference will fail.
#' @param index_factors Logical, defaults to \code{FALSE}. Should the factors 
#' be indexed, rather than using subscripts to display all levels?
#' @param show_distribution Logical. When fitting a logistic or probit
#'   regression, should the binomial distribution be displayed? Defaults to
#'   \code{FALSE}.
#' @param wrap Logical, defaults to \code{FALSE}. Should the terms on the
#'   right-hand side of the equation be split into multiple lines? This is
#'   helpful with models with many terms.
#' @param terms_per_line Integer, defaults to 4. The number of right-hand side
#'   terms to include per line. Used only when \code{wrap} is \code{TRUE}.
#' @param operator_location Character, one of \dQuote{end} (the default) or
#'   \dQuote{start}. When terms are split across multiple lines, they are split
#'   at mathematical operators like `+`. If set to \dQuote{end}, each line will
#'   end with a trailing operator (`+` or `-`). If set to \dQuote{start}, each
#'   line will begin with an operator.
#' @param align_env TeX environment to wrap around equation. Must be one of
#'   \code{aligned}, \code{aligned*}, \code{align}, or \code{align*}. Defaults
#'   to \code{aligned}.
#' @param use_coefs Logical, defaults to \code{FALSE}. Should the actual model
#'   estimates be included in the equation instead of math symbols?
#' @param coef_digits Integer, defaults to 2. The number of decimal places to
#'   round to when displaying model estimates.
#' @param fix_signs Logical, defaults to \code{TRUE}. If disabled,
#'   coefficient estimates that are negative are preceded with a "+" (e.g.
#'   `5(x) + -3(z)`). If enabled, the "+ -" is replaced with a "-" (e.g.
#'   `5(x) - 3(z)`).
#' @param font_size The font size of the equation. Defaults to default of
#'   the output format. Takes any of the standard LaTeX arguments (see 
#'   [here](https://www.overleaf.com/learn/latex/Font_sizes,_families,_and_styles#Font_styles)).
#' @param mean_separate Currently only support for \code{\link[lme4]{lmer}}
#'   models. Should the mean structure be inside or separated from the
#'   normal distribution? Defaults to \code{NULL}, in which case it will become
#'   \code{TRUE} if there are more than three fixed-effect parameters. If
#'   \code{TRUE}, the equation will be displayed as, for example,
#'   outcome ~ N(mu, sigma); mu = alpha + beta_1(wave). If \code{FALSE}, this
#'   same equation would be outcome ~ N(alpha + beta, sigma).
#' @param return_variances Logical. When \code{use_coefs = TRUE} with a mixed
#'   effects model (e.g., \code{lme4::lmer()}), should the variances and
#'   co-variances be returned? If \code{FALSE} (the default) standard deviations
#'   and correlations are returned instead.
#' @param se_subscripts Logical. If \code{se_subscripts = TRUE} then the 
#'   equation will include the standard errors below each coefficient. 
#'   This is supported for lm and glm models.
#' @param logit_notation Logical. If \code{TRUE}, then the equation uses a more
#'   compact notation for a logistic regression.
#' @param ... Additional arguments (for future development; not currently used).
#' @export
#'
#' @return A character of class \dQuote{equation}.
#' 
#' @details
#' The different methods all use the same arguments, but not all arguments are
#' suitable to all models. Check here above to determine if a feature is
#' implemented for a given model.
#' 
#' @examples
#' # Simple model
#' mod1 <- lm(mpg ~ cyl + disp, mtcars)
#' extract_eq(mod1)
#'
#' # Include all variables
#' mod2 <- lm(mpg ~ ., mtcars)
#' extract_eq(mod2)
#'
#' # Works for categorical variables too, putting levels as subscripts
#' data("penguins", package = "equatiomatic")
#' mod3 <- lm(body_mass_g ~ bill_length_mm + species, penguins)
#' extract_eq(mod3)
#'
#' set.seed(8675309)
#' d <- data.frame(
#'   cat1 = rep(letters[1:3], 100),
#'   cat2 = rep(LETTERS[1:3], each = 100),
#'   cont1 = rnorm(300, 100, 1),
#'   cont2 = rnorm(300, 50, 5),
#'   out = rnorm(300, 10, 0.5)
#' )
#' mod4 <- lm(out ~ ., d)
#' extract_eq(mod4)
#'
#' # Don't italicize terms
#' extract_eq(mod1, ital_vars = FALSE)
#'
#' # Wrap equations in an "aligned" environment
#' extract_eq(mod2, wrap = TRUE)
#'
#' # Wider equation wrapping
#' extract_eq(mod2, wrap = TRUE, terms_per_line = 4)
#'
#' # Include model estimates instead of Greek letters
#' extract_eq(mod2, wrap = TRUE, terms_per_line = 2, use_coefs = TRUE)
#'
#' # Don't fix doubled-up "+ -" signs
#' extract_eq(mod2, wrap = TRUE, terms_per_line = 4, use_coefs = TRUE, fix_signs = FALSE)
#'
#' # Use indices for factors instead of subscripts
#' extract_eq(mod2, wrap = TRUE, terms_per_line = 4, index_factors = TRUE)
#'
#' # Use other model types, like glm
#' set.seed(8675309)
#' d <- data.frame(
#'   out = sample(0:1, 100, replace = TRUE),
#'   cat1 = rep(letters[1:3], 100),
#'   cat2 = rep(LETTERS[1:3], each = 100),
#'   cont1 = rnorm(300, 100, 1),
#'   cont2 = rnorm(300, 50, 5)
#' )
#' mod5 <- glm(out ~ ., data = d, family = binomial(link = "logit"))
#' extract_eq(mod5, wrap = TRUE)
#' # More compact logit notation
#' extract_eq(mod5, wrap = TRUE, logit_notation = TRUE)
extract_eq <- function(model, intercept = "alpha", greek = "beta",
                       greek_colors = NULL, subscript_colors = NULL,
                       var_colors = NULL, var_subscript_colors = NULL, 
                       raw_tex = FALSE, 
                       swap_var_names = NULL, swap_subscript_names = NULL,
                       ital_vars = FALSE, label = NULL,
                       index_factors = FALSE, show_distribution = FALSE,
                       wrap = FALSE, terms_per_line = 4,
                       operator_location = "end", align_env = "aligned",
                       use_coefs = FALSE, coef_digits = 2,
                       fix_signs = TRUE, font_size = NULL,
                       mean_separate = NULL, return_variances = FALSE,
                       se_subscripts = FALSE, logit_notation = FALSE, ...) {
  UseMethod("extract_eq", model)
}

# PhG: we do not want to repeat usage, so no @describeIn extract_eq default `extract_eq()` method
#' Default function for extracting an equation from a model object
#'
#' @keywords internal
#' @export
#' @noRd
extract_eq.default <- function(model, intercept = "alpha", greek = "beta",
                               greek_colors = NULL, subscript_colors = NULL,
                               var_colors = NULL, var_subscript_colors = NULL, 
                               raw_tex = FALSE, 
                               swap_var_names = NULL, swap_subscript_names = NULL,
                               ital_vars = FALSE, label = NULL,
                               index_factors = FALSE, show_distribution = FALSE,
                               wrap = FALSE, terms_per_line = 4,
                               operator_location = "end", align_env = "aligned",
                               use_coefs = FALSE, coef_digits = 2,
                               fix_signs = TRUE, font_size = NULL,
                               mean_separate = NULL, return_variances = FALSE, 
                               se_subscripts = FALSE, logit_notation = FALSE,
                               ...) {
  if (index_factors & use_coefs) {
    stop("Coefficient estimates cannot be returned when factors are indexed.")
  }
  
  lhs <- extract_lhs(model, ital_vars, show_distribution, use_coefs, 
    swap_var_names, var_colors, logit_notation = logit_notation)
  rhs <- extract_rhs(model, index_factors)
  
  eq_raw <- create_eq(
    model, lhs, rhs, ital_vars, use_coefs, coef_digits,
    fix_signs, intercept, greek, 
    greek_colors, subscript_colors, var_colors, var_subscript_colors, raw_tex,
    index_factors, swap_var_names, swap_subscript_names
  )
  
  if (se_subscripts) {
    eq_raw$rhs[[1]] <- add_se(eq_raw$rhs[[1]], model)
  }

  if (wrap) {
    if (operator_location == "start") {
      line_end <- "\\\\\n&\\quad + "
    } else {
      line_end <- "\\ + \\\\\n&\\quad "
    }

    # Split all the RHS terms into groups of length terms_per_line
    rhs_groups <- lapply(eq_raw$rhs, function(x) {
      split(x, ceiling(seq_along(x) / terms_per_line))
    })

    # Collapse the terms with + within each group
    rhs_groups_collapsed <- lapply(rhs_groups, function(x) {
      vapply(x, paste0, collapse = " + ", FUN.VALUE = character(1))
    })

    # Collapse the collapsed groups with the line ending (trailing or leading +)
    rhs_combined <- lapply(rhs_groups_collapsed, function(x) {
      paste(x, collapse = line_end)
    })
  } else {
    rhs_combined <- lapply(eq_raw$rhs, function(x) {
      paste(x, collapse = " + ")
    })
  }

  if (wrap | 
      length(rhs_combined) > 1 | 
      show_distribution | 
      !is.null(font_size)) {
    needs_align <- TRUE
  } else {
    needs_align <- FALSE
  }

  # Combine RHS and LHS
  eq <- Map(function(.lhs, .rhs) {
    paste(.lhs, .rhs,
      sep = ifelse(needs_align, " &= ", " = ")
    )
  },
  .lhs = eq_raw$lhs,
  .rhs = wrap_rhs(model, rhs_combined)
  )

  if (use_coefs && fix_signs) {
    eq <- lapply(eq, fix_coef_signs)
  }
  
  if (use_coefs && fix_signs && se_subscripts) {
    eq <- lapply(eq, fix_coef_signs_se)
  }

  if (length(eq) > 1) {
    eq <- paste(eq, collapse = " \\\\\n")
  } else {
    eq <- eq[[1]]
  }

  # Add environment finally, if wrapping or if there are multiple equations
  # This comes later so that multiple equations don't get their own environments
  if (needs_align) {
    eq <- paste0(
      "\\begin{", align_env, "}\n",
      eq,
      "\n\\end{", align_env, "}"
    )
  }
  if (!is.null(label)) {
    eq <- paste0("\\label{eq: ", label, "}\n", eq)
  }
  if (!is.null(font_size)) {
    eq <- paste0("\\", font_size, "\n", eq)
  }
  
  if (
    any(
      !is.null(c(greek_colors, 
                 subscript_colors, 
                 var_colors, 
                 var_subscript_colors))
      )
    ) {
    
    full_colors <- unique(
      c(greek_colors, subscript_colors, var_colors, var_subscript_colors)
    ) 
  
    attributes(eq) <- list(
      latex_define_colors = define_latex_html_colors(full_colors)
    )
  }
  
  class(eq) <- c("equation", "character")
  
  return(eq)
}



# These args still need to be incorporated
# intercept, greek, raw_tex
# I haven't incorporated wrap yet either and we should think about if we want to
# It might be better to have an alternative for matrix notation

# PhG: we do not want to repeat usage, so no @describeIn extract_eq lmerMod `extract_eq()` method
#' @export
#' @noRd
extract_eq.lmerMod <- function(model, intercept = "alpha", greek = "beta",
                               greek_colors = NULL, subscript_colors = NULL,
                               var_colors = NULL, var_subscript_colors = NULL, 
                               raw_tex = FALSE,
                               swap_var_names = NULL, swap_subscript_names = NULL,
                               ital_vars = FALSE, label = NULL,
                               index_factors = FALSE, show_distribution = FALSE,
                               wrap = FALSE, terms_per_line = 4,
                               operator_location = "end", align_env = "aligned",
                               use_coefs = FALSE, coef_digits = 2,
                               fix_signs = TRUE, font_size = NULL,
                               mean_separate = NULL, return_variances = FALSE,
                               se_subscripts = FALSE, logit_notation = FALSE,
                               ...) {
  if (isTRUE(se_subscripts)) {
    warning("Standard errors are not supported for mixed effects models",
      call. = FALSE
    )
  }
  
  if (!is.null(greek_colors)) {
    warning(
      paste0("Colorization of greek notation not currently ",
             "implemented for merMod models"),
      call. = FALSE
    )
  }
  if (!is.null(subscript_colors)) {
    warning(
      paste0("Colorization of subscripts not currently ",
             "implemented for merMod models"),
      call. = FALSE
      )
  }
  l1 <- create_l1(model, mean_separate,
    ital_vars, wrap, terms_per_line,
    use_coefs, coef_digits, fix_signs,
    operator_location,
    sigma = "\\sigma^2", return_variances,
    swap_var_names, swap_subscript_names,
    var_colors, var_subscript_colors
  )
  vcv <- create_ranef_structure_merMod(
    model, ital_vars, use_coefs, coef_digits,
    fix_signs, return_variances, swap_var_names, swap_subscript_names,
    var_colors, var_subscript_colors
  )
  
  # check for double line breaks
  dbl_brk <- any(
    vapply(vcv, function(x) grepl("\n\\W+\n", x), FUN.VALUE = logical(1))
  )
  
  if (dbl_brk) {
      vcv <- lapply(vcv, function(x) {
        gsub("^\n(.+)", "\\1", x)
      })
  }

  eq <- paste(c(l1, vcv), collapse = " \\\\")
  eq <- gsub("\\sim", " &\\sim", eq, fixed = TRUE)
  eq <- paste(eq, collapse = " \\\\ \n")
  eq <- paste0(
    "\\begin{", align_env, "}\n",
    paste0("  ", eq),
    "\n\\end{", align_env, "}"
  )
  if (!is.null(label)) {
    eq <- paste0("\\label{eq: ", label, "}\n", eq)
  }
  if (!is.null(font_size)) {
    eq <- paste0("\\", font_size, "\n", eq)
  }
  class(eq) <- c("equation", "character")

  eq
}

# PhG: we do not want to repeat usage, so no @describeIn extract_eq glmerMod `extract_eq()` method
#' @export
#' @noRd
extract_eq.glmerMod <- function(model, intercept = "alpha", greek = "beta",
                                greek_colors = NULL, subscript_colors = NULL,
                                var_colors = NULL, var_subscript_colors = NULL, 
                                raw_tex = FALSE,
                                swap_var_names = NULL, swap_subscript_names = NULL,
                                ital_vars = FALSE, label = NULL,
                                index_factors = FALSE, show_distribution = FALSE,
                                wrap = FALSE, terms_per_line = 4,
                                operator_location = "end", align_env = "aligned",
                                use_coefs = FALSE, coef_digits = 2,
                                fix_signs = TRUE, font_size = NULL,
                                mean_separate = NULL, return_variances = FALSE,
                                se_subscripts = FALSE, logit_notation = FALSE,
                                ...) {
  if (!is.null(greek_colors)) {
    warning(
      paste0("Colorization of greek notation not currently ",
             "implemented for merMod models"))
  }
  if (!is.null(subscript_colors)) {
    warning(
      paste0("Colorization of subscripts not currently ",
             "implemented for merMod models"))
  }
  extract_eq.lmerMod(model, intercept = intercept, greek = greek,
    greek_colors = greek_colors, subscript_colors = subscript_colors,
    var_colors = var_colors, var_subscript_colors = var_subscript_colors,
    raw_tex = raw_tex, swap_var_names = swap_var_names,
    swap_subscript_names = swap_subscript_names, ital_vars = ital_vars,
    label = label, index_factors = index_factors,
    show_distribution = show_distribution, wrap = wrap,
    terms_per_line = terms_per_line, operator_location = operator_location,
    align_env = align_env, use_coefs = use_coefs, coef_digits = coef_digits,
    fix_signs = fix_signs, font_size = font_size, mean_separate = mean_separate,
    return_variances = return_variances, se_subscripts = se_subscripts, ...)
}

# PhG: we do not want to repeat usage, so no @describeIn extract_eq forecast_ARIMA `extract_eq()` method
#' @export
#' @noRd
extract_eq.forecast_ARIMA <- function(model, intercept = "alpha", greek = "beta",
                                      greek_colors = NULL, subscript_colors = NULL,
                                      var_colors = NULL, var_subscript_colors = NULL, 
                                      raw_tex = FALSE, 
                                      swap_var_names = NULL, swap_subscript_names = NULL,
                                      ital_vars = FALSE, label = NULL,
                                      index_factors = FALSE, show_distribution = FALSE,
                                      wrap = FALSE, terms_per_line = 4,
                                      operator_location = "end", align_env = "aligned",
                                      use_coefs = FALSE, coef_digits = 2,
                                      fix_signs = TRUE, font_size = NULL,
                                      mean_separate = NULL, return_variances = FALSE,
                                      se_subscripts = FALSE, logit_notation = FALSE, ...) {

  if (isTRUE(se_subscripts)) {
    warning("Standard errors are not supported for mixed effects models",
      call. = FALSE
    )
  }
  
  # Determine if we are working on Regression w/ Arima Errors
  regression <- helper_arima_is_regression(model)

  # Get each of the sides
  lhs <- extract_lhs(model)
  rhs <- extract_rhs(model)

  if (regression) {
    yt <- helper_arima_extract_lm(model)
  } else {
    yt <- NULL
  }

  # Extract the equation lists
  eq <- create_eq(
    model,
    lhs,
    rhs,
    yt,
    ital_vars,
    use_coefs,
    coef_digits,
    raw_tex,
    swap_var_names,
    swap_subscript_names
  )

  ##########
  # Wrapping has not been included due to the
  # Multiline nature of Regression w/ ARIMA errors
  ##########

  ##########
  # Fix signs is done automatically
  # This is necessary so that the Lag/Backshift equations will make sense.
  #########

  # Collapse down terms.
  eq <- lapply(eq, function(x) {
    lhs <- paste(x$lhs[[1]], collapse = " ")
    rhs <- paste(x$rhs[[1]], collapse = " ")
    rhs <- gsub("^\\+", "", rhs)

    # Alignment, if needed, will be added later.
    paste(lhs, rhs, sep = " = ")
  })

  # If regression w/ arima errors.
  if (regression) {
    # Add alignment to the regression function
    eq$lm_eq <- paste0("&\\text{let}\\quad &&", eq$lm_eq)

    # Add alignment and "where" to ARIMA line
    # Need to re-split the terms. This is redundant, but makes for less repeated code.
    # Ensure it is seen as a character vector first and foremost.
    split_arima <- strsplit(eq$arima_eq, "=")[[1]]
    names(split_arima) <- c("ar", "ma")

    split_arima["ar"] <- paste0("&\\text{where}\\quad  &&", split_arima["ar"])
    split_arima["ma"] <- paste0("& &&=", split_arima["ma"])

    eq$arima_eq <- paste(split_arima, collapse = " \\\\\n")

    # Add line (always the same) indicating the distribution of the residual.
    eq$err_dist <- "&\\text{where}\\quad &&\\varepsilon_{t} \\sim{WN(0, \\sigma^{2})}"

    # Add alignment to the equation structure.
    eq <- paste0(
      "\\begin{alignat}{2}\n",
      paste(eq, collapse = " \\\\\n"),
      "\n\\end{alignat}"
    )
    if (!is.null(font_size)) {
      eq <- paste0("\\", font_size, "\n", eq)
    }
  } else {
    # If arima only then we only need 1 line and no alignment.
    eq <- eq$arima_eq
    if (!is.null(label)) {
      eq <- paste0("\\label{eq: ", label, "}\n", eq)
    }
    if (!is.null(font_size)) {
      eq <- paste0(
        "\\begin{aligned}\n",
        paste(eq, collapse = " \\\\\n"),
        "\n\\end{aligned}"
      )
      eq <- paste0("\\", font_size, "\n", eq)
    }
  }
  
  # Set the class
  class(eq) <- c("equation", "character")

  # Explicit return
  return(eq)
}

#' Add Standard Errors Below Coefficients
#' 
#' @param coef character vector of model coefficients (from output of the function create_eq)
#' @param model a fitted model 
#' 
#' @return a character vector adding the errors beneath each term
#' @noRd
add_se <- function(coef, model) {
  errors <- summary(model)$coefficients[,"Std. Error"]
  errors <- as.character(round(errors, 3))
  if (length(coef) != length(errors)) {
    result <- paste0("\\underset{(", errors, ")}{", coef[-length(coef)], "}")
    result <- c(result, coef[length(coef)])
  }
  else {
    result <- paste0("\\underset{(", errors, ")}{", coef, "}")
  }
  return(result)
}



#' Fixes the Signs When Using se_subscripts Argument
#' 
#' @param equation list that contains the equation
#' 
#' @return a list containing the equation with fixed signs
#' @noRd
fix_coef_signs_se <- function(equation) {
  components <- strsplit(equation, " + ", fixed = TRUE)
  components <- unlist(components)
  terms <- components[2:length(components)]
  negative <- ifelse(grepl(terms, pattern = "-"), " - ", " + ")
  terms <- lapply(X = terms, FUN = gsub,  pattern = "-", replacement = "")
  terms <- paste0(negative, terms, collapse = "")
  list(paste(components[1], terms))
}

#' @export
#' @noRd
extract_eq.model_fit <- 
  function(model, intercept = "alpha", greek = "beta",
           greek_colors = NULL, subscript_colors = NULL,
           var_colors = NULL, var_subscript_colors = NULL, 
           raw_tex = FALSE, 
           swap_var_names = NULL, swap_subscript_names = NULL,
           ital_vars = FALSE, label = NULL,
           index_factors = FALSE, show_distribution = FALSE,
           wrap = FALSE, terms_per_line = 4,
           operator_location = "end", align_env = "aligned",
           use_coefs = FALSE, coef_digits = 2,
           fix_signs = TRUE, font_size = NULL,
           mean_separate = NULL, return_variances = FALSE, 
           se_subscripts = FALSE, logit_notation = FALSE, ...) {
    
  if ("fit" %in% names(model)) {
    fitted_model <- model$fit
    extract_eq(fitted_model, intercept = intercept, greek = greek,
      greek_colors = greek_colors, subscript_colors = subscript_colors,
      var_colors = var_colors, 
      var_subscript_colors = var_subscript_colors, 
      raw_tex = raw_tex, 
      swap_var_names = swap_var_names, 
      swap_subscript_names = swap_subscript_names,
      ital_vars = ital_vars, label = label,
      index_factors = index_factors, 
      show_distribution = show_distribution,
      wrap = wrap, terms_per_line = terms_per_line,
      operator_location = operator_location, 
      align_env = align_env,
      use_coefs = use_coefs, coef_digits = coef_digits,
      fix_signs = fix_signs, font_size = font_size,
      mean_separate = mean_separate, 
      return_variances = return_variances, 
      se_subscripts = se_subscripts, logit_notation = logit_notation, ...)  
  } else {
    stop("The 'model' does not appear to be a proper **model_fit** object ",
      "because it does not have a 'fit' component.")
  }
}

#' @export
#' @noRd
extract_eq.workflow <- 
  function(model, intercept = "alpha", greek = "beta",
           greek_colors = NULL, subscript_colors = NULL,
           var_colors = NULL, var_subscript_colors = NULL, 
           raw_tex = FALSE, 
           swap_var_names = NULL, swap_subscript_names = NULL,
           ital_vars = FALSE, label = NULL,
           index_factors = FALSE, show_distribution = FALSE,
           wrap = FALSE, terms_per_line = 4,
           operator_location = "end", align_env = "aligned",
           use_coefs = FALSE, coef_digits = 2,
           fix_signs = TRUE, font_size = NULL,
           mean_separate = NULL, return_variances = FALSE, 
           se_subscripts = FALSE, logit_notation = FALSE, ...) {
    
  if ("fit" %in% names(model)) {
    fitted_stage <- model$fit
    if ("fit" %in% names(fitted_stage)) {
      fitted_model <- fitted_stage$fit
      extract_eq(fitted_model, intercept = intercept, greek = greek,
        greek_colors = greek_colors, subscript_colors = subscript_colors,
        var_colors = var_colors, 
        var_subscript_colors = var_subscript_colors, 
        raw_tex = raw_tex, 
        swap_var_names = swap_var_names, 
        swap_subscript_names = swap_subscript_names,
        ital_vars = ital_vars, label = label,
        index_factors = index_factors, 
        show_distribution = show_distribution,
        wrap = wrap, terms_per_line = terms_per_line,
        operator_location = operator_location, 
        align_env = align_env,
        use_coefs = use_coefs, coef_digits = coef_digits,
        fix_signs = fix_signs, font_size = font_size,
        mean_separate = mean_separate, 
        return_variances = return_variances, 
        se_subscripts = se_subscripts, logit_notation = logit_notation, ...) 
    } else {
      stop("The 'model' does not appear to be a proper **workflow** object ",
        "because it does not have a proper 'fit' component.")
    }
  } else {
    stop("The 'model' does not appear to be a proper **workflow** object ",
      "because it does not have a 'fit' component.")
  }
}

# PhG: we do not want to repeat usage, so no @describeIn extract_eq list `extract_eq()` method
#' Extracting a series of equations from a list of models
#' @keywords internal
#' @export
#' @noRd
extract_eq.list <- function(model, intercept = "alpha", greek = "beta",
  greek_colors = NULL, subscript_colors = NULL,
  var_colors = NULL, var_subscript_colors = NULL, 
  raw_tex = FALSE, 
  swap_var_names = NULL, swap_subscript_names = NULL,
  ital_vars = FALSE, label = NULL,
  index_factors = FALSE, show_distribution = FALSE,
  wrap = FALSE, terms_per_line = 4,
  operator_location = "end", align_env = "aligned",
  use_coefs = FALSE, coef_digits = 2,
  fix_signs = TRUE, font_size = NULL,
  mean_separate = NULL, return_variances = FALSE, 
  se_subscripts = FALSE, logit_notation = FALSE, ...) {
  
  res <- sapply(model, extract_eq, intercept = intercept, greek = greek,
    greek_colors = greek_colors, subscript_colors = subscript_colors,
    var_colors = var_colors, 
    var_subscript_colors = var_subscript_colors, 
    raw_tex = raw_tex, 
    swap_var_names = swap_var_names, 
    swap_subscript_names = swap_subscript_names,
    ital_vars = ital_vars, label = label,
    index_factors = index_factors, 
    show_distribution = show_distribution,
    wrap = wrap, terms_per_line = terms_per_line,
    operator_location = operator_location, 
    align_env = align_env,
    use_coefs = use_coefs, coef_digits = coef_digits,
    fix_signs = fix_signs, font_size = font_size,
    mean_separate = mean_separate, 
    return_variances = return_variances, 
    se_subscripts = se_subscripts, logit_notation = logit_notation, ...)
  res <- as.character(res) # Make sure it is a character vector
  names(res) <- names(model)
  class(res) <- c("equation", "character")
  res
}

library(lme4)

test_that("Checking for random/fixed effects works", {
  # PhG: On MacOS the following model fails to converge, thus issuing
  # warnings that are not part of the test itself... So, I suppress them.
  m <- suppressWarnings(
    lmer(score ~ wave + (group | sid), data = sim_longitudinal)
  )
  expect_error(extract_eq(m))
})

test_that("colorizing works", {
  # calculate district means
  dist_mean <- tapply(
    sim_longitudinal$score,
    sim_longitudinal$district,
    mean
  )
  
  # put them in a df to merge
  dist_mean <- data.frame(
    district = names(dist_mean),
    dist_mean = dist_mean
  )
  
  # create a new df with dist_mean added
  d <- merge(sim_longitudinal, dist_mean, by = "district")
  
  suppressWarnings(
    cl_long3 <- lme4::lmer(
      score ~ wave * group * treatment + wave * prop_low * treatment +
        (wave | sid) + (wave | school) +
        (wave + treatment | district),
      sim_longitudinal
    )
  )
  
  expect_snapshot_output(
    extract_eq(
      cl_long3,
      var_colors = c(
        wave = "blue", 
        treatment = "red",
        score = "#FF00CC"
      ),
      var_subscript_colors = c(
        group = "orange"
      )
    )
  )
  
  expect_warning(
    extract_eq(
      cl_long3,
      var_colors = c(
        wave = "blue", 
        treatment = "red",
        score = "#FF00CC"
      ),
      var_subscript_colors = c(
        group = "orange"
      ),
      greek_colors = rainbow(15)
    )
  )
  
  expect_warning(
    extract_eq(
      cl_long3,
      var_colors = c(
        wave = "blue", 
        treatment = "red",
        score = "#FF00CC"
      ),
      var_subscript_colors = c(
        group = "orange"
      ),
      subscript_colors = rev(rainbow(15))
    )
  )
})

test_that("Math extraction works", {
  expect_warning(
    m1 <- lmer(Reaction ~ log(Days + 1) + exp(Days) + poly(Days, 4) + 
                 (1 | Subject), 
               data = sleepstudy)
  )
  m2 <- lmer(score ~ log(wave + 1) + exp(prop_low) +
                     (log(wave + 1)|sid),
             data = sim_longitudinal)
  
  m3 <- lmer(score ~ wave + poly(prop_low, 4) +
                     (wave|sid) + (wave|school) + (wave|district),
             data = sim_longitudinal)
  
  expect_snapshot(extract_eq(m1))
  expect_snapshot(extract_eq(m2))
  expect_snapshot(extract_eq(m3))
})
test_that("Implicit ID variables are handled", {
  splt <- split(sim_longitudinal, sim_longitudinal$school)
  splt <- lapply(splt, function(x) {
    x$sid <- as.numeric(as.factor(x$sid))
    x
  })
  d <- do.call(rbind, splt)
  
  m <- lme4::lmer(
    score ~ wave + treatment +
      (wave | sid) + (wave | school),
    data = d
  )
  
  expect_snapshot_output(extract_eq(m))
  
})


test_that("Renaming Variables works", {
  m5 <- lme4::lmer(
    score ~ wave + group + treatment + prop_low +
      (wave | sid) + (wave + group + treatment | school) +
      (wave + treatment + prop_low | district),
    sim_longitudinal
  )
  
  expect_snapshot_output(
    extract_eq(
      m5,
      swap_var_names = c(
        "wave" = "Wave",
        "prop_low" = "P(low income)",
        "score" = "Student Scores"),
      swap_subscript_names = c(
        "1" = ""
      )
    )
  )
})

test_that("Really big models work", {
  big_mod <- lmer(rt ~ 0 + n1_intercept + n1_warning1 + n1_cuing1 + x1_intercept +  
         x1_warning1 + x1_cuing1 + n2_intercept + n2_warning1 + n2_cuing1 +  
         x2_intercept + x2_warning1 + x2_cuing1 + 
         (0 + n1_intercept + n1_warning1 + n1_cuing1 + x1_intercept + 
            x1_warning1 + x1_cuing1 + n2_intercept + n2_warning1 + n2_cuing1 + 
            x2_intercept + x2_warning1 + x2_cuing1 | id),
         data = test_data)
  expect_snapshot(extract_eq(big_mod))
})


DF <- structure(list(brochure = structure(c(2L, 1L, 2L, 1L, 1L, 1L, 1L, 2L, 1L, 1L), 
                                          .Label = c("pictorial", "standard"), 
                                          class = "factor"), 
                     disease = structure(c(1L, 2L, 2L, 2L, 1L, 2L, 2L, 1L, 1L, 2L), 
                                         .Label = c("BC", "DS"), 
                                         class = "factor"), 
                     ID = c(1L, 2L, 3L, 4L, 5L, 1L, 2L, 3L, 4L, 5L), 
                     error = c(5, 3.9, 70, 73.5, 0.2, 19.9, 60.6, 50.8, 0.1, 0.3), 
                     pers = c(4, 2.3, 4.7, 3.3, 3.7, 3, 5, 4.6, 4.4, 5)), 
                row.names = c(NA, -10L), class = c("tbl_df", "tbl", "data.frame"))

test_that("Categorical variable level parsing works (from issue #140)", {
  cat_interaction <- lmer(error ~ brochure * disease  + (1 | ID), data = DF)
  expect_snapshot_output(extract_eq(cat_interaction))
})


test_that("Unconditional lmer models work", {
  um_long1 <- lmer(score ~ 1 + (1 | sid), data = sim_longitudinal)
  expect_snapshot_output(extract_eq(um_long1))

  um_long2 <- lmer(score ~ 1 + (1 | sid) + (1 | school), data = sim_longitudinal)
  expect_snapshot_output(extract_eq(um_long2))

  um_long3 <- lmer(score ~ 1 + (1 | sid) + (1 | school) + (1 | district),
    data = sim_longitudinal
  )
  expect_snapshot_output(extract_eq(um_long3))
})

test_that("Level 1 predictors work", {
  # lev 1 models used for multiple tests
  lev1_hsb <- lmer(math ~ female + ses + minority + (1 | sch.id), hsb)
  lev1_long <- lmer(score ~ wave + (1 | sid) + (1 | school) + (1 | district),
    data = sim_longitudinal
  )

  # Level 1 predictors HSB
  expect_snapshot_output(extract_eq(lev1_hsb))

  # Level 1 longitudinal
  expect_snapshot_output(extract_eq(lev1_long))
})

test_that("Mean separate works as expected", {
  # lev 1 models used for multiple tests
  lev1_hsb <- lmer(math ~ female + ses + minority + (1 | sch.id), hsb)
  lev1_long <- lmer(score ~ wave + (1 | sid) + (1 | school) + (1 | district),
    data = sim_longitudinal
  )

  # Mean separate HSB
  expect_snapshot_output(extract_eq(lev1_hsb, mean_separate = FALSE))

  # Mean separate longitudinal
  expect_snapshot_output(extract_eq(lev1_long, mean_separate = TRUE))
})

test_that("Wrapping works as expected", {
  # lev 1 models used for multiple tests
  lev1_hsb <- lmer(math ~ female + ses + minority + (1 | sch.id), hsb)
  lev1_long <- lmer(score ~ wave + (1 | sid) + (1 | school) + (1 | district),
    data = sim_longitudinal
  )

  # Wrapping HSB
  expect_snapshot_output(extract_eq(lev1_hsb, wrap = TRUE, terms_per_line = 2))
})

test_that("Unstructured variance-covariances work as expected", {
  # two-level models
  hsb1 <- lmer(
    math ~ female + ses + minority + (minority | sch.id),
    hsb
  )

  # Unstructured VCV, HSB 1
  expect_snapshot_output(extract_eq(hsb1))

  # Unstructured VCV, HSB 2
  hsb2 <- lmer(
    math ~ female + ses + minority + (ses + female | sch.id),
    hsb
  )
  expect_snapshot_output(extract_eq(hsb2))

  # Unstructured VCV, HSB 3
  hsb3 <- lmer(
    math ~ female * ses + minority + (ses * female | sch.id),
    hsb
  )
  expect_snapshot_output(extract_eq(hsb3))

  # Unstructured VCV, HSB 4
  suppressWarnings(
    hsb4 <- lmer(
      math ~ female * ses + minority +
        (ses * female + minority | sch.id),
      hsb
    )
  )
  expect_snapshot_output(extract_eq(hsb4))

  # four-level model
  # Unstructured VCV, Longitudinal 1
  long1 <- lmer(
    score ~ wave +
      (wave | sid) + (wave | school) + (wave | district),
    sim_longitudinal
  )
  expect_snapshot_output(extract_eq(long1))
})

test_that("Group-level predictors work as expected", {
  # level 2 variables
  suppressWarnings(
    long2 <- lmer(
      score ~ wave + group + treatment +
        (wave | sid) + (wave + group + treatment | school) +
        (wave + treatment | district),
      sim_longitudinal
    )
  )

  # Group-level predictors, Longitudinal (level 2)
  expect_snapshot_output(extract_eq(long2))

  # level 3 variable
  long3 <- lmer(
    score ~ wave + group + treatment + prop_low +
      (wave | sid) + (wave + group + treatment | school) +
      (wave + treatment + prop_low | district),
    sim_longitudinal
  )

  # Group-level predictors, Longitudinal (level 3)
  expect_snapshot_output(extract_eq(long3))

  # level 4 variable
  dist_mean <- tapply(
    sim_longitudinal$score,
    sim_longitudinal$district,
    mean
  )
  dist_mean <- data.frame(
    district = names(dist_mean),
    dist_mean = dist_mean
  )
  sim_longitudinal <- merge(sim_longitudinal, dist_mean, by = "district")

  long4 <- lmer(
    score ~ wave + group + treatment + prop_low + dist_mean +
      (wave | sid) + (wave + treatment | school) +
      (wave | district),
    sim_longitudinal
  )

  # Group-level predictors, Longitudinal (level 4)
  expect_snapshot_output(extract_eq(long4))
})

data("sim_longitudinal", package = "equatiomatic")

test_that("Interactions work as expected", {
  # l1 interaction
  l1_hsb_int <- lmer(math ~ minority * female + (1 | sch.id),
    data = hsb
  )

  # Level 1 interaction
  expect_snapshot_output(extract_eq(l1_hsb_int))

  # l2 interaction
  l2_long_int <- lmer(score ~ treatment * group + (1 | sid) + (treatment | school) +
    (treatment * group | district),
  data = sim_longitudinal
  )
  expect_snapshot_output(extract_eq(l2_long_int))

  # cross-level interaction w/random at the level
  suppressWarnings(
    cl_long1 <- lmer(score ~ treatment * wave + (wave | sid) + (1 | school) +
      (1 | district),
    data = sim_longitudinal
    )
  )
  expect_snapshot_output(extract_eq(cl_long1))

  # cross-level interaction w/o random at the level
  cl_long2 <- lmer(score ~ treatment * wave + (1 | sid) + (1 | school) +
    (1 | district),
  data = sim_longitudinal
  )
  expect_snapshot_output(extract_eq(cl_long2))

  # Multiple cross-level interactions
  cl_long3 <- lmer(
    score ~ wave * group * treatment + prop_low * treatment * wave +
      (wave | sid) + (wave | school) +
      (wave + treatment | district),
    sim_longitudinal
  )
  expect_snapshot_output(extract_eq(cl_long3))
})

# Random effect structures with no covars estimated
test_that("Alternate random effect VCV structures work", {
  hsb_varsonly <- lmer(math ~ minority * female + (minority * female || sch.id),
    data = hsb
  )

  # Variance components only
  expect_snapshot_output(extract_eq(hsb_varsonly))

  suppressWarnings(
    hsb_doublegroup <- lmer(math ~ minority * female +
      (minority | sch.id) + (female | sch.id),
    data = hsb
    )
  )

  # Double grouping (like cross-classified models)
  expect_snapshot_output(extract_eq(hsb_varsonly))

  long_mixed_ranef <- lmer(
    score ~ wave +
      (wave || sid) + (wave | school) + (1 | school) + (wave || district),
    sim_longitudinal
  )

  # Mixed random effect structures at diff levels
  expect_snapshot_output(extract_eq(long_mixed_ranef))
})

test_that("Nested model syntax works", {
  suppressWarnings(
    nested_m1 <- lmer(score ~ 1 + (1 | sid / school), sim_longitudinal)
  )

  # Nested random effects 1
  expect_snapshot_output(extract_eq(nested_m1))

  suppressWarnings(
    nested_m2 <- lmer(score ~ 1 + (1 | sid / school / district), sim_longitudinal)
  )

  # Nested random effects 2
  expect_snapshot_output(extract_eq(nested_m1))

  suppressWarnings(
    nested_m3 <- lmer(score ~ wave + group + prop_low +
      (1 | sid / school / district), sim_longitudinal)
  )

  # Nested random effects 2
  expect_snapshot_output(extract_eq(nested_m1))
})

# PhG: skipping these two tests for now because they fail. In fact,
# result is correct but coefficient roundings are slightly differents, at least
# on aarch64 (silicon) processors, it seems. expect_snapshot_output() is looking
# for an **exact** equivalence.
# TODO: a solution would be to make two different versions of this test, but
# that requires extended testings on different platforms.
test_that("use_coef works", {
  skip()
  #skip_on_os("mac", "aarch64")
  
  suppressWarnings(
    use_coef_m1 <- lmer(
      score ~ wave  + treatment +
        (wave | sid) + (wave | school) +
        (1 | district),
      sim_longitudinal
    )
  )
  # Nested random effects 3
  expect_snapshot_output(extract_eq(use_coef_m1, use_coefs = TRUE, coef.digits = 1))
})

test_that("return variances works", {
  skip()
  #skip_on_os("mac", "aarch64")
  
  suppressWarnings(
    use_coef_m1_var <- lmer(
      score ~ wave  + treatment +
        (wave | sid) + (wave | school) +
        (1 | district),
      sim_longitudinal
    )
  )
  # Nested random effects 3
  expect_snapshot_output(
    extract_eq(use_coef_m1_var, use_coefs = TRUE, return_variances = TRUE)
  )
})

library(lme4)

test_that("Checking for random/fixed effects works", {
  m <- glmer(bush ~ 1 + edu + (black|state),
             data = polls,
             family = binomial(link = "logit"))
  expect_error(extract_eq(m))
})

d <- arrests
totes <- tapply(d$arrests, d$precinct, sum)
tot_arrests <- data.frame(precinct = as.numeric(names(totes)),
                          total_arrests = totes)

d <- merge(d, tot_arrests, by = "precinct")


test_that("colorizing works", {
  suppressWarnings(
    m <- lme4::glmer(stops ~ eth + total_arrests + (1|precinct), 
                     data = d,
                     family = poisson(link = "log"))
  )
  
  expect_snapshot_output(
    extract_eq(
      m,
      swap_var_names = c(
        stops = "Stops",
        "eth" = "Ethnicity",
        "total_arrests" = "Total Arrests"
      ),
      var_colors = c(
        eth = "red",
        stops = "blue"
      ),
      var_subscript_colors = c(
        eth = "purple"
      )
    )
  )
  
  suppressWarnings(
  expect_warning(
    extract_eq(
      m,
      swap_var_names = c(
        stops = "Stops",
        "eth" = "Ethnicity",
        "total_arrests" = "Total Arrests"
      ),
      var_colors = c(
        stops = "blue",
        eth = "red"
      ),
      var_subscript_colors = c(
        eth = "purple"
      ),
      greek_colors = rainbow(7)
    )
  )
  )
})

test_that("Renaming Variables works", {
  suppressWarnings(
    m6 <- lme4::glmer(stops ~ eth + total_arrests + (1|precinct), 
                      data = d,
                      family = poisson(link = "log"))
  )
  
  expect_snapshot_output(
    extract_eq(
      m6,
      swap_var_names = c(
        "eth" = "Ethnicity",
        "total_arrests" = "Total Arrests"
      ),
      swap_subscript_names = c(
        "black" = "Black",
        "hispanic" = "Hispanic/Latino",
        "white" = "White"
      )
    )
  )
})

test_that("Standard Poisson regression models work", {
  p1 <- glmer(stops ~ eth + (1|precinct), 
              data = arrests,
              family = poisson(link = "log"))
  expect_snapshot_output(extract_eq(p1))
  
  
  suppressWarnings(
    p_complicated <- glmer(stops ~ eth*total_arrests + (eth|precinct), 
                           data = d,
                           family = poisson(link = "log"))
  )
  expect_snapshot_output(extract_eq(p_complicated))
})


test_that("Poisson regression models with an offset work", {
  p_offset1 <- glmer(stops ~ eth + (1|precinct), 
              data = arrests,
              family = poisson(link = "log"),
              offset = log(arrests))
  expect_snapshot_output(extract_eq(p_offset1))
  
  
  suppressWarnings(
    p_offset_complicated <- glmer(stops ~ eth*total_arrests + (eth|precinct), 
                           data = d,
                           family = poisson(link = "log"),
                           offset = log(arrests))
  )
  expect_snapshot_output(extract_eq(p_offset_complicated))
})



test_that("Binomial Logistic Regression models work", {
  m <- glmer(bush ~ 1 + black + female + edu + (black|state),
             data = polls,
             family = binomial(link = "logit"))
  expect_snapshot_output(extract_eq(m))
})

test_that("colorizing works", {
  # Required for R >= 4.5.0, which provides another version of the penguins
  # dataset with different column names!
  data("penguins", package = "equatiomatic")
  
  lr <- glm(sex ~ species * bill_length_mm,
            data = penguins,
            family = binomial(link = "logit")
  )
  
  pr <- glm(sex ~ species * bill_length_mm,
            data = penguins,
            family = binomial(link = "probit")
  )
  
  coef_colors <- c("#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
                   "#E6AB02")
  
  expect_snapshot_output(
    extract_eq(
      lr,
      swap_var_names = c(sex = "SEX"),
      var_colors = c(
        sex = "#0073ff",
        species = "green"
      ),
      var_subscript_colors = c(species = "orange"),
      greek_colors = coef_colors,
      subscript_colors = "blue",
      wrap = TRUE
    )
  )
  
  expect_snapshot_output(
    extract_eq(
      lr, 
      swap_var_names = c(sex = "SEX"),
      var_colors = c(
        sex = "#0073ff",
        species = "green"
      ),
      var_subscript_colors = c(species = "orange"),
      greek_colors = coef_colors,
      subscript_colors = "blue",
      wrap = TRUE,
      show_distribution = TRUE
    )
  )
  
  expect_snapshot_output(
    extract_eq(
      pr, 
      swap_var_names = c(sex = "SEX"),
      var_colors = c(
        sex = "#0073ff",
        species = "green"
      ),
      var_subscript_colors = c(species = "orange"),
      greek_colors = coef_colors,
      subscript_colors = "blue",
      wrap = TRUE
    )
  )
  
  expect_snapshot_output(
    extract_eq(
      pr, 
      swap_var_names = c(sex = "SEX"),
      var_colors = c(
        sex = "#0073ff",
        species = "green"
      ),
      var_subscript_colors = c(species = "orange"),
      greek_colors = coef_colors,
      subscript_colors = "blue",
      wrap = TRUE,
      show_distribution = TRUE
    )
  )
})

test_that("logit_notation works for logistic glm", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_logit <- glm(outcome ~ .,
    data = df,
    family = binomial(link = "logit")
  )

  # basic compact logit notation
  expect_snapshot(extract_eq(model_logit, logit_notation = TRUE))

  # wrapping works with logit notation
  expect_snapshot(extract_eq(model_logit, logit_notation = TRUE, wrap = TRUE))

  # use_coefs changes LHS to use \hat{outcome}
  expect_snapshot(extract_eq(model_logit, logit_notation = TRUE, use_coefs = TRUE))

  # logit_notation emits a message and is ignored when show_distribution = TRUE
  expect_message(
    extract_eq(model_logit, logit_notation = TRUE, show_distribution = TRUE),
    "logit_notation = TRUE ignored when show_distribution is TRUE"
  )
})
test_that("Renaming Variables works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 100, replace = TRUE),
    categorical = rep(letters[1:5], 20),
    continuous_1 = rnorm(100, 100, 1),
    continuous_2 = rnorm(100, 50, 5)
  )
  
  m2 <- glm(outcome ~ categorical*continuous_1*continuous_2,
            data = df,
            family = binomial(link = "logit")
  )
  
  expect_snapshot_output(
    extract_eq(
      m2,
      swap_var_names = c(
        "categorical" = "cat",
        "continuous_1" = "Continuous Variable [1]"),
      swap_subscript_names = c(
        "a" = "aaaaaaaaa",
        "d" = "dddd"
      ),
      wrap = TRUE,
      terms_per_line = 2
    )
  )
})

test_that("Math extraction works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 100, replace = TRUE),
    continuous_1 = rnorm(100, 100, 1),
    continuous_2 = rnorm(100, 50, 5)
  )
  
  model_logit <- glm(outcome ~ log(continuous_1) + exp(continuous_2) +
                       poly(continuous_2, 3),
                     data = df,
                     family = binomial(link = "logit")
  )
  model_probit <- glm(outcome ~ log(continuous_1) + exp(continuous_2) +
                        poly(continuous_2, 3),
                      data = df,
                      family = binomial(link = "probit")
  )
  expect_snapshot_output(extract_eq(model_logit))
  expect_snapshot_output(extract_eq(model_probit))
})

test_that("Collapsing glm factors works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 100, replace = TRUE),
    categorical = rep(letters[1:5], 20),
    continuous_1 = rnorm(100, 100, 1),
    continuous_2 = rnorm(100, 50, 5)
  )
  
  model_logit <- glm(outcome ~ categorical*continuous_1*continuous_2,
                     data = df,
                     family = binomial(link = "logit")
  )
  model_probit <- glm(outcome ~ categorical*continuous_1*continuous_2,
                     data = df,
                     family = binomial(link = "probit")
  )
  
  # no collapsing
  expect_snapshot(extract_eq(model_logit))
  expect_snapshot(extract_eq(model_probit))
  
  # collapsing
  expect_snapshot(extract_eq(model_logit, index_factors = TRUE))
  expect_snapshot(extract_eq(model_probit, index_factors = TRUE))
})


test_that("Logistic regression works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_logit <- glm(outcome ~ .,
    data = df,
    family = binomial(link = "logit")
  )

  # basic equation builds correctly
  expect_snapshot_output(extract_eq(model_logit))
})

test_that("Probit regression works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_probit <- glm(outcome ~ .,
    data = df,
    family = binomial(link = "probit")
  )

  # basic equation builds correctly
  expect_snapshot_output(extract_eq(model_probit))

  # Everything works when there are no categorical variables
  # equation sans categorical variables builds correctly
  model_gaussian <- glm(mpg ~ cyl + disp, data = mtcars, family = gaussian())
  expect_snapshot_output(extract_eq(model_gaussian))
})

test_that("Unsupported GLMs create a message", {
  model_gaussian <- glm(mpg ~ cyl + disp, data = mtcars, family = gaussian())
  model_gaussian$family$link <- "nothing"

  expect_message(extract_eq(model_gaussian))
})

test_that("Distribution-based equations work", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_logit <- glm(outcome ~ .,
    data = df,
    family = binomial(link = "logit")
  )

  model_probit <- glm(outcome ~ .,
    data = df,
    family = binomial(link = "probit")
  )

  expect_snapshot_output(extract_eq(model_logit, show_distribution = TRUE))
  expect_snapshot_output(extract_eq(model_probit, show_distribution = TRUE))

  # Everything works when there are no categorical variables
  # equation sans categorical variables builds correctly
  model_gaussian <- glm(mpg ~ cyl + disp, data = mtcars, family = gaussian())
  expect_snapshot_output(extract_eq(model_gaussian, show_distribution = TRUE))
})

test_that("Weights work", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:1, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5),
    weight = rep(c(1, 2), 150)
  )

  model_logit <- glm(outcome ~ .,
    data = df, weights = weight,
    family = binomial(link = "logit")
  )

  expect_warning(extract_eq(model_logit, show_distribution = TRUE))
})

test_that("non-binomial regression works", {
  set.seed(1234)
  df <- data.frame(
    outcome = sample(0:15, 300, replace = TRUE),
    categorical = rep(letters[1:3], 100),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_log <- glm(outcome ~ .,
    data = df,
    family = poisson(link = "log")
  )

  # basic log equation builds correctly
  expect_snapshot_output(extract_eq(model_log))

  expect_message(extract_eq(model_log, show_distribution = TRUE))

  model_id <- glm(outcome ~ .,
    data = df,
    family = gaussian(link = "identity")
  )
  expect_snapshot_output(extract_eq(model_log))

  # basic equation with offset builds correctly
  model_offset <- glm(outcome ~ . + offset(rep(1, 300)),
    data = df,
    family = poisson(link = "log")
  )
  expect_snapshot_output(extract_eq(model_offset))
})

test_that("colorizing works", {
  ordinal_ologit <- ordinal::clm(rating ~ temp * contact,
                                 data = ordinal::wine,
                                 link = "logit"
  )
  
  greek_col <- c("#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
                 "#E6AB02", "#A6761D", "#666666")
  
  expect_snapshot_output(
    extract_eq(
      ordinal_ologit,
      wrap = TRUE, 
      terms_per_line = 2,
      var_colors = c(temp = "blue"),
      greek_colors = greek_col,
      subscript_colors = rev(greek_col)
    )
  )
})

test_that("Renaming Variables works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    categorical = rep(letters[1:5], 20),
    continuous = rnorm(300, 100, 1)
  )
  expect_warning(
    m3 <- ordinal::clm(outcome ~ categorical*continuous,
                       data = df, 
                       link = "logit"
    )
  )
  expect_snapshot_output(
    extract_eq(
      m3,
      swap_var_names = c("categorical" = "cat"),
      swap_subscript_names = c(
        "a" = "Hey look! A new subscript",
        "d" = "Don't look at me though"
      ),
      wrap = TRUE,
      terms_per_line = 2
    )
  )
})

test_that("Math extraction works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    continuous = rnorm(300, 100, 1)
  )
  
  expect_warning(
    model_logit <- ordinal::clm(
      outcome ~ poly(continuous, 3) + exp(continuous) + log(continuous),
      data = df, 
      link = "logit"
    )
  )
  expect_warning(
    model_probit <- ordinal::clm(
      outcome ~ poly(continuous, 3) + exp(continuous) + log(continuous),
      data = df,
      link = "probit"
    )
  )
  
  expect_snapshot_output(extract_eq(model_logit))
  expect_snapshot_output(extract_eq(model_probit))
})

test_that("Collapsing clm factors works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    categorical = rep(letters[1:5], 20),
    continuous = rnorm(300, 100, 1)
  )
  expect_warning(
    model_logit <- ordinal::clm(outcome ~ categorical*continuous,
                                data = df, link = "logit"
    )
  )
  expect_warning(
    model_probit <- ordinal::clm(outcome ~ categorical*continuous,
                                 data = df, link = "probit"
    )
  )
  # no collapsing
  expect_snapshot(extract_eq(model_logit))
  expect_snapshot(extract_eq(model_probit))
  
  # collapsing
  expect_snapshot(extract_eq(model_logit, index_factors = TRUE))
  expect_snapshot(extract_eq(model_probit, index_factors = TRUE))
})

test_that("Ordered models with clm work", {
  set.seed(1234)
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
      levels = LETTERS[1:3],
      ordered = TRUE
    ),
    continuous_1 = rnorm(300, 1, 1),
    continuous_2 = rnorm(300, 5, 5)
  )

  model_logit <- ordinal::clm(outcome ~ continuous_1 + continuous_2,
    data = df, link = "logit"
  )
  model_probit <- ordinal::clm(outcome ~ continuous_1 + continuous_2,
    data = df, link = "probit"
  )

  expect_snapshot_output(extract_eq(model_logit, wrap = FALSE))
  expect_snapshot_output(extract_eq(model_logit, wrap = TRUE, terms_per_line = 2))
  expect_snapshot_output(extract_eq(model_probit, wrap = FALSE))
  expect_snapshot_output(extract_eq(model_probit, wrap = TRUE, terms_per_line = 2))

  # Coefficients instead of letters
  expect_snapshot_output(extract_eq(model_logit, use_coefs = TRUE))
})

test_that("Unsupported CLMs create a message", {
  set.seed(1234)
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
      levels = LETTERS[1:3],
      ordered = TRUE
    ),
    continuous_1 = rnorm(300, 1, 1),
    continuous_2 = rnorm(300, 5, 5)
  )

  model <- ordinal::clm(outcome ~ continuous_1 + continuous_2,
    data = df, link = "cauchit"
  )

  expect_message(extract_eq(model))
})

test_that("Coefficient digits work correctly", {
  model_simple <- lm(mpg ~ cyl + disp, data = mtcars)

  # coefficient rounding works
  expect_snapshot_output(
    extract_eq(model_simple, use_coefs = TRUE, coef_digits = 4)
  )

  # signs are doubled when fix_signs = FALSE
  expect_snapshot_output(
    extract_eq(model_simple, use_coefs = TRUE, fix_signs = FALSE)
  )
})

test_that("Wrapping works correctly", {
  model_big <- lm(mpg ~ ., data = mtcars)

  # wrapping with 4 terms works
  expect_snapshot_output(
    extract_eq(model_big, wrap = TRUE, terms_per_line = 4)
  )

  # wrapping with 2 terms works
  expect_snapshot_output(
    extract_eq(model_big, wrap = TRUE, terms_per_line = 2)
  )

  tex_end <- capture.output(extract_eq(model_big,
    wrap = TRUE,
    operator_location = "end"
  ))
  expect_match(tex_end[3], "\\+ \\\\\\\\$",
    label = "wrapped equation line ends with +"
  )
  expect_match(tex_end[4], "\\+ \\\\\\\\$",
    label = "other wrapped equation line ends with +"
  )


  tex_start <- capture.output(extract_eq(model_big,
    wrap = TRUE,
    operator_location = "start"
  ))
  expect_match(tex_start[3], "\\)\\\\\\\\$",
    label = "wrapped equation line doesn't end with +"
  )
  expect_match(tex_start[4], "&\\\\quad \\+ ",
    label = "wrapped equation line starts with +"
  )
  expect_match(tex_start[5], "&\\\\quad \\+ ",
    label = "other wrapped equation line starts with +"
  )


  tex_align <- capture.output(extract_eq(model_big,
    wrap = TRUE,
    align_env = "align"
  ))
  expect_equal(tex_align[2], "\\begin{align}",
    label = "different align environment used"
  )
})

library(forecast)

# Test Group A: Simple Arima Model
test_that("Basic ARIMA model functions", {

  # Build Arima (no regression)
  model <- forecast::Arima(simple_ts,
    order = c(1, 1, 1),
    seasonal = c(1, 0, 1),
    include.constant = TRUE
  )

  # Test 1: Works with greek letters
  # basic Arima + greek builds correctly
  expect_snapshot_output(extract_eq(model))

  # Test 2: Works with coefficients
  expect_snapshot_output(extract_eq(model, use_coefs = TRUE))
})

# Test Group B: Regression w/ ARIMA Errors
test_that("Regression w/ ARIMA Errors functions", {
  # Build Exogenous Regressors
  xregs <- as.matrix(data.frame(
    x1 = ts_reg_list$x1 + 5,
    x2 = ts_reg_list$x2 * 5
  ))

  regress_ts <- ts(ts_reg_list$ts_rnorm, freq = 4)

  # Build Regression Model
  model <- forecast::Arima(regress_ts,
    order = c(1, 1, 1),
    seasonal = c(1, 0, 1),
    xreg = xregs,
    include.constant = TRUE
  )

  # Test 1: Works with greek letters
  # Regession w/ Arima Errors + greek builds correctly
  expect_snapshot_output(extract_eq(model))

  # Test 2: Works with coefficients
  # Regession w/ Arima Errors + coefficients builds correctly
  expect_snapshot_output(extract_eq(model, use_coefs = TRUE))
})

test_that("colorizing works", {
  mass_ologit <- MASS::polr(rating ~ temp * contact,
                            data = ordinal::wine
  )
  
  greek_col <- c("#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
                 "#E6AB02", "#A6761D", "#666666")
  
  expect_snapshot_output(
    extract_eq(
      mass_ologit,
      wrap = TRUE, 
      terms_per_line = 2,
      var_colors = c(temp = "blue"),
      var_subscript_colors = c(contact = "orange"),
      greek_colors = greek_col,
      subscript_colors = rev(greek_col)
    )
  )
})

test_that("Renaming Variables works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    categorical = rep(letters[1:5], 20),
    continuous = rnorm(300, 100, 1)
  )
  
  m4 <- MASS::polr(
    outcome ~ categorical*continuous,
    Hess = TRUE,
    data = df,
    method = "probit"
  )
  expect_snapshot_output(
    extract_eq(
      m4,
      swap_var_names = c(
        "categorical" = "catty",
        "continuous" = "Cont Var"),
      swap_subscript_names = c(
        "a" = "alabama arkansas",
        "d" = "do do do"
      ),
      wrap = TRUE,
      terms_per_line = 2
    )
  )
})

test_that("Math extraction works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    continuous = rnorm(300, 100, 1)
  )
  model_logit <- MASS::polr(
    outcome ~ poly(continuous, 3) + log(continuous),
    Hess = TRUE,
    data = df, 
    method = "logistic"
  )
  model_probit <- MASS::polr(
    outcome ~ poly(continuous, 3) + log(continuous),
    Hess = TRUE,
    data = df,
    method = "probit"
  )
  
  expect_snapshot_output(extract_eq(model_logit))
  expect_snapshot_output(extract_eq(model_probit))
})

test_that("Collapsing polr factors works", {
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
                     levels = LETTERS[1:3],
                     ordered = TRUE
    ),
    categorical = rep(letters[1:5], 20),
    continuous = rnorm(300, 100, 1)
  )
  model_logit <- MASS::polr(
    outcome ~ categorical*continuous,
    Hess = TRUE,
    data = df, 
    method = "logistic"
  )
  model_probit <- MASS::polr(
    outcome ~ categorical*continuous,
    Hess = TRUE,
    data = df,
    method = "probit"
  )
  
  # no collapsing
  expect_snapshot(extract_eq(model_logit))
  expect_snapshot(extract_eq(model_probit))
  
  # collapsing
  expect_snapshot(extract_eq(model_logit, index_factors = TRUE))
  expect_snapshot(extract_eq(model_probit, index_factors = TRUE))
})

test_that("Ordered logistic regression works", {
  set.seed(1234)
  df <- data.frame(
    outcome = factor(rep(LETTERS[1:3], 100),
      levels = LETTERS[1:3],
      ordered = TRUE
    ),
    continuous_1 = rnorm(300, 100, 1),
    continuous_2 = rnorm(300, 50, 5)
  )

  model_polr <- MASS::polr(outcome ~ ., data = df, Hess = TRUE, method = "logistic")
  model_polr_probit <- MASS::polr(outcome ~ ., data = df, Hess = TRUE, method = "probit")

  # ordered logit builds correctly
  expect_snapshot_output(extract_eq(model_polr, wrap = FALSE))

  # ordered logit builds correctly in wrapped environment
  expect_snapshot_output(
    extract_eq(model_polr, wrap = TRUE, terms_per_line = 2)
  )

  # ordered probit builds correctly
  expect_snapshot_output(
    extract_eq(model_polr_probit, wrap = FALSE)
  )

  # ordered probit builds correctly in wrapped environment
  expect_snapshot_output(
    extract_eq(model_polr_probit, wrap = TRUE, terms_per_line = 2)
  )

  # ordered logit + coefs builds correctly
  expect_snapshot_output(extract_eq(model_polr, use_coefs = TRUE))
})

test_that("Equation is printed correctly", {
  model_simple <- lm(mpg ~ cyl + disp, data = mtcars)

  tex <- extract_eq(model_simple)
  actual <- paste(
    "\\operatorname{mpg} = \\alpha + \\beta_{1}(\\operatorname{cyl})",
    "+ \\beta_{2}(\\operatorname{disp}) + \\epsilon"
  )

  printed_tex <- capture.output(tex)

  expect_equal(printed_tex[1], "$$",
    label = "first line is $$"
  )
  expect_equal(printed_tex[2], actual,
    label = "second line is the formula"
  )
})

test_that("Equation is knit_print-ed correctly", {
  model_simple <- lm(mpg ~ cyl + disp, data = mtcars)

  knit_print_tex <- knitr::knit_print(extract_eq(model_simple))
  actual <- paste(
    "$$\n\\operatorname{mpg} = \\alpha + \\beta_{1}(\\operatorname{cyl}) +",
    "\\beta_{2}(\\operatorname{disp}) + \\epsilon\n$$\n"
  )

  expect_equal(as.character(knit_print_tex),
    actual,
    label = "second line is the formula"
  )
  expect_s3_class(
    knit_print_tex,
    "knit_asis"
  )
})

test_that("se_subscripts argument works with numeric coefficients", {
  
 # test lm with negative coefficient
  mod <- lm(disp ~ cyl + mpg, data = mtcars)
  
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               se_subscripts = TRUE)
  )
    
  # test glm with negative coefficients (linear model)
  mod <- glm(disp ~ cyl + mpg, data = mtcars)
    
    expect_snapshot_output(
      extract_eq(mod,
                 se_subscripts = TRUE)
  )
    
    # test glm with negative coefficients (logistic model)
    mod <- glm(am ~ mpg + wt, family = binomial, data = mtcars)
    
    expect_snapshot_output(
      extract_eq(mod,
                 se_subscripts = TRUE)
    )
})


test_that("se_subscripts argument works with wrapping", {
  
  # test lm  
  mod <- lm(disp ~ cyl + mpg + hp, data = mtcars)
  
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
  # test glm with linear model
  mod <- glm(disp ~ cyl + mpg, data = mtcars)
  
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
  # test glm with linear model
  mod <- glm(disp ~ cyl + mpg, data = mtcars)
  
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
  # test glm with logistic model
  mod <- glm(am ~ mpg + wt,
             family = binomial, 
             data = mtcars)
  
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
})


test_that("se_subscripts argument works with Greek letter terms", {
         
         # test lm  
         mod <- lm(disp ~ cyl + mpg + hp, data = mtcars)
         
         expect_snapshot_output(
           extract_eq(mod,
                      se_subscripts = TRUE)
         )
         
         
         # test lm with wrapping
         mod <- lm(disp ~ cyl + mpg + hp, data = mtcars)
         
         expect_snapshot_output(
           extract_eq(mod,
                      wrap = TRUE,
                      terms_per_line = 1,
                      se_subscripts = TRUE)
         )
         
         # test glm with linear model
         mod <- glm(disp ~ cyl + mpg, data = mtcars)
         
         expect_snapshot_output(
           extract_eq(mod,
                      wrap = TRUE,
                      terms_per_line = 1,
                      se_subscripts = TRUE)
         )
         
         # test glm with logistic model
         mod <- glm(am ~ mpg + wt,
                    family = binomial, 
                    data = mtcars)
         
         expect_snapshot_output(
           extract_eq(mod,
                      wrap = TRUE,
                      terms_per_line = 1,
                      se_subscripts = TRUE)
         )
         
         
})


test_that("se_subscripts argument works with transformed variables", {
  
  # test lm with log transformed variable 
  mod <- lm(disp ~ cyl + log(mpg) + hp, data = mtcars)
  
  # test without wrapping 
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               se_subscripts = TRUE)
  )
  
  # test with wrapping 
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
  # test glm with log transformed variable 
  mod <- glm(disp ~ cyl + log(mpg) + hp, data = mtcars)
  
  # test without wrapping 
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               se_subscripts = TRUE)
  )
  
  # test with wrapping 
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               wrap = TRUE,
               terms_per_line = 1,
               se_subscripts = TRUE)
  )
  
})


test_that("se_subscripts works with factor variable", {
  
  # test handling of model with factor variable
  mod <- glm(disp ~ cyl + log(mpg) + hp + factor(gear), data = mtcars)

  # with coefficients
  expect_snapshot_output(
    extract_eq(mod,
               use_coefs = TRUE,
               se_subscripts = TRUE)
  )
  
  # with Greek letters
  expect_snapshot_output(
    extract_eq(mod,
               se_subscripts = TRUE)
  )

  
})

test_that("font-size changes, lm", {
  mod1 <- lm(mpg ~ cyl + disp, data = mtcars)
  expect_snapshot_output(extract_eq(mod1, font_size = "large"))
  expect_snapshot_output(extract_eq(mod1, font_size = "Huge"))
  expect_snapshot_output(extract_eq(mod1, font_size = "tiny"))
})

test_that("font-size changes, lmer", {
  um_long1 <- lme4::lmer(score ~ 1 + (1 | sid), data = sim_longitudinal)
  expect_snapshot_output(extract_eq(um_long1, font_size = "scriptsize"))
  expect_snapshot_output(extract_eq(um_long1, font_size = "Large"))
  expect_snapshot_output(extract_eq(um_long1, font_size = "huge"))
})

test_that("font-size changes, arima", {
  model <- forecast::Arima(simple_ts,
                           order = c(1, 1, 1),
                           seasonal = c(1, 0, 1),
                           include.constant = TRUE
  )
  
  expect_snapshot_output(extract_eq(model, font_size = "footnotesize"))
  expect_snapshot_output(extract_eq(model, font_size = "small"))
  expect_snapshot_output(extract_eq(model, font_size = "LARGE"))
  
})

test_that("Overlapping names don't result in an error", {
  data("penguins", package = "equatiomatic")
  penguins2 <- penguins
  names(penguins2)[3:5] <- c("ca", "p", "gender_cat")
  
  #Model with the iris dataset variables renamed to show the problem
  m2 <- lm(body_mass_g ~ p*gender_cat + ca, penguins2)
  
  expect_snapshot_output(
    extract_eq(m2, wrap = TRUE, terms_per_line = 1,
               use_coefs = TRUE)
  )
})

test_that("Dropping intercept notation works", {
  m <- lm(bill_depth_mm ~ 0 + flipper_length_mm*island, penguins)
  expect_snapshot_output(extract_eq(m))
})

test_that("colorizing works", {
  m <- lm(bill_depth_mm ~ flipper_length_mm*island, penguins)
  
  coef_colors <- c("#1B9E77", "#D95F02", "#7570B3", "#E7298A", "#66A61E", 
                   "#E6AB02", "#A6761D")
  ss_colors <- rev(coef_colors)
  
  expect_snapshot_output(
    extract_eq(
      m,
      swap_var_names = c("flipper_length_mm" = "Flipper Length (MM)",
                         "island" = "ISLAND"),
      swap_subscript_names = c("Dream" = "super dreamy"),
      greek_colors = coef_colors, 
      subscript_colors = ss_colors,
      var_colors = c(
        "bill_depth_mm" = "#0f70f7",
        "flipper_length_mm" = "#b22222",
        "island" = "green"
      ),
      var_subscript_colors = c("island" = "cyan"),
      wrap = 2,
      terms_per_line = 2
    )
  )
})
test_that("Renaming Variables works", {
  m1 <- lm(body_mass_g ~ bill_length_mm * species + 
             flipper_length_mm * sex, 
           data = penguins)
  expect_snapshot_output(
    extract_eq(
      m1, 
      swap_var_names = c(
        "bill_length_mm" = "Bill Length (MM)",
        "flipper_length_mm" = "Flipper Length (MM)",
        "sex" = "SEX"
      ),
      swap_subscript_names = c(
        "male" = "Male",
        "Chinstrap" = "chinny chin chin"
      )
    )
  )
})

test_that("Math extraction works", {
  m_lm <- lm(bill_length_mm ~ poly(bill_depth_mm, 5) + 
               log(flipper_length_mm) +
               exp(bill_length_mm), 
             data = na.omit(penguins))
  
  expect_snapshot_output(extract_eq(m_lm))

  m1 <- lm(mpg ~ I(hp > 150), data = mtcars)
  m2 <- lm(mpg ~ I(hp < 250), data = mtcars)
  expect_snapshot_output(extract_eq(m1))
  expect_snapshot_output(extract_eq(m2))
})
test_that("Collapsing lm factors works", {
  d <- mtcars
  d$gear <- as.factor(d$gear)
  d$carb <- as.factor(d$carb)
  
  m <- lm(mpg ~ gear*carb*disp, data = d)
  
  # no collapsing
  expect_snapshot(extract_eq(m))
  
  # collapsing
  expect_snapshot(extract_eq(m, index_factors = TRUE))
})

test_that("Labeling works", {
  m <- lm(mpg ~ cyl + disp, data = mtcars)
  expect_snapshot_output(extract_eq(m, label = "mpg_mod"))
})

test_that("Simple lm models work", {
  model_simple <- lm(mpg ~ cyl + disp, data = mtcars)

  mtcars$gear <- as.factor(mtcars$gear)
  model_indicators <- lm(mpg ~ cyl + gear, data = mtcars)

  # basic equation builds correctly
  expect_snapshot_output(extract_eq(model_simple))

  # categorical subscripts work
  expect_snapshot_output(extract_eq(model_indicators))

  # basic equation + coefs builds correctly
  expect_snapshot_output(extract_eq(model_simple, use_coefs = TRUE))
})

test_that("Interactions work", {
  simple_int <- lm(body_mass_g ~ bill_length_mm * species, penguins)
  simple_int2 <- lm(mpg ~ hp * wt, mtcars)

  # Basic interaction with subscripts
  expect_snapshot_output(extract_eq(simple_int))

  # Basic interaction with no subscripts
  expect_snapshot_output(extract_eq(simple_int2))
})

test_that("Custom Greek works", {
  model_simple <- lm(mpg ~ cyl + disp, data = mtcars)

  # custom Greek coefficients work
  expect_snapshot_output(
    extract_eq(model_simple, greek = "\\hat{\\beta}", raw_tex = TRUE)
  )

  # custom Greek intercept works
  expect_snapshot_output(
    extract_eq(model_simple,
      intercept = "\\zeta",
      greek = "\\beta",
      raw_tex = TRUE
    )
  )
})

test_that("Hat is escaped correctly", {
  mtcars$carb <- ordered(mtcars$carb)
  m <- lm(mpg ~ carb, mtcars)
  expect_snapshot_output(extract_eq(m))
})

#' Arrest data from Gelman & Hill
#'
#' Arrest data from Gelman & Hill's book, used in Chapter 6 (and others).
#' The data have been aggregated by precinct and race/ethnicity, with the 
#' sum of prior arrests and stops calculated. You can download the original
#' data here: http://www.stat.columbia.edu/~gelman/arm/examples/police/
#'
#' @format A tibble with 225 rows and 4 variables:
#' \describe{
#'   \item{precinct}{An integer denoting the precinct identification number.}
#'   \item{eth}{A factor with the coded race/ethnicity}
#'   \item{stops}{The number of police stops}
#'   \item{arrests}{The number of prior arrests (this is used as an offset variable in the book)}
#' }
"arrests"

#' Generic function for extracting the left hand side from a model
#'
#' @keywords internal
#'
#' @param model A fitted model
#' @param \dots additional arguments passed to the specific extractor
#' @noRd

extract_lhs <- function(model, ...) {
  UseMethod("extract_lhs", model)
}

#' Extract left-hand side of an lm object object
#'
#' Extract a string of the outcome/dependent/y variable of a model
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd

extract_lhs.lm <- function(model, ital_vars, show_distribution, use_coefs,
                           swap_var_names, var_colors, logit_notation = FALSE,
                           ...) {

  check_dots(...)
  lhs <- rownames(attr(model$terms, "factors"))[1]
  lhs_nm <- lhs
  names(lhs) <- lhs_nm

  if (!is.null(swap_var_names)) {
    lhs <- swap_names(swap_var_names, lhs)[[1]]
  }

  lhs_escaped <- escape_tex(lhs)
  names(lhs_escaped) <- lhs_nm

  if (use_coefs) {
    lhs_escaped <- add_hat(lhs_escaped)
  }
  lhs_escaped <- add_tex_ital_v(lhs_escaped, ital_vars)
  colorize_terms(var_colors, list(lhs), list(lhs_escaped))
}

#' Extract left-hand side of an summary.lm object
#'
#' Extract a string of the outcome/dependent/y variable of a model
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd

extract_lhs.summary.lm <- extract_lhs.lm

#' Extract left-hand side of an lme4::lmer object
#'
#' Extract a string of the outcome/dependent/y variable of a model
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd
extract_lhs.lmerMod <- function(model, ital_vars, use_coefs, swap_var_names,
                                var_colors, logit_notation = FALSE,...) {
  check_dots(...)
  lhs <- all.vars(formula(model))[1]
  lhs_nm <- lhs
  names(lhs) <- lhs_nm

  if (!is.null(swap_var_names)) {
    lhs <- swap_names(swap_var_names, lhs)[[1]]
  }

  lhs_escaped <- escape_tex(lhs)
  if (use_coefs) {
    lhs_escaped <- add_hat(lhs_escaped)
  }
  lhs_escaped <- add_tex_ital_v(lhs_escaped, ital_vars)

  names(lhs_escaped) <-lhs_nm
  lhs_escaped <- colorize_terms(var_colors, list(lhs), list(lhs_escaped))
  paste0(lhs_escaped, "_{i}")
}

#' Extract left-hand side of an lme4::glmer object
#'
#' Extract a string of the outcome/dependent/y variable of a model
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd
extract_lhs.glmerMod <- function(model, ital_vars, use_coefs, ...) {
  extract_lhs.lmerMod(model, ital_vars, use_coefs, logit_notation = FALSE, ...)
}

#' Extract left-hand side of a glm object
#'
#' Extract a string of the outcome/dependent/y variable with the appropriate
#' link function.
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd

extract_lhs.glm <- function(model, ital_vars, show_distribution, use_coefs,
swap_var_names, var_colors, logit_notation = FALSE, ...) {
  
  if (show_distribution) {
    if (model$family$family == "binomial") {
      if (!isFALSE(logit_notation))
        message("logit_notation = TRUE ignored when show_distribution is TRUE.")
      return(extract_lhs2_binomial(model, ital_vars, use_coefs,
        swap_var_names, var_colors, logit_notation = logit_notation))
    } else {
      message("This distribution is not presently supported; the distribution assumption
      will not be displayed")
      lhs <- all.vars(formula(model))[1]
      lhs_nm <- lhs
      names(lhs) <- lhs_nm

      if (!is.null(swap_var_names)) {
        lhs <- swap_names(swap_var_names, lhs)[[1]]
      }

      lhs_escaped <- escape_tex(lhs)

      if (!is.null(var_colors)) {
        names(lhs) <- lhs_nm
        names(lhs_escaped) <- lhs_nm
        lhs_escaped <- colorize_terms(var_colors, list(lhs), list(lhs_escaped))
      }

      full_lhs <- paste("E(", add_tex_ital_v(lhs_escaped, ital_vars), ")")
      if (use_coefs) {
        full_lhs <- add_hat(full_lhs)
      }
      full_lhs <- modify_lhs_for_link(model, full_lhs)
      class(full_lhs) <- c("character", class(model))
      return(full_lhs)
    }
  }
  if (model$family$family == "binomial") {
    return(extract_lhs_binomial(model, ital_vars, use_coefs,
      swap_var_names, var_colors, logit_notation = logit_notation))
  } else {
    lhs <- all.vars(formula(model))[1]
    lhs_nm <- lhs
    names(lhs) <- lhs_nm

    if (!is.null(swap_var_names)) {
      lhs <- swap_names(swap_var_names, lhs)[[1]]
    }

    lhs_escaped <- escape_tex(lhs)

    if (!is.null(var_colors)) {
      names(lhs) <- lhs_nm
      names(lhs_escaped) <- lhs_nm
      lhs_escaped <- colorize_terms(var_colors, list(lhs), list(lhs_escaped))
    }

    full_lhs <- paste("E(", add_tex_ital(lhs, ital_vars), ")")
    if (use_coefs) {
      full_lhs <- add_hat(full_lhs)
    }
    full_lhs <- modify_lhs_for_link(model, full_lhs)
    class(full_lhs) <- c("character", class(model))
    return(full_lhs)
  }
}

#' @return A character string
#' @keywords internal
#' @noRd

extract_lhs_binomial <- function(model, ital_vars, use_coefs,
swap_var_names, var_colors, logit_notation = FALSE) {
  
  outcome <- all.vars(formula(model))[1]
  outcome_nm <- outcome
  names(outcome) <- outcome_nm

  if (!is.null(swap_var_names)) {
    outcome <- swap_names(swap_var_names, outcome)[[1]]
  }

  outcome_escaped <- escape_tex(outcome)
  outcome_escaped <- add_tex_ital_v(outcome_escaped, ital_vars)

  # This returns a 1x1 data.frame
  ss <- model$data[which(model$y == 1)[1], outcome_nm]
  # Convert to single character
  ss <- as.character(unlist(ss))
  ss_escaped <- escape_tex(ss)
  ss_escaped <- add_tex_ital_v(ss_escaped, ital_vars)

  if (!is.null(var_colors)) {
    names(outcome) <- outcome_nm
    names(outcome_escaped) <- outcome_nm
    outcome_escaped <- colorize_terms(var_colors, list(outcome), list(outcome_escaped))

    names(ss) <- outcome_nm
    names(ss_escaped) <- outcome_nm
    ss_escaped <- colorize_terms(var_colors, list(outcome), list(ss_escaped))
  }
  
  if (isTRUE(logit_notation)) {
    if (use_coefs) {
      full_lhs <- paste0(
        "\\operatorname{logit}\\left[ P(\\hat{",
        outcome_escaped,
        "}) \\right]"
      )
    } else {
      full_lhs <- paste0(
        "\\operatorname{logit}\\left[ P(",
        outcome_escaped,
        ") \\right]"
      )
    }
  } else {
    if (is.na(ss)) {
      full_lhs <- paste("P(", outcome_escaped, ")")
    } else {
      full_lhs <- paste(
        "P(", outcome_escaped,
        "=",
        ss_escaped, ")"
      )
    }
    if (use_coefs) {
      full_lhs <- add_hat(full_lhs)
    }
  }

  if (!isTRUE(logit_notation))
    full_lhs <- modify_lhs_for_link(model, full_lhs)
  class(full_lhs) <- c("character", class(model))
  full_lhs
}

#' @keywords internal
#' @noRd
extract_lhs2_binomial <- function(model, ital_vars, use_coefs, swap_var_names,
var_colors, logit_notation = FALSE) {
  
  outcome <- all.vars(formula(model))[1]
  outcome_nm <- outcome
  names(outcome) <- outcome_nm

  if (!is.null(swap_var_names)) {
    outcome <- swap_names(swap_var_names, outcome)[[1]]
  }

  n <- unique(model$model$`(weights)`)
  if (is.null(n)) {
    n <- nrow(model$data)
  }
  if (length(n) > 1) {
    warning(paste(
      "Unsure of how to handle a vector of weights in creation",
      "of the distribution portion of the equation. Please inspect",
      "carefully and modify by hand if necessary."
    ))
  }

  # This returns a 1x1 data.frame
  ss <- model$data[which(model$y == 1)[1], outcome_nm]

  # Convert to single character
  ss <- as.character(unlist(ss))

  outcome_escaped <- escape_tex(outcome)
  outcome_escaped <- add_tex_ital_v(outcome_escaped, ital_vars)

  ss_escaped <- escape_tex(ss)
  ss_escaped <- add_tex_ital_v(ss_escaped, ital_vars)

  if (!is.null(var_colors)) {
    names(outcome) <- outcome_nm
    names(outcome_escaped) <- outcome_nm
    outcome_escaped <- colorize_terms(var_colors, list(outcome), list(outcome_escaped))

    names(ss) <- outcome_nm
    names(ss_escaped) <- outcome_nm
    ss_escaped <- colorize_terms(var_colors, list(outcome), list(ss_escaped))
  }

  p <- paste0(
    "\\operatorname{prob}",
    add_tex_subscripts(
      paste0(
        outcome_escaped, " = ", ss_escaped
      )
    )
  )

  rhs <- paste0(
    "Bernoulli\\left(", p,
    "= \\hat{P}",
    "\\right)"
  )

  topline <- paste(outcome_escaped, "&\\sim", rhs)

  second_line <- modify_lhs_for_link(model, "\\hat{P}")

  full_lhs <- paste(topline, "\\\\\n", second_line, "\n")
  full_lhs
}


#' Extract left-hand side of a polr object
#'
#' Extract a string of the outcome/dependent/y variable with the appropriate
#' link function.
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd

extract_lhs.polr <- function(model, ital_vars, ...) {
  tidied <- broom::tidy(model)
  lhs <- tidied$term[tidied$coef.type == "scale"]
  lhs_escaped <- mapply_chr(escape_tex, lhs)
  threshold_focal <- lapply(
    strsplit(lhs_escaped, "\\|"),
    `[[`,
    1
  )
  lhs <- lapply(threshold_focal, add_tex_ital_v, ital_vars)
  outcome <- add_tex_ital_v(escape_tex(all.vars(formula(model))[1]), ital_vars)

  lhs <- lapply(lhs, function(.x) paste(outcome, " \\leq ", .x))
  lhs <- lapply(lhs, function(.x) paste0("P( ", .x, " )"))

  full_lhs <- lapply(lhs, function(.x) modify_lhs_for_link(model, .x))

  class(full_lhs) <- c("list", class(model))
  full_lhs
}


#' Extract left-hand side of a clm object
#'
#' Extract a string of the outcome/dependent/y variable with the appropriate
#' link function.
#'
#' @keywords internal
#'
#' @inheritParams extract_eq
#'
#' @return A character string
#' @noRd

extract_lhs.clm <- function(model, ital_vars, ...) {
  tidied <- broom::tidy(model)
  lhs <- tidied$term[tidied$coef.type == "intercept"]
  lhs_escaped <- mapply_chr(escape_tex, lhs)
  threshold_focal <- lapply(
    strsplit(lhs_escaped, "\\|"),
    `[[`,
    1
  )

  lhs <- lapply(threshold_focal, add_tex_ital_v, ital_vars)
  outcome <- add_tex_ital_v(escape_tex(all.vars(formula(model))[1]), ital_vars)

  lhs <- lapply(lhs, function(.x) paste(outcome, " \\leq ", .x))
  lhs <- lapply(lhs, function(.x) paste("P(", .x, ")"))
  full_lhs <- lapply(lhs, function(.x) modify_lhs_for_link(model, .x))

  class(full_lhs) <- c("list", class(model))
  full_lhs
}


#' modifies lhs of equations that include a link function
#' @keywords internal
#' @noRd
modify_lhs_for_link <- function(model, ...) {
  UseMethod("modify_lhs_for_link", model)
}

#' @keywords internal
#' @noRd
modify_lhs_for_link.glm <- function(model, lhs, ...) {
  check_dots(...)
  if (!(any(grepl(model$family$link, link_function_df$link_name)))) {
    message("This link function is not presently supported; using an identity
              function instead")
    model$family$link <- "identity"
  }
  matched_row_bool <- grepl(model$family$link, link_function_df$link_name)
  if (sum(matched_row_bool) > 1) {
    matched_row_bool[1] <- FALSE
  }
  filtered_link_formula <- link_function_df[matched_row_bool, "link_formula"]
  gsub("y", lhs, filtered_link_formula, fixed = TRUE)
}

#' @keywords internal
#' @noRd
modify_lhs_for_link.polr <- function(model, lhs, ...) {
  check_dots(...)
  matched_row_bool <- grepl(model$method, link_function_df$link_name)
  filtered_link_formula <- link_function_df[matched_row_bool, "link_formula"]

  gsub("y", lhs, filtered_link_formula, fixed = TRUE)
}

#' @keywords internal
#' @noRd
modify_lhs_for_link.clm <- function(model, lhs, ...) {

  check_dots(...)
  if (!(any(grepl(model$info$link, link_function_df$link_name)))) {
    message("This link function is not presently supported; using an identity
              function instead")
    model$info$link <- "identity"
  }
  matched_row_bool <- grepl(model$info$link, link_function_df$link_name)
  if (sum(matched_row_bool) > 1) {
    matched_row_bool[1] <- FALSE
  }
  filtered_link_formula <- link_function_df[matched_row_bool, "link_formula"]
  gsub("y", lhs, filtered_link_formula, fixed = TRUE)
}


# link-functions.R

link_name <- c(
  "logit, logistic",
  "probit",
  "inverse",
  # '1/mu^2', # inverse gaussian; removed until we're certain
  "log",
  "identity"
) # this isn't necessary as extract_eq() does this by default

# not sure how to address this one: quasi(link = "identity", variance = "constant")

link_formula <- c(
  "\\log\\left[ \\frac { y }{ 1 - y } \\right]",
  "y",
  "\\frac { 1 }{ y }",
  # "\\frac { 1 }{ 1/{ y }^{ 2 } } ", # inverse gaussian - correct?
  "\\log ({ y }) ",
  "y"
) # are the parentheses italicized here?

link_function_df <- data.frame(link_name, link_formula,
  stringsAsFactors = FALSE
)


#' Extract left-hand side of an forecast::Arima object
#'
#' Extract a dataframe of S/AR components
#'
#' @keywords internal
#'
#' @param model A forecast::Arima object
#'
#' @return A dataframe
#' @noRd
extract_lhs.forecast_ARIMA <- function(model, ...) {
  # LHS of ARIMA is the Auto Regressive side
  # Consists of Non-Seasonal AR (p), Seasonal AR (P), Non-Seasonal Differencing
  # (d), Seasonal Differencing(D), Constant Terms.
  # Constants are dealt with here if they go here and in LM if they go there.

  # This is more than needed, but we"re being explicit for readability.
  # Orders stucture in Arima model: c(p, q, P, Q, m, d, D)
  ords <- model$arma
  names(ords) <- c("p", "q", "P", "Q", "m", "d", "D")

  # Following the rest of the package.
  # Pull the full model with broom::tidy
  full_mdl <- broom::tidy(model)

  # Filter down to only the AR and SAR coefficients.
  full_lhs <- full_mdl[grepl("^s?ar", full_mdl$term), ]

  # Add a Primary column and set it to the backshift operator.
  full_lhs["primary"] <- rep("B", nrow(full_lhs))

  # Get the superscript for the backshift operator.
  ## This is equal to the number on the term for AR
  ## and the number on the term * the seasonal frequency for SAR.
  ## Powers of 1 are replaced with an empty string.
  lhs_super <- as.numeric(gsub("^s?ar", "", full_lhs$term))
  lhs_super[grepl("^sar", full_lhs$term)] <- lhs_super[grepl("^sar", full_lhs$term)] * ords["m"]

  lhs_super <- as.character(lhs_super)

  full_lhs["superscript"] <- lhs_super

  # The LHS has differencing and seasonal differencing.
  ## Setup the dataframe
  diff_df <- data.frame(
    term = c("zz_differencing", "zz_seas_Differencing"),
    estimate = rep(NA, 2),
    std.error = rep(NA, 2),
    primary = c("(1 - B)", paste0("(1 - B", "^{", ords["m"], "})")),
    superscript = c(ords["d"], ords["D"])
  )

  ## Remove what isn"t needed
  diff_df <- diff_df[ords[c("d", "D")] > 0, ]

  # Add the differencing lines to the end of the S/AR lines.
  full_lhs <- rbind(full_lhs, diff_df)

  # Reduce any "1" superscripts to blank
  full_lhs[full_lhs$superscript == "1", "superscript"] <- ""

  # Deal with potential constant terms
  # This will only come up if it is not a regression model
  regression <- helper_arima_is_regression(model)
  if (!regression) {
    # It is not a regression model
    c_df <- full_mdl[full_mdl$term %in% c("intercept", "drift", "mean"), ]

    c_df[c_df$term %in% c("intercept", "mean"), "primary"] <- ""
    c_df[c_df$term == "drift", "primary"] <- "t"
    c_df$superscript <- ""

    full_lhs <- rbind(full_lhs, c_df)
  }

  # Set subscripts so that create_term works later
  full_lhs$subscripts <- ""

  # Set the class
  class(full_lhs) <- c(class(model), "data.frame")

  # Explicit return
  return(full_lhs)
}

#' Display equations in shiny apps
#' 
#' `r lifecycle::badge("experimental")`
#' These are a set of functions designed to help render equations in 
#' Shiny applications (see the vignette about Shiny).
#'
#' @param expr An R expression, specifically a call to [extract_eq()]
#' @param outputId The identifier of the output from the server. Should be
#'   passed as a string.
#' @param env The environment
#' @param quoted Is the expression quoted?
#' @param outputArgs list of output arguments
#' 
#' @return Render the equation in a suitable way for Shiny for [renderEq()] in
#'   an [eqOutput()] equation output element that can be included in a panel.
#' @export
#' 

#' @describeIn renderEq Rendering function
renderEq <- function(expr, env = parent.frame(), quoted = FALSE, outputArgs = list()) {
  installExprFunction(expr = expr, name = "func", eval.env = env, quoted = quoted)
  createRenderFunction(func = func, function(value, session, name, ...) {
    as.character(withMathJax(format.equation(x = value)))
  }, eqOutput, outputArgs)
}

#' @describeIn renderEq Output function
#' @export
eqOutput <- function(outputId) {
  htmlOutput(outputId = outputId)
}

#' Strict versions of \code{base::\link[base]{mapply}}
#'
#' Return a vector of the corresponding type
#'
#' @keywords internal
#'
#' @param \dots arguments passed to \code{base::\link[base]{mapply}}
#'
#' @return A vector of the corresponding type, \code{chr} = character,
#'   \code{dbl} = double, \code{lgl} = logical, and \code{int} = logical
#' @noRd
mapply_chr <- function(...) {
  out <- mapply(...)
  stopifnot(is.character(out))
  out
}

mapply_lgl <- function(...) {
  out <- mapply(...)
  stopifnot(is.logical(out))
  out
}

mapply_int <- function(...) {
  out <- mapply(...)
  stopifnot(is.integer(out))
  out
}

mapply_dbl <- function(...) {
  out <- mapply(...)
  stopifnot(is.double(out))
  out
}

distinct <- function(df, col) {
  df[!duplicated(df[col]), ]
}

colorize <- function(col, x) {
  if(is.null(col)) {
    return(x)
  }
  if(is_latex_output()) {
    col <- strip_html_hash(col)
  }
  paste0("{\\color{", col, "}{", x, "}}")
}

colorize_terms <- function(colors, side_primary, primary_escaped) {
  missing <- setdiff(
    unique(names(unlist(side_primary))), 
    names(colors)
  )
  names(missing) <- missing
  full_name_vector <- c(colors, missing)
  
  color_matches <- lapply(side_primary, function(x) {
    full_name_vector[match(names(x), names(full_name_vector))]
  })
  
  color_matches <- lapply(color_matches, function(x) {
    x[x != names(x)]
  })
  
  Map(function(.x, .y) {
    matched <- .x[match(names(.y), names(.x))]
    out <- ifelse(is.na(matched), .y, colorize(matched, .y))
    names(out) <- names(.y)
    out
  },
  .x = color_matches, 
  .y = primary_escaped)
}

is_html_colorcode <- function(cols) {
  grepl("^#", cols)
}

strip_html_hash <- function(cols) {
  is_hex <- is_html_colorcode(cols)
  cols[is_hex] <- gsub("^#(.+)", "\\1", cols[is_hex])
  cols
}

define_latex_html_colors <- function(cols) {
  nms <- strip_html_hash(cols)
  nms <- nms[is_html_colorcode(cols)]
  vapply(nms, function(x) {
    paste0("\\definecolor{", x, "}{HTML}{", x, "}")
  }, FUN.VALUE = character(1))
}


## action: message, warning, stop
check_dots <- function(..., .ignore = NULL, .action="stop") {
    L <- list(...)
    if (length(.ignore)>0) {
        L <- L[!names(L) %in% .ignore]
    }
    if (length(L)>0) {
        FUN <- get(.action)
        FUN("unknown arguments: ",
            paste(names(L), collapse=","))
    }
    return(NULL)
}

test_that("Strict mapply_* functions work", {
  chr_function <- function(x) as.character(x)
  lgl_function <- function(x) TRUE
  int_function <- function(x) as.integer(x)
  dbl_function <- function(x) as.double(x)

  expect_equal(equatiomatic:::mapply_chr(chr_function, x = "test"),
    "test",
    label = "mapply_chr returns a character",
    ignore_attr = TRUE
  )

  expect_equal(equatiomatic:::mapply_lgl(lgl_function, x = "test"),
    TRUE,
    label = "mapply_lgl returns a logical",
    ignore_attr = TRUE
  )

  expect_equal(equatiomatic:::mapply_int(int_function, x = 1L),
    1L,
    label = "mapply_int returns an integer",
    ignore_attr = TRUE
  )

  expect_equal(equatiomatic:::mapply_dbl(dbl_function, x = 1.0),
    1.0,
    label = "mapply_dbl returns a double",
    ignore_attr = TRUE
  )

  expect_error(equatiomatic:::mapply_chr(lgl_function, x = "test"),
    label = "mapply_chr doesn't work with non-characters"
  )

  expect_error(equatiomatic:::mapply_lgl(chr_function, x = "test"),
    label = "mapply_lgl doesn't work with non-logicals"
  )

  expect_error(equatiomatic:::mapply_int(chr_function, x = "test"),
    label = "mapply_int doesn't work with non-integers"
  )

  expect_error(equatiomatic:::mapply_dbl(chr_function, x = "test"),
    label = "mapply_dbl doesn't work with non-doubles"
  )
})

#' The polls data from Gelman and Hill ()
#'
#' This is the dataset used in Gelman & Hill's book, Data Analysis Using 
#' Regression and Multilevel/Hierarchical Models. They are polling data on
#' the presidential election from 1988, collected one week before the election.
#' You can download all the data from the book here: http://www.stat.columbia.edu/~gelman/arm/examples/ARM_Data.zip.
#' Note that this is only a few of the variables from the original data supplied
#' with the book.
#'
#' @format A tibble with 13,544 rows and 7 variables:
#' \describe{
#'   \item{state}{An integer denoting the state identification number.}
#'   \item{edu}{An ordered factor  stating the education level of the respondent}
#'   \item{age}{An unordered factor stating the age of range of the respondent}
#'   \item{female}{A dummy variable (integer) denoting whether the respondent was 
#'     coded as male (female = 0) or female (female = 1).}
#'   \item{black}{A dummy variable (integer) denoting whether the respondent was 
#'     coded as Black (black = 1) or not Black (black = 0).}
#'   \item{weight}{A sampling weight}
#'   \item{bush}{Whether the respondent stated they were in favor of voting for
#'   George Bush Sr.}
#' }
"polls"

#' Simulated longitudinal data
#'
#' Data are simulated to be similar to longitudinal data collected within
#' schools/districts.
#'
#' @format A tibble with 1000 rows and 8 variables:
#' \describe{
#'   \item{sid}{An integer denoting the individual student. 
#'     There are 100 students.}
#'   \item{school}{An integer denoting the school There are 15 schools.}
#'   \item{district}{An integer denoting the school district. There are 
#'     5 districts.}
#'   \item{group}{A character variable denoting the instructional level of 
#'     the student, low, medium, or high.}
#'   \item{treatment}{A factor indicating whether the student received the 
#'     intervention treatment (0 = no treatment received; 1 = treatment 
#'     received).}
#'   \item{prop_low}{The proportion of student in the school in the low 
#'     instructional group.}
#'   \item{wave}{The assessment wave. Each student has nine waves of data 
#'     collection}
#'   \item{score}{The individual students' score at the given wave.}
#' }
"sim_longitudinal"

#' Size measurements for adult foraging penguins near Palmer Station, Antarctica
#'
#' Data originally from \href{https://allisonhorst.github.io/palmerpenguins/}{\code{palmerpenguins}}. Includes
#' measurements for penguin species, island in Palmer Archipelago,
#' size (flipper length, body mass, bill dimensions), and sex.
#'
#' @format A tibble with 344 rows and 8 variables:
#' \describe{
#'   \item{species}{a factor denoting penguin species (Adélie, Chinstrap and Gentoo)}
#'   \item{island}{a factor denoting island in Palmer Archipelago, Antarctica (Biscoe, Dream or Torgersen)}
#'   \item{bill_length_mm}{a number denoting bill length (millimeters)}
#'   \item{bill_depth_mm}{a number denoting bill depth (millimeters)}
#'   \item{flipper_length_mm}{an integer denoting flipper length (millimeters)}
#'   \item{body_mass_g}{an integer denoting body mass (grams)}
#'   \item{sex}{a factor denoting penguin sex (female, male)}
#'   \item{year}{an integer denoting the study year (2007, 2008, or 2009)}
#' }
#' @source {Adélie penguins: Palmer Station Antarctica LTER and K. Gorman. 2020. 
#'   Structural size measurements and isotopic signatures of foraging among 
#'   adult male and female Adélie penguins (Pygoscelis adeliae) nesting along 
#'   the Palmer Archipelago near Palmer Station, 2007-2009 ver 5. Environmental 
#'   Data Initiative} \doi{10.6073/pasta/98b16d7d563f265cb52372c8ca99e60f}
#' @source {Gentoo penguins: Palmer Station Antarctica LTER and K. Gorman. 2020. 
#'   Structural size measurements and isotopic signatures of foraging among 
#'   adult male and female Gentoo penguin (Pygoscelis papua) nesting along the 
#'   Palmer Archipelago near Palmer Station, 2007-2009 ver 5. Environmental 
#'   Data Initiative} \doi{10.6073/pasta/7fca67fb28d56ee2ffa3d9370ebda689}
#' @source {Chinstrap penguins: Palmer Station Antarctica LTER and K. Gorman. 
#'   2020. Structural size measurements and isotopic signatures of foraging 
#'   among adult male and female Chinstrap penguin (Pygoscelis antarcticus) 
#'   nesting along the Palmer Archipelago near Palmer Station, 2007-2009 ver 6. 
#'   Environmental Data Initiative} \doi{10.6073/pasta/c14dfcfada8ea13a17536e73eb6fbe9e}
#' @source {Originally published in: Gorman KB, Williams TD, Fraser WR (2014) 
#'   Ecological Sexual Dimorphism and Environmental Variability within a 
#'   Community of Antarctic Penguins (Genus Pygoscelis). PLoS ONE 9(3): e90081.} 
#'   \doi{10.1371/journal.pone.0090081}
"penguins"

#' Provides the order of the levels
#' @param rhs_random output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "ran_pars", ]}.
#' @noRd
get_order <- function(rhs_random) {
  sort(tapply(rhs_random$original_order, rhs_random$group, min))
} 

vary_higher_subscripts <- function(term, rhs_random, lev_omit = NULL) {
  splt_random <- split(rhs_random, rhs_random$group)
  lev_indexes <- letters[seq_along(splt_random) + 9]
  
  order <- get_order(rhs_random)
  splt_random <- splt_random[names(order)]
  
  if (!is.null(lev_omit)) {
    lev_indexes <- lev_indexes[-seq_along(grep(lev_omit, names(splt_random)))]
    splt_random <- splt_random[-seq_along(grep(lev_omit, names(splt_random)))]
  }
  
  out <- rep(NA_character_, length(lev_indexes))
  for (i in seq_along(splt_random)) {
    if (any(grepl(term, splt_random[[i]]$term))) {
      out[i] <- paste0(lev_indexes[i], "[i]")
    }
  }
  paste0(out[!is.na(out)], collapse = ",")
}

#' Create a vector to split by
#' The vector denotes which level the term (for the given row) 
#' should be added to in the equation (e.g., l1, l2, l3, etc.)
#' @param rhs_fixed output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "fixed", ]}
#' @param rhs_random output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "ran_pars", ]}
#' @return A named vector where the names represent the term and the 
#' values represent the level. For example, the output may look similar
#' to \code{c("(Intercept)" = "l1", "wave" = "l1", 
#'          "grouplow" = "sid", "groupmedium" = "sid")}
#' @noRd        
pred_level_split <- function(rhs_fixed, rhs_random) {

  order <- get_order(rhs_random)
  
  vapply(rhs_fixed$pred_level, function(x) {
    if (length(x) == 0) out <- "l1"
    if (length(x) == 1) out <- x
    if (length(unique(x)) == 1) out <- unique(x)
    if (length(unique(x)) > 1) {
      out <- order[x]
      out <- names(out[which.max(out)])
    }
    out
  }, FUN.VALUE = character(1))
}

pull_term_subscript <- function(greek_coef, n = 1) {
  regex <- paste0("(.+\\{.{", n, "}).+(\\})")
  gsub(regex, "\\1\\2", greek_coef)
}

pull_min_level <- function(one_crossdata, one_detected, one_lev, one_var, 
                           order) {
  all_vars <- mapply_chr(function(lev, var) {
    d <- one_crossdata[[lev]]
    
    out <- d[d$term == var, ]$greek
    if (length(out) == 0) {
      return("")
    }
    out
  }, one_lev, one_var)
  
  min_lev <- one_lev[which.min(order[one_lev])]
  all_vars[names(min_lev)]
}

pull_superscript <- function(slp_preds, detect_cross, cross_data, order) {
  order <- c("l1" = 0, order)
  levs <- Map(function(predlev, detected) {
    predlev[detected]
  }, slp_preds$pred_level, detect_cross)
  
  vars <- Map(function(splt, detected) {
    splt[detected]
  }, slp_preds$split, detect_cross)
  
  ss <- mapply_chr(function(cd, detected, lev, var) {
    pull_min_level(cd, detected, lev, var, order)
  }, cross_data, detect_cross, levs, vars)
  
  superscripts <- if (length(ss) == 0) {
    ""
  } else {
    ifelse(is.na(ss), "", ss)
  }
  superscripts
}


#' Creates the greek for the level 1 fixed effects
#' @param rhs_fixed output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "fixed", ]}
#' @param rhs_random output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "ran_pars", ]}
#' @return A character vector the same length at the number of rows in 
#'   \code{rhs_fixed} with the greek assigned for those that are at level 1 and 
#'   \code{NA_character_} otherwise (higher-level predictors).
#' @noRd
assign_l1_greek <- function(rhs_fixed, rhs_random) {
  beta_indices <- seq_along(rhs_fixed$l1[rhs_fixed$l1])
  if (any(rhs_fixed$term == "(Intercept)" & rhs_fixed$l1)) {
    beta_indices <- beta_indices[-length(beta_indices)]
  }
  l1 <- rep(NA_character_, length(rhs_fixed$term))
  l1[rhs_fixed$term == "(Intercept)" & rhs_fixed$l1] <- "\\alpha_{"
  l1[rhs_fixed$term != "(Intercept)" & rhs_fixed$l1] <- paste0("\\beta_{", beta_indices)
  
  terms <- rhs_fixed$term[!is.na(l1)]
  
  ss <- vapply(terms, function(x) vary_higher_subscripts(x, rhs_random),
               FUN.VALUE = character(1))
  l1[!is.na(l1)] <- paste0(l1[!is.na(l1)], ss, "}")
  l1  
}

#' Takes one level of the fixed effects data and assigns greek for that level
#' @param lev_data One level of the data, e.g. \code{splt[[1]]} where 
#'   \code{splt} is the \code{rhs_fixed} data split by the \code{predsplit} 
#'   column.
#' @param lev_data_name A character vector specifying the name of the level, 
#'   e.g., \code{"sid"}, \code{"school"}, \code{"site"}.
#' @param splt The \code{rhs_fixed} data split by the \code{predsplit} 
#'   column.
#' @param rhs_random output from \code{extract_rhs.lmerMod}, subset as 
#' \code{rhs[rhs$effect == "ran_pars", ]}
#' @return The full data \code{rhs_fixed} data frame for that level with the
#'   \code{greek} column filled in
#' @noRd
assign_higher_levels <- function(lev_data, lev_data_name, splt, rhs_random) {
  if (is.null(lev_data)) {
    return()
  }
  split_terms <- split(lev_data, lev_data$crosslevel)
  order <- get_order(rhs_random)
  
  # create intercepts
  int_preds <- split_terms$`FALSE`
  ss <- lapply(int_preds$term, function(x) vary_higher_subscripts(x, rhs_random, lev_data_name))
  int_preds$greek <- paste0("\\gamma_{", seq_len(nrow(int_preds)), ss, "}")
  
  # Cross-level interactions
  slp_preds <- split_terms$`TRUE`
  if (is.null(slp_preds)) {
    return(int_preds)
  }
  
  detect_cross <- lapply(slp_preds$pred_level, function(x) {
    !grepl(lev_data_name, x)
  })
  
  cross_data <- Map(function(pred_levs, cross_detected) {
    splt[pred_levs[cross_detected]]
  },
  slp_preds$pred_level,
  detect_cross)
  
  superscripts <- pull_superscript(slp_preds, detect_cross, cross_data, order)
  superscripts <- pull_term_subscript(superscripts)
  
  coef_number <- unlist(tapply(superscripts, superscripts, seq_along))
  
  subscripts <- vapply(slp_preds$term, function(x) {
    vary_higher_subscripts(x, rhs_random, lev_data_name)
  }, FUN.VALUE = character(1))
  
  slp_preds$greek <- paste0("\\gamma^{", superscripts, "}_{", coef_number, subscripts, "}")
  
  # return
  out <- rbind(int_preds, slp_preds)
  out[order(out$original_order), ]
}

#' Create the fixed effects portion of the equation
#' @param model The fitted \code{\link[lme4]{lmer}} model
#' @return The \code{extract_rhs} data frame where \code{effect == fixed} with
#'   a new \code{greek} column denoting the greek for the specific
#'   coefficient (including indexing for the levels the fixed effects vary
#'   at), and a new \code{predsplit} column that allows this data frame
#'   to be split by the level at which the variable predicts.
#' @noRd
create_fixef_greek_merMod <- function(model, return_variances) {
  rhs <- extract_rhs(model, return_variances)
  rhs_fixed <- rhs[rhs$effect == "fixed", ]
  rhs_random <- rhs[rhs$effect == "ran_pars", ]
  rhs_random <- rhs_random[rhs_random$group != "Residual", ]
  
  order <- get_order(rhs_random)
  
  # fixed effects
  rhs_fixed$greek <- assign_l1_greek(rhs_fixed, rhs_random)
  rhs_fixed$predsplit <- pred_level_split(rhs_fixed, rhs_random)
  
  splt <- split(rhs_fixed, rhs_fixed$predsplit)[c("l1", names(order))]
  splt <- splt[!vapply(splt, is.null, FUN.VALUE = logical(1))]
  
  for (i in seq_along(splt)[-1]) {
    splt[[i]] <- assign_higher_levels(splt[[i]], names(splt)[i], splt, rhs_random) 
  }
  
  Reduce(rbind, splt)
}

pull_cross_var <- function(cross_splt_frame, order) {
  if (nrow(cross_splt_frame) == 0) {
    return()
  }
  interaction_terms <- strsplit(cross_splt_frame$term, ":")
  order <- c("l1" = 0, order)
  orders <- lapply(cross_splt_frame$pred_level, function(x) {
    order[x]
  })
  
  # put in order from lowest to highest level
  interaction_terms <- Map(order_split, 
                           interaction_terms, 
                           cross_splt_frame$pred_level)
  
  term <- mapply_chr(function(interaction, order) {
    interaction[which.min(order)]
  }, interaction_terms, orders)
  unique(term)
}

rm_crosslev_int_redundancy <- function(model, lev_data, term) {
  formula_rhs <- labels(terms(formula(model)))
  formula_rhs <- formula_rhs[!grepl(":|\\|", formula_rhs)]
  terms <- vapply(term, function(x) {
    unlist(extract_primary_term(formula_rhs, x))
  }, FUN.VALUE = character(1))
  
  subset <- lapply(lev_data$primary, function(x) {
    exact <- x %in% paste0(terms, collapse = "|")
    detected <- grepl(paste0(terms, collapse = "|"), x)
    
    if (any(exact)) {
      !exact
    } else {
      !detected
    }
  })
  
  lev_data$primary <- Map(function(x, ss) x[ss], lev_data$primary, subset)
  lev_data$subscripts <- Map(function(x, ss) x[ss], lev_data$subscripts, subset)
  lev_data
}

# Make cross-level interactions go on the intercept level if the coef
# it predicts doesn't vary at this level (because there's no mean structure
# for it)
check_interact_vary <- function(splt_lev_fixed, splt_lev_random, order) {
  cross <- splt_lev_fixed[splt_lev_fixed$crosslevel, ]
  lower_var <- pull_cross_var(cross, order)
  check_vary <- lower_var %in% splt_lev_random$term
  
  check <- vapply(lower_var, function(x) {
    grepl(x, splt_lev_fixed$term)
  }, FUN.VALUE = logical(length(splt_lev_fixed$term)))
  as.logical(check %*% check_vary)
}

pull_intercept <- function(splt_lev_fixed, splt_lev_random, order, 
                           use_coefs, coef_digits, fix_signs) {

  int <- "\\alpha"

  if (is.null(splt_lev_fixed)) {
    return()
  }
  
  check <- check_interact_vary(splt_lev_fixed, splt_lev_random, order)
  
  # Check if lower-level variable in cross-level interactions varies at this level
  splt_lev_fixed$crosslevel <- check & splt_lev_fixed$crosslevel
    
  nocross <- splt_lev_fixed[!splt_lev_fixed$crosslevel, ]
  
  # remove any previous superscripts
  nocross$greek <- gsub("(.+)\\^.+(_\\{.+$)", "\\1\\2",  nocross$greek)
  
  # renumber
  nocross$greek <- paste0(gsub("(.+_\\{).+", "\\1", nocross$greek),
                          seq_along(nocross$greek),
                          gsub(".+_\\{.{1}(.+)", "\\1", nocross$greek))
  
  # add alpha superscript
  nocross$greek <- paste0(nocross$greek, paste0("^{", int, "}"))
  
  if (use_coefs) {
    coef_terms <- paste0(round(nocross$estimate, coef_digits), 
                         "_{", nocross$greek, "}",  
                         nocross$terms)
    if (length(coef_terms) == 0) {
      coef_terms <- 0
    }
    coef_terms <- paste0(coef_terms, collapse = " + ")
    if (fix_signs) {
      coef_terms <- fix_coef_signs(coef_terms)
    }
    out <- data.frame(term = "(Intercept)", 
                      greek = coef_terms,
                      stringsAsFactors = FALSE)
  } else {
    coef_terms <- paste0(nocross$greek, nocross$terms)
    # add intercept term
    coef_terms <- c(paste0("\\gamma_{0}^{", int, "}"), coef_terms)
    out <- data.frame(term = "(Intercept)", 
                      greek = paste0(coef_terms, collapse = " + "),
                      stringsAsFactors = FALSE)
    if (nrow(out) == 0) {
      return()
    }
  }

  out
}

pull_slope_superscript <- function(greek) {
  gsub(".+\\^\\{(.+)\\}_.+", "\\1", greek)
}

create_slope_intercept <- function(term) {
  gsub("(.+)(.{1})(\\}$)", "\\10\\3", term)
}

pull_slopes <- function(model, splt_lev_fixed, splt_lev_random, ital_vars,
                        order, use_coefs, coef_digits, fix_signs,
                        swap_var_names, swap_subscript_names,
                        var_colors, var_subscript_colors) {
  if (is.null(splt_lev_fixed)) {
    return()
  }
  
  check <- check_interact_vary(splt_lev_fixed, splt_lev_random, order)
  splt_lev_fixed$crosslevel <- check & splt_lev_fixed$crosslevel
  
  cross <- splt_lev_fixed[splt_lev_fixed$crosslevel, ]
  
  # pull the specific slope that is predicted by the cross-level interaction
  cross$slope_predicted <- pull_slope_superscript(cross$greek)
  
  # split the df by the slope that is predicted
  cross_splt <- split(cross, cross$slope_predicted)
  
  # pull the name of the variable being predicted
  terms_predicted <- lapply(cross_splt, pull_cross_var, order)
  
  # remove that variable from interaction so there's no redundancies
  cross_splt <- Map(function(cross, terms) {
    rm_crosslev_int_redundancy(model, cross, terms)
  }, cross_splt, terms_predicted)
  
  # recreate terms
  cross_splt <- lapply(cross_splt, function(x) {
    x$terms <- create_term(x, ital_vars, swap_var_names, 
                           swap_subscript_names,
                           var_colors, var_subscript_colors)
    
    x$terms <- vapply(x$terms, function(x) {
      if (nchar(x) == 0) {
        return("")
      }
      paste0("(", x, ")")
    }, character(1))
    x
  })
  
  if (use_coefs) {
    final_slopes <- lapply(cross_splt, function(x) {
      slopes <- paste0(round(x$estimate, coef_digits),
                       "_{",
                       #gsub("(^.+)\\^\\{.+\\}\\}(.+)", 
                        #    "\\1\\2", 
                        cross_splt[[1]]$greek, #),
                       "}",
                       x$terms)
      paste0(slopes, collapse = " + ")
    })
  } else {
    final_slopes <- lapply(cross_splt, function(x) {
      int <- create_slope_intercept(x$greek[1])
      slopes <- paste0(x$greek, x$terms)
      paste0(c(int, paste0(slopes, collapse = " + ")), collapse = " + ")
    })
  }
  if (fix_signs) {
    final_slopes <- lapply(final_slopes, fix_coef_signs)
  }
  out <- data.frame(term = unlist(terms_predicted),
                    greek = unlist(final_slopes),
                    stringsAsFactors = FALSE)
  if (nrow(out) == 0) {
    return()
  }
  out
}

rbind_named <- function(l) {
  null <- vapply(l, is.null, FUN.VALUE = logical(1))
  l <- Map(function(lst, lst_names) {
    lst$group <- lst_names
    lst
  }, l[!null], names(l)[!null])
  
  Reduce(rbind, l)
}

#' Create the mean structure for the VCV matrix
#' @param rhs Output from \code{extract_rhs}
#' @param fixed_greek_mermod Output from \code{create_fixef_greek_merMod}.
#' @param model The fitted \code{\link[lme4]{lmer}} model
#' @param ital_vars Logical, defaults to \code{FALSE}. Should the variable
#'   names not be wrapped in the \code{\\operatorname{}} command?
#' @return A data frame with each term that varies at each higher level (term), 
#' the level at which it varies (group), the greek for the mean structure 
#' (greek), and the greek for the term that is varying (greek_vary). The 
#' \code{greek_vary} column defines the left hand side of ~. The greek for the 
#' mean structure (greek) will include any group-level predictors. If there are 
#' none, it will be replaced by \code{"\\mu_{greek_vary}"} where 
#' \code{greek_vary} is actually substituted in.
#' @noRd
create_means_merMod <- function(rhs, fixed_greek_mermod, model, ital_vars,
                                use_coefs, coef_digits, fix_signs,
                                swap_var_names, swap_subscript_names,
                                var_colors, var_subscript_colors) {
  rhs_random <- rhs[rhs$effect == "ran_pars", ]
  rhs_random <- rhs_random[rhs_random$group != "Residual" &
                             !grepl("^cor__", rhs_random$term), ]
  rhs_random$term <- gsub("sd__", "", rhs_random$term)
  
  order <- get_order(rhs_random)
  
  fixed_greek_mermod$terms <- create_term(fixed_greek_mermod, ital_vars,
                                          swap_var_names, swap_subscript_names,
                                          var_colors, var_subscript_colors)
  
  fixed_greek_mermod$terms <- vapply(fixed_greek_mermod$terms, function(x) {
    if (nchar(x) == 0) {
      return("")
    }
    paste0("(", x, ")")
  }, character(1))
  
  splt_fixed <- split(fixed_greek_mermod, fixed_greek_mermod$predsplit)
  splt_fixed <- splt_fixed[names(order)]
  names(splt_fixed) <- names(order)
  
  splt_rand <- split(rhs_random, rhs_random$group)
  splt_rand <- splt_rand[names(order)]
  names(splt_rand) <- names(order)
  
  ints <- Map(function(fixed, rand) {
    pull_intercept(fixed, rand, order, use_coefs, coef_digits, fix_signs)
  }, splt_fixed, splt_rand)
  ints <- rbind_named(ints)
  
  slopes <- Map(function(fixed, rand) {
    pull_slopes(model, fixed, rand, ital_vars, order, 
                use_coefs, coef_digits, fix_signs,
                swap_var_names, swap_subscript_names,
                var_colors, var_subscript_colors)
  }, splt_fixed, splt_rand)
  slopes <- rbind_named(slopes)
  
  int_slopes <- rbind(ints, slopes)
  if (is.null(int_slopes)) {
    out <- rhs_random
    out$greek <- NA_character_
    
  } else {
    out <- merge(rhs_random, int_slopes, 
                 all.x = TRUE, by = c("group", "term")) 
  }
  out <- out[order(out$original_order), c("group", "term", "greek", "original_order")]  
  random_vary <- fixed_greek_mermod[fixed_greek_mermod$term %in% unique(out$term), ]
  random_vary$greek_vary <- gsub("(.+\\{\\d?\\d?\\d?).+", "\\1}", random_vary$greek)
  random_vary$new_order <- random_vary$original_order
  
  out <- merge(out, 
               random_vary[, c("term", "greek_vary", "new_order")], 
               by = "term", 
               all.x = TRUE)
  
  lev_indexes <- setNames(letters[seq_along(order) + 9], names(order))
  lev_indexes <- lev_indexes[match(out$group, names(lev_indexes))]
  out$greek_vary <- ifelse(
    is.na(out$greek_vary), 
    "",
    paste0(gsub("(.+)\\}", "\\1", out$greek_vary), lev_indexes, "}")
  )
  
  if (use_coefs) {
    out$greek <- ifelse(is.na(out$greek), 0, out$greek)
  } else {
    out$greek <- ifelse(is.na(out$greek), 
                        paste0("\\mu_{", out$greek_vary, "}"),
                        out$greek) 
  }

  out[order(out$new_order), ]
}

assign_vcov_greek <- function(rand_lev, means_merMod) {
  assign <- lapply(rand_lev$terms, function(x) {
    means_merMod$greek_vary[match(x, means_merMod$term)]
  })
  lapply(assign, function(x) {
    if (length(x) == 1) {
      return(c(x, x))
    } else {
      x
    }
  })
}

create_greek_matrix <- function(v, mat, use_coefs, coef_digits, est) {
  if (identical(use_coefs, FALSE)) {
    if (length(unique(v)) == 1) {
      greek_vcov <- paste0("\\sigma^2_{", v[1], "}")
    } else {
      greek_vcov <- paste0("\\rho_{", 
                           paste0(v, collapse = ""), 
                           "}", 
                           collapse = "")
    }
  } else {
    greek_vcov <- round(est, coef_digits)
  }
  
  mat[v[1], v[2]] <- greek_vcov
  mat
}

pull_var <- function(term) {
  if (grepl("^cor__", term)) {
    ran_part <- gsub("(.+\\.).+", "\\1", term)
    term <- gsub(ran_part, "", term, fixed = TRUE)
  } else if (grepl("^sd__", term)) {
    ran_part <- "sd__"
    term <- gsub(paste0("^", ran_part), "", term)
  }
  term
}


create_vcov_merMod <- function(rhs_random_lev, means_merMod, 
                               use_coefs, coef_digits) {

  rand_lev <- rhs_random_lev[, c("group", "term", "estimate")]
  rand_lev$terms <- gsub("sd__|cor__", "", rand_lev$term)
  rand_lev$terms <- strsplit(rand_lev$terms, "\\.")
  
  # add extra row for reverse on correlation
  cors <- rand_lev[grepl("^cor__", rand_lev$term), ]
  cors$terms <- lapply(cors$terms, rev)
  
  rand_lev <- rbind(rand_lev, cors)
  
  means <- means_merMod[means_merMod$group == unique(rand_lev$group), ]
  rand_lev$vcov_greek <- assign_vcov_greek(rand_lev, means)
  
  rand_lev$terms_var <- vapply(rand_lev$term, pull_var, FUN.VALUE = character(1))
  rand_lev <- merge(rand_lev, means[, c("term", "new_order")], 
                    by.x = "terms_var", 
                    by.y = "term", 
                    all.x = TRUE)
  rand_lev <- rand_lev[order(rand_lev$new_order), ]
  
  # Create matrix
  sd_rows <- grepl("^sd__", rand_lev$term)
  
  mat <- diag(sum(sd_rows))
  dimnames(mat) <- list(unique(unlist(rand_lev$vcov_greek[sd_rows])),
                        unique(unlist(rand_lev$vcov_greek[sd_rows])))
  for (i in seq_along(rand_lev$vcov_greek)) {
    mat <- create_greek_matrix(rand_lev$vcov_greek[[i]], mat, 
                               use_coefs, coef_digits, 
                               est = rand_lev$estimate[i])
  }
  mat
}

create_ranef_structure_merMod <- function(model, ital_vars, use_coefs,
                                          coef_digits, fix_signs, 
                                          return_variances, swap_var_names, 
                                          swap_subscript_names,
                                          var_colors, var_subscript_colors) {
  rhs <- extract_rhs(model, return_variances)
  rhs_random <- rhs[rhs$effect == "ran_pars", ]
  order <- get_order(rhs_random[rhs_random$group != "Residual", ])
  
  fixed_greek_mermod <- create_fixef_greek_merMod(model, return_variances)
  means_merMod <- create_means_merMod(rhs, fixed_greek_mermod, 
                                      model, ital_vars,
                                      use_coefs, coef_digits, 
                                      fix_signs, swap_var_names, 
                                      swap_subscript_names,
                                      var_colors, var_subscript_colors)
  
  means_splt <- split(means_merMod, means_merMod$group)[names(order)]
  names(means_splt) <- names(order)
  
  rhs_random_splt <- split(rhs_random, rhs_random$group)[names(order)]
  
  vcov_mats <- lapply(rhs_random_splt, function(x) {
    create_vcov_merMod(x, means_merMod, use_coefs, coef_digits)
  })
  
  # create each final latex piece
  lhs <- lapply(means_splt, function(x) create_onecol_array(x$greek_vary))
  means <- lapply(means_splt, function(x) create_onecol_array(x$greek)) 
  vcovs <- lapply(vcov_mats, convert_matrix)
  
  distributed <- Map(wrap_normal_dist, means, vcovs)
  
  Map(function(lhs, dist, name, index) {
    paste0("\n    ", lhs, " \\sim ", dist,
           "\n    \\text{, for ", escape_tex(name), " ", tolower(index), " = 1,}",
           " \\dots ", "\\text{,", toupper(index), "}")
  }, lhs, distributed, names(lhs), letters[seq_along(lhs) + 9])
}

#' Create a one column array from a vector
#' This may be able to be incorporated into the `convert_matrix` function
#' @param v character vector to put into a one-column array
#' @keywords internal
#' @noRd
#' @examples \dontrun {
#' equatiomatic:::create_onecol_array(c("\\alpha_{j}", "\\beta_{1j}"))
#'
#' }
#' # Note that the actual function does not return with the linebreak
create_onecol_array <- function(v) {
  if (length(v) == 1 | is.null(v)) {
    return(v)
  }
  v <- paste0("&", v)
  paste0("\n\\left(\n  \\begin{array}{c} \n    \\begin{aligned}\n",
    paste0("      ", v, collapse = " \\\\\n"),
    "\n",
    "    \\end{aligned}\n",
    "  \\end{array}\n\\right)\n"
  )
}

#' Convert a matrix to a LaTeX array
#' @param mat An R matrix
#' @keywords internal
#' @noRd
#' @examples \dontrun {
#' m <- matrix(c("\\sigma^2_{\\alpha_{j}}",
#'               "\\rho \\sigma_{\\beta_{1j}} \\sigma_{\\alpha_{j}}",
#'               "\\rho \\sigma_{\\alpha_{j}} \\sigma_{\\beta_{1j}}",
#'               "\\sigma^2_{\\beta_{1j}"),
#'             ncol = 2)
#' equatiomatic:::convert_matrix(m)
#' }
convert_matrix <- function(mat) {
  if (all(dim(mat) == c(1, 1))) {
    return(mat)
  }
  cols <- paste(rep("c", ncol(mat)), collapse = "")
  paste0("\n\\left(\n  \\begin{array}{", cols, "}\n",
    paste("    ", apply(mat, 1, paste, collapse = " & "), collapse = " \\\\ \n"),
    "\n  \\end{array}\n\\right)\n"
  )
}

create_l1_fixef <- function(model, ital_vars, use_coefs, coef_digits, 
                            mean_separate, fix_signs, wrap, terms_per_line, 
                            operator_location, return_variances, 
                            swap_var_names, swap_subscript_names,
                            var_colors, var_subscript_colors) {
  rhs <- extract_rhs(model, return_variances)
  lhs <- extract_lhs(model, ital_vars, use_coefs, swap_var_names, var_colors)
  greek <- create_fixef_greek_merMod(model, return_variances)
  terms <- create_term(greek, ital_vars, swap_var_names, 
                       swap_subscript_names,
                       var_colors, var_subscript_colors)
  
  terms <- vapply(terms, function(x) {
    if (nchar(x) == 0) {
      return("")
    }
    paste0("(", x, ")")
  }, character(1))
  
  if (use_coefs) {
    sigma <- round(sigma(model), coef_digits)
    l1 <- paste0(
      round(greek$estimate[greek$predsplit == "l1"], coef_digits),
      "_{", greek$greek[greek$predsplit == "l1"], "}",
      terms[greek$predsplit == "l1"]
    )
  } else {
    l1 <- paste0(greek$greek[greek$predsplit == "l1"], 
                 terms[greek$predsplit == "l1"])
    if(is_exposure_modeled(model)) {
      offset <- get_offset(model, ital_vars)
      l1 <- c(offset, l1)
    }
  }
  
  if (wrap) {
    if (operator_location == "start") {
      line_end <- "\\\\\n&\\quad + "
    } else {
      line_end <- "\\ + \\\\\n&\\quad "
    }
    l1 <- split(l1, ceiling(seq_along(l1) / terms_per_line))
    
    if (identical(mean_separate, FALSE)) {
      l1 <- lapply(l1, function(x) {
        terms_added <- paste0(x, collapse = " + ")
        paste0("&", terms_added)
      })
      l1 <- paste0("\\begin{aligned}\n", paste0(l1, collapse = "\\\\"), "\n\\end{aligned}")
      if (fix_signs) {
        l1 <- fix_coef_signs(l1)  
      }
    } else {
      l1 <- lapply(l1, paste0, collapse = " + ")
      l1 <- paste0(l1, collapse = line_end)
      if (fix_signs) {
        l1 <- fix_coef_signs(l1)  
      }
    }
  } else {
    l1 <- paste0(l1, collapse = " + ")
    if (fix_signs) {
      l1 <- fix_coef_signs(l1)  
    }
  }
}

create_l1 <- function(model, ...) {
  UseMethod("create_l1", model)
}

#' Create the full fixed-effects portion of an lmerMod
#'
#' @param model A fitted model from \code{\link[lme4]{lmer}}
#' @param ital_vars Logical, defaults to \code{FALSE}. Should the variable
#'   names not be wrapped in the \code{\\operatorname{}} command?
#' @param sigma The error term. Defaults to "\\sigma^2".
#' @keywords internal
#' @noRd
#' @examples \dontrun{
#' library(lme4)
#' fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
#' equatiomatic:::create_fixed_merMod(fm1, FALSE)
#' }
create_l1.lmerMod <- function(model, mean_separate,
                             ital_vars, wrap, terms_per_line,
                             use_coefs, coef_digits, fix_signs,
                             operator_location, sigma = "\\sigma^2",
                             return_variances, swap_var_names, 
                             swap_subscript_names,
                             var_colors, var_subscript_colors, ...) {
  check_dots(...)
  
  rhs <- extract_rhs(model, return_variances)
  lhs <- extract_lhs(model, ital_vars, use_coefs, swap_var_names, var_colors)
  l1 <- create_l1_fixef(model, ital_vars, use_coefs, coef_digits, 
                        mean_separate, fix_signs, wrap, terms_per_line, 
                        operator_location, return_variances,
                        swap_var_names, swap_subscript_names,
                        var_colors, var_subscript_colors)
  if (is.null(mean_separate)) {
    mean_separate <- sum(rhs$l1) > 3
  }
  if (mean_separate) {
    paste0(lhs, " \\sim ", wrap_normal_dist("\\mu", sigma),
           " \\\\\n    \\mu &=", l1)
  }  else {
    paste(lhs, "\\sim", wrap_normal_dist(l1, sigma))
  }
}

#' @export
#' @noRd
create_l1.glmerMod <- function(model, mean_separate,
                              ital_vars, wrap, terms_per_line,
                              use_coefs, coef_digits, fix_signs,
                              operator_location, sigma = "\\sigma^2",
                              return_variances, swap_var_names, 
                              swap_subscript_names,
                              var_colors, var_subscript_colors, ...) {
  check_dots(...)
  
  rhs <- extract_rhs(model, return_variances)
  lhs <- extract_lhs(model, ital_vars, use_coefs, swap_var_names, var_colors)
  l1 <- create_l1_fixef(model, ital_vars, use_coefs, coef_digits, 
                        mean_separate, fix_signs, wrap, terms_per_line, 
                        operator_location, return_variances,
                        swap_var_names, swap_subscript_names,
                        var_colors, var_subscript_colors)
  
  combo <- paste0(which_family(model), "-", which_link(model))
  
  switch(
    combo,
    "binomial-logit" = binomial_logit_l1(model, lhs, l1, ital_vars),
    "poisson-log" = poisson_log_l1(lhs, l1)
  )
}

#' @keywords internal
#' @importFrom stats terms formula coef setNames
#' @importFrom knitr knit_print asis_output is_html_output is_latex_output
#' @importFrom utils tail
#' @importFrom shiny installExprFunction createRenderFunction htmlOutput
#'   withMathJax helpText

"_PACKAGE"

utils::globalVariables("func")

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' Determine if the forecast::Arima model is Regression w/ Arima Errors
#' or Arima only.
#'
#' @keywords internal
#'
#' @param model A forecast::Arima object
#'
#' @return A boolean
#' @noRd
helper_arima_is_regression <- function(model) {
  # Determine if we are working on Regression w/ Arima Errors
  !(is.null(model$xreg) || ((NCOL(model$xreg) == 1) && is.element("drift", names(model$coef))))
}


#' Extract regression components from forecast::Arima when using Regression w/
#' Arima Errors. Similar to \code{extract_rhs} and \code{extract_lhs}.
#'
#' @keywords internal
#'
#' @param model A forecast::Arima object
#'
#' @return A dataframe
#' @noRd
helper_arima_extract_lm <- function(model) {
  # ARIMA is a linear model and Exogenous Regressors can be included.
  # This function will pull out the linear portion with the exception of constants.

  # Following the rest of the package.
  # Pull the full model with broom::tidy
  full_mdl <- broom::tidy(model)
  full_mdl$term <- as.character(full_mdl$term)

  # Filter down to only non-ARMA coefficients
  full_rhs <- full_mdl[!grepl("^s?ar|^s?ma", full_mdl$term), ]

  # Extract RHS from formula, excluding constants
  formula_rhs <- full_rhs$term[!grepl("^intercept|^drift", full_rhs$term)]

  # Extract unique (primary) terms from formula (no interactions)
  formula_rhs_terms <- formula_rhs[!grepl(":", formula_rhs)]

  # Similar to extract_rhs.lm.
  ## Split interactions split into character vectors
  full_rhs$split <- strsplit(full_rhs$term, ":")

  ## Parse primary coefficients.
  full_rhs$primary <- extract_primary_term(
    formula_rhs_terms,
    full_rhs$term
  )

  full_rhs$primary[full_rhs$term == "drift"] <- "t"

  ## Build up the subscripts (on the primary)
  ## All non-constant coefficients will need a "t" subscript to represent time.
  full_rhs$subscripts <- extract_all_subscripts(
    full_rhs$primary,
    full_rhs$split
  )

  full_rhs$subscripts[full_rhs$term %in% c("intercept", "drift")] <- ""

  rhs_t_subscript <- rep("t", nrow(full_rhs))
  rhs_t_subscript[grepl("^intercept|^drift", full_rhs$term)] <- ""

  full_rhs$subscripts <- paste(full_rhs$subscripts, rhs_t_subscript, sep = ",")
  full_rhs$subscripts <- gsub("^,", "", full_rhs$subscripts)

  ## If the coefficient is drift then we need to add a y0 term
  if ("drift" %in% full_rhs$term) {
    # Then there is a drift term
    # Draw up y0 row. It will need to end up at the top of the dataframe.
    y0_df <- data.frame(
      term = "y0",
      estimate = NA,
      std.error = NA,
      split = "y",
      primary = "y",
      subscripts = "0",
      stringsAsFactors = FALSE
    )

    # rbind to add y0 to the top
    full_rhs <- rbind(y0_df, full_rhs)
  }

  # Set subscripts so that create_term works later
  full_rhs$superscripts <- ""

  # Set the class
  class(full_rhs) <- c(class(model), "data.frame")

  # Explicit return
  return(full_rhs)
}

#' Simple simulated time series data
#'
#' Output from \code{set.seed(42); simple_ts <- ts(rnorm(1000),freq = 4)}.
#' This is included primarily for unit testing.
#'
#' @format A tibble with 1000 rows and 8 variables:
#' \describe{
#'   \item{Qtr1}{First quarter simulated values.}
#'   \item{Qtr2}{Second quarter simulated values.}
#'   \item{Qtr3}{Third quarter simulated values.}
#'   \item{Qtr4}{Fourth quarter simulated values.}
#' }
"simple_ts"

create_eq <- function(model, lhs, ...) {
  UseMethod("create_eq", model)
}

#' Create the full equation
#'
#' @keywords internal
#'
#' @param lhs A character string of the left-hand side variable extracted with
#'   \code{extract_lhs}
#' @param rhs A data frame of right-hand side variables extracted with
#'   \code{extract_rhs}.
#'
#' @inheritParams extract_eq
#' @noRd

create_eq.default <- function(model, lhs, rhs, ital_vars, use_coefs, coef_digits,
                              fix_signs, intercept, greek, 
                              greek_colors, subscript_colors, 
                              var_colors, var_subscript_colors,
                              raw_tex, index_factors, swap_var_names, 
                              swap_subscript_names, ...) {

   check_dots(...)
  rhs$final_terms <- create_term(rhs, ital_vars, 
                                 swap_var_names, swap_subscript_names, 
                                 var_colors, var_subscript_colors)

  if (use_coefs) {
    rhs$final_terms <- add_coefs(rhs, rhs$final_terms, coef_digits)
  } else if (index_factors) {
    rhs$final_terms <- ifelse(
      grepl("\\\\times", rhs$final_terms),
      paste0("\\left(", rhs$final_terms, "\\right)"),
      rhs$final_terms
    )
    rhs$final_terms <- ifelse(
      rhs$final_terms == "",
      "\\alpha",
      rhs$final_terms
    )
  } else {
    rhs$final_terms <- add_greek(rhs, rhs$final_terms, greek, intercept, 
                                 greek_colors, subscript_colors, 
                                 raw_tex)
  }

  # Add error row or not in lm
  if (!use_coefs) {
    error_row <- rhs[nrow(rhs) + 1, ]
    error_row$term <- "error"
    error_row$final_terms <- "\\epsilon"
    error_row$final_terms <- colorize(
      greek_colors[length(greek_colors)], 
      error_row$final_terms
    )
    rhs <- rbind(rhs, error_row)
  }

  list(lhs = list(lhs), rhs = list(rhs$final_terms))
}

#' @noRd
#' @inheritParams extract_eq
create_eq.glm <- function(model, lhs, rhs, ital_vars, use_coefs, coef_digits,
                          fix_signs, intercept, greek, 
                          greek_colors, subscript_colors, 
                          var_colors, var_subscript_colors,
                          raw_tex, index_factors,
                          swap_var_names, swap_subscript_names, ...) {
  check_dots(...)
  rhs$final_terms <- create_term(rhs, ital_vars, swap_var_names, 
                                 swap_subscript_names,
                                 var_colors, var_subscript_colors)

  if (use_coefs) {
    rhs$final_terms <- add_coefs(rhs, rhs$final_terms, coef_digits)
  } else if (index_factors) {
    rhs$final_terms <- ifelse(
      grepl("\\\\times", rhs$final_terms),
      paste0("\\left(", rhs$final_terms, "\\right)"),
      rhs$final_terms
    )
    rhs$final_terms <- ifelse(
      rhs$final_terms == "",
      "\\alpha",
      rhs$final_terms
    )
  } else {
    rhs$final_terms <- add_greek(rhs, rhs$final_terms, greek, intercept, 
                                 greek_colors, subscript_colors, raw_tex)
  }
  if (!is.null(model$offset)) {
    rhs <- rbind(rhs, c(
      rep(NA, (dim(rhs)[2] - 1)),
      add_tex_ital(tail(names(attr(model$terms, "dataClasses")), 1), ital_vars)
    ))
  }

  list(lhs = list(lhs), rhs = list(rhs$final_terms))
}

#' @noRd
create_eq.polr <- function(model, lhs, rhs, ital_vars, use_coefs, coef_digits,
                           fix_signs, intercept, greek, 
                           greek_colors, subscript_colors, 
                           var_colors, var_subscript_colors,
                           raw_tex, index_factors,
                           swap_var_names, swap_subscript_names, ...) {

  check_dots(...)
  rhs$final_terms <- create_term(rhs, ital_vars, swap_var_names, 
                                 swap_subscript_names,
                                 var_colors, var_subscript_colors)

  if (use_coefs) {
    rhs$final_terms <- add_coefs(rhs, rhs$final_terms, coef_digits)
  } else if (index_factors) {
    rhs$final_terms <- ifelse(
      grepl("\\\\times", rhs$final_terms),
      paste0("\\left(", rhs$final_terms, "\\right)"),
      rhs$final_terms
    )
    rhs$final_terms <- ifelse(
      rhs$final_terms == "",
      "\\alpha",
      rhs$final_terms
    )
  } else {
    rhs$final_terms <- add_greek(rhs, rhs$final_terms, greek, intercept, 
                                 greek_colors, subscript_colors, raw_tex)
  }

  splt <- split(rhs, rhs$coef.type)
  rhs_final <- lapply(splt$scale$final_terms, function(x) {
    c(x, splt$coefficient$final_terms)
  })
  attributes(lhs) <- NULL
  list(lhs = lhs, rhs = rhs_final)
}

#' @noRd
create_eq.clm <- function(model, lhs, rhs, ital_vars, use_coefs, coef_digits,
                          fix_signs, intercept, greek, 
                          greek_colors, subscript_colors, 
                          var_colors, var_subscript_colors,
                          raw_tex, index_factors,
                          swap_var_names, swap_subscript_names, ...) {
  check_dots(...)
  rhs$final_terms <- create_term(rhs, ital_vars, swap_var_names, 
                                 swap_subscript_names,
                                 var_colors, var_subscript_colors)

  if (use_coefs) {
    rhs$final_terms <- add_coefs(rhs, rhs$final_terms, coef_digits)
  } else if (index_factors) {
    rhs$final_terms <- ifelse(
      grepl("\\\\times", rhs$final_terms),
      paste0("\\left(", rhs$final_terms, "\\right)"),
      rhs$final_terms
    )
    rhs$final_terms <- ifelse(
      rhs$final_terms == "",
      "\\alpha",
      rhs$final_terms
    )
  } else {
    rhs$final_terms <- add_greek(rhs, rhs$final_terms, greek, intercept, 
                                 greek_colors, subscript_colors, raw_tex)
  }

  splt <- split(rhs, rhs$coef.type)
  rhs_final <- lapply(splt$intercept$final_terms, function(x) {
    c(x, splt$location$final_terms)
  })

  attributes(lhs) <- NULL
  list(lhs = lhs, rhs = rhs_final)
}

#' Create equations from forecast::Arima object
#'
#' @keywords internal
#'
#' @param lhs A data frame of right-hand side variables extracted with
#'   \code{extract_lhs}
#' @param rhs A data frame of right-hand side variables extracted with
#'   \code{extract_rhs}.
#' @param yt A data frame of regression variables extracted with
#'   \code{helper_arima_extract_lm}.
#'
#' @return A dataframe
#' @noRd
create_eq.forecast_ARIMA <- function(model, lhs, rhs, yt, ital_vars, use_coefs, 
                                     coef_digits, raw_tex, swap_var_names,
                                     swap_subscript_names, ...) {

  # Determine if we are working on Regression w/ Arima Errors
  regression <- helper_arima_is_regression(model)

  # Convert sides into LATEX-like terms
  lhs$final_terms <- create_term(lhs, ital_vars, swap_var_names, 
                                 swap_subscript_names)
  rhs$final_terms <- create_term(rhs, ital_vars, swap_var_names, 
                                 swap_subscript_names)

  # Combine coefs or greek letters with the terms
  if (use_coefs) {
    lhs$final_terms <- add_coefs(lhs, lhs$final_terms, coef_digits, side_sign = -1)
    rhs$final_terms <- add_coefs(rhs, rhs$final_terms, coef_digits, side_sign = 1)
  } else {
    lhs$final_terms <- add_greek(lhs, lhs$final_terms, regression, raw_tex, side_sign = -1)
    rhs$final_terms <- add_greek(rhs, rhs$final_terms, regression, raw_tex, side_sign = 1)
  }

  # Convert the final terms into proper Backshift notation for L/RHS
  ## Setup the parsing functions
  parsing_functions <- list(
    "ar" = function(x) paste("(1", paste0(x, collapse = " "), ")\\"),
    "diffs" = function(x) paste(x, collapse = "\\ "),
    "error" = function(x) x
  )

  parsing_functions["sar"] <- parsing_functions["ar"]
  parsing_functions["ma"] <- parsing_functions["ar"]
  parsing_functions["sma"] <- parsing_functions["ar"]


  ## Parse the LHS
  lhs_intercept <- lhs$final_terms[lhs$term %in% c("intercept", "drift")]

  if (length(lhs_intercept) > 0) {
    # An intercept or drift term exists for the ARIMA section.
    lhs_intercept <- paste0("(y_{t} ", lhs_intercept, ")")
  } else {
    lhs_intercept <- "y_{t}"
  }

  lhs_parse <- list(
    "ar" = lhs$final_terms[grepl("^ar", lhs$term)],
    "sar" = lhs$final_terms[grepl("^sar", lhs$term)],
    "diffs" = lhs$final_terms[lhs$term %in% c("zz_differencing", "zz_seas_Differencing")],
    "error" = if (regression) "\\eta_{t}" else lhs_intercept
  )

  lhs_parse <- lhs_parse[lengths(lhs_parse) > 0L]

  lhs_final <- Map(function(x, y) parsing_functions[x][[1]](y), names(lhs_parse), lhs_parse)

  ## Parse the RHS
  rhs_parse <- list(
    "ma" = rhs$final_terms[grepl("^ma", rhs$term)],
    "sma" = rhs$final_terms[grepl("^sma", rhs$term)],
    "diffs" = rhs$final_terms[rhs$term == "zz_seas_Differencing"],
    "error" = "\\varepsilon_{t}"
  )

  rhs_parse <- rhs_parse[lengths(rhs_parse) > 0L]

  rhs_final <- Map(function(x, y) parsing_functions[x][[1]](y), names(rhs_parse), rhs_parse)

  ## Output the ARIMA equation segments.
  arima_eq <- list(
    lhs = list(unlist(lhs_final)),
    rhs = list(unlist(rhs_final))
  )

  # If this is a linear model, also parse yt
  # Note that this is the same set of operations as earlier, but only with yt
  if (regression) {
    # Convert sides into LATEX-like terms
    yt$final_terms <- create_term(yt, ital_vars, swap_var_names, 
                                  swap_subscript_names)

    # Combine coefs or greek letters with the terms
    if (use_coefs) {
      yt$final_terms <- add_coefs(yt, yt$final_terms, coef_digits, side_sign = 1)
    } else {
      yt$final_terms <- add_greek(yt, yt$final_terms, regression, raw_tex, side_sign = 1)
    }

    # Output the LM equation segments
    lm_eq <- list(
      lhs = list("y_{t}"),
      rhs = list(c(yt$final_terms, "+\\eta_{t}"))
    )

    # Prep for final output
    eq_list <- list(
      lm_eq = lm_eq,
      arima_eq = arima_eq
    )
  } else {
    # No regression model
    # Prep for final output
    eq_list <- list(arima_eq = arima_eq)
  }

  # Explicit return
  return(eq_list)
}

create_term <- function(side, ...) {
  UseMethod("create_term", side)
}

#' @noRd
create_term.default <- function(side, ital_vars, swap_var_names, 
                                swap_subscript_names, var_colors,
                                var_subscript_colors, ...) {
  check_dots(...)
  side$primary <- lapply(side$primary, function(x) {
    names(x) <- x
    x
  })
  
  subscript_nms <- Map(function(.x, .y) {
    names(.x) <- .y
    .x
    }, 
    .x = side$subscripts,
    .y = side$primary
  )
  
  if (!is.null(swap_var_names)) {
    side$primary <- swap_names(swap_var_names, side$primary)
  }
  if (!is.null(swap_subscript_names)) {
    side$subscripts <- swap_names(swap_subscript_names, side$subscripts)
    side$subscripts <- Map(function(.x, .y) {
      names(.x) <- names(.y)
      .x
    }, 
    .x = side$subscripts,
    .y = subscript_nms
    )
  }
  
  prim_escaped <- lapply(side$primary, function(x) {
    vapply(x, escape_tex, FUN.VALUE = character(1))
  })
  prim_escaped <- add_math(prim_escaped, side$subscripts)
  prim <- lapply(prim_escaped, add_tex_ital_v, ital_vars)
  
  if (!is.null(var_colors)) {
    prim <- colorize_terms(var_colors, side$primary, prim)
  }
  
  drop_subscripts <- vapply(side$primary, 
                            function(x) any(grepl("poly|<|>", x)), 
                            FUN.VALUE = logical(1))
  
  subs <- ifelse(drop_subscripts, "", side$subscripts)
  
  subs_escaped <- lapply(subs, function(x) {
    vapply(x, escape_tex, FUN.VALUE = character(1))
  })
  subs <- lapply(subs_escaped, add_tex_ital_v, ital_vars)
  subs <- lapply(subs, add_tex_subscripts_v)
  
  if (!is.null(var_subscript_colors)) {
    subs <- Map(function(.x, .y) {
      names(.x) <- names(.y)
      .x
    }, 
    .x = subs,
    .y = subscript_nms
    )
    
    subs <- colorize_terms(var_subscript_colors, side$primary, subs)
  }

  final <- Map(paste0, prim, subs)

  vapply(final, add_tex_mult, FUN.VALUE = character(1))
}

# swap out log, exp
add_math <- function(primary, subscripts) {
  Map(check_math, primary, subscripts)
}

check_math <- function(primary, subscripts) {
  checks <- c("log", "exp", "poly\\((.+),.+", "I\\((.+)\\)")
  replacements <- c("\\\\log", "\\\\exp", paste0("\\1^", subscripts), "\\1")
  for(i in seq_along(checks)) {
    primary <- gsub(checks[i], replacements[i], primary)  
  }
  gsub("\\^1$", "", primary)
}

#' Create a full term w/subscripts
#'
#' @keywords internal
#'
#' @param side A data frame of right-hand side variables extracted with
#'   \code{extract_rhs} or \code{extract_rhs} or \code{helper_arima_extract_lm}.
#'
#' @inheritParams extract_eq
#' @noRd
create_term.forecast_ARIMA <- function(side, ital_vars, swap_var_names,
                                       swap_subscript_names, ...) {
  check_dots(...)
  if (!is.null(swap_var_names)) {
    side$primary <- swap_names(swap_var_names, side$primary)
  }
  if (!is.null(swap_subscript_names)) {
    side$subscripts <- swap_names(swap_subscript_names, side$subscripts)
  }
  # Get and format the primaries
  # Do not escape seasonal differecing primary
  prim_escaped <- lapply(side$primary, function(x) {
    vapply(x, escape_tex, FUN.VALUE = character(1))
  })
  term_check <- side$term %in% c("zz_differencing", "zz_seas_Differencing")
  prim_escaped[term_check] <- side[term_check, "primary"]

  prim <- lapply(prim_escaped, add_tex_ital_v, ital_vars)

  if (!ital_vars) {
    # We don"t want (1-B) inside \\operatorname
    prim[term_check] <- gsub("B", "\\\\operatorname{B}", side$primary[term_check])
  }

  # Get and format the subscripts
  subs_escaped <- lapply(side$subscripts, function(x) {
    vapply(x, escape_tex, FUN.VALUE = character(1))
  })

  subs <- lapply(subs_escaped, add_tex_ital_v, ital_vars)
  subs <- lapply(subs, add_tex_subscripts_v)

  # Get and format the superscripts
  supers_escaped <- lapply(side$superscript, function(x) {
    vapply(x, escape_tex, FUN.VALUE = character(1))
  })

  supers <- lapply(supers_escaped, add_tex_ital_v, ital_vars)
  supers <- lapply(supers, add_tex_superscripts_v)

  # Combine operators
  final <- Map(paste0, prim, subs, supers)

  # Add any multiplication for interaction terms
  final <- vapply(final, add_tex_mult, FUN.VALUE = character(1))

  # Explicit return
  return(final)
}

#' Swap out variable names for more human readable forms
#' @param name_vector A vector of the form c("old_var_name" = "new name"). For
#' example: c("bill_length_mm" = "Bill Length (MM)")
#' @param primary The primary column from \code{rhs}
#' @noRd
swap_names <- function(name_vector, primary) {
  missing <- setdiff(unique(unlist(primary)), names(name_vector))
  names(missing) <- missing
  full_name_vector <- c(name_vector, missing)
  
  lapply(primary, function(x) {
    full_name_vector[match(x, names(full_name_vector))]
  })
}

#' Escape TeX
#'
#' Escape special TeX characters.
#'
#' Ten characters have special meaning in TeX \code{& \% $ # _ { } ~ ^ \\}.
#' This function either escapes them with \\, or in the case of the last three,
#' replaces them with special TeX macros.
#'
#' @keywords internal
#'
#' @param term A character string to escape
#'
#' @return A character string
#' @noRd

escape_tex <- function(term) {
  unescaped <- c(" ", "&", "%", "$", "#", "_", "{", "}", "~", "^", "\\")
  
  hat <- ifelse(is_html_output(), "\\texttt{^}", "\\texttt{\\^{}}")
  
  escaped <- c(
    "\\ ", "\\&", "\\%", "\\$", "\\#", "\\_", "\\{", "\\}",
    "\\char`\\~", hat, "\\backslash "
  )

  # Split term into a vector of single characters
  characters <- strsplit(term, "")[[1]]

  # Go through term and replace all unescaped characters with their escaped versions
  replaced <- vapply(characters,
    function(x) {
      ifelse(x %in% unescaped,
        escaped[which(x == unescaped)],
        x
      )
    },
    FUN.VALUE = character(1)
  )

  # Return the reassembled term
  paste0(replaced, collapse = "")
}


#' Wrap text in \code{\\operatorname{}}
#'
#' Add tex code to make string not italicized within an equation
#'
#' @keywords internal
#'
#' @param term A character to wrap in \code{\\operatorname{}}
#' @param ital_vars Passed from \code{extract_eq}
#'
#' @return A character string
#' @noRd

add_tex_ital <- function(term, ital_vars) {
  if (any(nchar(term) == 0, ital_vars)) {
    return(term)
  }
  paste0("\\operatorname{", term, "}")
}


#' Wrap text in \code{\\operatorname{}} (vectorized)
#'
#' Add tex code to make string not italicized within an equation for a vector
#' of strings
#'
#' @keywords internal
#'
#' @return A vector of characters
#' @noRd

add_tex_ital_v <- function(term_v, ital_vars) {
  vapply(term_v, add_tex_ital, ital_vars, FUN.VALUE = character(1))
}


#' Wrap text in \code{_{}}
#'
#' Add tex code to make subscripts for a single string
#'
#' @keywords internal
#'
#' @param term A character string to TeXify
#'
#' @return A character string
#' @noRd

add_tex_subscripts <- function(term) {
  if (any(nchar(term) == 0)) {
    return(term)
  }
  paste0("_{", term, "}")
}


#' Wrap text in \code{_{}}
#'
#' Add tex code to make subscripts for a vector of strings
#'
#' @keywords internal
#'
#' @return A vector of characters
#' @noRd

add_tex_subscripts_v <- function(term_v) {
  vapply(term_v, add_tex_subscripts, FUN.VALUE = character(1))
}

#' Wrap text in \code{_{}}
#'
#' Add tex code to make subscripts for a single string
#'
#' @keywords internal
#'
#' @param term A character string to TeXify
#'
#' @return A character string
#' @noRd
add_tex_superscripts <- function(term) {
  if (any(nchar(term) == 0)) {
    return(term)
  }
  paste0("^{", term, "}")
}


#' Wrap text in \code{_{}}
#'
#' Add tex code to make subscripts for a vector of strings
#'
#' @keywords internal
#'
#' @return A vector of characters
#' @noRd
add_tex_superscripts_v <- function(term_v) {
  vapply(term_v, add_tex_superscripts, FUN.VALUE = character(1))
}

#' Add multiplication symbol for interaction terms
#'
#' @keywords internal
#' @noRd

add_tex_mult <- function(term) {
  paste(term, collapse = " \\times ")
}

#' Add a hat sign to the response variable in lm
#'
#' @keywords internal
#' @noRd

add_hat <- function(term) {
  paste0("\\widehat{", term, "}")
}



add_coefs <- function(rhs, ...) {
  UseMethod("add_coefs", rhs)
}

#' Add coefficient values to the equation
#'
#' @keywords internal
#' @noRd

add_coefs.default <- function(rhs, term, coef_digits, ...) {
  check_dots(...)
  ests <- round(rhs$estimate, coef_digits)
  ifelse(
    rhs$term == "(Intercept)",
    paste0(ests, term),
    paste0(ests, "(", term, ")")
  )
}

#' @keywords internal
#' @noRd

add_coefs.polr <- function(rhs, term, coef_digits, ...) {
  check_dots(...)
  ests <- round(rhs$estimate, coef_digits)
  ifelse(
    rhs$coef.type == "scale",
    paste0(ests, term),
    paste0(ests, "(", term, ")")
  )
}

#' @keywords internal
#' @noRd

add_coefs.clm <- function(rhs, term, coef_digits, ...) {
  check_dots(...)
  ests <- round(rhs$estimate, coef_digits)
  ifelse(
    rhs$coef.type == "intercept",
    paste0(ests, term),
    paste0(ests, "(", term, ")")
  )
}

#' Add coefficient values for forecast::Arima
#'
#' @keywords internal
#' @noRd
add_coefs.forecast_ARIMA <- function(rhs, term, coef_digits, side_sign = 1, ...) {
  check_dots(...)
  # Round the estimates and turn to a character vector
  ests <- round(rhs$estimate, coef_digits)

  # Use signif on anything where the round returns a zero
  ests[ests == 0 & !is.na(ests)] <- signif(rhs$estimate[ests == 0 & !is.na(ests)], 1)

  # Deal with signs
  ## Flip sign if needed
  ests <- ests * side_sign

  ## Setup for positive signs. Negative signs come automatically.
  signs <- ifelse(ests > 0, "+", "")

  # Deal with NAs
  signs[is.na(ests)] <- ""
  ests[is.na(ests)] <- ""

  # Combine final terms and the coeficients
  final_terms <- paste0(signs, ests, term)

  # Explicit return
  return(final_terms)
}

add_greek <- function(rhs, ...) {
  UseMethod("add_greek", rhs)
}

#' Adds greek symbols to the equation
#'
#' @keywords internal
#' @noRd

add_greek.default <- function(rhs, terms, greek = "beta", intercept = "alpha",
                              greek_colors, subscript_colors,
                              raw_tex = FALSE, ...) {
  check_dots(...)
  int <- switch(intercept,
    "alpha" = "\\alpha",
    "beta" = "\\beta_{0}"
  )
  if (raw_tex & !(intercept %in% c("alpha", "beta"))) {
    int <- intercept
  }
  
  indices <- if ("(Intercept)" %in% rhs$term) {
    seq_len(nrow(rhs)) - 1
  } else {
    seq_len(nrow(rhs))
  } 
  
  ifelse(rhs$term == "(Intercept)",
    colorize(greek_colors[1], int),
    anno_greek(
      greek, 
      indices, 
      terms, 
      greek_colors, 
      subscript_colors, 
      raw_tex
    )
  )
}

#' @keywords internal
#' @noRd

add_greek.polr <- function(rhs, terms, greek, intercept, greek_colors, 
                           subscript_colors, raw_tex, ...) {
  rhs$idx <- unlist(lapply(split(rhs, rhs$coef.type), function(x) {
    seq_along(x$coef.type)
  }))

  ifelse(
    rhs$coef.type == "scale",
    anno_greek(
      "alpha", 
      rhs$idx, 
      greek_colors = greek_colors[1], 
      subscript_colors = subscript_colors[1]
    ),
    anno_greek(
      "beta", 
      rhs$idx, 
      terms,
      greek_colors = greek_colors[-1],
      subscript_colors = subscript_colors[-1]
    )
  )
}

#' @keywords internal
#' @noRd

add_greek.clm <- function(rhs, terms, greek, intercept, greek_colors, 
                          subscript_colors, raw_tex, ...) {
  rhs$idx <- unlist(lapply(split(rhs, rhs$coef.type), function(x) {
    seq_along(x$coef.type)
  }))

  ifelse(
    rhs$coef.type == "intercept",
    anno_greek(
      "alpha", 
      rhs$idx, 
      greek_colors = greek_colors[1], 
      subscript_colors = subscript_colors[1]
    ),
    anno_greek(
      "beta", 
      rhs$idx, 
      terms,
      greek_colors = greek_colors[-1],
      subscript_colors = subscript_colors[-1]
    )
  )
}

#' Add greek placeholders for forecast::Arima
#'
#' @keywords internal
#' @noRd
add_greek.forecast_ARIMA <- function(rhs, terms, regression, raw_tex = FALSE, side_sign = 1, ...) {
  # These are the greek letters we need to use in REGEX
  # Others will be assigned manually
  greek_letters <- c(
    "^ar" = "phi",
    "^sar" = "Phi",
    "^ma" = "theta",
    "^sma" = "Theta",
    "^intercept$" = "mu",
    "^drift$" = "delta"
  )

  # We are going to use lapply to walk and change things for us.
  # Note the <<-. This affects something outside the function.
  # This is just a fancy for loop.
  greek <- rep("", nrow(rhs))
  invisible(
    lapply(names(greek_letters), function(x) {
      greek[grepl(x, rhs$term)] <<- greek_letters[x]
    })
  )


  # To make life easier, we are including the greek parsing for
  # the linear component here too.
  if (sum(grepl("^s?ar|^s?ma", rhs$term)) == 0) {
    # Then we are dealing with the linear component and not the ARIMA component

    # Beta will serve as the main letter for the linear component
    # Anything in greek that is blank, wasn"t accounted for with known paramters.
    greek[greek == "" & rhs$term != "y0"] <- "beta"

    # Generate the subs for the greek letters
    ## Initialize the vector
    subs <- rep("", nrow(rhs))

    ## Give numbers to non-constants
    int_drift_y0 <- !(rhs$term %in% c("intercept", "drift", "y0"))
    subs[int_drift_y0] <- as.character(1:sum(int_drift_y0))
  } else {
    # Deal with ARIMA component

    # Format the greek letters to vibe with the final_terms
    # This is done, in part, with anno_greek for other functions.
    subs <- rep("", nrow(rhs))
    subs[grepl("^s?ar|^s?ma", rhs$term)] <- gsub("^s?ar|^s?ma", "", rhs$term[grepl("^s?ar|^s?ma", rhs$term)])
  }

  # Add subscripts to greek
  greek[subs != ""] <- paste0(greek[subs != ""], "_{", subs[subs != ""], "}")

  # Finalize greek based on raw_tex
  if (!raw_tex) {
    greek[greek != ""] <- paste0("\\", greek[greek != ""])
  }

  # Deal with signs on greek letters
  signs <- rep("", length(greek))
  signs[!(rhs$term %in% c("zz_differencing", "zz_seas_Differencing"))] <- if (side_sign > 0) "+" else "-"

  # Combine the final terms with the greek letters
  final_terms <- paste0(signs, greek, terms)

  # Explicit return
  return(final_terms)
}

#' Intermediary function to wrap text in `\\beta_{}`
#'
#' @keywords internal
#' @noRd
anno_greek <- function(greek, nums, terms = NULL, 
                       greek_colors = NULL, subscript_colors = NULL, 
                       raw_tex = FALSE) {
  
  if (raw_tex) {
    out <- paste0(greek, "_{", nums, "}")
  } else {
    coefs <- colorize(greek_colors, paste0("\\", greek))
    subscripts <- colorize(subscript_colors, nums)
    out <- paste0(coefs, "_{", subscripts, "}")
  }
    
  if (!is.null(terms)) {
    out <- paste0(out, "(", terms, ")")
  }
  out
}


#' Deduplicate operators
#'
#' Convert "+ -" to "-"
#'
#' @keywords internal
#'
#' @param eq String containing a LaTeX equation
#' @noRd

fix_coef_signs <- function(eq) {
  # Side-by-side + -
  eq_clean <- gsub("\\+ -", "- ", eq)

  # + - that spans lines
  eq_clean <- gsub(
    "\\+ \\\\\\\\\\n&\\\\quad -",
    "- \\\\\\\\\n&\\\\quad ",
    eq_clean
  )

  eq_clean
}

#' A subset of the full 1982 High School and Beyond Survey
#'
#' This is the dataset used throughout Raudenbush & Bryk (2002).
#'
#' @format A tibble with 7185 rows and 8 variables:
#' \describe{
#'   \item{sch.id}{An integer denoting the school identification number. 
#'     There are 160 unique schools.}
#'   \item{math}{Individual students' math score.}
#'   \item{size}{The number of students in the school.}
#'   \item{sector}{A dummy variable (integer) denoting whether the school is 
#'     public (sector = 0) or catholic (sector = 1). There are 90 public schools 
#'     and 70 catholic.}
#'   \item{meanses}{A group-mean centered SES variable at the school level}
#'   \item{minority}{A dummy variable indicating if the student was coded as 
#'     white (minority = 0) or not (minority = 1).}
#'   \item{female}{A dummy variable indicating if the student was coded as 
#'     female (female = 1) or not (female = 0).}
#'   \item{ses}{A student-level composite variable indicating the students' 
#'     socio-economic status.}
#' }
"hsb"

---
title: "Tidymodels with {equatiomatic}"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tidymodels with {equatiomatic}}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
library(equatiomatic)
library(recipes)
library(parsnip)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

To paraphrase its web site, [Tidymodels](https://www.tidymodels.org/) provides a series of packages for modeling and machine learning using the [tidyverse](https://tidyverse.org/) principles. {equatiomatic} is now partly compatible with it, meaning that it can extract the equation of certain models.

Here is an example (adapted from the {workflows} main page):

```{r cars}
# Preparation of the dataset using {recipes}
spline_cars <- recipe(mpg ~ ., data = mtcars) |>
  step_ns(disp, deg_free = 10)
spline_cars_prepped <- prep(spline_cars, mtcars)
```

Here is a simple (tidy)model:

```{r lm}
# Fitting of a least-square linear model
lm_fit <- linear_reg() |>
  fit(mpg ~ ., data = juice(spline_cars_prepped))
```

We can extract the equation of this model with `extract_eq()`:

```{r lm_equation}
extract_eq(lm_fit, wrap = TRUE)
```

## Working with workflows

The {equatiomatic} `extract_eq()` also works with models fitted using the {workflows} package.

```{r wflow}
library(workflows)

# A model compatible with {equatiomatic}
linear_lm <- linear_reg()

# A workflow object
car_wflow <- workflow() |>
  add_recipe(spline_cars) |>
  add_model(linear_lm)
```

Now you can prepare the recipe and estimate the model via a single call to `fit()`:

```{r wflow_fit}
wflow_fit <- fit(car_wflow, data = mtcars)
```

You can also extract the equation from `wflow_fit`:

```{r wflow_equation}
extract_eq(wflow_fit, wrap = TRUE)
```

You notice that the original name of the dependent variable is lost, but you can reset it manually using `swapt_var_names=`:

```{r wflow_equation2}
extract_eq(wflow_fit, wrap = TRUE, swap_var_names = c(..y = "mpg"))
```

## Models requiring {broom.mixed}

For some models, {broom} is not enough. You need also to `library(broom.mixed)` before you can extract the equation. This is the case of a Bayes linear model using `"stan"`. Note: this code is not run in the vignette to avoid heavy extra-dependencies, but you can run this code in your R process.

```{r bayes_fit, eval=FALSE}
library(broom.mixed) # Required for some models, or extract_eq() will choke!

bayes_fit <- linear_reg() |>
  set_engine("stan") |>
  fit(mpg ~ hp + drat, data = mtcars)
```

And the equation would be obtained with:

```{r bayes_equation, eval=FALSE}
extract_eq(bayes_fit)
```

```{r bayes_equation2, echo=FALSE}
equation("E( \\operatorname{mpg} ) = \\alpha + \\beta_{1}(\\operatorname{hp}) + \\beta_{2}(\\operatorname{drat})")
```

---
title: "Tests and Coverage"
date: "`r format(Sys.time(), '%d %B, %Y %H:%M:%S')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{covrpage: Tests and Coverage}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
options(rmarkdown.html_vignette.check_title = FALSE)
```

<!--- Placeholder for tests/README.md Do Not Edit--->
  - [Coverage](#coverage)
  - [Unit Tests](#unit-tests)

This output is created by
[covrpage](https://github.com/yonicd/covrpage).

## Coverage

Coverage summary is created using the
[covr](https://github.com/r-lib/covr) package.

| Object                                 | Coverage (%) |
| :------------------------------------- | :----------: |
| equatiomatic                           |    99.66     |
| [R/extract\_lhs.R](https://github.com/datalorax/equatiomatic/blob/master/R/extract_lhs.R) |    98.70     |
| [R/create\_eq.R](https://github.com/datalorax/equatiomatic/blob/master/R/create_eq.R)     |    100.00    |
| [R/extract\_eq.R](https://github.com/datalorax/equatiomatic/blob/master/R/extract_eq.R)   |    100.00    |
| [R/extract\_rhs.R](https://github.com/datalorax/equatiomatic/blob/master/R/extract_rhs.R) |    100.00    |
| [R/print.R](https://github.com/datalorax/equatiomatic/blob/master/R/print.R)              |    100.00    |
| [R/utils.R](https://github.com/datalorax/equatiomatic/blob/master/R/utils.R)              |    100.00    |

<br>

## Unit Tests

Unit Test summary is created using the
[testthat](https://github.com/r-lib/testthat) package.

| file                                                              |  n |  time | error | failed | skipped | warning |
| :---------------------------------------------------------------- | -: | ----: | ----: | -----: | ------: | ------: |
| [test-clm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-clm.R)                                 |  6 | 0.051 |     0 |      0 |       0 |       0 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R)                                 |  8 | 0.057 |     0 |      0 |       0 |       0 |
| [test-lm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-lm.R)                                   |  7 | 0.037 |     0 |      0 |       0 |       0 |
| [test-polr.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-polr.R)                               |  5 | 0.052 |     0 |      0 |       0 |       0 |
| [test-print.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-print.R)                             |  2 | 0.005 |     0 |      0 |       0 |       0 |
| [test-utils.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-utils.R)                             |  8 | 0.028 |     0 |      0 |       0 |       0 |
| [test-wrapping-formatting.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-wrapping-formatting.R) | 10 | 0.040 |     0 |      0 |       0 |       0 |

<details closed>

<summary> Show Detailed Test Results </summary>

| file                                                                      | context                 | test                              | status | n |  time |
| :------------------------------------------------------------------------ | :---------------------- | :-------------------------------- | :----- | -: | ----: |
| [test-clm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-clm.R#L46_L47)                                 | CLMs                    | Ordered models with clm work      | PASS   | 5 | 0.039 |
| [test-clm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-clm.R#L79)                                     | CLMs                    | Unsupported CLMs create a message | PASS   | 1 | 0.012 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R#L16_L17)                                 | GLMs                    | Logistic regression works         | PASS   | 1 | 0.009 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R#L33_L34)                                 | GLMs                    | Probit regression works           | PASS   | 2 | 0.013 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R#L49)                                     | GLMs                    | Unsupported GLMs create a message | PASS   | 1 | 0.007 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R#L78_L79)                                 | GLMs                    | Distribution-based equations work | PASS   | 3 | 0.019 |
| [test-glm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-glm.R#L107)                                    | GLMs                    | Weights work                      | PASS   | 1 | 0.009 |
| [test-lm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-lm.R#L11_L12)                                   | Linear models           | Simple lm models work             | PASS   | 3 | 0.016 |
| [test-lm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-lm.R#L32_L33)                                   | Linear models           | Interactions work                 | PASS   | 2 | 0.011 |
| [test-lm.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-lm.R#L48_L49)                                   | Linear models           | Custom Greek works                | PASS   | 2 | 0.010 |
| [test-polr.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-polr.R#L44_L45)                               | polr                    | Ordered logistic regression works | PASS   | 5 | 0.052 |
| [test-print.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-print.R#L11_L12)                             | Printing                | Equation is printed correctly     | PASS   | 2 | 0.005 |
| [test-utils.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-utils.R#L9_L11)                              | Utility functions       | Strict mapply\_\* functions work  | PASS   | 8 | 0.028 |
| [test-wrapping-formatting.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-wrapping-formatting.R#L8_L9)   | Wrapping and formatting | Coefficient digits work correctly | PASS   | 2 | 0.010 |
| [test-wrapping-formatting.R](https://github.com/datalorax/equatiomatic/blob/master/tests/testthat/test-wrapping-formatting.R#L26_L27) | Wrapping and formatting | Wrapping works correctly          | PASS   | 8 | 0.030 |

</details>

<details>

<summary> Session Info </summary>

| Field    | Value                             |
| :------- | :-------------------------------- |
| Version  | R version 4.0.2 (2020-06-22)      |
| Platform | x86\_64-apple-darwin17.0 (64-bit) |
| Running  | macOS Catalina 10.15.6            |
| Language | en\_US                            |
| Timezone | America/Los\_Angeles              |

| Package  | Version |
| :------- | :------ |
| testthat | 2.3.2   |
| covr     | 3.5.0   |
| covrpage | 0.0.71  |

</details>

<!--- Final Status : pass --->

---
title: "Introduction to {equatiomatic}"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to {equatiomatic}}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
library(equatiomatic)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Background and Motivation

If you use R to statistically analyze your data, you might be used to seeing and 
interpreting the output from functions for models, like `lm()` and `glm()`. For example,
here is the code and output for a single regression model, fit using the `lm()`
function. We'll examine how the depth of penguin's bills relates to their bill
length using data from the [{palmerpenguins}](https://github.com/allisonhorst/palmerpenguins)
package:

```{r, warning = FALSE, results = 'markup'}
lm(bill_length_mm ~ bill_depth_mm, penguins)
```

At the same time, you might have come across---or written!---equations that
appear in books, journal articles, and reports. Equations that look like this:

```{r, echo = FALSE}
library(equatiomatic)
extract_eq(lm(bill_length_mm ~ bill_depth_mm, penguins))
```

The purpose of  [{equatiomatic}](https://datalorax.github.io/equatiomatic/) is to help you go from model output, like the `lm()` output above,
to the equation we just saw. 

As we detail in this vignette, {equatiomatic} provides the underlying equation
corresponding to the statistical model output. It does this through the use of
the [TeX](https://en.wikipedia.org/wiki/TeX) equation formatting system, which
can then be included in documents written in a number of formats, including 
(importantly) R Markdown documents rendered to either HTML or PDF formats.

# Example Usage

## A basic example for a multiple linear regression

Let's look at another basic example, again using the {palmerpenguins} data:

```{r fit-m1}
library(equatiomatic)

# fit a basic multiple linear regression model
m <- lm(bill_length_mm ~ bill_depth_mm + flipper_length_mm, penguins)
```

Now we can pull the TeX code with `extract_eq`

```{r extract_eq1}
extract_eq(m)
```

You can of course set `echo = FALSE` as well, and then you'll get just the equation, which will look like the below.

```{r extract_eq1-no-echo, echo = FALSE}
extract_eq(m)
```

Let's dive into a bit more complex example.

## An example with a slightly more complicated model

The above was super simple, but---often---you have categorical variables with lots of
levels and interactions, which can make the output of your models (and writing a 
model equation for your models) a bit more complicated. {equatiomatic} is 
intended to "smooth out" some of these issues, requiring the same process as in 
the above example to extract (and present) the model's equation:

For example, here is an example using a categorical variable (`island`) in an
interaction with the `bill_depth_mm` variable we used in the example above:

```{r eq2}
m2 <- lm(bill_length_mm ~ bill_depth_mm * island, penguins)
extract_eq(m2)
```

Sometimes, for such models, the equations can get overly long. 

That's where the `wrap` and `terms_per_line` arguments come in.

```{r eq2-wrap}
extract_eq(m2, wrap = TRUE) # default terms_per_line = 4
extract_eq(m2, wrap = TRUE, terms_per_line = 2)
```

Maybe you want different intercept notation, such as $\beta_0$? Simply pass
"beta" to the `intercept` argument, as follows. 

```{r eq2-intercept-beta}
extract_eq(m2, wrap = TRUE, intercept = "beta")
```

We also wrap all the variable names in `\operatorname` by default so they show
up as plain text, but if you'd like your variable names to be italicized just
set `ital_vars = TRUE`.

```{r eq2-ital-vars}
extract_eq(m2, wrap = TRUE, ital_vars = TRUE)
```

## Raw TeX code

Currently, the `intercept` argument defaults to `"alpha"` and only takes one
additional argument, `"beta"`. However, the `raw_tex` and `greek` arguments
allows you to specify whatever syntax you would like both for the intercept and for the coefficients. For example

```{r raw-tex}
extract_eq(m2,
  wrap = TRUE,
  intercept = "\\hat{\\phi}",
  greek = "\\hat{\\gamma}",
  raw_tex = TRUE
)
```

## Coefficients
In many cases, you're interested more in including the coefficient estimates
(e.g., `3.04`), than the Greek symbols (e.g., $\\beta\_1$). This may be
helpful for communicating the results of a model (and, possibly, for teaching 
about the statistical model). To do this, simply change the `use_coefs` argument:

```{r use_coefs}
extract_eq(m2, wrap = TRUE, use_coefs = TRUE)
```

# Other models

While the above examples focused on regression models (and the `lm()` function), 
{equatiomatic} supports output from other model types as well, specifically:

```{r echo = FALSE}
supported <- data.frame(
  model = c(
    "linear regression",
    "logistic regression",
    "probit regression",
    "ordinal logistic regression",
    "ordinal probit regression",
    "auto-regressive integrated moving average",
    "regression with auto-regressive integrated moving average errors"
  ),
  packages = c(
    "`stats::lm`",
    "`stats::glm(family = binomial(link = 'logit'))`",
    "`stats::glm(family = binomial(link = 'probit'))`",
    "`MASS::polr(method = 'logistic')`; `ordinal::clm(link = 'logit')`",
    "`MASS::polr(method = 'probit')`; `ordinal::clm(link = 'probit')`",
    "`forecast::Arima`",
    "`forecast::Arima`"
  )
)

knitr::kable(supported, col.names = c("Model", "Packages/Functions"))
```

Here are a few basic examples using these supported model types:

## Logistic Regression
```{r log-reg1}
lr <- glm(sex ~ species * bill_length_mm,
  data = penguins,
  family = binomial(link = "logit")
)

extract_eq(lr, wrap = TRUE)
```

You can also (optionally) show the how the data are assumed to be distributed.

```{r log-reg2}
extract_eq(lr, wrap = TRUE, show_distribution = TRUE)
```

## Probit Regression

Probit regression works similarly to logistic regression:
```{r prob-reg1}
pr <- glm(sex ~ species * bill_length_mm,
  data = penguins,
  family = binomial(link = "probit")
)

extract_eq(pr, wrap = TRUE)
```

Again, you can (optionally) show the how the data are assumed distributed.

```{r prob-reg2}
extract_eq(pr, wrap = TRUE, show_distribution = TRUE)
```

## Ordinal Regression
For these examples, we'll use wine tasting data from the [{ordinal}](https://github.com/runehaubo/ordinal) package. If you don't already have this package, you can download it with

```{r install-ordinal, eval = FALSE}
install.packages("ordinal")
```

The data look like this

```{r wine-data, echo = FALSE}
knitr::kable(head(ordinal::wine), align = "l")
```

We'll fit a model from the documentation, with the ordinal rating response predicted by an interaction between the temperature and and contact. 

You can fit ordinal response models with either `MASS::polr` or `ordinal::clm`, and {equatiomatic} works with either, and with logistic or probit link functions.

### {MASS}
#### Logit method

```{r mass-ologit}
mass_ologit <- MASS::polr(rating ~ temp * contact,
  data = ordinal::wine
)

extract_eq(mass_ologit, wrap = TRUE, terms_per_line = 2)
```

#### Probit method

```{r mass-oprobit}
mass_oprobit <- MASS::polr(rating ~ temp * contact,
  data = ordinal::wine,
  method = "probit"
)

extract_eq(mass_oprobit, wrap = TRUE, terms_per_line = 2)
```

### {ordinal} 
And we can do the same thing with the {ordinal} package

#### Logit method
```{r ordinal-ologit}
ordinal_ologit <- ordinal::clm(rating ~ temp * contact,
  data = ordinal::wine,
  link = "logit"
)

extract_eq(ordinal_ologit, wrap = TRUE, terms_per_line = 2)
```

#### Probit method

```{r ordinal-oprobit}
ordinal_probit <- ordinal::clm(rating ~ temp * contact,
  data = ordinal::wine,
  link = "probit"
)

extract_eq(ordinal_probit, wrap = TRUE, terms_per_line = 2)
```

# Things the package does not yet do

We are aware of a few things the package doesn't yet do, but that we hope to add
later. These include:

  - Math functions (e.g., `log`, `exp`, `sqrt`)
  - Polynomial (e.g., `lm(y ~ poly(x, 3))`)
  - A range of other models, including multi-level (or mixed effects) models

Regarding this last point, we are hopeful that we can incorporate essentially
all the models covered by [broom](https://broom.tidymodels.org). Multilevel
models are particularly high on our wish list. But we have not yet had the time
to develop these.

# Contributing

We would LOVE to have you as a contributor! Is there a model that we don't
currently fit that you want implemented? There are a few ways to go about this.
You can either (a) fork the repository and implement the method on your own, then
submit a PR, or (b) file an issue.

If you file an issue it would be *really* helpful if you could provide an
example of a fitted model and what the equation for that model should look like.
We will try to get to these as soon as possible.

Also, the next planned vignette (at this particular moment) is on contributing
to the package, with a step-by-step example of implementing a new method. So
stay tuned for that, if you're interested in (a) but not yet sure how to get
started.

---
title: "Equations in R Markdown and Quarto"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Equations in R Markdown and Quarto}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
library(equatiomatic)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette demonstrates how to include equations in R Markdown and Quarto documents using the {equatiomatic} package and the new functions `equation()`, `eq_()` and `eq__()` in version \>= 0.4.0.

## Display equations from an R chunk

The most natural way to include equations in R Markdown or Quarto documents is to just print the equation generated by `extract_eq()` in an R chunk. The `knit_print()` method of {knitr} will then be used to generate a display equation (a LaTeX equation surrounded by double dollars `$$...$$`). Now, the `equation()` function can now be used in place of `extract_eq()`. It can use `label` and `units` attributes set to variables in the dataset used in the model. This is a convenient way to use friendlier variable names (and possibly units) in the equation.

Here is an example using the (overused) `iris` dataset. We add `label` and `units` attributes to our variables inside the data.frame:

```{r iris}
data(iris)
# Attach `label` and `units` attributes to the variables
# Note, this is done manually, but there are functions in various packages to
# ease the process (see packages {Hmisc}, {labelled}, {LabelR}, {labelVector})
attr(iris$Sepal.Length, "label") <- "Sepal length"
attr(iris$Sepal.Length, "units") <- "cm"
attr(iris$Sepal.Width, "label") <- "Sepal width"
attr(iris$Sepal.Width, "units") <- "cm"
attr(iris$Petal.Length, "label") <- "Petal length"
attr(iris$Petal.Length, "units") <- "cm"
attr(iris$Petal.Width, "label") <- "Petal width"
attr(iris$Petal.Width, "units") <- "cm"

str(iris)
```

Now, let's fit a simple linear model to those "label-augmented" `iris` data:

```{r}
lm1 <- lm(Sepal.Length ~ Sepal.Width, data = iris)
lm1
```

If we use `extract_eq()`, we got the equation using variable names (unless we specify the `swap_var_names=` argument, of course):

```{r extracteq}
extract_eq(lm1)
```

Now, using `equation()` instead, we use the labels and units attributes of the variables in the dataset:

```{r equation-iris}
equation(lm1)
```

In case the model does not retain the original data with their labels, one can use the `origdata=` argument to pass the original data set to the `equation()` function. Also the `auto.labs=` argument can be set to `FALSE` to avoid using the labels and units attributes (it is `TRUE` by default).

## Inline equations in R Markdown or Quarto documents

Now, if you want an inline equation (an equation inside the text), you can easily use an inline R chunk calling the `eq_()` function like this: `` `r eq_(lm1)` ``. This produces `r eq_(lm1)`, thus right inside the text. Note that `eq_()` also handles `label` and `units` attributes, the same way `equation()` does. Moreover, if you issue `eq_(lm1)` in the R console or terminal (at the R prompt), you got a preview of your equation in a temporary HTML document, or in the Viewer pane if you work in RStudio. This is convenient to check the appearance of your equation without having to render the whole document.

## Display equations with labels

The `extract_eq()` function has a `label=` argument that allows you to set a label for the equation, but in LaTeX documents only (thus for PDF but not for HTML final documents). It does not ease cross-referencing of the equation because the label is not known from within Markdown and your Markdown editor (RStudio, VSCode, Positron...). The `eq__()` function allows to insert an equation generated with {equatiomatic} inside a `$$...$$` construct in a Markdown text. The construct can potentially have a label, like `$$...$${#eq-model1}` that can be easily cross-referenced. You just have to place an R chunk with your equation inside the `$$...$$` construct, very similarly to what you did with the inline R chunk. Thus, you would have something like this in your document:

```         
$$
`r eq__(lm1)`
$${#eq-model1}
```

which translates into:

$$
`r eq__(lm1)`
$${#eq-model1}

In documents that support cross-referencing (unfortunately not this vignette document), the equation is numbered at the right and you can reference it in the text with `@eq-model1`. The `eq__()` function is also aware of `label` and `units` attributes and it also allows to preview the equation when its code is issued at the R prompt.

# Customizing equations

All three functions `equation()`, `eq_()` and `eq__()` also support all the arguments of `extract_eq()` (for colors, parameters change, line wrapping, etc.). Here is an example:

```{r}
equation(lm1, var_colors = c(Sepal.Width  = "red"),
  ital_vars = TRUE, use_coefs = TRUE, coef_digits = 3)
```

---
title: "Usage with shiny"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Usage with shiny}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

# TL;DR

See a working application [here](https://djanderson07.shinyapps.io/equatiomatic-shiny/)

# More detail

The equatiomatic package exports `renderEq()` and `eqOutput()` functions to make it's usage with Shiny more straightforward.

Imagine, for example, that we wanted to have an app that showed us the coefficients of a model in a nice table. We could do that with something like:

```{r}
library(equatiomatic)
library(shiny)
library(shinyWidgets)
library(gtsummary)
library(gt)

data("penguins", package = "equatiomatic")

ui <- fluidPage(
  titlePanel("equatiomatic w/Shiny"),
  sidebarLayout(
    sidebarPanel(
      multiInput(
        inputId = "xvars", label = "Select predictor variables :",
        choices = names(penguins)[-3],
        selected = "island"
      )
    ),
    mainPanel(
      gt_output("tbl")
    )
  )
)

server <- function(input, output) {
  model <- reactive({
    form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
    lm(as.formula(form), penguins)
  })
  
  output$tbl <- render_gt({
    as_gt(tbl_regression(model()))
  })
  
}

shinyApp(ui = ui, server = server)
```

But, naturally, we'd probably also want the equation rendered too. We can extend the above with {equatiomatic} as follows.

```{r}
library(equatiomatic)
library(shiny)
library(shinyWidgets)
library(gtsummary)
library(gt)

ui <- fluidPage(
  titlePanel("equatiomatic w/Shiny"),
  withMathJax(), # Initialize mathJax so the equation renders properly
  sidebarLayout(
    sidebarPanel(
      multiInput(
        inputId = "xvars", label = "Select predictor variables :",
        choices = names(penguins)[-3],
        selected = "island"
      )
    ),
    mainPanel(
      eqOutput("equation"),
      gt_output("tbl")
    )
  )
)

server <- function(input, output) {
  model <- reactive({
    form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
    lm(as.formula(form), penguins)
  })
  
  output$equation <- renderEq(
    extract_eq(model())
  )
  
  output$tbl <- render_gt({
    as_gt(tbl_regression(model()))
  })
  
}

shinyApp(ui = ui, server = server)
```

Note that, beyond the `renderEq()` and `eqOutput()` function, we also have to initialize mathJax with the `shiny::withMathJax()` function somewhere in the UI. Without this line, you'll get the text version of the equation.

Finally, we can of course add additional arguments `extract_eq()`, and it all works as expected.

```{r}
library(equatiomatic)
library(shiny)
library(shinyWidgets)
library(gtsummary)
library(gt)

ui <- fluidPage(
  titlePanel("equatiomatic w/Shiny"),
  withMathJax(),
  sidebarLayout(
    sidebarPanel(
      multiInput(
        inputId = "xvars", 
        label = "Select predictor variables :",
        choices = names(penguins)[-3],
        selected = "island"
      )
    ),
    mainPanel(
      eqOutput("eq2"),
      eqOutput("equation"),
      gt_output("tbl")
    )
  )
)

server <- function(input, output) {
  model <- reactive({
    form <- paste("bill_length_mm ~ ", paste(input$xvars, collapse = " + "))
    lm(as.formula(form), penguins)
  })
  
  output$equation <- renderEq(
      extract_eq(
        model(), 
        wrap = TRUE,
        terms_per_line = 2,
        use_coefs = TRUE,
        font_size = "Huge"
    )
  )

  output$tbl <- render_gt({
    as_gt(tbl_regression(model()))
  })
  
}

shinyApp(ui = ui, server = server)
```

---
title: "lme4::lmer Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{lme4::lmer Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This vignette is intended to show the wide variety of `lme4::lmer` models that can be handled by {equatiomatic}. The output uses the notation from [Gelman and Hill](
https://doi.org/10.1017/CBO9780511790942). If you notice any errors in the notation, please [file an issue](https://github.com/datalorax/equatiomatic/issues). Similarly, if you try to use {equatiomatic} with an **lmer** model and end up with an error (either in notation or code) I would *really* appreciate if you could file an issue with a reproducible example.

This vignette displays many of the models that are covered by the package tests. It also illustrates many of the features of {equatiomatic}, including

* Automatic detection of group-level predictor variables (and the level at which they predict)
* Handling of interactions, both within a level and across levels
* Handling of any number of higher-level (or cross-classified) grouping factors (i.e., number of levels or random effects is infinite)
* Some support for variance-covariance structure outside of standard unstructured specifications

## Setup
```{r setup}
library(equatiomatic)
library(lme4)
```

## Unconditional models

A basic two-level unconditional model:

```{r unconditional1}
um_hsb <- lmer(math ~ 1 + (1 | sch.id), data = hsb)
extract_eq(um_hsb)
```

And a model with multiple levels:

```{r unconditional2}
um_long3 <- lmer(score ~ 1 + (1 | sid) + (1 | school) + (1 | district),
  data = sim_longitudinal)
extract_eq(um_long3)
```

## Models with predictors at level 1
```{r l1-preds}
lev1_hsb <- lmer(math ~ female + ses + minority + (1 | sch.id), data = hsb)
extract_eq(lev1_hsb)
```

Note that in the above, the mean structure at level 1 is broken out into a separate line. You can override this with the `mean_separate` argument.

```{r mean_separate}
extract_eq(lev1_hsb, mean_separate = FALSE)
```

Similarly, just like with standard `lm` models, you can specify wrapping, and how many terms per line:

```{r wrapping}
extract_eq(lev1_hsb, wrap = TRUE, terms_per_line = 2)
```

And one more example with multiple levels.

```{r l1-preds2}
lev1_long <- lmer(score ~ wave + (1 | sid) + (1 | school) + (1 | district),
  data = sim_longitudinal)
extract_eq(lev1_long)
```

## Models with unstructured variance-covariance specifications

This should generally work regardless of the complexity.

```{r unstructured-vcv1}
hsb1 <- lmer(math ~ female + ses + minority + (minority | sch.id),
  data = hsb)
extract_eq(hsb1)
```

Notice that it correctly parses which variable is randomly varying here. We can also make it more complicated.

```{r unstructured-vcv2}
hsb2 <- lmer(math ~ female + ses + minority + (female + ses | sch.id),
  data = hsb)
extract_eq(hsb2)
```

And even really complicated. Note the model below gives a warning.

```{r unstructured-vcv3}
hsb3 <- lmer(math ~ female * ses + minority + (ses * female + minority | sch.id),
  data = hsb)
extract_eq(hsb3)
```

# Group level predictors

In the `sim_longitudinal` data that comes with the package, the only level 1 predictor is `wave`. The `group` and `treatement` variables are at the student level (level 2) and `prop_low` is at the school level. Let's also add a district level variable (just the average score for each district).

```{r group-level-preds-prep}
# calculate district means
dist_mean <- tapply(sim_longitudinal$score, sim_longitudinal$district, mean)

# put them in a df to merge
dist_mean <- data.frame(
  district  = names(dist_mean),
  dist_mean = dist_mean
)

# create a new df with dist_mean added
d <- merge(sim_longitudinal, dist_mean, by = "district")
```

Now we can fit a model that should have predictors at every level. We'll allow `wave` to vary randomly at each level too.

```{r group-preds-model1}
group_preds_m1 <- lmer(score ~ wave + group + treatment + prop_low + dist_mean +
  (wave | sid) + (wave | school) + (wave | district), data = d)
extract_eq(group_preds_m1)
```

## Interactions

Interactions with multilevel models can be tricky because they can be within or across levels, and the notation changes depending on whether the random effect for the lower level (in the interaction term) is specified as varying randomly within the given level. Luckily, {equatiomatic} handles all of this for you.

First, let's look at a model with interactions only at level 1.

```{r l1-interaction}
l1_hsb_int <- lmer(math ~ minority * female + (1 | sch.id), data = hsb)
extract_eq(l1_hsb_int)
```

And now an interaction at only level 2

```{r l2-interaction}
l2_hsb_int <- lmer(math ~ meanses * sector + (1 | sch.id),
  data = hsb
)
extract_eq(l2_hsb_int)
```

But more complicated are cross level interactions. Here's a quick example.

```{r cross-level1}
cl_long1 <- lmer(score ~ wave * treatment + (wave | sid) + (1 | school) +
  (1 | district), data = sim_longitudinal)
extract_eq(cl_long1)
```

Note that the `treatement` variable is shown as a predictor of the level 1 intercept and the level 1 slope. But if the slope is not randomly varying at this level, then the notation has to change.

```{r cross-level2}
cl_long2 <- lmer(score ~ wave * treatment + (1 | sid) + (1 | school) +
  (1 | district), data = sim_longitudinal)
extract_eq(cl_long2)
```

This works even for really complicated models, including three-way interactions that contain a cross-level interaction. For example:

```{r cross-level3}
cl_long3 <- lmer(score ~ wave * group * treatment + wave * prop_low * treatment +
  (wave | sid) + (wave | school) + (wave + treatment | district),
  data = sim_longitudinal)
extract_eq(cl_long3)
```

## Alternative variance-covariance specifications

Finally, there is some support for alternative variance-covariance specifications. For example, you may want to specify a model with only the variance terms estimated, and not the covariances.

```{r vars-only}
hsb_varsonly <- lmer(math ~ minority * female + (minority * female || sch.id),
  data = hsb)
extract_eq(hsb_varsonly)
```

Or maybe you want to group by the same thing multiple times. In this exact model I don't think this makes any sense, but there are cases where it can. Note that, again, this model produces a warning.

```{r doublegroup}
hsb_doublegroup <- lmer(math ~ minority * female +
  (minority | sch.id) + (female | sch.id), data = hsb)
extract_eq(hsb_doublegroup)
```

And finally, you can have a mix of different things and it should generally still work.

```{r mix}
long_mixed_ranef <- lmer(score ~ wave + (wave || sid) + (wave | school) +
  (1 | school) + (wave || district), data = sim_longitudinal)
extract_eq(long_mixed_ranef)
```

With that said, this is the part of the code that I would consider the most "experimental" at this point, so please, do reach out if you have issues or run into use cases that are not supported.

## Using estimated coefficients 

Because of the multilevel nature of the notation used by {equatiomatic}, tracking the estimated coefficients can at times be a little difficult. To help with this, we keep the Greek notation with the estimated coefficients as subscripts. For example, let's extract the equation from our model with group-level predictors.

```{r}
extract_eq(group_preds_m1, use_coef = TRUE)
```

## Future developments

There are still a few things I'm planning to implement that I haven't quite gotten around to yet. These include:

* Implementation of other features currently included in the `lm` extraction, including `intercept=`, and `raw_tex=` 
* Inevitable bug fixes

Additionally, the Greek notation as subscripts may not always be desirable, particularly if the model is fairly simple. Future developments will likely make this optional.

The range of models that you can fit with `lme4::lmer` is huge, but I hope this will handle a wide range of models. I also recognize that some users may want a different notation for the models. At this point, I'm not planning to implement other notations, but that could change if there's enough demand for it. I'm open to any/all suggestions for how to improve the package, and would *love* pull requests to help support the package, big or small.

---
title: "Introduction to {equatiomatic}"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to {equatiomatic}}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
library(equatiomatic)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

# Background and Motivation

If you use R to statistically analyze your data, you might be used to seeing and interpreting the output from functions for models, like `lm()` and `glm()`. For example, here is the code and output for a single regression model, fit using the `lm()` function. We'll examine how the depth of penguin's bills relates to their bill length using data from the [{palmerpenguins}](https://github.com/allisonhorst/palmerpenguins) package:

```{r, warning=FALSE, results='markup'}
data("penguins", package = "equatiomatic")
penguins_lm <- lm(bill_length_mm ~ bill_depth_mm, penguins)
penguins_lm
```

At the same time, you might have come across---or written!---equations that appear in books, journal articles, and reports. Equations that look like this:

```{r, echo=FALSE}
library(equatiomatic)
extract_eq(penguins_lm)
```

The purpose of [{equatiomatic}](https://datalorax.github.io/equatiomatic/) is to help you go from model output, like the `lm()` output above,
to the equation we just saw. As we detail in this vignette, {equatiomatic} provides the underlying equation corresponding to the statistical model output. It does this through the use of the TeX equation formatting system, which can then be included in documents written in a number of formats, including (importantly) R Markdown or Quarto documents rendered to either HTML or PDF formats.

# Example Usage

## A basic example for a multiple linear regression

Let's look at another basic example, again using the `palmerpenguins` data:

```{r fit-m1}
library(equatiomatic)

# fit a basic multiple linear regression model
m <- lm(bill_length_mm ~ bill_depth_mm + flipper_length_mm, data = penguins)
```

Now we can pull the TeX code with `extract_eq`:

```{r extract_eq1}
extract_eq(m)
```

You can, of course, set `echo=FALSE` as well in the chunk header, and then you'll get just the equation, which will look like the below.

```{r extract_eq1-no-echo, echo=FALSE}
extract_eq(m)
```

Let's dive into a bit more complex example.

## An example with a slightly more complicated model

The above was super simple, but---often---you have categorical variables with lots of levels and interactions, which can make the output of your models (and writing a model equation for your models) a bit more complicated. {equatiomatic} is  intended to "smooth out" some of these issues, requiring the same process as in the above example to extract (and present) the model's equation.

For instance, here is an example using a categorical variable (`island`) in an interaction with the `bill_depth_mm` variable we used in the regression above:

```{r eq2}
m2 <- lm(bill_length_mm ~ bill_depth_mm * island, data = penguins)
extract_eq(m2)
```

Sometimes, for such models, the equations can get overly long. That's where the `wrap=` and `terms_per_line=` arguments come in.

```{r eq2-wrap}
extract_eq(m2, wrap = TRUE) # default terms_per_line = 4
extract_eq(m2, wrap = TRUE, terms_per_line = 2)
```

Maybe you want different intercept notation, such as $\beta_0$? Simply pass `"beta"` to the `intercept=` argument, as follows. 

```{r eq2-intercept-beta}
extract_eq(m2, wrap = TRUE, intercept = "beta")
```

We also wrap all the variable names in `\operatorname{}` by default so they show up as plain text, but if you'd like your variable names to be italicized just set `ital_vars = TRUE`.

```{r eq2-ital-vars}
extract_eq(m2, wrap = TRUE, ital_vars = TRUE)
```

## Raw TeX code

Currently, the `intercept=` argument defaults to `"alpha"` and only takes one additional argument, `"beta"`. However, the `raw_tex=` and `greek=` arguments allows you to specify whatever syntax you would like both for the intercept and for the coefficients. For example:

```{r raw-tex}
extract_eq(m2, wrap = TRUE, intercept = "\\hat{\\phi}", greek = "\\hat{\\gamma}",
  raw_tex = TRUE)
```

## Coefficients

In many cases, you're interested more in including the coefficient estimates (e.g., `3.04`), than the Greek symbols (e.g., $\\beta\_1$). This may be helpful for communicating the results of a model (and, possibly, for teaching  about the statistical model). To do this, simply change the `use_coefs=` argument:

```{r use_coefs}
extract_eq(m2, wrap = TRUE, use_coefs = TRUE)
```

# Other models

While the above examples focused on regression models (and the `lm()` function), {equatiomatic} supports output from other model types as well, specifically:

```{r echo=FALSE}
supported <- data.frame(
  model = c(
    "linear regression",
    "logistic regression",
    "probit regression",
    "ordinal logistic regression",
    "ordinal probit regression",
    "auto-regressive integrated moving average",
    "regression with auto-regressive integrated moving average errors"
  ),
  packages = c(
    "`stats::lm`",
    "`stats::glm(family = binomial(link = 'logit'))`",
    "`stats::glm(family = binomial(link = 'probit'))`",
    "`MASS::polr(method = 'logistic')`; `ordinal::clm(link = 'logit')`",
    "`MASS::polr(method = 'probit')`; `ordinal::clm(link = 'probit')`",
    "`forecast::Arima`",
    "`forecast::Arima`"
  )
)

knitr::kable(supported, col.names = c("Model", "Packages/Functions"))
```

Here are a few basic examples using these supported model types:

## Logistic Regression

```{r log-reg1}
lr <- glm(sex ~ species * bill_length_mm, data = penguins,
  family = binomial(link = "logit"))

extract_eq(lr, wrap = TRUE)
```

You can also (optionally) show the how the data are assumed to be distributed.

```{r log-reg2}
extract_eq(lr, wrap = TRUE, show_distribution = TRUE)
```

## Probit Regression

Probit regression works similarly to logistic regression:

```{r prob-reg1}
pr <- glm(sex ~ species * bill_length_mm, data = penguins,
  family = binomial(link = "probit"))

extract_eq(pr, wrap = TRUE)
```

Again, you can (optionally) show the how the data are assumed distributed.

```{r prob-reg2}
extract_eq(pr, wrap = TRUE, show_distribution = TRUE)
```

## Ordinal Regression

For these examples, we'll use wine tasting data from the [{ordinal}](https://github.com/runehaubo/ordinal) package. If you don't already have this package, you can download it with:

```{r install-ordinal, eval=FALSE}
install.packages("ordinal")
```

The data look like this:

```{r wine-data, echo=FALSE}
knitr::kable(head(ordinal::wine), align = "l")
```

We'll fit a model from the documentation, with the ordinal rating response predicted by an interaction between the temperature and and contact. You can fit ordinal response models with either `MASS::polr()` or `ordinal::clm()`, and {equatiomatic} works with either, and with logistic or probit link functions.

### {MASS}

#### Logit method

```{r mass-ologit}
mass_ologit <- MASS::polr(rating ~ temp * contact, data = ordinal::wine)
extract_eq(mass_ologit, wrap = TRUE, terms_per_line = 2)
```

#### Probit method

```{r mass-oprobit}
mass_oprobit <- MASS::polr(rating ~ temp * contact, data = ordinal::wine,
  method = "probit")
extract_eq(mass_oprobit, wrap = TRUE, terms_per_line = 2)
```

### {ordinal}

And we can do the same thing with the {ordinal} package.

#### Logit method

```{r ordinal-ologit}
ordinal_ologit <- ordinal::clm(rating ~ temp * contact, data = ordinal::wine,
  link = "logit")
extract_eq(ordinal_ologit, wrap = TRUE, terms_per_line = 2)
```

#### Probit method

```{r ordinal-oprobit}
ordinal_probit <- ordinal::clm(rating ~ temp * contact, data = ordinal::wine,
  link = "probit")
extract_eq(ordinal_probit, wrap = TRUE, terms_per_line = 2)
```

# Working with a list of models

Starting from version 0.4.2 {equatiomatic} can also extract equations from a list of models. Here is an example:

```{r list}
# Create a series of regressions and place results in a data.frame
# (note: this is much better handled with a tibble, but we avoid
# extra dependencies here)
data(iris)
res <- data.frame(name = c("lm1", "lm2", "lm3"))
models <- list(
  lm1 = lm(Sepal.Length ~ Sepal.Width, data = iris),
  lm2 = lm(Sepal.Length ~ Sepal.Width + Petal.Length, data = iris),
  lm3 = lm(Sepal.Length ~ Sepal.Width * Petal.Length, data = iris)
)
# Add a column in the data.frame with the three corresponding equations
res$equation <- extract_eq(models)
knitr::kable(res)
```


# Things the package does not yet do

We are aware of a few things the package doesn't yet do, but that we hope to add later. These include:

- Math functions (e.g., `log`, `exp`, `sqrt`)
- Polynomial (e.g., `lm(y ~ poly(x, 3))`)
- A range of other models, including multi-level (or mixed effects) models

Regarding this last point, we are hopeful that we can incorporate essentially all the models covered by [{broom}](https://broom.tidymodels.org). Multilevel models are particularly high on our wish list. But we have not yet had the time to develop these.

# Contributing

We would **love** to have you as a contributor! Is there a model that we don't currently fit that you want implemented? There are a few ways to go about this. You can either (a) fork the repository and implement the method on your own, then submit a PR, or (b) file an issue.

If you file an issue it would be *really* helpful if you could provide an example of a fitted model and what the equation for that model should look like. We will try to get to these as soon as possible.

Also, the next planned vignette (at this particular moment) is on contributing to the package, with a step-by-step example of implementing a new method. So stay tuned for that, if you're interested in, but not yet sure how to get started.

# Coefficient digits work correctly

    $$
    \operatorname{\widehat{mpg}} = 34.661 - 1.5873(\operatorname{cyl}) - 0.0206(\operatorname{disp})
    $$

---

    $$
    \operatorname{\widehat{mpg}} = 34.66 + -1.59(\operatorname{cyl}) + -0.02(\operatorname{disp})
    $$

# Wrapping works correctly

    $$
    \begin{aligned}
    \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\ + \\
    &\quad \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt}) + \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\ + \\
    &\quad \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear}) + \beta_{10}(\operatorname{carb}) + \epsilon
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl})\ + \\
    &\quad \beta_{2}(\operatorname{disp}) + \beta_{3}(\operatorname{hp})\ + \\
    &\quad \beta_{4}(\operatorname{drat}) + \beta_{5}(\operatorname{wt})\ + \\
    &\quad \beta_{6}(\operatorname{qsec}) + \beta_{7}(\operatorname{vs})\ + \\
    &\quad \beta_{8}(\operatorname{am}) + \beta_{9}(\operatorname{gear})\ + \\
    &\quad \beta_{10}(\operatorname{carb}) + \epsilon
    \end{aligned}
    $$

# colorizing works

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{rating}  \leq  \operatorname{1} ) }{ 1 - P( \operatorname{rating}  \leq  \operatorname{1} ) } \right] &= {\color{#1B9E77}{\alpha}}_{{\color{#666666}{1}}} + {\color{#D95F02}{\beta}}_{{\color{#A6761D}{1}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}})\ + \\
    &\quad {\color{#7570B3}{\beta}}_{{\color{#E6AB02}{2}}}(\operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) + {\color{#E7298A}{\beta}}_{{\color{#66A61E}{3}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}} \times \operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) \\
    \log\left[ \frac { P( \operatorname{rating}  \leq  \operatorname{2} ) }{ 1 - P( \operatorname{rating}  \leq  \operatorname{2} ) } \right] &= {\color{#1B9E77}{\alpha}}_{{\color{#666666}{2}}} + {\color{#D95F02}{\beta}}_{{\color{#A6761D}{1}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}})\ + \\
    &\quad {\color{#7570B3}{\beta}}_{{\color{#E6AB02}{2}}}(\operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) + {\color{#E7298A}{\beta}}_{{\color{#66A61E}{3}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}} \times \operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) \\
    \log\left[ \frac { P( \operatorname{rating}  \leq  \operatorname{3} ) }{ 1 - P( \operatorname{rating}  \leq  \operatorname{3} ) } \right] &= {\color{#1B9E77}{\alpha}}_{{\color{#666666}{3}}} + {\color{#D95F02}{\beta}}_{{\color{#A6761D}{1}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}})\ + \\
    &\quad {\color{#7570B3}{\beta}}_{{\color{#E6AB02}{2}}}(\operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) + {\color{#E7298A}{\beta}}_{{\color{#66A61E}{3}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}} \times \operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) \\
    \log\left[ \frac { P( \operatorname{rating}  \leq  \operatorname{4} ) }{ 1 - P( \operatorname{rating}  \leq  \operatorname{4} ) } \right] &= {\color{#1B9E77}{\alpha}}_{{\color{#666666}{4}}} + {\color{#D95F02}{\beta}}_{{\color{#A6761D}{1}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}})\ + \\
    &\quad {\color{#7570B3}{\beta}}_{{\color{#E6AB02}{2}}}(\operatorname{contact}{\color{orange}{_{\operatorname{yes}}}}) + {\color{#E7298A}{\beta}}_{{\color{#66A61E}{3}}}({\color{blue}{\operatorname{temp}}}_{\operatorname{warm}} \times \operatorname{contact}{\color{orange}{_{\operatorname{yes}}}})
    \end{aligned}
    $$

# Renaming Variables works

    $$
    \begin{aligned}
    P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{catty}_{\operatorname{b}})\ + \\
    &\qquad\ \beta_{2}(\operatorname{catty}_{\operatorname{c}}) + \beta_{3}(\operatorname{catty}_{\operatorname{do\ do\ do}})\ + \\
    &\qquad\ \beta_{4}(\operatorname{catty}_{\operatorname{e}}) + \beta_{5}(\operatorname{Cont\ Var})\ + \\
    &\qquad\ \beta_{6}(\operatorname{catty}_{\operatorname{b}} \times \operatorname{Cont\ Var}) + \beta_{7}(\operatorname{catty}_{\operatorname{c}} \times \operatorname{Cont\ Var})\ + \\
    &\qquad\ \beta_{8}(\operatorname{catty}_{\operatorname{do\ do\ do}} \times \operatorname{Cont\ Var}) + \beta_{9}(\operatorname{catty}_{\operatorname{e}} \times \operatorname{Cont\ Var})] \\
    P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{catty}_{\operatorname{b}})\ + \\
    &\qquad\ \beta_{2}(\operatorname{catty}_{\operatorname{c}}) + \beta_{3}(\operatorname{catty}_{\operatorname{do\ do\ do}})\ + \\
    &\qquad\ \beta_{4}(\operatorname{catty}_{\operatorname{e}}) + \beta_{5}(\operatorname{Cont\ Var})\ + \\
    &\qquad\ \beta_{6}(\operatorname{catty}_{\operatorname{b}} \times \operatorname{Cont\ Var}) + \beta_{7}(\operatorname{catty}_{\operatorname{c}} \times \operatorname{Cont\ Var})\ + \\
    &\qquad\ \beta_{8}(\operatorname{catty}_{\operatorname{do\ do\ do}} \times \operatorname{Cont\ Var}) + \beta_{9}(\operatorname{catty}_{\operatorname{e}} \times \operatorname{Cont\ Var})]
    \end{aligned}
    $$

# Math extraction works

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous}) + \beta_{2}(\operatorname{continuous^2}) + \beta_{3}(\operatorname{continuous^3}) + \beta_{4}(\operatorname{\log(continuous)}) \\
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous}) + \beta_{2}(\operatorname{continuous^2}) + \beta_{3}(\operatorname{continuous^3}) + \beta_{4}(\operatorname{\log(continuous)})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous}) + \beta_{2}(\operatorname{continuous^2}) + \beta_{3}(\operatorname{continuous^3}) + \beta_{4}(\operatorname{\log(continuous)})] \\
    P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous}) + \beta_{2}(\operatorname{continuous^2}) + \beta_{3}(\operatorname{continuous^3}) + \beta_{4}(\operatorname{\log(continuous)})]
    \end{aligned}
    $$

# Collapsing polr factors works

    Code
      extract_eq(model_logit)
    Output
      $$
      \begin{aligned}
      \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous}) + \beta_{6}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous}) + \beta_{7}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous}) + \beta_{8}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous}) + \beta_{9}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous}) \\
      \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous}) + \beta_{6}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous}) + \beta_{7}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous}) + \beta_{8}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous}) + \beta_{9}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous})
      \end{aligned}
      $$

---

    Code
      extract_eq(model_probit)
    Output
      $$
      \begin{aligned}
      P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous}) + \beta_{6}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous}) + \beta_{7}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous}) + \beta_{8}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous}) + \beta_{9}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous})] \\
      P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous}) + \beta_{6}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous}) + \beta_{7}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous}) + \beta_{8}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous}) + \beta_{9}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous})]
      \end{aligned}
      $$

---

    Code
      extract_eq(model_logit, index_factors = TRUE)
    Output
      $$
      \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] = \alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous}\right) \\
      \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] = \alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous}\right)
      $$

---

    Code
      extract_eq(model_probit, index_factors = TRUE)
    Output
      $$
      P( \operatorname{outcome}  \leq  \operatorname{A} ) = \Phi[\alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous}\right)] \\
      P( \operatorname{outcome}  \leq  \operatorname{B} ) = \Phi[\alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous}\right)]
      $$

# Ordered logistic regression works

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2}) \\
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= \alpha_{1} + \beta_{1}(\operatorname{continuous\_1})\ + \\
    &\quad \beta_{2}(\operatorname{continuous\_2}) \\
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= \alpha_{2} + \beta_{1}(\operatorname{continuous\_1})\ + \\
    &\quad \beta_{2}(\operatorname{continuous\_2})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})] \\
    P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1}) + \beta_{2}(\operatorname{continuous\_2})]
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    P( \operatorname{outcome}  \leq  \operatorname{A} ) &= \Phi[\alpha_{1} + \beta_{1}(\operatorname{continuous\_1})\ + \\
    &\qquad\ \beta_{2}(\operatorname{continuous\_2})] \\
    P( \operatorname{outcome}  \leq  \operatorname{B} ) &= \Phi[\alpha_{2} + \beta_{1}(\operatorname{continuous\_1})\ + \\
    &\qquad\ \beta_{2}(\operatorname{continuous\_2})]
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{A} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{A} ) } \right] &= 1.09 + 0.03(\operatorname{continuous\_1}) - 0.03(\operatorname{continuous\_2}) \\
    \log\left[ \frac { P( \operatorname{outcome}  \leq  \operatorname{B} ) }{ 1 - P( \operatorname{outcome}  \leq  \operatorname{B} ) } \right] &= 2.48 + 0.03(\operatorname{continuous\_1}) - 0.03(\operatorname{continuous\_2})
    \end{aligned}
    $$

# colorizing works

    $$
    \begin{aligned}
    \log\left[ \frac { P( {\color{#0073ff}{\operatorname{SEX}}} = {\color{#0073ff}{\operatorname{male}}} ) }{ 1 - P( {\color{#0073ff}{\operatorname{SEX}}} = {\color{#0073ff}{\operatorname{male}}} ) } \right] &= {\color{#1B9E77}{\alpha}} + {\color{#D95F02}{\beta}}_{{\color{blue}{1}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}}) + {\color{#7570B3}{\beta}}_{{\color{blue}{2}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}}) + {\color{#E7298A}{\beta}}_{{\color{blue}{3}}}(\operatorname{bill\_length\_mm})\ + \\
    &\quad {\color{#66A61E}{\beta}}_{{\color{blue}{4}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}} \times \operatorname{bill\_length\_mm}) + {\color{#E6AB02}{\beta}}_{{\color{blue}{5}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}} \times \operatorname{bill\_length\_mm})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    {\color{#0073ff}{\operatorname{SEX}}} &\sim Bernoulli\left(\operatorname{prob}_{{\color{#0073ff}{\operatorname{SEX}}} = {\color{#0073ff}{\operatorname{male}}}}= \hat{P}\right) \\
     \log\left[ \frac { \hat{P} }{ 1 - \hat{P} } \right] 
     &= {\color{#1B9E77}{\alpha}} + {\color{#D95F02}{\beta}}_{{\color{blue}{1}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}}) + {\color{#7570B3}{\beta}}_{{\color{blue}{2}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}}) + {\color{#E7298A}{\beta}}_{{\color{blue}{3}}}(\operatorname{bill\_length\_mm})\ + \\
    &\quad {\color{#66A61E}{\beta}}_{{\color{blue}{4}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}} \times \operatorname{bill\_length\_mm}) + {\color{#E6AB02}{\beta}}_{{\color{blue}{5}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}} \times \operatorname{bill\_length\_mm})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    P( {\color{#0073ff}{\operatorname{SEX}}} = {\color{#0073ff}{\operatorname{male}}} ) &= \Phi[{\color{#1B9E77}{\alpha}} + {\color{#D95F02}{\beta}}_{{\color{blue}{1}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}}) + {\color{#7570B3}{\beta}}_{{\color{blue}{2}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}}) + {\color{#E7298A}{\beta}}_{{\color{blue}{3}}}(\operatorname{bill\_length\_mm})\ + \\
    &\qquad\ {\color{#66A61E}{\beta}}_{{\color{blue}{4}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}} \times \operatorname{bill\_length\_mm}) + {\color{#E6AB02}{\beta}}_{{\color{blue}{5}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}} \times \operatorname{bill\_length\_mm})]
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    {\color{#0073ff}{\operatorname{SEX}}} &\sim Bernoulli\left(\operatorname{prob}_{{\color{#0073ff}{\operatorname{SEX}}} = {\color{#0073ff}{\operatorname{male}}}}= \hat{P}\right) \\
     \hat{P} 
     &= \Phi[{\color{#1B9E77}{\alpha}} + {\color{#D95F02}{\beta}}_{{\color{blue}{1}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}}) + {\color{#7570B3}{\beta}}_{{\color{blue}{2}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}}) + {\color{#E7298A}{\beta}}_{{\color{blue}{3}}}(\operatorname{bill\_length\_mm})\ + \\
    &\qquad\ {\color{#66A61E}{\beta}}_{{\color{blue}{4}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Chinstrap}}}} \times \operatorname{bill\_length\_mm}) + {\color{#E6AB02}{\beta}}_{{\color{blue}{5}}}({\color{green}{\operatorname{species}}}{\color{orange}{_{\operatorname{Gentoo}}}} \times \operatorname{bill\_length\_mm})]
    \end{aligned}
    $$

# logit_notation works for logistic glm

    Code
      extract_eq(model_logit, logit_notation = TRUE)
    Output
      $$
      \operatorname{logit}\left[ P(\operatorname{outcome}) \right] = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})
      $$

---

    Code
      extract_eq(model_logit, logit_notation = TRUE, wrap = TRUE)
    Output
      $$
      \begin{aligned}
      \operatorname{logit}\left[ P(\operatorname{outcome}) \right] &= \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1})\ + \\
      &\quad \beta_{4}(\operatorname{continuous\_2})
      \end{aligned}
      $$

---

    Code
      extract_eq(model_logit, logit_notation = TRUE, use_coefs = TRUE)
    Output
      $$
      \operatorname{logit}\left[ P(\hat{\operatorname{outcome}}) \right] = 5.33 + 0.33(\operatorname{categorical}_{\operatorname{b}}) - 0.12(\operatorname{categorical}_{\operatorname{c}}) - 0.06(\operatorname{continuous\_1}) + 0.01(\operatorname{continuous\_2})
      $$

# Renaming Variables works

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{outcome} = \operatorname{1} ) }{ 1 - P( \operatorname{outcome} = \operatorname{1} ) } \right] &= \alpha + \beta_{1}(\operatorname{cat}_{\operatorname{b}})\ + \\
    &\quad \beta_{2}(\operatorname{cat}_{\operatorname{c}}) + \beta_{3}(\operatorname{cat}_{\operatorname{dddd}})\ + \\
    &\quad \beta_{4}(\operatorname{cat}_{\operatorname{e}}) + \beta_{5}(\operatorname{Continuous\ Variable\ [1]})\ + \\
    &\quad \beta_{6}(\operatorname{continuous\_2}) + \beta_{7}(\operatorname{cat}_{\operatorname{b}} \times \operatorname{Continuous\ Variable\ [1]})\ + \\
    &\quad \beta_{8}(\operatorname{cat}_{\operatorname{c}} \times \operatorname{Continuous\ Variable\ [1]}) + \beta_{9}(\operatorname{cat}_{\operatorname{dddd}} \times \operatorname{Continuous\ Variable\ [1]})\ + \\
    &\quad \beta_{10}(\operatorname{cat}_{\operatorname{e}} \times \operatorname{Continuous\ Variable\ [1]}) + \beta_{11}(\operatorname{cat}_{\operatorname{b}} \times \operatorname{continuous\_2})\ + \\
    &\quad \beta_{12}(\operatorname{cat}_{\operatorname{c}} \times \operatorname{continuous\_2}) + \beta_{13}(\operatorname{cat}_{\operatorname{dddd}} \times \operatorname{continuous\_2})\ + \\
    &\quad \beta_{14}(\operatorname{cat}_{\operatorname{e}} \times \operatorname{continuous\_2}) + \beta_{15}(\operatorname{Continuous\ Variable\ [1]} \times \operatorname{continuous\_2})\ + \\
    &\quad \beta_{16}(\operatorname{cat}_{\operatorname{b}} \times \operatorname{Continuous\ Variable\ [1]} \times \operatorname{continuous\_2}) + \beta_{17}(\operatorname{cat}_{\operatorname{c}} \times \operatorname{Continuous\ Variable\ [1]} \times \operatorname{continuous\_2})\ + \\
    &\quad \beta_{18}(\operatorname{cat}_{\operatorname{dddd}} \times \operatorname{Continuous\ Variable\ [1]} \times \operatorname{continuous\_2}) + \beta_{19}(\operatorname{cat}_{\operatorname{e}} \times \operatorname{Continuous\ Variable\ [1]} \times \operatorname{continuous\_2})
    \end{aligned}
    $$

# Math extraction works

    $$
    \log\left[ \frac { P( \operatorname{outcome} = \operatorname{1} ) }{ 1 - P( \operatorname{outcome} = \operatorname{1} ) } \right] = \alpha + \beta_{1}(\operatorname{\log(continuous\_1)}) + \beta_{2}(\operatorname{\exp(continuous\_2)}) + \beta_{3}(\operatorname{continuous\_2}) + \beta_{4}(\operatorname{continuous\_2^2}) + \beta_{5}(\operatorname{continuous\_2^3})
    $$

---

    $$
    P( \operatorname{outcome} = \operatorname{1} ) = \Phi[\alpha + \beta_{1}(\operatorname{\log(continuous\_1)}) + \beta_{2}(\operatorname{\exp(continuous\_2)}) + \beta_{3}(\operatorname{continuous\_2}) + \beta_{4}(\operatorname{continuous\_2^2}) + \beta_{5}(\operatorname{continuous\_2^3})]
    $$

# Collapsing glm factors works

    Code
      extract_eq(model_logit)
    Output
      $$
      \log\left[ \frac { P( \operatorname{outcome} = \operatorname{1} ) }{ 1 - P( \operatorname{outcome} = \operatorname{1} ) } \right] = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous\_1}) + \beta_{6}(\operatorname{continuous\_2}) + \beta_{7}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_1}) + \beta_{8}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_1}) + \beta_{9}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_1}) + \beta_{10}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_1}) + \beta_{11}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_2}) + \beta_{12}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_2}) + \beta_{13}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_2}) + \beta_{14}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_2}) + \beta_{15}(\operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{16}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{17}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{18}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{19}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2})
      $$

---

    Code
      extract_eq(model_probit)
    Output
      $$
      P( \operatorname{outcome} = \operatorname{1} ) = \Phi[\alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{categorical}_{\operatorname{d}}) + \beta_{4}(\operatorname{categorical}_{\operatorname{e}}) + \beta_{5}(\operatorname{continuous\_1}) + \beta_{6}(\operatorname{continuous\_2}) + \beta_{7}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_1}) + \beta_{8}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_1}) + \beta_{9}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_1}) + \beta_{10}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_1}) + \beta_{11}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_2}) + \beta_{12}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_2}) + \beta_{13}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_2}) + \beta_{14}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_2}) + \beta_{15}(\operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{16}(\operatorname{categorical}_{\operatorname{b}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{17}(\operatorname{categorical}_{\operatorname{c}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{18}(\operatorname{categorical}_{\operatorname{d}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}) + \beta_{19}(\operatorname{categorical}_{\operatorname{e}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2})]
      $$

---

    Code
      extract_eq(model_logit, index_factors = TRUE)
    Output
      $$
      \log\left[ \frac { P( \operatorname{outcome} = \operatorname{1} ) }{ 1 - P( \operatorname{outcome} = \operatorname{1} ) } \right] = \alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous\_1} + \operatorname{continuous\_2} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_1}\right) + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_2}\right) + \left(\operatorname{continuous\_1} \times \operatorname{continuous\_2}\right) + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}\right)
      $$

---

    Code
      extract_eq(model_probit, index_factors = TRUE)
    Output
      $$
      P( \operatorname{outcome} = \operatorname{1} ) = \Phi[\alpha + \operatorname{categorical}_{\operatorname{i}} + \operatorname{continuous\_1} + \operatorname{continuous\_2} + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_1}\right) + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_2}\right) + \left(\operatorname{continuous\_1} \times \operatorname{continuous\_2}\right) + \left(\operatorname{categorical}_{\operatorname{i}} \times \operatorname{continuous\_1} \times \operatorname{continuous\_2}\right)]
      $$

# Logistic regression works

    $$
    \log\left[ \frac { P( \operatorname{outcome} = \operatorname{1} ) }{ 1 - P( \operatorname{outcome} = \operatorname{1} ) } \right] = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})
    $$

# Probit regression works

    $$
    P( \operatorname{outcome} = \operatorname{1} ) = \Phi[\alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})]
    $$

---

    $$
    E( \operatorname{mpg} ) = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp})
    $$

# Distribution-based equations work

    $$
    \begin{aligned}
    \operatorname{outcome} &\sim Bernoulli\left(\operatorname{prob}_{\operatorname{outcome} = \operatorname{1}}= \hat{P}\right) \\
     \log\left[ \frac { \hat{P} }{ 1 - \hat{P} } \right] 
     &= \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \operatorname{outcome} &\sim Bernoulli\left(\operatorname{prob}_{\operatorname{outcome} = \operatorname{1}}= \hat{P}\right) \\
     \hat{P} 
     &= \Phi[\alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})]
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    E( \operatorname{mpg} ) &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp})
    \end{aligned}
    $$

# non-binomial regression works

    $$
    \log ({ E( \operatorname{outcome} ) })  = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})
    $$

---

    $$
    \log ({ E( \operatorname{outcome} ) })  = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2})
    $$

---

    $$
    \log ({ E( \operatorname{outcome} ) })  = \alpha + \beta_{1}(\operatorname{categorical}_{\operatorname{b}}) + \beta_{2}(\operatorname{categorical}_{\operatorname{c}}) + \beta_{3}(\operatorname{continuous\_1}) + \beta_{4}(\operatorname{continuous\_2}) + \operatorname{offset(rep(1, 300))}
    $$

# colorizing works

    $$
    \begin{aligned}
      {\color{#FF00CC}{\operatorname{score}}}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}({\color{blue}{\operatorname{wave}}}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{group}{\color{orange}{_{\operatorname{low}}}}) + \gamma_{2}^{\alpha}(\operatorname{group}{\color{orange}{_{\operatorname{medium}}}}) + \gamma_{3l[i]}^{\alpha}({\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) + \gamma_{4}^{\alpha}(\operatorname{group}{\color{orange}{_{\operatorname{low}}}} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) + \gamma_{5}^{\alpha}(\operatorname{group}{\color{orange}{_{\operatorname{medium}}}} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{group}{\color{orange}{_{\operatorname{low}}}}) + \gamma^{\beta_{1}}_{2}(\operatorname{group}{\color{orange}{_{\operatorname{medium}}}}) + \gamma^{\beta_{1}}_{3}({\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) + \gamma^{\beta_{1}}_{4}(\operatorname{group}{\color{orange}{_{\operatorname{low}}}} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) + \gamma^{\beta_{1}}_{5}(\operatorname{group}{\color{orange}{_{\operatorname{medium}}}} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{prop\_low}) + \gamma_{2}^{\alpha}(\operatorname{prop\_low} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{prop\_low}) + \gamma^{\beta_{1}}_{1}(\operatorname{prop\_low} \times {\color{red}{\operatorname{treatment}}}_{\operatorname{1}})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l} \\
          &\gamma_{3l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}} \\
          &\mu_{\gamma_{3l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} & \rho_{\alpha_{l}\gamma_{3l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}} & \rho_{\beta_{1l}\gamma_{3l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\beta_{1l}} & \sigma^2_{\gamma_{3l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Math extraction works

    Code
      extract_eq(m1)
    Output
      $$
      \begin{aligned}
        \operatorname{Reaction}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
          \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{\log(Days\ +\ 1)}) + \beta_{2}(\operatorname{\exp(Days)}) + \beta_{3}(\operatorname{Days}) + \beta_{4}(\operatorname{Days^2}) + \beta_{5}(\operatorname{Days^3}) + \beta_{6}(\operatorname{Days^4}) \\
          \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
          \text{, for Subject j = 1,} \dots \text{,J}
      \end{aligned}
      $$

---

    Code
      extract_eq(m2)
    Output
      $$
      \begin{aligned}
        \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i]} + \beta_{1}(\operatorname{\log(wave\ +\ 1)}), \sigma^2 \right) \\    
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\alpha_{j} \\
            &\beta_{1j}
          \end{aligned}
        \end{array}
      \right)
        &\sim N \left(
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{\exp(prop\_low)}) \\
            &\mu_{\beta_{1j}}
          \end{aligned}
        \end{array}
      \right)
      , 
      \left(
        \begin{array}{cc}
           \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
           \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
        \end{array}
      \right)
       \right)
          \text{, for sid j = 1,} \dots \text{,J}
      \end{aligned}
      $$

---

    Code
      extract_eq(m3)
    Output
      $$
      \begin{aligned}
        \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\alpha_{j} \\
            &\beta_{1j}
          \end{aligned}
        \end{array}
      \right)
        &\sim N \left(
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\mu_{\alpha_{j}} \\
            &\mu_{\beta_{1j}}
          \end{aligned}
        \end{array}
      \right)
      , 
      \left(
        \begin{array}{cc}
           \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
           \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
        \end{array}
      \right)
       \right)
          \text{, for sid j = 1,} \dots \text{,J} \\    
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\alpha_{k} \\
            &\beta_{1k}
          \end{aligned}
        \end{array}
      \right)
        &\sim N \left(
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{prop\_low}) + \gamma_{2}^{\alpha}(\operatorname{prop\_low^2}) + \gamma_{3}^{\alpha}(\operatorname{prop\_low^3}) + \gamma_{4}^{\alpha}(\operatorname{prop\_low^4}) \\
            &\mu_{\beta_{1k}}
          \end{aligned}
        \end{array}
      \right)
      , 
      \left(
        \begin{array}{cc}
           \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
           \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
        \end{array}
      \right)
       \right)
          \text{, for school k = 1,} \dots \text{,K} \\    
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\alpha_{l} \\
            &\beta_{1l}
          \end{aligned}
        \end{array}
      \right)
        &\sim N \left(
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\mu_{\alpha_{l}} \\
            &\mu_{\beta_{1l}}
          \end{aligned}
        \end{array}
      \right)
      , 
      \left(
        \begin{array}{cc}
           \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} \\ 
           \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}}
        \end{array}
      \right)
       \right)
          \text{, for district l = 1,} \dots \text{,L}
      \end{aligned}
      $$

# Implicit ID variables are handled

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i]} + \beta_{1j[i],k[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for school j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{k}} \\
          &\mu_{\beta_{1k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for sid k = 1,} \dots \text{,K}
    \end{aligned}
    $$

# Renaming Variables works

    $$
    \begin{aligned}
      \operatorname{Student\ Scores}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{Wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1k[i]}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{2k[i]}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{3k[i],l[i]}^{\alpha}(\operatorname{treatment}) \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k} \\
          &\gamma_{1k} \\
          &\gamma_{2k} \\
          &\gamma_{3k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1l[i]}^{\alpha}(\operatorname{P(low\ income)}) \\
          &\mu_{\beta_{1k}} \\
          &\mu_{\gamma_{1k}} \\
          &\mu_{\gamma_{2k}} \\
          &\mu_{\gamma_{3k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccccc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} & \rho_{\alpha_{k}\gamma_{1k}} & \rho_{\alpha_{k}\gamma_{2k}} & \rho_{\alpha_{k}\gamma_{3k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}} & \rho_{\beta_{1k}\gamma_{1k}} & \rho_{\beta_{1k}\gamma_{2k}} & \rho_{\beta_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{1k}\alpha_{k}} & \rho_{\gamma_{1k}\beta_{1k}} & \sigma^2_{\gamma_{1k}} & \rho_{\gamma_{1k}\gamma_{2k}} & \rho_{\gamma_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{2k}\alpha_{k}} & \rho_{\gamma_{2k}\beta_{1k}} & \rho_{\gamma_{2k}\gamma_{1k}} & \sigma^2_{\gamma_{2k}} & \rho_{\gamma_{2k}\gamma_{3k}} \\ 
         \rho_{\gamma_{3k}\alpha_{k}} & \rho_{\gamma_{3k}\beta_{1k}} & \rho_{\gamma_{3k}\gamma_{1k}} & \rho_{\gamma_{3k}\gamma_{2k}} & \sigma^2_{\gamma_{3k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l} \\
          &\gamma_{3l} \\
          &\gamma_{1l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}} \\
          &\mu_{\gamma_{3l}} \\
          &\mu_{\gamma_{1l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} & \rho_{\alpha_{l}\gamma_{3l}} & \rho_{\alpha_{l}\gamma_{1l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}} & \rho_{\beta_{1l}\gamma_{3l}} & \rho_{\beta_{1l}\gamma_{1l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\beta_{1l}} & \sigma^2_{\gamma_{3l}} & \rho_{\gamma_{3l}\gamma_{1l}} \\ 
         \rho_{\gamma_{1l}\alpha_{l}} & \rho_{\gamma_{1l}\beta_{1l}} & \rho_{\gamma_{1l}\gamma_{3l}} & \sigma^2_{\gamma_{1l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Really big models work

    Code
      extract_eq(big_mod)
    Output
      $$
      \begin{aligned}
        \operatorname{rt}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
          \mu &=\beta_{1j[i]}(\operatorname{n1\_intercept}) + \beta_{2j[i]}(\operatorname{n1\_warning1}) + \beta_{3j[i]}(\operatorname{n1\_cuing1}) + \beta_{4j[i]}(\operatorname{x1\_intercept}) + \beta_{5j[i]}(\operatorname{x1\_warning1}) + \beta_{6j[i]}(\operatorname{x1\_cuing1}) + \beta_{7j[i]}(\operatorname{n2\_intercept}) + \beta_{8j[i]}(\operatorname{n2\_warning1}) + \beta_{9j[i]}(\operatorname{n2\_cuing1}) + \beta_{10j[i]}(\operatorname{x2\_intercept}) + \beta_{11j[i]}(\operatorname{x2\_warning1}) + \beta_{12j[i]}(\operatorname{x2\_cuing1}) \\    
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\beta_{1j} \\
            &\beta_{2j} \\
            &\beta_{3j} \\
            &\beta_{4j} \\
            &\beta_{5j} \\
            &\beta_{6j} \\
            &\beta_{7j} \\
            &\beta_{8j} \\
            &\beta_{9j} \\
            &\beta_{10j} \\
            &\beta_{11j} \\
            &\beta_{12j}
          \end{aligned}
        \end{array}
      \right)
        &\sim N \left(
      \left(
        \begin{array}{c} 
          \begin{aligned}
            &\mu_{\beta_{1j}} \\
            &\mu_{\beta_{2j}} \\
            &\mu_{\beta_{3j}} \\
            &\mu_{\beta_{4j}} \\
            &\mu_{\beta_{5j}} \\
            &\mu_{\beta_{6j}} \\
            &\mu_{\beta_{7j}} \\
            &\mu_{\beta_{8j}} \\
            &\mu_{\beta_{9j}} \\
            &\mu_{\beta_{10j}} \\
            &\mu_{\beta_{11j}} \\
            &\mu_{\beta_{12j}}
          \end{aligned}
        \end{array}
      \right)
      , 
      \left(
        \begin{array}{cccccccccccc}
           \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} & \rho_{\beta_{1j}\beta_{3j}} & \rho_{\beta_{1j}\beta_{4j}} & \rho_{\beta_{1j}\beta_{5j}} & \rho_{\beta_{1j}\beta_{6j}} & \rho_{\beta_{1j}\beta_{7j}} & \rho_{\beta_{1j}\beta_{8j}} & \rho_{\beta_{1j}\beta_{9j}} & \rho_{\beta_{1j}\beta_{10j}} & \rho_{\beta_{1j}\beta_{11j}} & \rho_{\beta_{1j}\beta_{12j}} \\ 
           \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}} & \rho_{\beta_{2j}\beta_{3j}} & \rho_{\beta_{2j}\beta_{4j}} & \rho_{\beta_{2j}\beta_{5j}} & \rho_{\beta_{2j}\beta_{6j}} & \rho_{\beta_{2j}\beta_{7j}} & \rho_{\beta_{2j}\beta_{8j}} & \rho_{\beta_{2j}\beta_{9j}} & \rho_{\beta_{2j}\beta_{10j}} & \rho_{\beta_{2j}\beta_{11j}} & \rho_{\beta_{2j}\beta_{12j}} \\ 
           \rho_{\beta_{3j}\beta_{1j}} & \rho_{\beta_{3j}\beta_{2j}} & \sigma^2_{\beta_{3j}} & \rho_{\beta_{3j}\beta_{4j}} & \rho_{\beta_{3j}\beta_{5j}} & \rho_{\beta_{3j}\beta_{6j}} & \rho_{\beta_{3j}\beta_{7j}} & \rho_{\beta_{3j}\beta_{8j}} & \rho_{\beta_{3j}\beta_{9j}} & \rho_{\beta_{3j}\beta_{10j}} & \rho_{\beta_{3j}\beta_{11j}} & \rho_{\beta_{3j}\beta_{12j}} \\ 
           \rho_{\beta_{4j}\beta_{1j}} & \rho_{\beta_{4j}\beta_{2j}} & \rho_{\beta_{4j}\beta_{3j}} & \sigma^2_{\beta_{4j}} & \rho_{\beta_{4j}\beta_{5j}} & \rho_{\beta_{4j}\beta_{6j}} & \rho_{\beta_{4j}\beta_{7j}} & \rho_{\beta_{4j}\beta_{8j}} & \rho_{\beta_{4j}\beta_{9j}} & \rho_{\beta_{4j}\beta_{10j}} & \rho_{\beta_{4j}\beta_{11j}} & \rho_{\beta_{4j}\beta_{12j}} \\ 
           \rho_{\beta_{5j}\beta_{1j}} & \rho_{\beta_{5j}\beta_{2j}} & \rho_{\beta_{5j}\beta_{3j}} & \rho_{\beta_{5j}\beta_{4j}} & \sigma^2_{\beta_{5j}} & \rho_{\beta_{5j}\beta_{6j}} & \rho_{\beta_{5j}\beta_{7j}} & \rho_{\beta_{5j}\beta_{8j}} & \rho_{\beta_{5j}\beta_{9j}} & \rho_{\beta_{5j}\beta_{10j}} & \rho_{\beta_{5j}\beta_{11j}} & \rho_{\beta_{5j}\beta_{12j}} \\ 
           \rho_{\beta_{6j}\beta_{1j}} & \rho_{\beta_{6j}\beta_{2j}} & \rho_{\beta_{6j}\beta_{3j}} & \rho_{\beta_{6j}\beta_{4j}} & \rho_{\beta_{6j}\beta_{5j}} & \sigma^2_{\beta_{6j}} & \rho_{\beta_{6j}\beta_{7j}} & \rho_{\beta_{6j}\beta_{8j}} & \rho_{\beta_{6j}\beta_{9j}} & \rho_{\beta_{6j}\beta_{10j}} & \rho_{\beta_{6j}\beta_{11j}} & \rho_{\beta_{6j}\beta_{12j}} \\ 
           \rho_{\beta_{7j}\beta_{1j}} & \rho_{\beta_{7j}\beta_{2j}} & \rho_{\beta_{7j}\beta_{3j}} & \rho_{\beta_{7j}\beta_{4j}} & \rho_{\beta_{7j}\beta_{5j}} & \rho_{\beta_{7j}\beta_{6j}} & \sigma^2_{\beta_{7j}} & \rho_{\beta_{7j}\beta_{8j}} & \rho_{\beta_{7j}\beta_{9j}} & \rho_{\beta_{7j}\beta_{10j}} & \rho_{\beta_{7j}\beta_{11j}} & \rho_{\beta_{7j}\beta_{12j}} \\ 
           \rho_{\beta_{8j}\beta_{1j}} & \rho_{\beta_{8j}\beta_{2j}} & \rho_{\beta_{8j}\beta_{3j}} & \rho_{\beta_{8j}\beta_{4j}} & \rho_{\beta_{8j}\beta_{5j}} & \rho_{\beta_{8j}\beta_{6j}} & \rho_{\beta_{8j}\beta_{7j}} & \sigma^2_{\beta_{8j}} & \rho_{\beta_{8j}\beta_{9j}} & \rho_{\beta_{8j}\beta_{10j}} & \rho_{\beta_{8j}\beta_{11j}} & \rho_{\beta_{8j}\beta_{12j}} \\ 
           \rho_{\beta_{9j}\beta_{1j}} & \rho_{\beta_{9j}\beta_{2j}} & \rho_{\beta_{9j}\beta_{3j}} & \rho_{\beta_{9j}\beta_{4j}} & \rho_{\beta_{9j}\beta_{5j}} & \rho_{\beta_{9j}\beta_{6j}} & \rho_{\beta_{9j}\beta_{7j}} & \rho_{\beta_{9j}\beta_{8j}} & \sigma^2_{\beta_{9j}} & \rho_{\beta_{9j}\beta_{10j}} & \rho_{\beta_{9j}\beta_{11j}} & \rho_{\beta_{9j}\beta_{12j}} \\ 
           \rho_{\beta_{10j}\beta_{1j}} & \rho_{\beta_{10j}\beta_{2j}} & \rho_{\beta_{10j}\beta_{3j}} & \rho_{\beta_{10j}\beta_{4j}} & \rho_{\beta_{10j}\beta_{5j}} & \rho_{\beta_{10j}\beta_{6j}} & \rho_{\beta_{10j}\beta_{7j}} & \rho_{\beta_{10j}\beta_{8j}} & \rho_{\beta_{10j}\beta_{9j}} & \sigma^2_{\beta_{10j}} & \rho_{\beta_{10j}\beta_{11j}} & \rho_{\beta_{10j}\beta_{12j}} \\ 
           \rho_{\beta_{11j}\beta_{1j}} & \rho_{\beta_{11j}\beta_{2j}} & \rho_{\beta_{11j}\beta_{3j}} & \rho_{\beta_{11j}\beta_{4j}} & \rho_{\beta_{11j}\beta_{5j}} & \rho_{\beta_{11j}\beta_{6j}} & \rho_{\beta_{11j}\beta_{7j}} & \rho_{\beta_{11j}\beta_{8j}} & \rho_{\beta_{11j}\beta_{9j}} & \rho_{\beta_{11j}\beta_{10j}} & \sigma^2_{\beta_{11j}} & \rho_{\beta_{11j}\beta_{12j}} \\ 
           \rho_{\beta_{12j}\beta_{1j}} & \rho_{\beta_{12j}\beta_{2j}} & \rho_{\beta_{12j}\beta_{3j}} & \rho_{\beta_{12j}\beta_{4j}} & \rho_{\beta_{12j}\beta_{5j}} & \rho_{\beta_{12j}\beta_{6j}} & \rho_{\beta_{12j}\beta_{7j}} & \rho_{\beta_{12j}\beta_{8j}} & \rho_{\beta_{12j}\beta_{9j}} & \rho_{\beta_{12j}\beta_{10j}} & \rho_{\beta_{12j}\beta_{11j}} & \sigma^2_{\beta_{12j}}
        \end{array}
      \right)
       \right)
          \text{, for id j = 1,} \dots \text{,J}
      \end{aligned}
      $$

# Categorical variable level parsing works (from issue #140)

    $$
    \begin{aligned}
      \operatorname{error}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{brochure}_{\operatorname{standard}}) + \beta_{2}(\operatorname{disease}_{\operatorname{DS}}) + \beta_{3}(\operatorname{brochure}_{\operatorname{standard}} \times \operatorname{disease}_{\operatorname{DS}}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for ID j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Unconditional lmer models work

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K} \\
        \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Level 1 predictors work

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{female}) + \beta_{2}(\operatorname{ses}) + \beta_{3}(\operatorname{minority}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1}(\operatorname{wave}), \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K} \\
        \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Mean separate works as expected

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\alpha_{j[i]} + \beta_{1}(\operatorname{female}) + \beta_{2}(\operatorname{ses}) + \beta_{3}(\operatorname{minority}), \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i],k[i],l[i]} + \beta_{1}(\operatorname{wave}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K} \\
        \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Wrapping works as expected

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{female})\ + \\
    &\quad \beta_{2}(\operatorname{ses}) + \beta_{3}(\operatorname{minority}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Unstructured variance-covariances work as expected

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{female}) + \beta_{2}(\operatorname{ses}) + \beta_{3j[i]}(\operatorname{minority}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{3j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{3j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{3j}} \\ 
         \rho_{\beta_{3j}\alpha_{j}} & \sigma^2_{\beta_{3j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{female}) + \beta_{2j[i]}(\operatorname{ses}) + \beta_{3}(\operatorname{minority}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}} \\
          &\mu_{\beta_{2j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} & \rho_{\alpha_{j}\beta_{2j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} \\ 
         \rho_{\beta_{2j}\alpha_{j}} & \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{female}) + \beta_{2j[i]}(\operatorname{ses}) + \beta_{3}(\operatorname{minority}) + \beta_{4j[i]}(\operatorname{female} \times \operatorname{ses}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j} \\
          &\beta_{4j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}} \\
          &\mu_{\beta_{2j}} \\
          &\mu_{\beta_{4j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} & \rho_{\alpha_{j}\beta_{2j}} & \rho_{\alpha_{j}\beta_{4j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} & \rho_{\beta_{1j}\beta_{4j}} \\ 
         \rho_{\beta_{2j}\alpha_{j}} & \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}} & \rho_{\beta_{2j}\beta_{4j}} \\ 
         \rho_{\beta_{4j}\alpha_{j}} & \rho_{\beta_{4j}\beta_{1j}} & \rho_{\beta_{4j}\beta_{2j}} & \sigma^2_{\beta_{4j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{female}) + \beta_{2j[i]}(\operatorname{ses}) + \beta_{3j[i]}(\operatorname{minority}) + \beta_{4j[i]}(\operatorname{female} \times \operatorname{ses}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j} \\
          &\beta_{3j} \\
          &\beta_{4j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}} \\
          &\mu_{\beta_{2j}} \\
          &\mu_{\beta_{3j}} \\
          &\mu_{\beta_{4j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccccc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} & \rho_{\alpha_{j}\beta_{2j}} & \rho_{\alpha_{j}\beta_{3j}} & \rho_{\alpha_{j}\beta_{4j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} & \rho_{\beta_{1j}\beta_{3j}} & \rho_{\beta_{1j}\beta_{4j}} \\ 
         \rho_{\beta_{2j}\alpha_{j}} & \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}} & \rho_{\beta_{2j}\beta_{3j}} & \rho_{\beta_{2j}\beta_{4j}} \\ 
         \rho_{\beta_{3j}\alpha_{j}} & \rho_{\beta_{3j}\beta_{1j}} & \rho_{\beta_{3j}\beta_{2j}} & \sigma^2_{\beta_{3j}} & \rho_{\beta_{3j}\beta_{4j}} \\ 
         \rho_{\beta_{4j}\alpha_{j}} & \rho_{\beta_{4j}\beta_{1j}} & \rho_{\beta_{4j}\beta_{2j}} & \rho_{\beta_{4j}\beta_{3j}} & \sigma^2_{\beta_{4j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{k}} \\
          &\mu_{\beta_{1k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Group-level predictors work as expected

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1k[i]}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{2k[i]}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{3k[i],l[i]}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k} \\
          &\gamma_{1k} \\
          &\gamma_{2k} \\
          &\gamma_{3k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{k}} \\
          &\mu_{\beta_{1k}} \\
          &\mu_{\gamma_{1k}} \\
          &\mu_{\gamma_{2k}} \\
          &\mu_{\gamma_{3k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccccc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} & \rho_{\alpha_{k}\gamma_{1k}} & \rho_{\alpha_{k}\gamma_{2k}} & \rho_{\alpha_{k}\gamma_{3k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}} & \rho_{\beta_{1k}\gamma_{1k}} & \rho_{\beta_{1k}\gamma_{2k}} & \rho_{\beta_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{1k}\alpha_{k}} & \rho_{\gamma_{1k}\beta_{1k}} & \sigma^2_{\gamma_{1k}} & \rho_{\gamma_{1k}\gamma_{2k}} & \rho_{\gamma_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{2k}\alpha_{k}} & \rho_{\gamma_{2k}\beta_{1k}} & \rho_{\gamma_{2k}\gamma_{1k}} & \sigma^2_{\gamma_{2k}} & \rho_{\gamma_{2k}\gamma_{3k}} \\ 
         \rho_{\gamma_{3k}\alpha_{k}} & \rho_{\gamma_{3k}\beta_{1k}} & \rho_{\gamma_{3k}\gamma_{1k}} & \rho_{\gamma_{3k}\gamma_{2k}} & \sigma^2_{\gamma_{3k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l} \\
          &\gamma_{3l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}} \\
          &\mu_{\gamma_{3l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} & \rho_{\alpha_{l}\gamma_{3l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}} & \rho_{\beta_{1l}\gamma_{3l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\beta_{1l}} & \sigma^2_{\gamma_{3l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1k[i]}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{2k[i]}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{3k[i],l[i]}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k} \\
          &\gamma_{1k} \\
          &\gamma_{2k} \\
          &\gamma_{3k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1l[i]}^{\alpha}(\operatorname{prop\_low}) \\
          &\mu_{\beta_{1k}} \\
          &\mu_{\gamma_{1k}} \\
          &\mu_{\gamma_{2k}} \\
          &\mu_{\gamma_{3k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccccc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} & \rho_{\alpha_{k}\gamma_{1k}} & \rho_{\alpha_{k}\gamma_{2k}} & \rho_{\alpha_{k}\gamma_{3k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}} & \rho_{\beta_{1k}\gamma_{1k}} & \rho_{\beta_{1k}\gamma_{2k}} & \rho_{\beta_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{1k}\alpha_{k}} & \rho_{\gamma_{1k}\beta_{1k}} & \sigma^2_{\gamma_{1k}} & \rho_{\gamma_{1k}\gamma_{2k}} & \rho_{\gamma_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{2k}\alpha_{k}} & \rho_{\gamma_{2k}\beta_{1k}} & \rho_{\gamma_{2k}\gamma_{1k}} & \sigma^2_{\gamma_{2k}} & \rho_{\gamma_{2k}\gamma_{3k}} \\ 
         \rho_{\gamma_{3k}\alpha_{k}} & \rho_{\gamma_{3k}\beta_{1k}} & \rho_{\gamma_{3k}\gamma_{1k}} & \rho_{\gamma_{3k}\gamma_{2k}} & \sigma^2_{\gamma_{3k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l} \\
          &\gamma_{3l} \\
          &\gamma_{1l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}} \\
          &\mu_{\gamma_{3l}} \\
          &\mu_{\gamma_{1l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} & \rho_{\alpha_{l}\gamma_{3l}} & \rho_{\alpha_{l}\gamma_{1l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}} & \rho_{\beta_{1l}\gamma_{3l}} & \rho_{\beta_{1l}\gamma_{1l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\beta_{1l}} & \sigma^2_{\gamma_{3l}} & \rho_{\gamma_{3l}\gamma_{1l}} \\ 
         \rho_{\gamma_{1l}\alpha_{l}} & \rho_{\gamma_{1l}\beta_{1l}} & \rho_{\gamma_{1l}\gamma_{3l}} & \sigma^2_{\gamma_{1l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{2}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{3k[i]}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k} \\
          &\gamma_{3k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{prop\_low}) \\
          &\mu_{\beta_{1k}} \\
          &\mu_{\gamma_{3k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} & \rho_{\alpha_{k}\gamma_{3k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}} & \rho_{\beta_{1k}\gamma_{3k}} \\ 
         \rho_{\gamma_{3k}\alpha_{k}} & \rho_{\gamma_{3k}\beta_{1k}} & \sigma^2_{\gamma_{3k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{dist\_mean}) \\
          &\mu_{\beta_{1l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Interactions work as expected

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1}(\operatorname{minority}) + \beta_{2}(\operatorname{female}) + \beta_{3}(\operatorname{female} \times \operatorname{minority}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]}, \sigma^2 \right) \\    \alpha_{j}  &\sim N \left(\gamma_{0}^{\alpha} + \gamma_{1k[i],l[i]}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) + \gamma_{2l[i]}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{3l[i]}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{4l[i]}^{\alpha}(\operatorname{group}_{\operatorname{low}} \times \operatorname{treatment}_{\operatorname{1}}) + \gamma_{5l[i]}^{\alpha}(\operatorname{group}_{\operatorname{medium}} \times \operatorname{treatment}_{\operatorname{1}}), \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\gamma_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{k}} \\
          &\mu_{\gamma_{1k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\gamma_{1k}} \\ 
         \rho_{\gamma_{1k}\alpha_{k}} & \sigma^2_{\gamma_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\gamma_{1l} \\
          &\gamma_{2l} \\
          &\gamma_{3l} \\
          &\gamma_{4l} \\
          &\gamma_{5l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\gamma_{1l}} \\
          &\mu_{\gamma_{2l}} \\
          &\mu_{\gamma_{3l}} \\
          &\mu_{\gamma_{4l}} \\
          &\mu_{\gamma_{5l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\gamma_{1l}} & \rho_{\alpha_{l}\gamma_{2l}} & \rho_{\alpha_{l}\gamma_{3l}} & \rho_{\alpha_{l}\gamma_{4l}} & \rho_{\alpha_{l}\gamma_{5l}} \\ 
         \rho_{\gamma_{1l}\alpha_{l}} & \sigma^2_{\gamma_{1l}} & \rho_{\gamma_{1l}\gamma_{2l}} & \rho_{\gamma_{1l}\gamma_{3l}} & \rho_{\gamma_{1l}\gamma_{4l}} & \rho_{\gamma_{1l}\gamma_{5l}} \\ 
         \rho_{\gamma_{2l}\alpha_{l}} & \rho_{\gamma_{2l}\gamma_{1l}} & \sigma^2_{\gamma_{2l}} & \rho_{\gamma_{2l}\gamma_{3l}} & \rho_{\gamma_{2l}\gamma_{4l}} & \rho_{\gamma_{2l}\gamma_{5l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\gamma_{1l}} & \rho_{\gamma_{3l}\gamma_{2l}} & \sigma^2_{\gamma_{3l}} & \rho_{\gamma_{3l}\gamma_{4l}} & \rho_{\gamma_{3l}\gamma_{5l}} \\ 
         \rho_{\gamma_{4l}\alpha_{l}} & \rho_{\gamma_{4l}\gamma_{1l}} & \rho_{\gamma_{4l}\gamma_{2l}} & \rho_{\gamma_{4l}\gamma_{3l}} & \sigma^2_{\gamma_{4l}} & \rho_{\gamma_{4l}\gamma_{5l}} \\ 
         \rho_{\gamma_{5l}\alpha_{l}} & \rho_{\gamma_{5l}\gamma_{1l}} & \rho_{\gamma_{5l}\gamma_{2l}} & \rho_{\gamma_{5l}\gamma_{3l}} & \rho_{\gamma_{5l}\gamma_{4l}} & \sigma^2_{\gamma_{5l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{treatment}_{\operatorname{1}})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K} \\    \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1}(\operatorname{wave}), \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) + \gamma_{2}^{\alpha}(\operatorname{treatment}_{\operatorname{1}} \times \operatorname{wave}), \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for school k = 1,} \dots \text{,K} \\
        \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i]} + \beta_{1j[i],k[i],l[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{group}_{\operatorname{low}}) + \gamma_{2}^{\alpha}(\operatorname{group}_{\operatorname{medium}}) + \gamma_{3l[i]}^{\alpha}(\operatorname{treatment}_{\operatorname{1}}) + \gamma_{4}^{\alpha}(\operatorname{group}_{\operatorname{low}} \times \operatorname{treatment}_{\operatorname{1}}) + \gamma_{5}^{\alpha}(\operatorname{group}_{\operatorname{medium}} \times \operatorname{treatment}_{\operatorname{1}}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{group}_{\operatorname{low}}) + \gamma^{\beta_{1}}_{2}(\operatorname{group}_{\operatorname{medium}}) + \gamma^{\beta_{1}}_{3}(\operatorname{treatment}_{\operatorname{1}}) + \gamma^{\beta_{1}}_{4}(\operatorname{group}_{\operatorname{low}} \times \operatorname{treatment}_{\operatorname{1}}) + \gamma^{\beta_{1}}_{5}(\operatorname{group}_{\operatorname{medium}} \times \operatorname{treatment}_{\operatorname{1}})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{prop\_low}) + \gamma_{2}^{\alpha}(\operatorname{prop\_low} \times \operatorname{treatment}_{\operatorname{1}}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{2}(\operatorname{prop\_low}) + \gamma^{\beta_{1}}_{1}(\operatorname{prop\_low} \times \operatorname{treatment}_{\operatorname{1}})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{l} \\
          &\beta_{1l} \\
          &\gamma_{3l}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{l}} \\
          &\mu_{\beta_{1l}} \\
          &\mu_{\gamma_{3l}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{l}} & \rho_{\alpha_{l}\beta_{1l}} & \rho_{\alpha_{l}\gamma_{3l}} \\ 
         \rho_{\beta_{1l}\alpha_{l}} & \sigma^2_{\beta_{1l}} & \rho_{\beta_{1l}\gamma_{3l}} \\ 
         \rho_{\gamma_{3l}\alpha_{l}} & \rho_{\gamma_{3l}\beta_{1l}} & \sigma^2_{\gamma_{3l}}
      \end{array}
    \right)
     \right)
        \text{, for district l = 1,} \dots \text{,L}
    \end{aligned}
    $$

# Alternate random effect VCV structures work

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{minority}) + \beta_{2j[i]}(\operatorname{female}) + \beta_{3j[i]}(\operatorname{female} \times \operatorname{minority}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j} \\
          &\beta_{3j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}} \\
          &\mu_{\beta_{2j}} \\
          &\mu_{\beta_{3j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccc}
         \sigma^2_{\alpha_{j}} & 0 & 0 & 0 \\ 
         0 & \sigma^2_{\beta_{1j}} & 0 & 0 \\ 
         0 & 0 & \sigma^2_{\beta_{2j}} & 0 \\ 
         0 & 0 & 0 & \sigma^2_{\beta_{3j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{math}_{i}  &\sim N \left(\mu, \sigma^2 \right) \\
        \mu &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{minority}) + \beta_{2j[i]}(\operatorname{female}) + \beta_{3j[i]}(\operatorname{female} \times \operatorname{minority}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j} \\
          &\beta_{3j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}} \\
          &\mu_{\beta_{2j}} \\
          &\mu_{\beta_{3j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cccc}
         \sigma^2_{\alpha_{j}} & 0 & 0 & 0 \\ 
         0 & \sigma^2_{\beta_{1j}} & 0 & 0 \\ 
         0 & 0 & \sigma^2_{\beta_{2j}} & 0 \\ 
         0 & 0 & 0 & \sigma^2_{\beta_{3j}}
      \end{array}
    \right)
     \right)
        \text{, for sch.id j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i],l[i],m[i]} + \beta_{1j[i],k[i],m[i]}(\operatorname{wave}), \sigma^2 \right) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & 0 \\ 
         0 & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for sid j = 1,} \dots \text{,J} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{k} \\
          &\beta_{1k}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{k}} \\
          &\mu_{\beta_{1k}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{k}} & \rho_{\alpha_{k}\beta_{1k}} \\ 
         \rho_{\beta_{1k}\alpha_{k}} & \sigma^2_{\beta_{1k}}
      \end{array}
    \right)
     \right)
        \text{, for school k = 1,} \dots \text{,K} \\    \alpha_{l}  &\sim N \left(\mu_{\alpha_{l}}, \sigma^2_{\alpha_{l}} \right)
        \text{, for school.1 l = 1,} \dots \text{,L} \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{m} \\
          &\beta_{1m}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{m}} \\
          &\mu_{\beta_{1m}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{m}} & 0 \\ 
         0 & \sigma^2_{\beta_{1m}}
      \end{array}
    \right)
     \right)
        \text{, for district m = 1,} \dots \text{,M}
    \end{aligned}
    $$

# Nested model syntax works

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for school:sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for sid k = 1,} \dots \text{,K}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for school:sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for sid k = 1,} \dots \text{,K}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i],k[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for school:sid j = 1,} \dots \text{,J} \\
        \alpha_{k}  &\sim N \left(\mu_{\alpha_{k}}, \sigma^2_{\alpha_{k}} \right)
        \text{, for sid k = 1,} \dots \text{,K}
    \end{aligned}
    $$

# font-size changes, lm

    $$
    \large
    \begin{aligned}
    \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    \end{aligned}
    $$

---

    $$
    \Huge
    \begin{aligned}
    \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    \end{aligned}
    $$

---

    $$
    \tiny
    \begin{aligned}
    \operatorname{mpg} &= \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    \end{aligned}
    $$

# font-size changes, lmer

    $$
    \scriptsize
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \Large
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \huge
    \begin{aligned}
      \operatorname{score}_{i}  &\sim N \left(\alpha_{j[i]}, \sigma^2 \right) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for sid j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# font-size changes, arima

    $$
    \footnotesize
    \begin{aligned}
    (1 -\phi_{1}\operatorname{B} )\ (1 -\Phi_{1}\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) (y_{t} -\delta\operatorname{t}) = (1 +\theta_{1}\operatorname{B} )\ (1 +\Theta_{1}\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t}
    \end{aligned}
    $$

---

    $$
    \small
    \begin{aligned}
    (1 -\phi_{1}\operatorname{B} )\ (1 -\Phi_{1}\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) (y_{t} -\delta\operatorname{t}) = (1 +\theta_{1}\operatorname{B} )\ (1 +\Theta_{1}\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t}
    \end{aligned}
    $$

---

    $$
    \LARGE
    \begin{aligned}
    (1 -\phi_{1}\operatorname{B} )\ (1 -\Phi_{1}\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) (y_{t} -\delta\operatorname{t}) = (1 +\theta_{1}\operatorname{B} )\ (1 +\Theta_{1}\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t}
    \end{aligned}
    $$

# Overlapping names don't result in an error

    $$
    \begin{aligned}
    \operatorname{\widehat{body\_mass\_g}} &= -36659.51\ + \\
    &\quad 1813.23(\operatorname{p})\ + \\
    &\quad 197.51(\operatorname{gender\_cat})\ + \\
    &\quad 7.8(\operatorname{ca})\ - \\
    &\quad 8.82(\operatorname{p} \times \operatorname{gender\_cat})
    \end{aligned}
    $$

# Dropping intercept notation works

    $$
    \operatorname{bill\_depth\_mm} = \beta_{1}(\operatorname{flipper\_length\_mm}) + \beta_{2}(\operatorname{island}_{\operatorname{Biscoe}}) + \beta_{3}(\operatorname{island}_{\operatorname{Dream}}) + \beta_{4}(\operatorname{island}_{\operatorname{Torgersen}}) + \beta_{5}(\operatorname{flipper\_length\_mm} \times \operatorname{island}_{\operatorname{Dream}}) + \beta_{6}(\operatorname{flipper\_length\_mm} \times \operatorname{island}_{\operatorname{Torgersen}}) + \epsilon
    $$

# colorizing works

    $$
    \begin{aligned}
    {\color{#0f70f7}{\operatorname{bill\_depth\_mm}}} &= {\color{#1B9E77}{\alpha}} + {\color{#D95F02}{\beta}}_{{\color{#E6AB02}{1}}}({\color{#b22222}{\operatorname{Flipper\ Length\ (MM)}}})\ + \\
    &\quad {\color{#7570B3}{\beta}}_{{\color{#66A61E}{2}}}({\color{green}{\operatorname{ISLAND}}}{\color{cyan}{_{\operatorname{super\ dreamy}}}}) + {\color{#E7298A}{\beta}}_{{\color{#E7298A}{3}}}({\color{green}{\operatorname{ISLAND}}}{\color{cyan}{_{\operatorname{Torgersen}}}})\ + \\
    &\quad {\color{#66A61E}{\beta}}_{{\color{#7570B3}{4}}}({\color{#b22222}{\operatorname{Flipper\ Length\ (MM)}}} \times {\color{green}{\operatorname{ISLAND}}}{\color{cyan}{_{\operatorname{super\ dreamy}}}}) + {\color{#E6AB02}{\beta}}_{{\color{#D95F02}{5}}}({\color{#b22222}{\operatorname{Flipper\ Length\ (MM)}}} \times {\color{green}{\operatorname{ISLAND}}}{\color{cyan}{_{\operatorname{Torgersen}}}})\ + \\
    &\quad {\color{#A6761D}{\epsilon}}
    \end{aligned}
    $$

# Renaming Variables works

    $$
    \operatorname{body\_mass\_g} = \alpha + \beta_{1}(\operatorname{Bill\ Length\ (MM)}) + \beta_{2}(\operatorname{species}_{\operatorname{chinny\ chin\ chin}}) + \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}}) + \beta_{4}(\operatorname{Flipper\ Length\ (MM)}) + \beta_{5}(\operatorname{SEX}_{\operatorname{Male}}) + \beta_{6}(\operatorname{Bill\ Length\ (MM)} \times \operatorname{species}_{\operatorname{chinny\ chin\ chin}}) + \beta_{7}(\operatorname{Bill\ Length\ (MM)} \times \operatorname{species}_{\operatorname{Gentoo}}) + \beta_{8}(\operatorname{Flipper\ Length\ (MM)} \times \operatorname{SEX}_{\operatorname{Male}}) + \epsilon
    $$

# Math extraction works

    $$
    \operatorname{bill\_length\_mm} = \alpha + \beta_{1}(\operatorname{bill\_depth\_mm}) + \beta_{2}(\operatorname{bill\_depth\_mm^2}) + \beta_{3}(\operatorname{bill\_depth\_mm^3}) + \beta_{4}(\operatorname{bill\_depth\_mm^4}) + \beta_{5}(\operatorname{bill\_depth\_mm^5}) + \beta_{6}(\operatorname{\log(flipper\_length\_mm)}) + \beta_{7}(\operatorname{\exp(bill\_length\_mm)}) + \epsilon
    $$

---

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{hp\ >\ 150}) + \epsilon
    $$

---

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{hp\ <\ 250}) + \epsilon
    $$

# Collapsing lm factors works

    Code
      extract_eq(m)
    Output
      $$
      \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{gear}_{\operatorname{4}}) + \beta_{2}(\operatorname{gear}_{\operatorname{5}}) + \beta_{3}(\operatorname{carb}_{\operatorname{2}}) + \beta_{4}(\operatorname{carb}_{\operatorname{3}}) + \beta_{5}(\operatorname{carb}_{\operatorname{4}}) + \beta_{6}(\operatorname{carb}_{\operatorname{6}}) + \beta_{7}(\operatorname{carb}_{\operatorname{8}}) + \beta_{8}(\operatorname{disp}) + \beta_{9}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{2}}) + \beta_{10}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{2}}) + \beta_{11}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{3}}) + \beta_{12}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{3}}) + \beta_{13}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{4}}) + \beta_{14}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{4}}) + \beta_{15}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{6}}) + \beta_{16}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{6}}) + \beta_{17}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{8}}) + \beta_{18}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{8}}) + \beta_{19}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{disp}) + \beta_{20}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{disp}) + \beta_{21}(\operatorname{carb}_{\operatorname{2}} \times \operatorname{disp}) + \beta_{22}(\operatorname{carb}_{\operatorname{3}} \times \operatorname{disp}) + \beta_{23}(\operatorname{carb}_{\operatorname{4}} \times \operatorname{disp}) + \beta_{24}(\operatorname{carb}_{\operatorname{6}} \times \operatorname{disp}) + \beta_{25}(\operatorname{carb}_{\operatorname{8}} \times \operatorname{disp}) + \beta_{26}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{2}} \times \operatorname{disp}) + \beta_{27}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{2}} \times \operatorname{disp}) + \beta_{28}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{3}} \times \operatorname{disp}) + \beta_{29}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{3}} \times \operatorname{disp}) + \beta_{30}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{4}} \times \operatorname{disp}) + \beta_{31}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{4}} \times \operatorname{disp}) + \beta_{32}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{6}} \times \operatorname{disp}) + \beta_{33}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{6}} \times \operatorname{disp}) + \beta_{34}(\operatorname{gear}_{\operatorname{4}} \times \operatorname{carb}_{\operatorname{8}} \times \operatorname{disp}) + \beta_{35}(\operatorname{gear}_{\operatorname{5}} \times \operatorname{carb}_{\operatorname{8}} \times \operatorname{disp}) + \epsilon
      $$

---

    Code
      extract_eq(m, index_factors = TRUE)
    Output
      $$
      \operatorname{mpg} = \alpha + \operatorname{gear}_{\operatorname{i}} + \operatorname{carb}_{\operatorname{j}} + \operatorname{disp} + \left(\operatorname{gear}_{\operatorname{i}} \times \operatorname{carb}_{\operatorname{j}}\right) + \left(\operatorname{gear}_{\operatorname{i}} \times \operatorname{disp}\right) + \left(\operatorname{carb}_{\operatorname{j}} \times \operatorname{disp}\right) + \left(\operatorname{gear}_{\operatorname{i}} \times \operatorname{carb}_{\operatorname{j}} \times \operatorname{disp}\right) + \epsilon
      $$

# Labeling works

    $$
    \label{eq: mpg_mod}
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    $$

# Simple lm models work

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    $$

---

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{gear}_{\operatorname{4}}) + \beta_{3}(\operatorname{gear}_{\operatorname{5}}) + \epsilon
    $$

---

    $$
    \operatorname{\widehat{mpg}} = 34.66 - 1.59(\operatorname{cyl}) - 0.02(\operatorname{disp})
    $$

# Interactions work

    $$
    \operatorname{body\_mass\_g} = \alpha + \beta_{1}(\operatorname{bill\_length\_mm}) + \beta_{2}(\operatorname{species}_{\operatorname{Chinstrap}}) + \beta_{3}(\operatorname{species}_{\operatorname{Gentoo}}) + \beta_{4}(\operatorname{bill\_length\_mm} \times \operatorname{species}_{\operatorname{Chinstrap}}) + \beta_{5}(\operatorname{bill\_length\_mm} \times \operatorname{species}_{\operatorname{Gentoo}}) + \epsilon
    $$

---

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{hp}) + \beta_{2}(\operatorname{wt}) + \beta_{3}(\operatorname{hp} \times \operatorname{wt}) + \epsilon
    $$

# Custom Greek works

    $$
    \operatorname{mpg} = \alpha + \hat{\beta}_{1}(\operatorname{cyl}) + \hat{\beta}_{2}(\operatorname{disp}) + \epsilon
    $$

---

    $$
    \operatorname{mpg} = \zeta + \beta_{1}(\operatorname{cyl}) + \beta_{2}(\operatorname{disp}) + \epsilon
    $$

# Hat is escaped correctly

    $$
    \operatorname{mpg} = \alpha + \beta_{1}(\operatorname{carb}_{\operatorname{.L}}) + \beta_{2}(\operatorname{carb}_{\operatorname{.Q}}) + \beta_{3}(\operatorname{carb}_{\operatorname{.C}}) + \beta_{4}(\operatorname{carb}_{\operatorname{\texttt{\^{}}4}}) + \beta_{5}(\operatorname{carb}_{\operatorname{\texttt{\^{}}5}}) + \epsilon
    $$

# colorizing works

    $$
    \begin{aligned}
      {\color{blue}{\operatorname{Stops}}}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\alpha_{j[i]} + \beta_{1}({\color{red}{\operatorname{Ethnicity}}}{\color{purple}{_{\operatorname{hispanic}}}}) + \beta_{2}({\color{red}{\operatorname{Ethnicity}}}{\color{purple}{_{\operatorname{white}}}}) \\
        \alpha_{j}  &\sim N \left(\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{Total\ Arrests}), \sigma^2_{\alpha_{j}} \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Renaming Variables works

    $$
    \begin{aligned}
      \operatorname{stops}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\alpha_{j[i]} + \beta_{1}(\operatorname{Ethnicity}_{\operatorname{Hispanic/Latino}}) + \beta_{2}(\operatorname{Ethnicity}_{\operatorname{White}}) \\
        \alpha_{j}  &\sim N \left(\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{Total\ Arrests}), \sigma^2_{\alpha_{j}} \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Standard Poisson regression models work

    $$
    \begin{aligned}
      \operatorname{stops}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\alpha_{j[i]} + \beta_{1}(\operatorname{eth}_{\operatorname{hispanic}}) + \beta_{2}(\operatorname{eth}_{\operatorname{white}}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{stops}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{eth}_{\operatorname{hispanic}}) + \beta_{2j[i]}(\operatorname{eth}_{\operatorname{white}}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{total\_arrests}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{total\_arrests}) \\
          &\gamma^{\beta_{2}}_{0} + \gamma^{\beta_{2}}_{1}(\operatorname{total\_arrests})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} & \rho_{\alpha_{j}\beta_{2j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} \\ 
         \rho_{\beta_{2j}\alpha_{j}} & \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}}
      \end{array}
    \right)
     \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Poisson regression models with an offset work

    $$
    \begin{aligned}
      \operatorname{stops}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\log(\operatorname{arrests}) + \alpha_{j[i]} + \beta_{1}(\operatorname{eth}_{\operatorname{hispanic}}) + \beta_{2}(\operatorname{eth}_{\operatorname{white}}) \\
        \alpha_{j}  &\sim N \left(\mu_{\alpha_{j}}, \sigma^2_{\alpha_{j}} \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
      \operatorname{stops}_{i}  &\sim \operatorname{Poisson}(\lambda_i) \\
        \log(\lambda_i) &=\log(\operatorname{arrests}) + \alpha_{j[i]} + \beta_{1j[i]}(\operatorname{eth}_{\operatorname{hispanic}}) + \beta_{2j[i]}(\operatorname{eth}_{\operatorname{white}}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j} \\
          &\beta_{2j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\gamma_{0}^{\alpha} + \gamma_{1}^{\alpha}(\operatorname{total\_arrests}) \\
          &\gamma^{\beta_{1}}_{0} + \gamma^{\beta_{1}}_{1}(\operatorname{total\_arrests}) \\
          &\gamma^{\beta_{2}}_{0} + \gamma^{\beta_{2}}_{1}(\operatorname{total\_arrests})
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{ccc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} & \rho_{\alpha_{j}\beta_{2j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}} & \rho_{\beta_{1j}\beta_{2j}} \\ 
         \rho_{\beta_{2j}\alpha_{j}} & \rho_{\beta_{2j}\beta_{1j}} & \sigma^2_{\beta_{2j}}
      \end{array}
    \right)
     \right)
        \text{, for precinct j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# Binomial Logistic Regression models work

    $$
    \begin{aligned}
      \operatorname{bush}_{i}  &\sim \operatorname{Binomial}(n = 1, \operatorname{prob}_{\operatorname{bush} = 1} = \widehat{P}) \\
        \log\left[\frac{\hat{P}}{1 - \hat{P}} \right] &=\alpha_{j[i]} + \beta_{1j[i]}(\operatorname{black}) + \beta_{2}(\operatorname{female}) + \beta_{3}(\operatorname{edu}) \\    
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\alpha_{j} \\
          &\beta_{1j}
        \end{aligned}
      \end{array}
    \right)
      &\sim N \left(
    \left(
      \begin{array}{c} 
        \begin{aligned}
          &\mu_{\alpha_{j}} \\
          &\mu_{\beta_{1j}}
        \end{aligned}
      \end{array}
    \right)
    , 
    \left(
      \begin{array}{cc}
         \sigma^2_{\alpha_{j}} & \rho_{\alpha_{j}\beta_{1j}} \\ 
         \rho_{\beta_{1j}\alpha_{j}} & \sigma^2_{\beta_{1j}}
      \end{array}
    \right)
     \right)
        \text{, for state j = 1,} \dots \text{,J}
    \end{aligned}
    $$

# se_subscripts argument works with numeric coefficients

    $$
    \operatorname{\widehat{disp}} = \underset{(116.748)}{67.8}  + \underset{(9.962)}{45.56(\operatorname{cyl})} - \underset{(2.952)}{5.92(\operatorname{mpg})}
    $$

---

    $$
    E( \operatorname{disp} ) = \underset{(116.748)}{\alpha} + \underset{(9.962)}{\beta_{1}(\operatorname{cyl})} + \underset{(2.952)}{\beta_{2}(\operatorname{mpg})}
    $$

---

    $$
    \log\left[ \frac { P( \operatorname{am} = \operatorname{1} ) }{ 1 - P( \operatorname{am} = \operatorname{1} ) } \right] = \underset{(12.194)}{\alpha} + \underset{(0.239)}{\beta_{1}(\operatorname{mpg})} + \underset{(2.547)}{\beta_{2}(\operatorname{wt})}
    $$

# se_subscripts argument works with wrapping

    $$
    \begin{aligned}
    \operatorname{\widehat{disp}} &= \underset{(119.073)}{60.8}\  + \\
    &\quad \underset{(11.801)}{42.46(\operatorname{cyl})}\ - \\
    &\quad \underset{(3.073)}{5.56(\operatorname{mpg})}\ + \\
    &\quad \underset{(0.255)}{0.13(\operatorname{hp})}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \widehat{E( \operatorname{disp} )} &= \underset{(116.748)}{67.8}\  + \\
    &\quad \underset{(9.962)}{45.56(\operatorname{cyl})}\ - \\
    &\quad \underset{(2.952)}{5.92(\operatorname{mpg})}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \widehat{E( \operatorname{disp} )} &= \underset{(116.748)}{67.8}\  + \\
    &\quad \underset{(9.962)}{45.56(\operatorname{cyl})}\ - \\
    &\quad \underset{(2.952)}{5.92(\operatorname{mpg})}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \log\left[ \frac { \widehat{P( \operatorname{am} = \operatorname{1} )} }{ 1 - \widehat{P( \operatorname{am} = \operatorname{1} )} } \right] &= \underset{(12.194)}{25.89}\  - \\
    &\quad \underset{(0.239)}{0.32(\operatorname{mpg})}\ - \\
    &\quad \underset{(2.547)}{6.42(\operatorname{wt})}
    \end{aligned}
    $$

# se_subscripts argument works with Greek letter terms

    $$
    \operatorname{disp} = \underset{(119.073)}{\alpha} + \underset{(11.801)}{\beta_{1}(\operatorname{cyl})} + \underset{(3.073)}{\beta_{2}(\operatorname{mpg})} + \underset{(0.255)}{\beta_{3}(\operatorname{hp})} + \epsilon
    $$

---

    $$
    \begin{aligned}
    \operatorname{disp} &= \underset{(119.073)}{\alpha}\ + \\
    &\quad \underset{(11.801)}{\beta_{1}(\operatorname{cyl})}\ + \\
    &\quad \underset{(3.073)}{\beta_{2}(\operatorname{mpg})}\ + \\
    &\quad \underset{(0.255)}{\beta_{3}(\operatorname{hp})}\ + \\
    &\quad \epsilon
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    E( \operatorname{disp} ) &= \underset{(116.748)}{\alpha}\ + \\
    &\quad \underset{(9.962)}{\beta_{1}(\operatorname{cyl})}\ + \\
    &\quad \underset{(2.952)}{\beta_{2}(\operatorname{mpg})}
    \end{aligned}
    $$

---

    $$
    \begin{aligned}
    \log\left[ \frac { P( \operatorname{am} = \operatorname{1} ) }{ 1 - P( \operatorname{am} = \operatorname{1} ) } \right] &= \underset{(12.194)}{\alpha}\ + \\
    &\quad \underset{(0.239)}{\beta_{1}(\operatorname{mpg})}\ + \\
    &\quad \underset{(2.547)}{\beta_{2}(\operatorname{wt})}
    \end{aligned}
    $$

# se_subscripts argument works with transformed variables

    $$
    \operatorname{\widehat{disp}} = \underset{(226.662)}{481.42}  + \underset{(10.942)}{37.32(\operatorname{cyl})} - \underset{(59.243)}{165.48(\operatorname{\log(mpg)})} + \underset{(0.241)}{0.05(\operatorname{hp})}
    $$

---

    $$
    \begin{aligned}
    \operatorname{\widehat{disp}} &= \underset{(226.662)}{481.42}\  + \\
    &\quad \underset{(10.942)}{37.32(\operatorname{cyl})}\ - \\
    &\quad \underset{(59.243)}{165.48(\operatorname{\log(mpg)})}\ + \\
    &\quad \underset{(0.241)}{0.05(\operatorname{hp})}
    \end{aligned}
    $$

---

    $$
    \widehat{E( \operatorname{disp} )} = \underset{(226.662)}{481.42}  + \underset{(10.942)}{37.32(\operatorname{cyl})} - \underset{(59.243)}{165.48(\operatorname{\log(mpg)})} + \underset{(0.241)}{0.05(\operatorname{hp})}
    $$

---

    $$
    \begin{aligned}
    \widehat{E( \operatorname{disp} )} &= \underset{(226.662)}{481.42}\  + \\
    &\quad \underset{(10.942)}{37.32(\operatorname{cyl})}\ - \\
    &\quad \underset{(59.243)}{165.48(\operatorname{\log(mpg)})}\ + \\
    &\quad \underset{(0.241)}{0.05(\operatorname{hp})}
    \end{aligned}
    $$

# se_subscripts works with factor variable

    $$
    \widehat{E( \operatorname{disp} )} = \underset{(224.642)}{427.66}  + \underset{(12.253)}{21.95(\operatorname{cyl})} - \underset{(61.235)}{120.45(\operatorname{\log(mpg)})} + \underset{(0.32)}{0.38(\operatorname{hp})} - \underset{(26.627)}{58.06(\operatorname{factor(gear)}_{\operatorname{4}})} - \underset{(35.762)}{66.88(\operatorname{factor(gear)}_{\operatorname{5}})}
    $$

---

    $$
    E( \operatorname{disp} ) = \underset{(224.642)}{\alpha} + \underset{(12.253)}{\beta_{1}(\operatorname{cyl})} + \underset{(61.235)}{\beta_{2}(\operatorname{\log(mpg)})} + \underset{(0.32)}{\beta_{3}(\operatorname{hp})} + \underset{(26.627)}{\beta_{4}(\operatorname{factor(gear)}_{\operatorname{4}})} + \underset{(35.762)}{\beta_{5}(\operatorname{factor(gear)}_{\operatorname{5}})}
    $$

# Basic ARIMA model functions

    $$
    (1 -\phi_{1}\operatorname{B} )\ (1 -\Phi_{1}\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) (y_{t} -\delta\operatorname{t}) = (1 +\theta_{1}\operatorname{B} )\ (1 +\Theta_{1}\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t}
    $$

---

    $$
    (1 -0.004\operatorname{B} )\ (1 -0.39\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) (y_{t} -3e-05\operatorname{t}) = (1 -1\operatorname{B} )\ (1 -0.36\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t}
    $$

# Regression w/ ARIMA Errors functions

    $$
    \begin{alignat}{2}
    &\text{let}\quad &&y_{t} = \operatorname{y}_{\operatorname{0}} +\delta\operatorname{t} +\beta_{1}\operatorname{x1}_{\operatorname{t}} +\beta_{2}\operatorname{x2}_{\operatorname{t}} +\eta_{t} \\
    &\text{where}\quad  &&(1 -\phi_{1}\operatorname{B} )\ (1 -\Phi_{1}\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) \eta_{t}  \\
    & &&= (1 +\theta_{1}\operatorname{B} )\ (1 +\Theta_{1}\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t} \\
    &\text{where}\quad &&\varepsilon_{t} \sim{WN(0, \sigma^{2})}
    \end{alignat}
    $$

---

    $$
    \begin{alignat}{2}
    &\text{let}\quad &&y_{t} = \operatorname{y}_{\operatorname{0}} -1e-04\operatorname{t} -0.02\operatorname{x1}_{\operatorname{t}} -0.004\operatorname{x2}_{\operatorname{t}} +\eta_{t} \\
    &\text{where}\quad  &&(1 -0.01\operatorname{B} )\ (1 +0.78\operatorname{B}^{\operatorname{4}} )\ (1 - \operatorname{B}) \eta_{t}  \\
    & &&= (1 -1\operatorname{B} )\ (1 +0.82\operatorname{B}^{\operatorname{4}} )\ \varepsilon_{t} \\
    &\text{where}\quad &&\varepsilon_{t} \sim{WN(0, \sigma^{2})}
    \end{alignat}
    $$

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equation.R
\name{equation}
\alias{equation}
\alias{equation.default}
\alias{equation.character}
\alias{eq__}
\alias{eq_}
\alias{print.inline_equation}
\title{Get a 'LaTeX' equation from a model or from 'LaTeX' code in a R Markdown or Quarto document}
\usage{
equation(object, ...)

\method{equation}{default}(
  object,
  auto.labs = TRUE,
  origdata = NULL,
  labs = NULL,
  swap_var_names = NULL,
  ...
)

\method{equation}{character}(object, ...)

eq__(object, ...)

eq_(object, ...)

\method{print}{inline_equation}(x, ...)
}
\arguments{
\item{object}{An object with a model whose the equation is constructed. If a
\strong{character} object is provided, it is concatenated into a single
character string and the \strong{equation} class, otherwise it is not
transformed (it is supposed to be a valid 'LaTeX' equation). Remember that
backslashes must be doubled in regular R strings, or use the \code{r"(...)"}
notation, which may be more comfortable here, see examples.}

\item{...}{Further parameters passed to \code{\link[=extract_eq]{extract_eq()}}.}

\item{auto.labs}{If \code{TRUE} (by default), use labels (and units) automatically
from data or \verb{origdata=}.}

\item{origdata}{The original data set this model was fitted to. By default it
is \code{NULL} and no label is used.}

\item{labs}{Labels to change the names of elements in the \code{term} column of
the table. By default it is \code{NULL} and no term is changed.}

\item{swap_var_names}{Change the variable names for these values, regardless
of the values in \verb{auto.labs=} or \verb{labs=} that are ignored if this argument
is used. Provide a named character string with name being the variables and
strings the new names. You can use \code{^} or \verb{_} to indicate next character,
or next integer should be super- or subscript in the equation.}

\item{x}{An \strong{inline_equation} object generated by \code{eq_()} or \code{eq__()}.}
}
\value{
An object of class \code{c("equation", "character")}.
}
\description{
Extract or create a 'LaTeX' equation to describe a model, or directly from
'LaTeX' code. All objects supported by \code{\link[=extract_eq]{extract_eq()}} are also supported by
the default method description. If labels (and units) are defined as
attributes to the data, they are automatically used in the equation in place
of the variable names.
}
\details{
There are slight differences between \code{equation()}, \code{eq_()} and \code{eq__()}:
\itemize{
\item \code{equation()} returns a string with 'LaTeX' code and prints 'LaTeX' code at
the R console.
\item \code{eq_()} returns the 'LaTeX' code surrounded by a dollar sign \verb{$...$} and is
suitable to build inline equations in R Markdown/Quarto documents by using
inline R code. It prints the rendered inline equation in the RStudio
Viewer or in the browser. So it is advised to rapidly preview the
resulting equation.
\item \code{eq__()} returns the 'LaTeX' code \emph{not} surrounded by dollar signs in a
single character string. It also prints the rendered inline equation in
the RStudio Viewer or in the browser.
It should be used in an R inline chunk inside a \verb{$$...$$}
construct in a Markdown text. The result is a display equation that can
also be cross referenced in Quarto in the usual way if you label it, e.g.,
you use \verb{$$...$$ \{#eq-label\}}.
}
}
\examples{
iris_lm <- lm(data = iris, Petal.Length ~ Sepal.Length + Species)
summary(iris_lm)
equation(iris_lm)
# Providing directly the LaTeX code of the equation (variance of a sample)
equation("S^2_y = \\\\sum_{i=1}^{n-1} \\\\frac{(y_i - \\\\bar{Y})^2}{n}")
# Easier to write like this (avoiding the double backslashes):
eq1 <- equation(r"(S^2_y = \sum_{i=1}^{n-1} \frac{(y_i - \bar{Y})^2}{n})")
# Print raw text:
eq1
# Get a preview of the equation
if (interactive())
  eq__(eq1)
# The same function can be used inside a `$$...$$ {#eq-label}` construct in
# R Markdown or Quarto to calcule a display equation that is also recognized
# by the cross referencing system of Quarto.
# Get a string suitable for inclusion inline in R Markdown with `r eq__(eq1)`
# (inside Markdown text and without the dollar signs around it)
# For inline equations in a Markdown text, you are better to use `r eq_(eq1)`
if (interactive())
  eq_(eq1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penguins.R
\docType{data}
\name{penguins}
\alias{penguins}
\title{Size measurements for adult foraging penguins near Palmer Station, Antarctica}
\format{
A tibble with 344 rows and 8 variables:
\describe{
\item{species}{a factor denoting penguin species (Adélie, Chinstrap and Gentoo)}
\item{island}{a factor denoting island in Palmer Archipelago, Antarctica (Biscoe, Dream or Torgersen)}
\item{bill_length_mm}{a number denoting bill length (millimeters)}
\item{bill_depth_mm}{a number denoting bill depth (millimeters)}
\item{flipper_length_mm}{an integer denoting flipper length (millimeters)}
\item{body_mass_g}{an integer denoting body mass (grams)}
\item{sex}{a factor denoting penguin sex (female, male)}
\item{year}{an integer denoting the study year (2007, 2008, or 2009)}
}
}
\source{
{Adélie penguins: Palmer Station Antarctica LTER and K. Gorman. 2020.
Structural size measurements and isotopic signatures of foraging among
adult male and female Adélie penguins (Pygoscelis adeliae) nesting along
the Palmer Archipelago near Palmer Station, 2007-2009 ver 5. Environmental
Data Initiative} \doi{10.6073/pasta/98b16d7d563f265cb52372c8ca99e60f}

{Gentoo penguins: Palmer Station Antarctica LTER and K. Gorman. 2020.
Structural size measurements and isotopic signatures of foraging among
adult male and female Gentoo penguin (Pygoscelis papua) nesting along the
Palmer Archipelago near Palmer Station, 2007-2009 ver 5. Environmental
Data Initiative} \doi{10.6073/pasta/7fca67fb28d56ee2ffa3d9370ebda689}

{Chinstrap penguins: Palmer Station Antarctica LTER and K. Gorman.
2020. Structural size measurements and isotopic signatures of foraging
among adult male and female Chinstrap penguin (Pygoscelis antarcticus)
nesting along the Palmer Archipelago near Palmer Station, 2007-2009 ver 6.
Environmental Data Initiative} \doi{10.6073/pasta/c14dfcfada8ea13a17536e73eb6fbe9e}

{Originally published in: Gorman KB, Williams TD, Fraser WR (2014)
Ecological Sexual Dimorphism and Environmental Variability within a
Community of Antarctic Penguins (Genus Pygoscelis). PLoS ONE 9(3): e90081.}
\doi{10.1371/journal.pone.0090081}
}
\usage{
penguins
}
\description{
Data originally from \href{https://allisonhorst.github.io/palmerpenguins/}{\code{palmerpenguins}}. Includes
measurements for penguin species, island in Palmer Archipelago,
size (flipper length, body mass, bill dimensions), and sex.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ts_reg_list.R
\docType{data}
\name{ts_reg_list}
\alias{ts_reg_list}
\title{Simulated data for time-series regression}
\format{
A tibble with 1000 rows and 8 variables:
\describe{
\item{x1}{Random normal simulated data.}
\item{x2}{Random normal simulated data.}
\item{ts_rnorm}{Random normal simulated data.}
}
}
\usage{
ts_reg_list
}
\description{
Output from \code{set.seed(42); ts_reg_list <- list(x1 = rnorm(1000), x2 = rnorm(1000), ts_rnorm = rnorm(1000))}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hsb.R
\docType{data}
\name{hsb}
\alias{hsb}
\title{A subset of the full 1982 High School and Beyond Survey}
\format{
A tibble with 7185 rows and 8 variables:
\describe{
\item{sch.id}{An integer denoting the school identification number.
There are 160 unique schools.}
\item{math}{Individual students' math score.}
\item{size}{The number of students in the school.}
\item{sector}{A dummy variable (integer) denoting whether the school is
public (sector = 0) or catholic (sector = 1). There are 90 public schools
and 70 catholic.}
\item{meanses}{A group-mean centered SES variable at the school level}
\item{minority}{A dummy variable indicating if the student was coded as
white (minority = 0) or not (minority = 1).}
\item{female}{A dummy variable indicating if the student was coded as
female (female = 1) or not (female = 0).}
\item{ses}{A student-level composite variable indicating the students'
socio-economic status.}
}
}
\usage{
hsb
}
\description{
This is the dataset used throughout Raudenbush & Bryk (2002).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{preview_eq}
\alias{preview_eq}
\title{Preview an equation in a web browser}
\usage{
preview_eq(x, ...)
}
\arguments{
\item{x}{'LaTeX' equation built with \code{\link{extract_eq}}}

\item{...}{not used}
}
\value{
The path to the temporary html file that was created to preview the
equation is returned invisibly.
}
\description{
Preview 'LaTeX' equations built with \code{\link{extract_eq}}.
}
\examples{
if (interactive()) {
  mod1 <- lm(mpg ~ cyl + disp, mtcars)
  eq1 <- extract_eq(mod1)
  eq1 # Not that nice
  preview_eq(eq1)
  # or easier...
  preview_eq(mod1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{format.equation}
\alias{format.equation}
\title{Format 'LaTeX' equations}
\usage{
\method{format}{equation}(x, ..., latex = knitr::is_latex_output())
}
\arguments{
\item{x}{'LaTeX' equation built with \code{\link{extract_eq}}}

\item{...}{not used}

\item{latex}{Logical, whether the output is LaTeX or not. The default
value uses \code{\link[knitr:output_type]{knitr::is_latex_output()}} to determine the current output format.}
}
\value{
A character string with the equation formatted either as proper
LaTeX code, or as a display equation tag (surrounded by \verb{$$...$$}) for R
Markdown or Quarto documents.
}
\description{
Format 'LaTeX' equations built with \code{\link{extract_eq}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrests.R
\docType{data}
\name{arrests}
\alias{arrests}
\title{Arrest data from Gelman & Hill}
\format{
A tibble with 225 rows and 4 variables:
\describe{
\item{precinct}{An integer denoting the precinct identification number.}
\item{eth}{A factor with the coded race/ethnicity}
\item{stops}{The number of police stops}
\item{arrests}{The number of prior arrests (this is used as an offset variable in the book)}
}
}
\usage{
arrests
}
\description{
Arrest data from Gelman & Hill's book, used in Chapter 6 (and others).
The data have been aggregated by precinct and race/ethnicity, with the
sum of prior arrests and stops calculated. You can download the original
data here: http://www.stat.columbia.edu/~gelman/arm/examples/police/
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{knit_print.equation}
\alias{knit_print.equation}
\title{Print 'LaTeX' equations in R Markdown environments}
\usage{
\method{knit_print}{equation}(
  x,
  ...,
  tex_packages = "\\\\renewcommand*\\\\familydefault{\\\\rmdefault}"
)
}
\arguments{
\item{x}{'LaTeX' equation built with \code{\link{extract_eq}}}

\item{...}{not used}

\item{tex_packages}{A string with LaTeX code to include in the header,
usually to include LaTeX packages in the output.}
}
\value{
A string with the equation formatted according to R Markdown's output
format (different output for HTML, PDF, docx, gfm, markdown_strict). The
format is detected automatically, so, you do not have to worry about it.
}
\description{
Print 'LaTeX' equations built with \code{\link{extract_eq}} nicely in R Markdown environments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.equation}
\alias{print.equation}
\title{Print 'LaTeX' equations}
\usage{
\method{print}{equation}(x, ...)
}
\arguments{
\item{x}{'LaTeX' equation built with \code{\link{extract_eq}}}

\item{...}{not used}
}
\value{
The unmodified object 'x' is returned invisibly. The function is
used for its side effect of printing the equation.
}
\description{
Print 'LaTeX' equations built with \code{\link{extract_eq}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_eq.R
\name{extract_eq}
\alias{extract_eq}
\title{'LaTeX' equation for R models}
\usage{
extract_eq(
  model,
  intercept = "alpha",
  greek = "beta",
  greek_colors = NULL,
  subscript_colors = NULL,
  var_colors = NULL,
  var_subscript_colors = NULL,
  raw_tex = FALSE,
  swap_var_names = NULL,
  swap_subscript_names = NULL,
  ital_vars = FALSE,
  label = NULL,
  index_factors = FALSE,
  show_distribution = FALSE,
  wrap = FALSE,
  terms_per_line = 4,
  operator_location = "end",
  align_env = "aligned",
  use_coefs = FALSE,
  coef_digits = 2,
  fix_signs = TRUE,
  font_size = NULL,
  mean_separate = NULL,
  return_variances = FALSE,
  se_subscripts = FALSE,
  logit_notation = FALSE,
  ...
)
}
\arguments{
\item{model}{A fitted model}

\item{intercept}{How should the intercept be displayed? Default is \code{"alpha"},
but can also accept \code{"beta"}, in which case the it will be displayed
as beta zero.}

\item{greek}{What notation should be used for
coefficients? Currently only accepts \code{"beta"} (with plans for future
development). Can be used in combination with \code{raw_tex} to use any
notation, e.g., \code{"\\hat{\\beta}"}.}

\item{greek_colors}{The colors of the greek notation in the equation. Must
be a single color (named or HTML hex code) or a vector of colors (which
will be recycled if smaller than the number of terms in the model). When
rendering to PDF, I suggest using HTML hex codes, as not all named colors
are recognized by LaTeX, but equatiomatic will internally create the
color definitions for you if HTML codes are supplied. Note that this is
not yet implemented for mixed effects models (lme4).}

\item{subscript_colors}{The colors of the subscripts for the greek notation.
The argument structure is equivalent to \code{greek_colors} (i.e., see
above for more detail).}

\item{var_colors}{The color of the variable names. This takes a named vector
of the form \code{c("variable" = "color")}. For example
\code{c("bill_length_mm" = "#00d4fa", "island" = "#00fa85")}. Colors can
be names (e.g., \code{"red"}) or HTML hex codes, as shown in the example.}

\item{var_subscript_colors}{The colors of the factor subscripts for
categorical variables. The interface for this is equivalent to
\code{var_colors}, and all subscripts for a given variable will be
displayed in the provided color. For example, the code
\code{c("island" = "green")} would result in the subscripts for "Dream"
and "Torgersen" being green (assuming "Biscoe" was the reference group).}

\item{raw_tex}{Logical. Is the greek code being passed to denote coefficients
raw tex code?}

\item{swap_var_names}{A vector of the form c("old_var_name" = "new name").
For example: c("bill_length_mm" = "Bill Length (MM)").}

\item{swap_subscript_names}{A vector of the form
c("old_subscript_name" = "new name"). For example:
c("f" = "Female").}

\item{ital_vars}{Logical, defaults to \code{FALSE}. Should the variable names
not be wrapped in the \code{\\operatorname{}} command?}

\item{label}{A label for the equation, which can then be used for in-text
references. See example \href{https://www.overleaf.com/learn/latex/Cross_referencing_sections,_equations_and_floats#Referencing_equations.2C_figures_and_tables}{here}.
Note that this \strong{only works for PDF output}. The in-text references also
must match the label exactly, and must be formatted as
\code{\\ref{eq: label}}, where \code{label} is a place holder for the
specific label. Notice the space after the colon before the label. This
also must be there, or the cross-reference will fail.}

\item{index_factors}{Logical, defaults to \code{FALSE}. Should the factors
be indexed, rather than using subscripts to display all levels?}

\item{show_distribution}{Logical. When fitting a logistic or probit
regression, should the binomial distribution be displayed? Defaults to
\code{FALSE}.}

\item{wrap}{Logical, defaults to \code{FALSE}. Should the terms on the
right-hand side of the equation be split into multiple lines? This is
helpful with models with many terms.}

\item{terms_per_line}{Integer, defaults to 4. The number of right-hand side
terms to include per line. Used only when \code{wrap} is \code{TRUE}.}

\item{operator_location}{Character, one of \dQuote{end} (the default) or
\dQuote{start}. When terms are split across multiple lines, they are split
at mathematical operators like \code{+}. If set to \dQuote{end}, each line will
end with a trailing operator (\code{+} or \code{-}). If set to \dQuote{start}, each
line will begin with an operator.}

\item{align_env}{TeX environment to wrap around equation. Must be one of
\code{aligned}, \code{aligned*}, \code{align}, or \code{align*}. Defaults
to \code{aligned}.}

\item{use_coefs}{Logical, defaults to \code{FALSE}. Should the actual model
estimates be included in the equation instead of math symbols?}

\item{coef_digits}{Integer, defaults to 2. The number of decimal places to
round to when displaying model estimates.}

\item{fix_signs}{Logical, defaults to \code{TRUE}. If disabled,
coefficient estimates that are negative are preceded with a "+" (e.g.
\code{5(x) + -3(z)}). If enabled, the "+ -" is replaced with a "-" (e.g.
\code{5(x) - 3(z)}).}

\item{font_size}{The font size of the equation. Defaults to default of
the output format. Takes any of the standard LaTeX arguments (see
\href{https://www.overleaf.com/learn/latex/Font_sizes,_families,_and_styles#Font_styles}{here}).}

\item{mean_separate}{Currently only support for \code{\link[lme4]{lmer}}
models. Should the mean structure be inside or separated from the
normal distribution? Defaults to \code{NULL}, in which case it will become
\code{TRUE} if there are more than three fixed-effect parameters. If
\code{TRUE}, the equation will be displayed as, for example,
outcome ~ N(mu, sigma); mu = alpha + beta_1(wave). If \code{FALSE}, this
same equation would be outcome ~ N(alpha + beta, sigma).}

\item{return_variances}{Logical. When \code{use_coefs = TRUE} with a mixed
effects model (e.g., \code{lme4::lmer()}), should the variances and
co-variances be returned? If \code{FALSE} (the default) standard deviations
and correlations are returned instead.}

\item{se_subscripts}{Logical. If \code{se_subscripts = TRUE} then the
equation will include the standard errors below each coefficient.
This is supported for lm and glm models.}

\item{logit_notation}{Logical. If \code{TRUE}, then the equation uses a more
compact notation for a logistic regression.}

\item{...}{Additional arguments (for future development; not currently used).}
}
\value{
A character of class \dQuote{equation}.
}
\description{
Extract the variable names from a model to produce a 'LaTeX' equation.
Supports any model where there is a \code{\link[broom:reexports]{broom::tidy()}} method. This is a generic
function with methods for lmerMod objects obtained with \code{\link[lme4:lmer]{lme4::lmer()}},
glmerMod objects with \code{\link[lme4:glmer]{lme4::glmer()}}, forecast_ARIMA with
\code{\link[forecast:Arima]{forecast::Arima()}} and default, with the later further covering most "base"
R models implemented in \code{\link[broom:reexports]{broom::tidy()}} like lm objects with \code{\link[stats:lm]{stats::lm()}},
glm objects with \code{\link[stats:glm]{stats::glm()}} or polr objects with \code{\link[MASS:polr]{MASS::polr()}}. The
default method also supports clm objects obtained with \code{\link[ordinal:clm]{ordinal::clm()}}.
}
\details{
The different methods all use the same arguments, but not all arguments are
suitable to all models. Check here above to determine if a feature is
implemented for a given model.
}
\examples{
# Simple model
mod1 <- lm(mpg ~ cyl + disp, mtcars)
extract_eq(mod1)

# Include all variables
mod2 <- lm(mpg ~ ., mtcars)
extract_eq(mod2)

# Works for categorical variables too, putting levels as subscripts
data("penguins", package = "equatiomatic")
mod3 <- lm(body_mass_g ~ bill_length_mm + species, penguins)
extract_eq(mod3)

set.seed(8675309)
d <- data.frame(
  cat1 = rep(letters[1:3], 100),
  cat2 = rep(LETTERS[1:3], each = 100),
  cont1 = rnorm(300, 100, 1),
  cont2 = rnorm(300, 50, 5),
  out = rnorm(300, 10, 0.5)
)
mod4 <- lm(out ~ ., d)
extract_eq(mod4)

# Don't italicize terms
extract_eq(mod1, ital_vars = FALSE)

# Wrap equations in an "aligned" environment
extract_eq(mod2, wrap = TRUE)

# Wider equation wrapping
extract_eq(mod2, wrap = TRUE, terms_per_line = 4)

# Include model estimates instead of Greek letters
extract_eq(mod2, wrap = TRUE, terms_per_line = 2, use_coefs = TRUE)

# Don't fix doubled-up "+ -" signs
extract_eq(mod2, wrap = TRUE, terms_per_line = 4, use_coefs = TRUE, fix_signs = FALSE)

# Use indices for factors instead of subscripts
extract_eq(mod2, wrap = TRUE, terms_per_line = 4, index_factors = TRUE)

# Use other model types, like glm
set.seed(8675309)
d <- data.frame(
  out = sample(0:1, 100, replace = TRUE),
  cat1 = rep(letters[1:3], 100),
  cat2 = rep(LETTERS[1:3], each = 100),
  cont1 = rnorm(300, 100, 1),
  cont2 = rnorm(300, 50, 5)
)
mod5 <- glm(out ~ ., data = d, family = binomial(link = "logit"))
extract_eq(mod5, wrap = TRUE)
# More compact logit notation
extract_eq(mod5, wrap = TRUE, logit_notation = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{renderEq}
\alias{renderEq}
\alias{eqOutput}
\title{Display equations in shiny apps}
\usage{
renderEq(expr, env = parent.frame(), quoted = FALSE, outputArgs = list())

eqOutput(outputId)
}
\arguments{
\item{expr}{An R expression, specifically a call to \code{\link[=extract_eq]{extract_eq()}}}

\item{env}{The environment}

\item{quoted}{Is the expression quoted?}

\item{outputArgs}{list of output arguments}

\item{outputId}{The identifier of the output from the server. Should be
passed as a string.}
}
\value{
Render the equation in a suitable way for Shiny for \code{\link[=renderEq]{renderEq()}} in
an \code{\link[=eqOutput]{eqOutput()}} equation output element that can be included in a panel.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
These are a set of functions designed to help render equations in
Shiny applications (see the vignette about Shiny).
}
\section{Functions}{
\itemize{
\item \code{renderEq()}: Rendering function

\item \code{eqOutput()}: Output function

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polls.R
\docType{data}
\name{polls}
\alias{polls}
\title{The polls data from Gelman and Hill ()}
\format{
A tibble with 13,544 rows and 7 variables:
\describe{
\item{state}{An integer denoting the state identification number.}
\item{edu}{An ordered factor  stating the education level of the respondent}
\item{age}{An unordered factor stating the age of range of the respondent}
\item{female}{A dummy variable (integer) denoting whether the respondent was
coded as male (female = 0) or female (female = 1).}
\item{black}{A dummy variable (integer) denoting whether the respondent was
coded as Black (black = 1) or not Black (black = 0).}
\item{weight}{A sampling weight}
\item{bush}{Whether the respondent stated they were in favor of voting for
George Bush Sr.}
}
}
\usage{
polls
}
\description{
This is the dataset used in Gelman & Hill's book, Data Analysis Using
Regression and Multilevel/Hierarchical Models. They are polling data on
the presidential election from 1988, collected one week before the election.
You can download all the data from the book here: http://www.stat.columbia.edu/~gelman/arm/examples/ARM_Data.zip.
Note that this is only a few of the variables from the original data supplied
with the book.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equatiomatic-package.R
\docType{package}
\name{equatiomatic-package}
\alias{equatiomatic}
\alias{equatiomatic-package}
\title{equatiomatic: Transform Models into 'LaTeX' Equations}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The goal of 'equatiomatic' is to reduce the pain associated with writing 'LaTeX' formulas from fitted models. The primary function of the package, extract_eq(), takes a fitted model object as its input and returns the corresponding 'LaTeX' code for the model.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/datalorax/equatiomatic}
  \item \url{https://datalorax.github.io/equatiomatic/}
  \item Report bugs at \url{https://github.com/datalorax/equatiomatic/issues}
}

}
\author{
\strong{Maintainer}: Philippe Grosjean \email{phgrosjean@sciviews.org} (\href{https://orcid.org/0000-0002-2694-9471}{ORCID})

Authors:
\itemize{
  \item Daniel Anderson \email{daniela@uoregon.edu} (\href{https://orcid.org/0000-0003-4699-4680}{ORCID})
  \item Andrew Heiss \email{andrew@andrewheiss.com} (\href{https://orcid.org/0000-0002-3948-3914}{ORCID})
  \item Jay Sumners \email{jay.sumners@gmail.com}
}

Other contributors:
\itemize{
  \item Joshua Rosenberg \email{jmrosenberg@utk.edu} (\href{https://orcid.org/0000-0003-2170-0447}{ORCID}) [contributor]
  \item Jonathan Sidi \email{yonis@metrumrg.com} (\href{https://orcid.org/0000-0002-4222-1819}{ORCID}) [contributor]
  \item Ellis Hughes \email{ellishughes@live.com} (\href{https://orcid.org/0000-0003-0637-4436}{ORCID}) [contributor]
  \item Thomas Fung \email{thomas.fung.dr@gmail.com} (\href{https://orcid.org/0000-0003-2601-0728}{ORCID}) [contributor]
  \item Reza Norouzian \email{rnorouzian@gmail.com} (\href{https://orcid.org/0000-0002-0531-6886}{ORCID}) [contributor]
  \item Indrajeet Patil \email{patilindrajeet.science@gmail.com} (\href{https://orcid.org/0000-0003-1995-6531}{ORCID}) [contributor]
  \item Quinn White \email{quinnarlise@gmail.com} (\href{https://orcid.org/0000-0001-5399-0237}{ORCID}) [contributor]
  \item David Kane \email{dave.kane@gmail.com} [contributor]
  \item Teun van den Brand \email{t.vandenbrand@nki.nl} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID}) [contributor]
  \item Guyliann Engels \email{guyliann.engels@umons.ac.be} (\href{https://orcid.org/0000-0001-9514-1014}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple_ts.R
\docType{data}
\name{simple_ts}
\alias{simple_ts}
\title{Simple simulated time series data}
\format{
A tibble with 1000 rows and 8 variables:
\describe{
\item{Qtr1}{First quarter simulated values.}
\item{Qtr2}{Second quarter simulated values.}
\item{Qtr3}{Third quarter simulated values.}
\item{Qtr4}{Fourth quarter simulated values.}
}
}
\usage{
simple_ts
}
\description{
Output from \code{set.seed(42); simple_ts <- ts(rnorm(1000),freq = 4)}.
This is included primarily for unit testing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_longitudinal.R
\docType{data}
\name{sim_longitudinal}
\alias{sim_longitudinal}
\title{Simulated longitudinal data}
\format{
A tibble with 1000 rows and 8 variables:
\describe{
\item{sid}{An integer denoting the individual student.
There are 100 students.}
\item{school}{An integer denoting the school There are 15 schools.}
\item{district}{An integer denoting the school district. There are
5 districts.}
\item{group}{A character variable denoting the instructional level of
the student, low, medium, or high.}
\item{treatment}{A factor indicating whether the student received the
intervention treatment (0 = no treatment received; 1 = treatment
received).}
\item{prop_low}{The proportion of student in the school in the low
instructional group.}
\item{wave}{The assessment wave. Each student has nine waves of data
collection}
\item{score}{The individual students' score at the given wave.}
}
}
\usage{
sim_longitudinal
}
\description{
Data are simulated to be similar to longitudinal data collected within
schools/districts.
}
\keyword{datasets}
